<?

 /*****************************************************************************
 * Filename: include/data.base.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");

 define("REGEXP_DATA_PROJECT", "/^([a-z0-9]{1,25})$/i");
 define("REGEXP_DATA_MODULE", "/^([a-z]{1,25})$/i");
 define("REGEXP_DATA_PLUGIN", "/^([a-z0-9\_]{1,25}|.tmp)$/i");
 define("REGEXP_DATA_PLUGIN_MODULE", "/^([a-z0-9\_]{1,25})_([a-z]{1,25})$/i");
 define("REGEXP_DATA_ACTION", "/^([a-z]{1,25})$/i");
 define("REGEXP_DATA_EXTRA", "/^([a-z]{1,25})$/i");
 define("REGEXP_DATA_FIELDNAME", "/^([a-z0-9_]{1,50})$/i");
 define("REGEXP_DATA_ENUMOPTION", "/^([a-z0-9 \-\_\.]{0,50})$/i");
 define("REGEXP_DATA_LISTOPTION", "/^([^\"\;]*)$/i");
 define("REGEXP_DATA_HOSTNAME", "/^([a-z0-9_\-\.]+)$/i");
 define("REGEXP_DATA_DOMAIN", "/^([a-z0-9_\-\.]+)\.([a-z0-9]+)$/i");
 define("REGEXP_DATA_EMAIL", "/^([a-z0-9_\-\.]+)\@([a-z0-9_\-\.]+)\.([a-z0-9]+)$/i");
 define("REGEXP_DATA_URL", "/^([a-z0-9]+\:\/\/)?(([^:]*):?(.*)@)?([a-z0-9][a-z0-9_\-\.]*)(:(\d+))?(\/[^\?]*)?(\?.*)?$/i");
 define("REGEXP_DATA_IP", "/^(((\d{1,3}\.){3}\d{1,3})|(([a-f0-9]{1,4}:{1,2}){1,7}[a-f0-9]{1,4}))$/i");
 define("REGEXP_DATA_IPV6", "/^(([a-f0-9]{1,4}:{1,2}){1,7}[a-f0-9]{1,4})$/i");
 define("REGEXP_DATA_IPV4", "/^((\d{1,3}\.){3}\d{1,3})$/i");


/*
 Base class for Data class
 Contains all useful function for creating modules etc
 All data manipulation function can be found in Data class

 init(): Initialization of static array etc


*/

 //abstract class BaseData //php5
 class BaseData
 {
  function __construct()
  {
   $this->e=new Errors();

   $this->l=&$this->e->l;
   $this->l->read("data");
  }

  function __clone()
  {
   //save original
   $tmpE=&$this->e;
   $tmpL=&$this->e->l;

   //restore original language (dereference), so it's cloned correctly
   unset($this->e->l);
   $this->e->l=$tmpL;

   //clone object
   unset($this->e);
   $this->e=clone $tmpE;

   //restore original
   unset($tmpE->l);
   $tmpE->l=&$tmpL;

   //update references
   unset($this->l);
   $this->l=&$this->e->l;
  }


  function init()
  {
   global $_config;

//rights reference array
   $this->rights["category"]=Array();
   $this->rights["category"]["view"]    =0;
   $this->rights["category"]["search"]  =0;
   $this->rights["category"]["tree"]    =0;
   $this->rights["category"]["add"]     =1;
   $this->rights["category"]["copy"]    =1;
   $this->rights["category"]["edit"]    =2;
   $this->rights["category"]["delete"]  =3;
   $this->rights["category"]["undelete"]=3;
   $this->rights["category"]["move"]    =3;
   $this->rights["category"]["owner"]   =5;//reversed, looks prettier in cms
   $this->rights["category"]["rights"]  =4;
   $this->rights["content"]=Array();
   $this->rights["content"]["view"]     =8;
   $this->rights["content"]["search"]   =8;
   $this->rights["content"]["tree"]     =8;
   $this->rights["content"]["add"]      =9;
   $this->rights["content"]["copy"]     =9;
   $this->rights["content"]["edit"]     =10;
   $this->rights["content"]["delete"]   =11;
   $this->rights["content"]["undelete"] =11;
   $this->rights["content"]["move"]     =11;
   $this->rights["content"]["owner"]    =12;
   $this->rights["content"]["user"]     =13;

   $this->format=Array();

   //sql (database) data formatting
   $this->format["sql"]["dateformat"]="Y-m-d";
   $this->format["sql"]["timeformat"]="H:i:s";
   $this->format["sql"]["datetimeformat"]="Y-m-d H:i:s";
   $this->format["sql"]["decimals"]="10";
   $this->format["sql"]["decimalseparator"]=".";
   $this->format["sql"]["thousandsseparator"]="";

   $config=$_config->get(false); //get configuration

   if(isset($config["sqltimezone"]))
    $this->format["sql"]["timezone"]=$config["sqltimezone"];
   else
    $this->format["sql"]["timezone"]="UTC";

   //html (output) data formatting
   if(isset($config["timezone"]))
    $this->format["html"]["timezone"]=$config["timezone"];
   else
    $this->format["html"]["timezone"]=date("e");

   if(isset($config["dateformat"]))
    $this->format["html"]["dateformat"]=$config["dateformat"];
   else
    $this->format["html"]["dateformat"]="d-m-Y";

   if(isset($config["timeformat"]))
    $this->format["html"]["timeformat"]=$config["timeformat"];
   else
    $this->format["html"]["timeformat"]="H:i:s";

   if(isset($config["datetimeformat"]))
    $this->format["html"]["datetimeformat"]=$config["datetimeformat"];
   else
    $this->format["html"]["datetimeformat"]="H:i:s d-m-Y";

   if(isset($config["decimals"]))
    $this->format["html"]["decimals"]=$config["decimals"];
   else
    $this->format["html"]["decimals"]="2";

   if(isset($config["decimalseparator"]))
    $this->format["html"]["decimalseparator"]=$config["decimalseparator"];
   else
    $this->format["html"]["decimalseparator"]=".";

   if(isset($config["thousandsseparator"]))
    $this->format["html"]["thousandsseparator"]=$config["thousandsseparator"];
   else
    $this->format["html"]["thousandsseparator"]="";

   //text (output) data formatting
   $this->format["text"]=$this->format["html"];

   //form (input) data formatting
   if(isset($config["formtimezone"]))
    $this->format["form"]["timezone"]=$config["formtimezone"];
   else
    $this->format["form"]["timezone"]=date("e");

   if(isset($config["formdateformat"]))
    $this->format["form"]["dateformat"]=$config["formdateformat"];
   else
    $this->format["form"]["dateformat"]="d-m-Y";

   if(isset($config["formtimeformat"]))
    $this->format["form"]["timeformat"]=$config["formtimeformat"];
   else
    $this->format["form"]["timeformat"]="H:i:s";

   if(isset($config["formdatetimeformat"]))
    $this->format["form"]["datetimeformat"]=$config["formdatetimeformat"];
   else
    $this->format["form"]["datetimeformat"]="H:i:s d-m-Y";

   if(isset($config["formdecimals"]))
    $this->format["form"]["decimals"]=$config["formdecimals"];
   else
    $this->format["form"]["decimals"]="2";

   if(isset($config["formdecimalseparator"]))
    $this->format["form"]["decimalseparator"]=$config["formdecimalseparator"];
   else
    $this->format["form"]["decimalseparator"]=".";

   if(isset($config["formthousandsseparator"]))
    $this->format["form"]["thousandsseparator"]=$config["formthousandsseparator"];
   else
    $this->format["form"]["thousandsseparator"]="";

   //API data formatting
   if(isset($config["apitimezone"]))
    $this->format["api"]["timezone"]=$config["apitimezone"];
   else
    $this->format["api"]["timezone"]="UTC";

   if(isset($config["apidateformat"]))
    $this->format["api"]["dateformat"]=$config["apidateformat"];
   else
    $this->format["api"]["dateformat"]="Y-m-d";

   if(isset($config["apitimeformat"]))
    $this->format["api"]["timeformat"]=$config["apitimeformat"];
   else
    $this->format["api"]["timeformat"]="H:i:s";

   if(isset($config["apidatetimeformat"]))
    $this->format["api"]["datetimeformat"]=$config["apidatetimeformat"];
   else
    $this->format["api"]["datetimeformat"]="Y-m-d H:i:s";

   if(isset($config["apidecimals"]))
    $this->format["api"]["decimals"]=$config["apidecimals"];
   else
    $this->format["api"]["decimals"]="10";

   if(isset($config["apidecimalseparator"]))
    $this->format["api"]["decimalseparator"]=$config["apidecimalseparator"];
   else
    $this->format["api"]["decimalseparator"]=".";

   if(isset($config["apithousandsseparator"]))
    $this->format["api"]["thousandsseparator"]=$config["apithousandsseparator"];
   else
    $this->format["api"]["thousandsseparator"]="";

//default available actions
   $this->actionList=Array();
   $this->addActions("view", "search");
   $this->addActions("add", "copy", "edit", "delete");
   $this->addActions("database");
   $this->addActions("reference");

   $this->extraList=Array();
//first option is default
   $this->addExtra("view", "item", "list", "all"); //and deleted
   $this->addExtra("add", "item");
   $this->addExtra("copy", "item");
   $this->addExtra("edit", "item");
   $this->addExtra("move", "item", "list");
   $this->addExtra("order", "up", "down");
   $this->addExtra("delete", "item", "list", "all");
   $this->addExtra("undelete", "item", "list", "all");
   $this->addExtra("rights", "item", "list", "all");
   $this->addExtra("owner", "item", "list", "all");
   $this->addExtra("user", "login", "logout", "password");
   $this->addExtra("search", "title", "field", "show", "full"); //and deleted
   $this->addExtra("tree", "up", "down");
   $this->addExtra("lost", "andfound");
   $this->addExtra("database", "info", "create", "alter");
   $this->addExtra("reference", "check");

   //set default action+extra
   $this->action="view";
   $this->extra="item";

   $this->fieldTypes=Array(
    "nr", "float", "price", "version",
    "html", "text", "title", "search", "alphabetic", "url", "hostname", "domain", "email", "regexp", "phone",
    "file", "image", "filename", "localfile", "localfiles", "localdirectory",
    "date", "time", "datetime",
    "ip", "ipv4", "ipv6", "multilist", "modules", "list",
    "enum", "boolean", "password", "captcha", "filedata",
    "rights", "action", "extra",
    "module", "plugin", "project",
    "data", "filedata", "array",
    "submit", "label", //CMS fields, not allowed in database
   );

   $this->mappableFieldTypes=Array(
    "nr",
    "ip", "ipv4", "ipv6",
    "date", "time", "datetime",
   );

   $this->sqlMaxRequiredFieldTypes=Array(
    "url", "html", "text", "title", "phone", "domain", "hostname", "email", "regexp", "filename", "alphabetic", "captcha",
    "list", "multilist", "module", "modules", "plugin", "action", "extra", "project",
    "filedata",
    "password", //only when nocrypt property is used
   );

   $this->searchableFieldTypes=Array(
    "html", "text", "title", "search", "alphabetic", "url", "hostname", "domain", "email", "regexp", "phone",
    "file", "image", "filename", "localfile", "localfiles", "localdirectory",
    "date", "time", "datetime",
    "multilist", "modules", "list",
    "enum", "password", "captcha",
    "action", "extra",
    "module", "plugin", "project",
   );

   $this->fieldOptionsFieldTypes=Array(
    "enum", "list", "multilist",
    "module", "modules", "plugin",
    "project",
    "nr", "float", "price",
    "text", "html", "url", "title", "hostname", "domain", "email", "phone", "regexp", "search", "filename", "alphabetic", "ip", "ipv4", "ipv6",
    "localfile", "localfiles", "localdirectory",
   );

   $this->fieldProperties=Array(
    "default"    => Array( //applies to all field types, including the ones below
     "default", "global", "control", "title", "trim", //normal
     "forceUpdate", "noUpdate", //update
     "owner", "username", "password", "rightsReference",  //users/category
     "reload", "hidden", "disabled", "options", //cms
    ),

    "nr"         => Array("identifier", "order", "big", "dropdown"),
    "text"       => Array("identifier", "markup"),
    "html"       => Array("wysiwyg"),
    "rights"     => Array("allusers"),
    "version"    => Array("singleField"),

    "module"     => Array("exists"),
    "modules"    => Array("exists"),

    "boolean"    => Array("deleted"),

    "password"   => Array("nocrypt", "md5"),

    "file"           => Array("md5", "count"),
    "localfile"      => Array("recursive", "count"),
    "localfiles"     => Array("recursive"),
    "localdirectory" => Array("recursive"),
   );

   $this->fieldMutilations=Array(
    "nr"         => Array("humanReadableNumber", "humanReadableFileSize", "leadingZeros", "replace"),
    "float"      => Array("humanReadableNumber", "humanReadableFileSize", "leadingZeros", "replace"),
    "price"      => Array("humanReadableNumber", "humanReadableFileSize", "leadingZeros", "replace"),
    "version"    => Array("replace"),

    "text"       => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "html"       => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "title"      => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "alphabetic" => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "url"        => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "hostname"   => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "domain"     => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "email"      => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode", "scrambleEmail"),
    "phone"      => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "search"     => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "filename"   => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),

    "data"       => Array(),
    "regexp"     => Array(),
    "array"      => Array("humanReadableNumber", "humanReadableFileSize", "leadingZeros", "strip", "replace", "case", "trim", "stripTags", "encode", "decode", "scrambleEmail"),

    "list"       => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "multilist"  => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),

    "module"     => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "modules"    => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "plugin"     => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "action"     => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "extra"      => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "project"    => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),

    "enum"       => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "boolean"    => Array("strip", "replace", "case"),
    "rights"     => Array("replace", "leadingZeros"),

    "password"   => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "captcha"    => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),

    "file"       => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "image"      => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "filedata"   => Array("humanReadableNumber", "humanReadableFileSize", "leadingZeros", "strip", "replace", "case", "trim", "stripTags", "encode", "decode", "scrambleEmail"),

    "localfile"  => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "localfiles" => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),
    "localdirectory" => Array("strip", "replace", "case", "trim", "stripTags", "encode", "decode"),

    "ip"         => Array("strip", "replace", "case", "trim", "encode", "decode"),
    "ipv4"       => Array("strip", "replace"),
    "ipv6"       => Array("strip", "replace", "case", "trim", "encode", "decode"),

    "date"       => Array("dateConvert", "timeAgo", "replace", "encode", "decode"),
    "time"       => Array("dateConvert", "timeAgo", "replace", "encode", "decode"),
    "datetime"   => Array("dateConvert", "timeAgo", "timezoneConvert", "replace", "encode", "decode"),
   );


   //first is default
   $this->fieldOperators=Array(
    "default"    => Array("=", ">=", "<=", ">", "<", "!="), //nr, float, price

    "html"       => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "text"       => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "title"      => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "search"     => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "alphabetic" => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "url"        => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "hostname"   => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "domain"     => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "email"      => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "regexp"     => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "phone"      => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),

    "file"       => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "image"      => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "filename"   => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "filedata"   => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "localfile"  => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "localfiles" => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "localdirectory" => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),

    "date"       => Array("=", ">=", "<=", ">", "<", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "time"       => Array("=", ">=", "<=", ">", "<", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "datetime"   => Array("=", ">=", "<=", ">", "<", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),

    "ip"         => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "ipv4"       => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "ipv6"       => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),

    "multilist"  => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "modules"    => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),
    "list"       => Array("=", "!=", "like", "not like", "start like", "not start like", "end like", "not end like"),

    "enum"       => Array("=", "!="),
    "boolean"    => Array("=", "!="),
    "password"   => Array("=", "!="),
    "captcha"    => Array("=", "!="),

    "rights"     => Array("=", "!=", "&", "|"),
    "action"     => Array("=", "!="),
    "extra"      => Array("=", "!="),

    "module"     => Array("=", "!="),
    "plugin"     => Array("=", "!="),
    "project"    => Array("=", "!="),
   );

   $this->referenceTypes=Array("1:1", "*:0", "*:1", "0:*", "1:*", "*:*");
   $this->defaultOperant="AND";

   $this->ID=false;
   $this->rightsReference=false;
   $this->selfReference=false;

   $this->extends=Array();
   $this->extendedBy=Array();
  }


/******************************************************************
addAction

Adds a new action

******************************************************************/

  function addActions()
  {
   $args=func_get_args();
   $argc=sizeof($args);

   for($c=0;$c<$argc;$c++)
    if(!$this->validAction($args[$c]))
     $this->actionList[]=$args[$c];

   sort($this->actionList);

   return true;
  }



/******************************************************************
removeActions

XXX: untested
******************************************************************/

  function removeActions()
  {
   $args=func_get_args();
   $argc=sizeof($args);

   for($c=0;$c<$argc;$c++)
    if(($key=array_search($args[$c], $this->actionList))!==false)
     unset($this->actionList[$key]);

   sort($this->actionList);

   return true;
  }


/******************************************************************
validAction
Returns true if action is valid and false if not
If no action is given the global action is checked


******************************************************************/

  function validAction($action=false)
  {
   if(!$action){$action=$this->action;}
   $l=sizeof($this->actionList);

   for($c=0;$c<$l;$c++)
    if($this->actionList[$c]==$action)
     return true;

   return false; //illegal action
  }


/******************************************************************
addExtra

Adds an extra option to action

******************************************************************/

  function addExtra($action)
  {
   $args=func_get_args();
   $argc=sizeof($args);

   if(!isset($this->extraList[$action]))
    $this->extraList[$action]=Array();

   for($c=1;$c<$argc;$c++)
    if(!$this->validExtra($args[$c], $action))
     $this->extraList[$action][]=$args[$c];

   return true;
  }


/******************************************************************
removeExtra

XXX: untested
******************************************************************/

  function removeExtra($action)
  {
   $args=func_get_args();
   $argc=sizeof($args);

   for($c=1;$c<$argc;$c++)
    if(($key=array_search($args[$c], $this->extraList[$action]))!==false)
     unset($this->extraList[$action][$key]);

   sort($this->extraList[$action]);

   return true;
  }



/******************************************************************
validExtra
Returns true if extra options is valid for current action and false if not
If no action or extra is given the global action/extra is checked

******************************************************************/

  function validExtra($extra=false, $action=false)
  {
   if(!$extra){$extra=$this->extra;}
   if(!$action){$action=$this->action;}

   if(isset($this->extraList[$action])&&in_array($extra, $this->extraList[$action]))
    return true;

   return false; //illegal action
  }


  function getReferences($field, $type=false)
  {
   $references=Array();
   foreach(array_keys($this->reference) as $name)
   {
    if($this->reference[$name]["from"]==$field)
    {
     if(!$type||$this->reference[$name]["type"]==$type)
      $references[]=$name;
    }
   }

   return $references;
  }


/******************************************************************
getReferenceDepth
Returns the depth level of show references (can be used in as this->get($depth) parameter

******************************************************************/

  function getReferenceDepth()
  {
   //detect if any non control fields are shown
   $depth=false;
   foreach($this->check("use", "show") as $name)
   {
    if(!$this->check("properties", "control", $name))
    {
     $depth=true;
     break;
    }
   }

   if(!$depth) //no fields to gather for this module
    return false;

   $depth=0;
   foreach(array_keys($this->reference) as $name)
   {
    if($this->check("use", "show", $name))
    {
     switch($this->reference[$name]["type"])
     {
      case "*:*":
      {
       //value already in current module so depth only increases if reference is active
       if(isset($this->reference[$name]["data"])) //reference exist
       {
        if(($depth2=$this->reference[$name]["data"]->getReferenceDepth())!==false) //fields to show for reference
        {
         if($depth2>$depth)
          $depth=$depth2;
        }
       }

       break;
      }
      default:
      //case "0:*":
      //case "1:*":
      //case "*:0":
      //case "*:1":
      //case "1:1":
      {
       //value already in current module so depth only increases if reference is active
       if(isset($this->reference[$name]["data"])) //reference exist
       {
        if(($depth2=$this->reference[$name]["data"]->getReferenceDepth())!==false) //fields to show for reference
        {
         if($depth2+1>$depth)
          $depth=$depth2+1;
        }
       }

       break;
      }
     }
    }
   }

   return $depth;
  }



/******************************************************************
setDataFormat
Validates and set the new data format to the module and all referenced (active) modules

******************************************************************/

  function setDataFormat($dataformat)
  {
   if($dataformat==$this->dataformat)
    return true;

   if(!isset($this->format[$dataformat]))
   {
    $this->e->add("Unknown data format: \"".$dataformat."\"", "error");
    return false;
   }

   $this->dataformat=$dataformat;

   foreach(array_keys($this->reference) as $name)
    if(isset($this->reference[$name]["data"]))
     if(!$this->reference[$name]["data"]->setDataFormat($dataformat))
      return false;

   return true;
  }



/* Field functions */

  function mapValue($name, $entry=0)
  {
   global $_config;

   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't map value for unknown field: \"".$name."\"", "system");
    return false;
   }

   if(!in_array($this->field[$name]["type"], $this->mappableFieldTypes))
   {
    $this->e->add("Can't map value for unsupported type: \"".$this->field[$name]["type"]."\" for field: \"".$name."\"", "system");
    return false;
   }

   if(!isset($this->field[$name]["value"][$entry]))
   {
    $this->e->add("Can't map value for unknown entry: \"".$entry."\" for field: \"".$name."\"", "system");
    return false;
   }

   $type=$this->field[$name]["type"];

   if($type=="datetime"||$type=="date"||$type=="time")
   {
    //replace date/time "now" with actual date
    if(preg_match("/^now([\-\+]?)(\d*)$/", $this->field[$name]["value"][$entry], $m))
    {
     $time=time();

     //add time difference
     if($m[1]&&$m[2])
     {
      if($m[1]=="-")
       $time-=$m[2];
      else
       $time+=$m[2];
     }

     $this->field[$name]["value"][$entry]=date($this->field[$name]["format"][$this->dataformat][$type."format"], $time);

     //rectify datetime timezone if field timezone doesn't match default timezone
     if($type=="datetime")
      if($this->field[$name]["format"][$this->dataformat]["timezone"]!=date_default_timezone_get())
       $this->field[$name]["value"][$entry]=DT::timezoneConvertS($this->field[$name]["value"][$entry], date_default_timezone_get(), $this->field[$name]["format"][$this->dataformat]["timezone"], $this->field[$name]["format"][$this->dataformat]["datetimeformat"]);
    }
   }
   else if($type=="nr")
   {
    //usernr/rootnr
    if($this->field[$name]["value"][$entry]==="usernr")
     $this->field[$name]["value"][$entry]=$_config->get("usernr");
    else if($this->field[$name]["value"][$entry]==="rootnr")
     $this->field[$name]["value"][$entry]=$_config->get("rootnr");
   }
   else if($type=="ip"||$type=="ipv4"||$type=="ipv6")
   {
    if($this->field[$name]["value"][$entry]==="ip")
    {
     if(isset($_SERVER["REMOTE_ADDR"]))
      $this->field[$name]["value"][$entry]=$_SERVER["REMOTE_ADDR"];
     else if($type=="ipv6")
      $this->field[$name]["value"][$entry]="0::0";
     else
      $this->field[$name]["value"][$entry]="0.0.0.0";
    }
   }

   return true;
  }

  function mapValues($name=false)
  {
   $success=true;

   //name not set, do all fields
   if(!$name)
   {
    foreach(array_keys($this->field) as $name)
     if(in_array($this->field[$name]["type"], $this->mappableFieldTypes))
      if(!$this->mapValues($name))
       $success=false;

    return $success;
   }

   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't map values for unknown field: \"".$name."\"", "system");
    return false;
   }

   if(!in_array($this->field[$name]["type"], $this->mappableFieldTypes))
   {
    $this->e->add("Can't map values for unsupported type: \"".$this->field[$name]["type"]."\" for field: \"".$name."\"", "system");
    return false;
   }

   //map all values for mappableFieldTypes
   foreach(array_keys($this->field[$name]["value"]) as $key)
    if(!$this->mapValue($name, $key))
     $success=false;

   return $success;
  }


  function setValue($name, $value, $entry=0)
  {
   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't set value for unknown field: \"".$name."\"", "system");
    return false;
   }

   $this->field[$name]["valuetype"]="set";
   $this->field[$name]["value"][$entry]=$value;
   $this->field[$name]["operant"][$entry]=$this->defaultOperant;

   if($entry>=$this->vmax)
    $this->vmax=$entry+1;

   if(in_array($this->field[$name]["type"], $this->mappableFieldTypes))
    if(!$this->mapValue($name, $entry))
     return false;

   return true;
  }

  function unsetValue($name, $entry=0)
  {
   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't unset set value for unknown field: \"".$name."\"", "system");
    return false;
   }

   $this->field[$name]["valuetype"]="none";
   $this->field[$name]["value"][$entry]=false;
   $this->field[$name]["operant"][$entry]=$this->defaultOperant;

   return true;
  }

  function setDefaultValue($name, $entry=0)
  {
   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't set default value for unknown field: \"".$name."\"", "system");
    return false;
   }

   $this->field[$name]["valuetype"]="default";
   $this->field[$name]["value"][$entry]=$this->field[$name]["defaultvalue"];
   $this->field[$name]["operant"][$entry]=$this->defaultOperant;

   if($entry>=$this->vmax)
    $this->vmax=$entry+1;

   if(in_array($this->field[$name]["type"], $this->mappableFieldTypes))
    if(!$this->mapValue($name, $entry))
     return false;

   return true;
  }

  function getDefaultValue($name)
  {
   if(isset($this->field[$name])&&isset($this->field[$name]["defaultvalue"]))
    return $this->field[$name]["defaultvalue"];
   else
    return false;
  }

  function getValue($name, $entry=0)
  {
   if(isset($this->field[$name])&&isset($this->field[$name]["value"][$entry]))
    return $this->field[$name]["value"][$entry];
   else
    return false;
  }

  function getValues($name)
  {
   if(isset($this->field[$name])&&isset($this->field[$name]["value"]))
    return $this->field[$name]["value"];
   else
    return false;
  }

  function getValueType($name)
  {
   if(isset($this->field[$name]))
    return $this->field[$name]["valuetype"];
   else
    return false;
  }

  function searchValue($name, $value)
  {
   if(isset($this->field[$name])&&false!==($entry=array_search($value, $this->field[$name]["value"])))
    return $entry;
   else
    return false;
  }

  function clearValues($name=false)
  {
   if($name)
   {
    if(!isset($this->field[$name]))
    {
     $this->e->add("Can't clear values for unknown field: \"".$name."\"", "system");
     return false;
    }

    $this->field[$name]["valuetype"]="none";
    $this->field[$name]["value"]=Array();
    $this->field[$name]["operant"]=Array();
   }
   else //clear all values
   {
    foreach(array_keys($this->field) as $field)
     $this->clearValues($field);

    $this->vmax=0;
    $this->vc=0;
   }

   return true;
  }


  function addEntry($location=false)
  {
   if($location>=$this->vmax)
    $location=false;

   foreach(array_keys($this->field) as $name)
   {
    if(!$this->check("use", "show", $name)&&!$this->check("use", "sql", $name))
     continue;

    if($location===false) //push all entry at the end of array
    {
     $this->field[$name]["value"][]=$this->field[$name]["defaultvalue"];
     $this->field[$name]["operant"][]=$this->defaultOperant;

     if(in_array($this->field[$name]["type"], $this->mappableFieldTypes))
      if(!$this->mapValue($name, sizeof($this->field[$name]["value"])-1))
       return false;
    }
    else //add after entry
    {
     $left=array_slice($this->field[$name]["value"], 0, $location); //left side of array, include location
     $right=array_slice($this->field[$name]["value"], $location); //right side of array, starting location
     $left[]=$this->field[$name]["defaultvalue"]; //add copy location
     $this->field[$name]["value"]=array_merge($left, $right); //merge entries

     $left=array_slice($this->field[$name]["operant"], 0, $location); //left side of array, include location
     $right=array_slice($this->field[$name]["operant"], $location); //right side of array, starting location
     $left[]=$this->defaultOperant;
     $this->field[$name]["operant"]=array_merge($left, $right); //merge entries

     if(in_array($this->field[$name]["type"], $this->mappableFieldTypes))
      if(!$this->mapValue($name, $location))
       return false;
    }
   }

   $this->vmax++;
   return true;
  }

  function copyEntry($entry, $location=false)
  {
   if($location>=$this->vmax)
    $location=false;

   $copied=false;
   foreach(array_keys($this->field) as $name)
   {
    if(!$this->check("use", "show", $name)&&!$this->check("use", "sql", $name))
     continue;

    if(!isset($this->field[$name]["value"][$entry]))
     continue;

    $copied=true;
    if($location===false) //push all entry at the end of array
     $this->field[$name]["value"][]=$this->field[$name]["value"][$entry];
    else //copy to after entry
    {
     $left=array_slice($this->field[$name]["value"], 0, $location); //left side of array, include location
     $right=array_slice($this->field[$name]["value"], $location); //right side of array, starting location
     $left[]=$this->field[$name]["value"][$entry]; //add copy location
     $this->field[$name]["value"]=array_merge($left, $right); //merge entries

     $left=array_slice($this->field[$name]["operant"], 0, $location); //left side of array, include location
     $right=array_slice($this->field[$name]["operant"], $location); //right side of array, starting location
     $left[]=$this->defaultOperant;
     $this->field[$name]["operant"]=array_merge($left, $right); //merge entries
    }
   }

   if(!$copied)
    return false;

   $this->vmax++;
   return true;
  }

  function removeEntry($entry)
  {
   $removed=false;
   foreach(array_keys($this->field) as $name)
   {
    if(!isset($this->field[$name]["value"][$entry]))
     continue;

    $removed=true;
    if($entry==($this->vmax-1)) //remove last entry from arrays
    {
     unset($this->field[$name]["value"][$entry]);
     unset($this->field[$name]["operant"][$entry]);
    }
    else //remove middle entry
    {
     $left=array_slice($this->field[$name]["value"], 0, $entry); //left side of array, exclude entry
     $right=array_slice($this->field[$name]["value"], $entry+1); //right side of array, exclude entry
     $this->field[$name]["value"]=array_merge($left, $right); //merge entries

     $left=array_slice($this->field[$name]["operant"], 0, $entry); //left side of array, include afterEntry
     $right=array_slice($this->field[$name]["operant"], $entry+1); //right side of array, starting afterEntry
     $this->field[$name]["operant"]=array_merge($left, $right); //merge entries
    }
   }

   if(!$removed)
    return false;

   $this->vmax--;
   return true;
  }

/******************************************************************
 values
 Returns numbers of values in the data container

******************************************************************/

  function values()
  {
   return $this->vmax;
  }



/******************************************************************
 operator functions

******************************************************************/

  function setOperator($name, $operator, $entry=0)
  {
   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't set operator for unknown field: \"".$name."\"", "system");
    return false;
   }

   if(isset($this->fieldOperators[$this->field[$name]["type"]]))
    $type=$this->field[$name]["type"];
   else
    $type="default";

   if(!in_array($operator, $this->fieldOperators[$type]))
   {
    $this->e->add("Unknown operator: \"".$operator."\" for field: \"".$name."\" (".$this->field[$name]["type"].")", "system");
    return false;
   }

   $this->field[$name]["operator"][$entry]=$operator;

   return true;
  }

  function getOperator($name, $entry=0)
  {
   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't get operator for unknown field: \"".$name."\"", "system");
    return false;
   }

   if(isset($this->field[$name]["operator"][$entry])) //set
    return $this->field[$name]["operator"][$entry];

   if(isset($this->fieldOperators[$this->field[$name]["type"]])) //return default operator for field type
    $type=$this->field[$name]["type"];
   else
    $type="default";

   return $this->fieldOperators[$type][0];
  }

  function setOperant($name, $operant, $entry=0)
  {
   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't set operant for unknown field: \"".$name."\"", "system");
    return false;
   }

   if($operant==$this->defaultOperant)
    $this->field[$name]["operant"][$entry]=$this->defaultOperant;
   else
    $this->field[$name]["operant"][$entry]="OR";

   return true;
  }

  function switchOperant($name, $entry=0)
  {
   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't set operant for unknown field: \"".$name."\"", "system");
    return false;
   }

   if($this->field[$name]["operant"][$entry]==$this->defaultOperant)
    $this->field[$name]["operant"][$entry]="OR";
   else
    $this->field[$name]["operant"][$entry]=$this->defaultOperant;

   return true;
  }

  function getOperant($name, $entry=0)
  {
   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't get operant for unknown field: \"".$name."\"", "system");
    return false;
   }

   return $this->field[$name]["operant"][$entry];
  }



/******************************************************************

******************************************************************/
  function title($name=false, $translate=false)
  {
   if(!$name)
    return ($translate?$this->l->t($this->title):$this->title);
   else if(isset($this->field[$name])&&isset($this->field[$name]["title"]))
    return ($translate?$this->l->t($this->field[$name]["title"]):$this->field[$name]["title"]);
   else
    return false;
  }

/******************************************************************
 r
 Returns:
 -1 / "binary": binary rights string
 0 / "view":    true/false for view rights
 1 / "add":     true/false for add rights
 2 / "update":  true/false for update rights
 3 / "move":    true/false for move rights
 3 / "delete":  true/false for delete rights
 4 / "users":   true/false for user rights

 8 / "viewcontent":    true/false for view content rights
 9 / "addcontent":     true/false for add content rights
10 / "updatecontent":  true/false for update content rights
11 / "movecontent":    true/false for move content rights
11 / "deletecontent":  true/false for delete content rights

******************************************************************/

  function r($key=false, $group=false)
  {
   global $_config;

   if($_config->userRoot())
    return true;

   //determine key
   if(!$key) //no key
    $key=$this->action; //use current action

   //determine group
   if(!$group)
   {
    if($this->module=="categories"||isset($this->extends["categories"]))
     $group="category";
    else
     $group="content";
   }

   $entry=$this->rights[$group][$key];

   if($entry>=0&&strlen($this->r)>$entry&&$this->r[$entry]==1)
    return true;

   return false;
  }


  function addRightsEntry($group, $key, $entry)
  {
   if(!isset($this->rights[$group]))
    $this->rights[$group]=Array();

   $this->rights[$group][$key]=$entry;

   return true;
  }


  function removeRightsEntry($group, $key)
  {
   if(!isset($this->rights[$group]))
   {
    $this->e->add("Can't remove rights group: \"".$group."\"", "warning", "Group doesn't exist");
    return false;
   }

   if(!isset($this->rights[$group][$key]))
   {
    $this->e->add("Can't remove rights group: \"".$group."\" key: \"".$key."\"", "warning", "Group key doesn't exist");
    return false;
   }

   //remove entry
   unset($this->rights[$group][$key]);

   if(!$this->rights[$group]) //no more entries left in rights group
    unset($this->rights[$group]); //remove group

   return true;
  }


  function getRootRights()
  {
   $rootrights=0;
   foreach(array_keys($this->rights) as $group)
    foreach($this->rights[$group] as $key => $entry)
     $rootrights|=pow(2, $entry);

   return $rootrights;
  }


  function checkProperties($name, $properties)
  {
   if(!$properties)
    return true;

   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't check properties for unknown field: \"".$name."\"", "system");
    return false;
   }

   $ret=true;

   if(!is_array($properties))
    $properties=explode(";", $properties);

   foreach($properties as $property)
   {
    if($property)
    {
     if(!in_array($property, $this->fieldProperties["default"])&&(!isset($this->fieldProperties[$this->field[$name]["type"]])||!in_array($property, $this->fieldProperties[$this->field[$name]["type"]])))
     {
      $this->e->add("Unknown property: \"".$property."\" for field: \"".$name."\" (".$this->field[$name]["type"].")", "module");
      $ret=false;
     }
    }
   }

   return $ret;
  }

  function addMutilation($name, $mutilation)
  {
   $args=func_get_args();
   array_shift($args); //remove name
   array_shift($args); //remove mutilation

   //gather arguments
   if(isset($args[0])&&is_array($args[0]))
    $args=$args[0];

   //check if field exists
   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't add mutilation for unknown field: \"".$name."\"", "system");
    return false;
   }

   //check if mutilation is allowed for this field type
   if(!isset($this->fieldMutilations[$this->field[$name]["type"]])||!in_array($mutilation, $this->fieldMutilations[$this->field[$name]["type"]]))
   {
    $this->e->add("Unknown mutilation: \"".$mutilation."\" for field: \"".$name."\" (".$this->field[$name]["type"].")", "error");
    return false;
   }

   //functions that need at least 1 argument
   if(sizeof($args)<1&&(
    $mutilation=="leadingZeros"||
    $mutilation=="replace"||
    $mutilation=="case"||
    $mutilation=="dateConvert"||
    $mutilation=="timezoneConvert"
   ))
   {
    $this->e->add("Mutilation: \"".$mutilation."\" needs at least 1 argument for field: \"".$name."\" (".$this->field[$name]["type"].")", "error");
    return false;
   }

   //replace: check if pattern is valid
   if($mutilation=="replace"&&@preg_match($args[0], "")===false)
   {
    $this->e->add("Mutilation: \"".$mutilation."\" 1st argument is an invalid regexp pattern for field: \"".$name."\" (".$this->field[$name]["type"].")", "error");
    return false;
   }

   //replace: check if pattern is valid
   if($mutilation=="timezoneConvert"&&isset($this->field[$name]["mutilation"]["dateConvert"]))
   {
    $this->e->add("Mutilation: \"".$mutilation."\" should be done before \"dateConvert\" mutilation for field: \"".$name."\" (".$this->field[$name]["type"].")", "error");
    return false;
   }

   if($mutilation=="timeAgo")
   {
    if(isset($args[0])&&
     $args[0]!="years"&&
     $args[0]!="months"&&
     $args[0]!="weeks"&&
     $args[0]!="days"&&
     $args[0]!="hours"&&
     $args[0]!="minutes"&&
     $args[0]!="seconds"
    )
    {
     $this->e->add("Mutilation: \"".$mutilation."\" interval: \"".$args[0]."\" is invalid for field: \"".$name."\" (".$this->field[$name]["type"].")", "error");
     return false;
    }
   }

   if($mutilation=="case")
   {
    if(
     $args[0]!="lower"&&
     $args[0]!="upper"&&
     $args[0]!="first"&&
     $args[0]!="words"
    )
    {
     $this->e->add("Mutilation: \"".$mutilation."\" type: \"".$args[0]."\" is invalid for field: \"".$name."\" (".$this->field[$name]["type"].")", "error");
     return false;
    }
   }

   if($mutilation=="trim")
   {
    if(isset($args[1])&&
     $args[1]!="left"&&
     $args[1]!="begin"&&
     $args[1]!="ltrim"&&
     $args[1]!="right"&&
     $args[1]!="end"&&
     $args[1]!="rtrim"&&
     $args[1]!="both"
    )
    {
     $this->e->add("Mutilation: \"".$mutilation."\" type: \"".$args[1]."\" is invalid for field: \"".$name."\" (".$this->field[$name]["type"].")", "error");
     return false;
    }
   }

   if($mutilation=="encode"||$mutilation=="decode")
   {
    if(isset($args[0]))
    {
     $valid=Array("html", "markup", "url", "base64");
     foreach(explode(";", $args[0]) as $arg)
     {
      if(!in_array($arg, $valid))
      {
       $this->e->add("Mutilation: \"".$mutilation."\" type: \"".$arg."\" is invalid for field: \"".$name."\" (".$this->field[$name]["type"].")", "error");
       return false;
      }
     }
    }
   }

   $this->field[$name]["mutilation"][$mutilation]=$args;
   return true;
  }

  function removeMutilation($name, $mutilation)
  {
   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't remove mutilation for unknown field: \"".$name."\"", "system");
    return false;
   }

   if(!isset($this->field[$name]["mutilation"][$mutilation]))
   {
    $this->e->add("No mutilation: \"".$mutilation."\" for field: \"".$name."\"", "debug");
    return true; //debug info
   }

   unset($this->field[$name]["mutilation"][$mutilation]);
   return true;
  }


  function applyMutilations($name, $value)
  {
   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't apply mutilation for unknown field: \"".$name."\"", "system");
    return false;
   }

   if(!$this->field[$name]["mutilation"])
   {
    $this->e->add("No mutilations to apply for field: \"".$name."\"", "warning");
    return $value;
   }

   foreach(array_keys($this->field[$name]["mutilation"]) as $mutilation)
   {
    switch($mutilation)
    {
     case "strip":
     {
      $value=Mutilate::stripS($value, (isset($this->field[$name]["mutilation"][$mutilation][0])?$this->field[$name]["mutilation"][$mutilation][0]:false), (isset($this->field[$name]["mutilation"][$mutilation][1])?$this->field[$name]["mutilation"][$mutilation][1]:false));
      break;
     }
     case "replace":
     {
      $value=Mutilate::replaceS($value, $this->field[$name]["mutilation"][$mutilation][0], (isset($this->field[$name]["mutilation"][$mutilation][1])?$this->field[$name]["mutilation"][$mutilation][1]:false));
      break;
     }
     case "case":
     {
      $value=Mutilate::changeCaseS($value, $this->field[$name]["mutilation"][$mutilation][0]);
      break;
     }
     case "trim":
     {
      $value=Mutilate::trimS($value, (isset($this->field[$name]["mutilation"][$mutilation][0])?$this->field[$name]["mutilation"][$mutilation][0]:false), (isset($this->field[$name]["mutilation"][$mutilation][1])?$this->field[$name]["mutilation"][$mutilation][1]:false));
      break;
     }

     case "stripTags":
     {
      $value=Mutilate::stripTagsS($value);
      break;
     }
     case "encode":
     {
      if(isset($this->field[$name]["mutilation"][$mutilation][0])) //encoding set in mutilation
       $type=$this->field[$name]["mutilation"][$mutilation][0];
      else if($this->field[$name]["type"]=="html") //no encoding set in mutilation, autodetect
       $type="html";
      else if($this->field[$name]["type"]=="text")
       $type="markup";
      else if($this->field[$name]["type"]=="url")
       $type="url";
      else if($this->field[$name]["type"]=="filedata")
       $type="base64";
      else //no encoding
       break;

      if(strstr($type, "markup"))
       $value=Mutilate::markupEncodeS($value);
      if(strstr($type, "url"))
       $value=Mutilate::urlEncodeS($value);
      if(strstr($type, "html"))
       $value=Mutilate::htmlEncodeS($value);
      if(strstr($type, "base64"))
       $value=Mutilate::base64EncodeS($value);

      break;
     }
     case "decode":
     {
      if(isset($this->field[$name]["mutilation"][$mutilation][0])) //decoding set in mutilation
       $type=$this->field[$name]["mutilation"][$mutilation][0];
      else if($this->field[$name]["type"]=="html") //no decoding set in mutilation
       $type="html";
      else if($this->field[$name]["type"]=="text")
       $type="markup";
      else if($this->field[$name]["type"]=="url")
       $type="url";
      else if($this->field[$name]["type"]=="filedata")
       $type="base64";
      else //no decoding
       break;

      if(strstr($type, "base64"))
       $value=Mutilate::base64DecodeS($value);
      if(strstr($type, "html"))
       $value=Mutilate::htmlDecodeS($value);
      if(strstr($type, "url"))
       $value=Mutilate::urlDecodeS($value);
      if(strstr($type, "markup"))
       $value=Mutilate::markupDecodeS($value);

      break;
     }

     case "humanReadableNumber":
     {
      $value=Mutilate::humanReadableNumberS($value, (isset($this->field[$name]["mutilation"][$mutilation][0])?$this->field[$name]["mutilation"][$mutilation][0]:$this->format[$this->dataformat]["decimals"]), $this->format[$this->dataformat]["decimalseparator"]);
      break;
     }
     case "humanReadableFileSize":
     {
      $value=Mutilate::humanReadableFileSizeS($value, (isset($this->field[$name]["mutilation"][$mutilation][0])?$this->field[$name]["mutilation"][$mutilation][0]:$this->format[$this->dataformat]["decimals"]), $this->format[$this->dataformat]["decimalseparator"]);
      break;
     }
     case "leadingZeros":
     {
      $value=Mutilate::leadingZerosS($value, $this->field[$name]["mutilation"][$mutilation][0]);
      break;
     }

     case "dateConvert":
     {
      $value=Mutilate::dateConvertS($value, $this->field[$name]["format"][$this->dataformat][$this->field[$name]["type"]."format"], $this->field[$name]["mutilation"][$mutilation][0]);
      break;
     }
     case "timezoneConvert":
     {
      $value=Mutilate::timezoneConvertS($value, $this->field[$name]["format"][$this->dataformat]["timezone"], $this->field[$name]["mutilation"][$mutilation][0], $this->field[$name]["format"][$this->dataformat][$this->field[$name]["type"]."format"]);
      break;
     }
     case "timeAgo":
     {
      $value=Mutilate::timeAgoS($value, $this->field[$name]["format"][$this->dataformat][$this->field[$name]["type"]."format"], (isset($this->field[$name]["mutilation"][$mutilation][0])?$this->field[$name]["mutilation"][$mutilation][0]:false));
      break;
     }

     case "scrambleEmail":
     {
      $value=Mutilate::scrambleEmailS($value);
      break;
     }

     default:
     {
      $this->e->add("Can't apply unknown mutilation: \"".$mutilation."\" for field: \"".$name."\"", "system");
      break;
     }
    }
   }

   return $value;
  }


/******************************************************************
 rights2bin
 Converts rights database column to a binary string
 returns reversed binary array (reversed so it's easy to add new values)
 + rights: rights from database

******************************************************************/
  function rights2bin($rights)
  {
   if($rights==-1) //OWNER !!! (workaround)
    $rights=$this->getRootRights();

   $tmp=decbin($rights); //get rights
   $l=strlen($tmp);

   $bin="";
   for($c=0;$c<$l;$c++)
   {
    if($tmp[(($l-1)-$c)]==1){$bin.="1";}
    else{$bin.="0";}
   }

   for($c=$l;$c<16;$c++)
    $bin.="0";

   return $bin;
  }

/******************************************************************
 bin2rights
 Converts a binary string to rights database column
 returns nr
 + bin: binary rights string

******************************************************************/
  function bin2rights($bin)
  {
   $rights=0;
   $l=strlen($bin);

   for($c=0;$c<$l;$c++)
   {
    if($bin[$c]=="1"){$rights+=pow(2,$c);}
   }
   return $rights;
  }

  function __sleep()
  {
   if($this->moduleClassName) //serialize sub objects
   {
    $this->moduleObject->data=false;
    $this->moduleObject=serialize($this->moduleObject);
   }

   return array_keys(get_object_vars($this));
  }


  function __wakeup()
  {
   global $_path;

   if($this->moduleClassName)
   {
    //load module definition in case it isn't loaded
    if(preg_match(REGEXP_DATA_PLUGIN_MODULE, $this->module, $tmp)) //plugin module
    {
     $module=$tmp[2];

     if(!isset($this->plugin)||!$this->plugin) //plugin not yet initialized
      $this->plugin=new Plugin($tmp[1]); //init plugin
     else //plugin already initialized
      $this->plugin->initPath(); //update cached relative path

     if(isset($this->plugin->path["modules"])) //could be a *:* empty module
      if(file_exists($this->plugin->path["modules"].$module.".php")) //could be a *:* empty module
       require_once($this->plugin->path["modules"].$module.".php");
    }
    else //normal module
    {
     $modulePaths=Array();

     if(isset($_path["project"]["modules"]))
      $modulePaths[]=$_path["project"]["modules"];

     if(isset($_path["modules"]))
      $modulePaths[]=$_path["modules"];

     foreach($modulePaths as $path)
     {
      if(file_exists($path.$this->module.".php"))
      {
       require_once($path.$this->module.".php");
       break;
      }
     }
    }

    //create real object
    $this->moduleObject=unserialize($this->moduleObject);
    $this->moduleObject->data=&$this;
   }
  }


 }
?>
