<?

 /*****************************************************************************
 * Filename: include/errors.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."data.php");
 require_once($_path["include"]."language.php");


/******************************************************************
 Errors class

 functions:
 - Errors:      Constructor
 + add:        Add's an error
 + any:        Any notes, warings or errors ?
 + isABC:      Checks of txt is alfabetic


******************************************************************/

 class Errors
 {


/******************************************************************
 Errors
 Constructor, initialises arrays

******************************************************************/
  function __construct()
  {
   $this->errorType=Array("debug", "note", "warning", "error", "hack", "sql", "system", "module", "plugin");
   $this->logErrorTypes=Array("error", "hack", "sql", "system", "module", "plugin");

   $this->clear(); //init main vars
   $this->logging(); //get logging from config

   $this->l=new Language();
   $this->l->read("error");
  }

  function __clone()
  {
   //clone objects
   $this->l=clone $this->l;
  }

  function clear()
  {
   $this->errors=0; //error counter
   $this->error=Array(); //errors array
   $this->extra=Array(); //extra info array
   $this->errorlevel=Array(); //errors level array
  }

  function getErrors(){return $this->errors;}
  function getError($entry){return $this->error[$entry];}
  function getExtra($entry){return $this->extra[$entry];}
  function getLevel($entry){return $this->level($this->errorlevel[$entry]);}

  function logging($logging=NULL)
  {
   global $_config;

   if($logging===NULL) //get from config
    $logging=($_config&&$_config->get("logging"));

   $this->logging=($logging);
   return true;
  }


/******************************************************************
 level
 Converts level string to nr or nr to string

 + level: level string or nr
 returns: error level, default: error
    0 = note
    1 = warning
    2 = error
    3 = hack attempt

******************************************************************/
  function level($level)
  {
   if(is_numeric($level))
    return $this->errorType[$level];
   else
   {
    $l=sizeof($this->errorType);

    for($c=0;$c<$l;$c++)
     if($this->errorType[$c]==$level){return $c;}
   }

   return 2;
  }

/******************************************************************
 add
 Add's a new array to the array

 + text: error text
 - level: error level string

******************************************************************/
  function add($error, $level="error", $extra=false, $log=true)
  {
   global $_config;

   //store all errors in Error object (without logging them)
   if(is_object($error)&&strtolower(get_class($error))=="errors")
   {
    $success=true;
    for($c=0;$c<$error->errors;$c++)
     if(!$this->add($error->error[$c], $this->level($error->errorlevel[$c]), $error->extra[$c], false))
      $success=false;

    if($success)
     $success=$this->l->read($error->l);

    return $success;
   }

   //store error
   $this->error[$this->errors]=$error;
   $this->extra[$this->errors]=$extra;
   $this->errorlevel[$this->errors]=$this->level($level);
   $this->errors++;

   if(!$log||!in_array($level, $this->logErrorTypes)) //don't log error
    return true;


   //log to stderr
   if(php_sapi_name()=="cli")
   {
    file_put_contents("php://stderr", strtoupper($level).": ".$this->l->t($error)."\n");

    if($_config&&$_config->get("debug")&&$extra)
     file_put_contents("php://stderr", $extra."\n");

    file_put_contents("php://stderr", "\n");
   }

   if(!$this->logging) //don't log error to database
    return true;

   //log to database
   $d=Data::cache("log", "add", "item", Array("format" => "form"));
   $d->e->logging(false); //disable logging
   $d->clearValues();

   $password=false; //find user password field
   if($du=$d->getReferenceData("usernr")) //get user module
    if($password=$du->check("properties", "password")) //get password field
     if($du->check("use", "login", $password[0]))
      $password=$password[0];

   //basic information
   $d->setValue("level", $level);
   $d->setValue("title", $error);
   $d->setValue("text", $extra);

   //request
   if(isset($_SERVER["REQUEST_URI"]))
    $d->setValue("url", (isset($_SERVER["HTTPS"])&&$_SERVER["HTTPS"]?"https":"http")."://".$_SERVER["HTTP_HOST"].$_SERVER["REQUEST_URI"]);
   else
    $d->setValue("url", "");

   $req="";
   foreach($_REQUEST as $k => $v)
   {
    if($password&&$k==$password) //skip password field
     continue;

    if(is_array($v))
    {
     foreach($v as $k2 => $v2)
     {
      if($password&&$k2==$password) //skip password field
       continue;

      if(!is_array($v2)&&strlen($v2)<512)
       $req.=$k."[".$k2."] => ".addslashes($v2)."\n";
     }
    }
    else if(strlen($v)<512)
     $req.=$k." => ".addslashes($v)."\n";
   }

   $d->setValue("vars", $req);

   //function info
   $bt=debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2);
   $d->setValue("function", (isset($bt[1])?(isset($bt[1]["class"])?$bt[1]["class"]."::":"").$bt[1]["function"]:""));
   $d->setValue("file", $bt[0]["file"]);
   $d->setValue("line", $bt[0]["line"]);

   if(!$d->submit()) //error on submitting
    return false;

   return true;
  }


/******************************************************************
 any
 return if there are notes, warnings or errors

 + level: error level text, either:
   - note
   - warnings
   - error
   - hack

******************************************************************/
  function any($level=false)
  {
   global $_config;

   if(!$level) //any errors at all?
   {
    if($_config->get("debug")) //really check all error types
    {
     if($this->errors)
      return true;
     else
      return false;
    }
    else //all except debug
    {
     $errorTypes=Array("note", "warning", "error", "hack", "sql", "system", "module", "plugin");

     foreach($errorTypes as $errorType)
      if($this->any($errorType))
       return true;

     return false;
    }
   }
   else if($level=="fatal") //only fatal errors
   {
    $errorTypes=Array("error", "hack", "sql", "system", "module", "plugin");

    foreach($errorTypes as $errorType)
     if($this->any($errorType))
      return true;

    return false;
   }
   else if($level=="show") //only showable errors
   {
    $errorTypes=Array("note", "warning", "error");

    foreach($errorTypes as $errorType)
     if($this->any($errorType))
      return true;

    return false;
   }

   $level=$this->level($level); //convert error level

   for($c=0;$c<$this->errors;$c++)
    if($this->errorlevel[$c]==$level){return true;}

   return false;
  }


/******************************************************************
 show
 returns a string with all error of a certain level

 + level: error level text, either:
   - note
   - warnings
   - error
   - hack

******************************************************************/
  function show($level=false)
  {
   global $_config;

   $show="";

   if(!$level)
   {
    if($_config->get("debug"))
     $errorTypes=Array("debug", "note", "warning", "error", "hack", "sql", "system", "module", "plugin");
    else
     $errorTypes=Array("note", "warning", "error", "hack", "sql", "system", "module", "plugin");

    foreach(array_reverse($errorTypes) as $errorType)
     if($this->any($errorType))
      $show.=$this->_showErrors($errorType)."\n";

    if($show)
     $show="<ul class=\"errors\">\n".$show."</ul>\n";

    return $show;
   }
   else if($level=="fatal") //errors or hacks
   {
    $errorTypes=Array("error", "hack", "sql", "system", "module", "plugin");

    foreach(array_reverse($errorTypes) as $errorType)
     if($this->any($errorType))
      $show.=$this->_showErrors($errorType)."\n";

    if($show)
     $show="<ul class=\"errors ".$level."\">\n".$show."</ul>\n";

    return $show;
   }
   else if($level=="show") //useful for users
   {
    $errorTypes=Array("note", "warning", "error");

    foreach(array_reverse($errorTypes) as $errorType)
     if($this->any($errorType))
      $show.=$this->_showErrors($errorType)."\n";

    if($show)
     $show="<ul class=\"errors ".$level."\">\n".$show."</ul>\n";

    return $show;
   }

   if($this->any($level))
    $show.=$this->_showErrors($level)."\n";

   if($show)
    $show="<ul class=\"errors ".$level."\">\n".$show."</ul>\n";

   return $show;
  }

  function _showErrors($level=false)
  {
   global $_config;

   $show="";

   $level=$this->level($level); //convert error level

   for($c=0;$c<$this->errors;$c++)
   {
    if($this->errorlevel[$c]==$level)
    {
     $show.="<li class=\"".$this->getLevel($c)."\">".htmlentities($this->l->t($this->error[$c]));

     if($_config->get("debug")&&$this->extra[$c]!="")
      $show.=" ".htmlentities($this->extra[$c])."<br/>";

     $show.="</li>\n";
    }
   }

   return $show;
  }
 }
?>
