<?

 /*****************************************************************************
 * Filename: include/format/email.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."socket.php");
 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."mime.php");
 require_once($_path["include"]."fs.php");
 require_once($_path["format"]."format.php");

 define("REGEXP_EMAIL_ADDRESS", "/^([a-z0-9_\-\.]+)\@([a-z0-9_\-\.]+)\.([a-z0-9]+)$/i");

 class Format_Email extends Format
 {
  function __construct($config=false)
  {
   global $_config;

   parent::__construct();

   $this->mime=new Mime();

   //get Email config
   $this->config=$_config->merge("Email", $config);

   $this->contentType="message/rfc822";
   $this->encoding=$this->config["encoding"];

   $this->mimeVersion=number_format($this->config["mimeVersion"], 1);
   $this->userAgent=$this->config["userAgent"];
   $this->defaultDomain=$this->config["defaultDomain"];

   $this->wordwrap=$this->config["wordwrap"];
   $this->boundaryPrefix="--";
   $this->headerLinebreak="\r\n";
   $this->bodyLinebreak="\r\n";

   $this->clear();
  }

  function __clone()
  {
   parent::__clone();

   //clone objects
   $this->mime=clone $this->mime;
  }

  function clear($key=false)
  {
   if($key)
   {
    if(isset($this->presets[$key]))
    {
     $this->data[$key]=Array();
     $this->vmax[$key]=0;
     $this->vc[$key]=0;
    }
   }
   else //all
   {
    parent::clear();

    $this->presets=Array(
     "Return-Path" => "single",
     //"Delivered-To" => "multi", //property of Received to ensure correct ordering
     "Received" => "multi",

     "From" => "single",
     "To" => "list",
     "Cc" => "list",
     "Bcc" => "list",

     "Subject" => "single",

     "Reply-To" => "single",
     "In-Reply-To" => "single",
     "Message-ID" => "single",

     "X-Priority" => "single",
     "Importance" => "single",

     "Attachment" => "multi",
    );

    $this->data=Array();
    $this->vmax=Array();
    $this->vc=Array();
    foreach(array_keys($this->presets) as $key)
    {
     $this->data[$key]=Array();
     $this->vmax[$key]=0;
     $this->vc[$key]=0;
    }
   }
  }

  function set($key, $value, $nr=false)
  {
   if(!isset($this->presets[$key]))
   {
    $this->presets[$key]="multi";
    $this->data[$key]=Array();
    $this->vmax[$key]=0;
    $this->vc[$key]=0;
   }

   if($nr===false) //add
   {
    if($this->presets[$key]=="single")
    {
     $this->data[$key][0]=$value;
     $this->vmax[$key]=1;
    }
    else //list / multi
    {
     $this->data[$key][]=$value;
     $this->vmax[$key]++;
    }
   }
   else //update
   {
    if($nr<0||$nr>=$this->vmax[$key])
    {
     $this->e->add("Unable to set value for key: \"".$key."\" index: \"".$nr."\" out of bounds", "error", $this->filename);
     return false;
    }

    $this->data[$key][$nr]=$value;
   }

   return true;
  }

  function get($key, $nr=0)
  {
   if(!isset($this->data[$key]))
    $this->data[$key]=Array(); //allow multiple values

   if($nr<0||$nr>=$this->vmax[$key])
   {
    $this->e->add("Unable to get value for key: \"".$key."\" index: \"".$nr."\" out of bounds", "error", $this->filename);
    return false;
   }

   return $this->data[$key][$nr];
  }


  /*
   * Iterate
   */

  function reset($key=false)
  {
   if($key)
   {
    if(isset($this->data[$key]))
     $this->vc[$key]=0;
   }
   else //all
   {
    $this->vc=Array();
    foreach(array_keys($this->presets) as $key)
     $this->vc[$key]=0;
   }

   $this->v=false;
  }

  function next($key="")
  {
   if(!isset($this->data[$key]))
   {
    $this->e->add("Unknown data part: ".$key, "error", $this->filename);
    $this->v=false;
    return false;
   }

   //set all values to the v variable
   if(!$this->value($key, $this->vc[$key]))
   {
    if(isset($this->vc[$key]))
     $this->vc[$key]=0; //restart

    return false;
   }

   $this->vc[$key]++; //go to next entry
   return true;
  }

  function previous($key="")
  {
   if(!isset($this->data[$key]))
   {
    $this->e->add("Unknown data part: ".$key, "error", $this->filename);
    $this->v=false;
    return false;
   }

   //set all values to the v variable
   if(!$this->value($key, $this->vmax[$key]-($this->vc[$key]+1))) //vc from end
   {
    if(isset($this->vc[$key]))
     $this->vc[$key]=0; //restart

    return false;
   }

   $this->vc[$key]++; //go to "next" entry
   return true;
  }

  function first($key="")
  {
   if(!isset($this->data[$key]))
   {
    $this->e->add("Unknown data part: ".$key, "error", $this->filename);
    $this->v=false;
    return false;
   }

   //set all values to the v variable
   if(!$this->value($key, 0))
    return false;

   return true;
  }

  function last($key="")
  {
   if(!isset($this->data[$key]))
   {
    $this->e->add("Unknown data part: ".$key, "error", $this->filename);
    $this->v=false;
    return false;
   }

   //set all values to the v variable
   if(!$this->value($key, $this->vmax[$key]-1))
    return false;

   return true;
  }

  function search($key, $value="")
  {
   if(!isset($this->data[$key]))
   {
    $this->e->add("Unknown data part: ".$key, "error", $this->filename);
    $this->v=false;
    return false;
   }

   //search if the value exists
   for($c=$this->vc[$key];$c<$this->vmax[$key];$c++)
   {
    if(stristr($this->data[$key][$c], $value))
    {
     //set all values to the v variable
     if(!$this->value($key, $c))
      return false;

     $this->vc[$key]=$c+1; //go to next entry
     return true;
    }
   }

   $this->reset();
   return false;
  }

  function value($key, $nr=0)
  {
   if(!isset($this->data[$key]))
   {
    $this->e->add("Unknown data part: ".$key, "error", $this->filename);
    $this->v=false;
    return false;
   }

//illegal entry
   if($nr<0||$nr>=$this->vmax[$key])
   {
    $this->v=false;
    return false;
   }

   $this->v=$this->data[$key][$nr];
   return true;
  }


  /*
   * Parse
   */

  function parse($data)
  {
   $this->output=$data;

   if(!$this->original)
    $this->original=$data;

   //clear data
   foreach(array_keys($this->presets) as $key)
    $this->clear($key);

   if(!$data)
   {
    $this->e->add("No data to parse", "error", $this->filename);
    return false;
   }

   //split header + body
   $linebreak=Mutilate::detectLineBreaksS($data);
   list($headers, $body)=explode($linebreak.$linebreak, $data, 2);

   //merge multiline headers
   $headers=preg_replace("/".preg_quote($linebreak, "/")."\s+/", " ", $headers);

   //parse headers
   $content=Array(
    "ID" => false,
    "type" => false,
    "disposition" => false,
    "encoding" => false,
   );
   foreach(explode($linebreak, $headers) as $header)
   {
    if(!$header) //skip empty lines
     continue;

    //get header key
    list($key, $value)=explode(": ", $header, 2);

    switch($key)
    {
     case "From":
     case "To":
     case "Cc":
     case "Bcc":
     {
      //parse email addresses
      foreach(explode(", ", $value) as $address)
      {
       if(preg_match("/^\"?([^\"]+)?\"?\s?<([^>]+)>$/U", $address, $m)) //"name" <email@address.com>
       {
        $values=Array(
         "name" => ($m[1]?$m[1]:false),
         "address" => $m[2],
        );
       }
       else //email@address.com
       {
        $values=Array(
         "name" => false,
         "address" => $address,
        );
       }

       if(!$this->_validAddress($values["address"]))
        return false;

       if(!$this->set($key, $values))
        return false;
      }

      break;
     }
     case "Delivered-To":
     {
      //merged with received
      $values=Array(
       "Delivered-To" => $value,
      );

      if(!$this->set("Received", $values))
       return false;

      break;
     }
     case "Received":
     {
      //detect info
      if(preg_match("/from (.+) by (.+) with (.+); (.+)/", $value, $m))
      {
       $values=Array(
        "from" => $m[1],
        "by" => $m[2],
        "with" => $m[3],
        "text" => false,
        "date" => $m[4],
        "Delivered-To" => false,
       );
      }
      else if(preg_match("/(.+); (.+)/", $value, $m))
      {
       $values=Array(
        "from" => false,
        "by" => false,
        "with" => false,
        "text" => $m[1],
        "date" => $m[2],
        "Delivered-To" => false,
       );
      }
      else
      {
       $values=Array(
        "from" => false,
        "by" => false,
        "with" => false,
        "text" => $value,
        "date" => false,
        "Delivered-To" => false,
       );
      }

      //merge with previous Received entry
      if($this->last($key))
      {
       if(array_keys($this->v)==Array("Delivered-To")) //only Delivered-To set
       {
        $values["Delivered-To"]=$this->v["Delivered-To"]; //update Delivered-To for new entry

        if(!$this->set($key, $values, $this->vmax[$key]-1)) //overwrite last entry
         return false;

        break;
       }
      }

      //add without Delivered-To
      if(!$this->set($key, $values))
       return false;

      break;
     }
     case "Content-ID":
     {
      $content["ID"]=$value; //store for attachments parsing
      break; //don't store header
     }
     case "Content-Type":
     {
      $content["type"]=$value; //store for attachments parsing
      break; //don't store header
     }
     case "Content-Disposition":
     {
      $content["disposition"]=$value; //store for attachments parsing
      break; //don't store header
     }
     case "Content-Transfer-Encoding":
     {
      $content["encoding"]=$value; //store for attachments parsing
      break; //don't store header
     }
     default:
     {
      if(!$this->set($key, $value))
       return false;

      break;
     }
    }
   }


   //parse body
   if(!$content["type"]) //no content-type
    if(!$this->attach($body, false, $content["encoding"], false, "message")) //just add body
     return false;


   //parse attachments
   $queue=Array();
   $queue[]=Array(
    "ID" => $content["ID"],
    "type" => $content["type"], //parsed below
    "parent" => false, //first has no parent
    "boundary" => false, //filled below
    "encoding" => $content["encoding"],
    "disposition" => $content["disposition"],
    "content" => $body,
   );

   //queue loop
   while($queue)
   {
    //handle next item in the queue
    $item=array_shift($queue);

    //parse type/disposition
    foreach(Array("type", "disposition") as $key)
    {
     $value=explode("; ", $item[$key]);
     $item[$key]=$value[0]; //set actual value

     //add additional keys as defined in content-value
     for($x=1;$x<sizeof($value);$x++)
     {
      list($k, $v)=explode("=", $value[$x]);
      $item[$k]=$v;
     }
    }

    //handle queue
    if(substr($item["type"], 0, 9)=="multipart")
    {
     //detect boundary prefix
     if($dashes=substr_count($item["boundary"], "-"))
     {
      if($dashes!=strlen($this->boundaryPrefix)) //update prefix
       $this->boundaryPrefix=str_repeat("-", $dashes);

      $item["boundary"]=substr($item["boundary"], $dashes); //remove prefix from boundary
     }

     //get sub queue
     if(strpos($item["content"], $linebreak)!==0)
      $item["content"]=$linebreak.$item["content"];

     $sub=explode($linebreak."--".$this->boundaryPrefix.$item["boundary"], $item["content"]);
     if(sizeof($sub)==1)
     {
      $this->e->add("Unable to gather queue from attachment content: \"".$item["content"]."\"", "error", $this->filename);
      return false;
     }

     //parse sub queue
     foreach($sub as $attachment)
     {
      if(!$attachment) //skip empty lines (usually first)
       continue;

      if(strpos($attachment, "--")===0) //end of attachment
       break;

      //get attachment header
      $attachment=explode($linebreak.$linebreak, $attachment, 2);
      if(sizeof($attachment)==1)
      {
       $this->e->add("Unable to get attachment content from attachment: \"".$attachment[0]."\"", "error", $this->filename);
       return false;
      }

      //parse attachment header
      $content=Array(
       "ID" => false,
       "type" => false,
       "disposition" => false,
       "encoding" => false,
      );
      foreach(explode($linebreak, $attachment[0]) as $header)
      {
       if(!$header) //skip empty lines (usually first)
        continue;

       //get attachment header key
       list($key, $value)=explode(": ", $header, 2);

       switch($key)
       {
        case "Content-ID":
        {
         $content["ID"]=$value; //store for queue parsing
         break;
        }
        case "Content-Type":
        {
         $content["type"]=$value; //store for queue parsing
         break;
        }
        case "Content-Disposition":
        {
         $content["disposition"]=$value; //store for queue parsing
         break;
        }
        case "Content-Transfer-Encoding":
        {
         $content["encoding"]=$value; //store for queue parsing
         break;
        }
        default: break; //skip other headers
       }
      }

      //add attachment
      $queue[]=Array(
       "ID" => $content["ID"],
       "type" => $content["type"], //parsed at next iteration
       "parent" => $this->vmax["Attachment"], //next added attachment
       "boundary" => false, //filled at next iteration
       "encoding" => $content["encoding"],
       "disposition" => $content["disposition"],
       "content" => $attachment[1],
      );
     }


     //add multipart attachment
     $this->data["Attachment"][]=Array(
      "name" => false,
      "type" => false,
      "parent" => $item["parent"],
      "boundary" => $item["boundary"],
      "content" => false,
      "contentType" => $item["type"],
      "contentCharset" => false,
      "contentEncoding" => false,
     );

     $this->vmax["Attachment"]++;
    }
    else //normal/text attachment
    {
     //message/embedded attachment
     $type=false;//normal attachment by default
     if($item["parent"]===false||$this->data["Attachment"][$item["parent"]]["contentType"]=="multipart/alternative") //whole body or part of multipart/alternative attachment
     {
      if($item["type"]=="text/plain"||$item["type"]=="text/html") //text message
       $type="message"; //message attachment
     }
     else if($this->data["Attachment"][$item["parent"]]["contentType"]=="multipart/related") //part of multipart/related attachment
      $type="embedded"; //embedded attachment

     //add attachment
     $this->data["Attachment"][]=Array(
      "name" => (isset($item["filename"])?$item["filename"]:false),
      "type" => $type,
      "parent" => $item["parent"],
      "boundary" => false,
      "content" => Mutilate::fixLineBreaksS($item["content"], $this->bodyLinebreak),
      "contentType" => $item["type"],
      "contentCharset" => (isset($item["charset"])?$item["charset"]:false),
      "contentEncoding" => ($item["encoding"]?$item["encoding"]:"7bit"),
     );

     $this->vmax["Attachment"]++;
    }
   }

   return true;
  }


  /*
   * Set header values
   */

  function _validAddress($address)
  {
   if(!preg_match(REGEXP_EMAIL_ADDRESS, $address))
   {
    $this->e->add("Illegal E-mail address: \"".$address."\"", "error", $this->filename);
    return false;
   }

   return true;
  }


  function from($address, $name=false)
  {
   if(!$this->_validAddress($address))
    return false;

   if(!$this->set("From", Array(
    "address" => $address,
    "name" => str_replace("\"", "", $name),
   )))
    return false;

   return true;
  }

  function to($address, $name=false)
  {
   if(!$this->_validAddress($address))
    return false;

   if(!$this->set("To", Array(
    "address" => $address,
    "name" => str_replace("\"", "", $name),
   )))
    return false;

   return true;
  }

  function cc($address, $name=false)
  {
   if(!$this->_validAddress($address))
    return false;

   if(!$this->set("Cc", Array(
    "address" => $address,
    "name" => str_replace("\"", "", $name),
   )))
    return false;

   return true;
  }

  function bcc($address, $name=false)
  {
   if(!$this->_validAddress($address))
    return false;

   if(!$this->set("Bcc", Array(
    "address" => $address,
    "name" => str_replace("\"", "", $name),
   )))
    return false;

   return true;
  }


  function subject($subject)
  {
   if(!$this->set("Subject", str_replace("\n", "", $subject)))
    return false;

   return true;
  }



  /*
   * Set attachment/message values
   */

  function attachFile($file, $contentType=false, $contentEncoding=false, $type=false)
  {
   //read attachment from file
   if(($content=FS::readS($file))===false)
   {
    $this->e->add("Unable to read attachment from file: \"".$file."\"", "error", $this->filename);
    return false;
   }

   //determine content type
   if(!$contentType) //not preset, get from filename
    $contentType=$this->mime->filename($file);

   return $this->attach($content, $contentType, $contentEncoding, FS::filenameS($file), $type);
  }

  function attach($content, $contentType=false, $contentEncoding=false, $name=false, $type=false)
  {
   //setup base attachment
   $attachment=Array(
    "name" => $name,
    "type" => $type,
    "parent" => false,
    "boundary" => md5(uniqid(mt_rand(), true)),
    "content" => false,
    "contentType" => false,
    "contentCharset" => false,
    "contentEncoding" => false,
   );

   //determine content type
   if($contentType) //preset
    $attachment["contentType"]=$contentType;
   else //get from content
    $attachment["contentType"]=$this->mime->content($content);

   //set charset
   if($attachment["contentType"]=="text/plain"||$attachment["contentType"]=="text/html") //text/html
    $attachment["contentCharset"]=$this->encoding;

   //set encoding
   if($contentEncoding) //preset
    $attachment["contentEncoding"]=$contentEncoding;
   else if(strpos($attachment["contentType"], "text")===0) //text
    $attachment["contentEncoding"]="7bit";
   else if(strpos($attachment["contentType"], "multipart")===0) //multipart
    $attachment["contentEncoding"]=false;
   else //default
    $attachment["contentEncoding"]="base64";

   //set (encoded) content
   switch($attachment["contentEncoding"])
   {
    case "7bit":
    {
     $attachment["content"]=Mutilate::fixLineBreaksS($content, $this->bodyLinebreak);
     break;
    }
    case "Quoted-Printable":
    {
     $attachment["content"]=quoted_printable_encode(Mutilate::fixLineBreaksS($content, $this->bodyLinebreak));
     break;
    }
    case "base64":
    {
     $attachment["content"]=base64_encode($content);
     break;
    }
    default:
    {
     $attachment["content"]=$content;
     break;
    }
   }

   //determine where to place attachment based on type
   //no attachments:
   //+ add attachment
   //embedded:
   //- multipart/related exists
   //  + add embedded attachment with multipart/related as parent
   //- HTML message attachment exists
   //  + copy message attachment to end and reparent to multipart/related
   //  + set old message attachment to multipart/related with keeping parent
   //  + add embedded attachment with new multipart/related as parent
   //- else (no HTML message)
   //  + fail
   //message:
   //- original exists with same contentType:
   //+ overwrite
   //- multipart/alternative exists:
   //  + add message attachment with multipart/alternative as parent
   //- message attachment exists
   //  + copy message attachment to end and reparent to multipart/alternative
   //  + set old message attachment to multipart/alternative with keeping parent
   //  + add message attachment with new multipart/alternative as parent
   //- multipart/mixed exists:
   //  + add message attachment with multipart/mixed as parent
   //- else (only 1 normal attachment)
   //  + add multipart/mixed at start
   //  + attachments without parent get new multipart/mixed as parent
   //  + attachments with parent get parent++
   //  + add message attachment with new multipart/mixed as parent
   //attachment:
   //- multipart/mixed:
   //  + add attachment with multipart/mixed as parent
   //- else (only 1 normal attachment)
   //  + add multipart/mixed at start
   //  + attachments without parent get new multipart/mixed as parent
   //  + attachments with parent get parent++
   //  + add attachment with new multipart/mixed as parent

   if($type=="message") //message attachment
    $attachment["name"]=false; //no name

   if($this->vmax["Attachment"])
   {
    //lookup mixed, alternative and message (with same contentType) attachments
    $original=$alternative=$message=$mixed=$htmlMessage=$related=false;
    for($c=0;$c<$this->vmax["Attachment"];$c++)
    {
     if($original===false)
      if($this->data["Attachment"][$c]["type"]=="message") //message attachment found
       if($this->data["Attachment"][$c]["contentType"]==$attachment["contentType"]) //message attachment with same content type found
        $original=$c; //store message entry for overwrite (below)

     if($alternative===false)
      if($this->data["Attachment"][$c]["contentType"]=="multipart/alternative") //attachment with multipart/alternative content type found
       $alternative=$c; //store attachment entry

     if($message===false)
      if($this->data["Attachment"][$c]["type"]=="message") //message attachment found
       $message=$c; //store message attachment entry

     if($mixed===false)
      if($this->data["Attachment"][$c]["contentType"]=="multipart/mixed") //attachment with multipart/mixed content type found
       $mixed=$c; //store attachment entry

     if($htmlMessage===false)
      if($this->data["Attachment"][$c]["type"]=="message"&&$this->data["Attachment"][$c]["contentType"]=="text/html") //html message attachment found
       $htmlMessage=$c; //store html message attachment entry

     if($related===false)
      if($this->data["Attachment"][$c]["contentType"]=="multipart/related") //attachment with multipart/related content type found
       $related=$c; //store attachment entry
    }

    if($type=="embedded") //embedded message attachment
    {
     if($related!==false) //multipart/related exists
     {
      $attachment["parent"]=$related; //added new message to multipart/related container
      //added below
     }
     else if($htmlMessage!==false) //html message attachment exists
     {
      //generate new multipart/related attachment
      $multipart=Array(
       "name" => false,
       "type" => false,
       "parent" => $this->data["Attachment"][$htmlMessage]["parent"], //keep under same parent
       "boundary" => md5(uniqid(mt_rand(), true)),
       "content" => false,
       "contentType" => "multipart/related",
       "contentCharset" => false,
       "contentEncoding" => false,
      );

      //copy message to end
      $tmp=$this->data["Attachment"][$htmlMessage];
      $tmp["parent"]=$htmlMessage; //reparent

      if(!$this->set("Attachment", $tmp))
       return false;

      unset($tmp);

      //reuse attachment for multipart
      $this->data["Attachment"][$htmlMessage]=$multipart;

      //set multipart as parent for new message attachment
      $attachment["parent"]=$htmlMessage;
      //added below
     }
     else //no html message
     {
      $this->e->add("Unable to add embedded attachment, no HTML message found", "error", $this->filename);
      return false;
     }
    }
    else if($type=="message") //message attachment
    {
     if($original!==false) //message with same content type exists
     {
      $this->data["Attachment"][$original]["content"]=$attachment["content"];
      $this->data["Attachment"][$original]["contentEncoding"]=$attachment["contentEncoding"];
      return true; //done (don't add below)
     }
     else if($alternative!==false) //multipart/alternative exists
     {
      $attachment["parent"]=$alternative; //added new message to multipart/alternative container
      //added below
     }
     else if($message!==false) //message attachment exists
     {
      //generate new multipart/alternative attachment
      $multipart=Array(
       "name" => false,
       "type" => false,
       "parent" => $this->data["Attachment"][$message]["parent"], //keep under same parent
       "boundary" => md5(uniqid(mt_rand(), true)),
       "content" => false,
       "contentType" => "multipart/alternative",
       "contentCharset" => false,
       "contentEncoding" => false,
      );

      //copy message to end
      $tmp=$this->data["Attachment"][$message];
      $tmp["parent"]=$message; //reparent

      if(!$this->set("Attachment", $tmp))
       return false;

      unset($tmp);

      //reuse attachment for multipart
      $this->data["Attachment"][$message]=$multipart;

      //set multipart as parent for new message attachment
      $attachment["parent"]=$message;
      //added below
     }
     else if($mixed!==false) //multipart/mixed exists
     {
      $attachment["parent"]=$mixed; //added new message to multipart/mixed container
      //added below
     }
     else //no original, alternative, message or mixed attachment (i.e. just 1 (normal attachment)
      $mixed=true; //add multipart/mixed below
    }
    else //normal attachment
    {
     if($mixed!==false) //multipart/mixed exists
     {
      $attachment["parent"]=$mixed; //added new message to multipart/mixed container
      //added below
     }
     else //no original, alternative, message or mixed attachment (i.e. just 1 (normal attachment)
      $mixed=true; //add multipart/mixed below
    }

    //add multipart/mixed
    if($mixed===true)
    {
     //generate new multipart/mixed attachment
     $multipart=Array(
      "name" => false,
      "type" => false,
      "parent" => false,
      "boundary" => md5(uniqid(mt_rand(), true)),
      "content" => false,
      "contentType" => "multipart/mixed",
      "contentCharset" => false,
      "contentEncoding" => false,
     );

     //add multipart/mixed at start
     array_unshift($this->data["Attachment"], $multipart);
     $this->vmax["Attachment"]++;

     //reparent existing attachments
     for($c=1;$c<$this->vmax["Attachment"];$c++) //skip first (multipart/mixed)
     {
      if($this->data["Attachment"][$c]["parent"]===false) //parentless, attachment to new multipart/mixed
       $this->data["Attachment"][$c]["parent"]=0;
      else //parented
       $this->data["Attachment"][$c]["parent"]++; //correct parent for newly added entry
     }

     //add attachment with new multipart/mixed as parent
     $attachment["parent"]=0; //added new message to multipart/mixed container
     //added below
    }
   }

   //store attachment
   $this->data["Attachment"][]=$attachment;
   $this->vmax["Attachment"]++;
   return true;
  }


  /*
   * Detach attachment
   */

  function detachFile($index, $file)
  {
   //get contents and filename
   if(($content=$this->detach($index, $name))===false)
    return false;

   //append filename if directory is given
   if(is_dir($file))
   {
    if(!$name)
    {
     $this->e->add("Unable to write attachment to directory: \"".$file."\", attachment has no filename", "error", $this->filename);
     return false;
    }

    if(substr($file, -1)!="/") //append end slash to directory
     $file.="/";

    $file.=$name;
   }

   //write attachment to file
   if((FS::writeS($file, $content))===false)
   {
    $this->e->add("Unable to write attachment to file: \"".$file."\"", "error", $this->filename);
    return false;
   }

   return true;
  }

  function detach($index, &$name=false)
  {
   if(!preg_match("/^\d+$/", $index)) //not an array index
   {
    //lookup filename
    $found=false;
    for($c=0;$c<$this->vmax["Attachment"];$c++)
    {
     if($this->data["Attachment"][$c]["name"]==$index)
     {
      $found=true;
      $index=$c;
      break;
     }
    }

    if(!$found)
    {
     $this->e->add("Unknown detach file: \"".$index."\"", "error", $this->filename);
     return false;
    }
   }

   //check boundaries
   if($index<0||$index>$this->vmax["Attachment"])
   {
    $this->e->add("Out-of-bound detach nr: \"".$index."\"", "error", $this->filename);
    return false;
   }

   //skip multipart
   if(substr($this->data["Attachment"][$index]["contentType"], 0, 9)=="multipart")
   {
    $this->e->add("Detach nr: \"".$index."\" is a multipart attachment", "error", $this->filename);
    return false;
   }

   //decode attachment
   switch($this->data["Attachment"][$index]["contentEncoding"])
   {
    case "7bit":
    {
     $content=$this->data["Attachment"][$index]["content"];
     break;
    }
    case "Quoted-Printable":
    {
     $content=quoted_printable_decode($this->data["Attachment"][$index]["content"]);
     break;
    }
    case "base64":
    {
     $content=base64_decode($this->data["Attachment"][$index]["content"]);
     break;
    }
    default:
    {
     $content=$this->data["Attachment"][$index]["content"];
     break;
    }
   }

   //get attachment name
   $name=$this->data["Attachment"][$index]["name"];

   return $content;
  }


  /*
   * Set message values
   */

  function messageFile($file, $alternativeContent=false)
  {
   //read message attachment from file
   if(($content=FS::readS($file))===false)
   {
    $this->e->add("Unable to read message attachment from file: \"".$file."\"", "error", $this->filename);
    return false;
   }

   return $this->message($content, $alternativeContent);
  }

  function message($content, $alternativeContent=false)
  {
   //fix line breaks
   $content=Mutilate::fixLineBreaksS($content, $this->bodyLinebreak);

   if(!is_bool($alternativeContent))
    $alternativeContent=Mutilate::fixLineBreaksS($alternativeContent, $this->bodyLinebreak);


   //determine content type
   $contentType=$this->mime->content($content);

   if($contentType=="text/plain") //text message
   {
    if($alternativeContent===true) //generate alternative HTML message
    {
     //generate HTML version
     $alternativeContent="<html>".$this->bodyLinebreak;
     $alternativeContent.=" <head>".$this->bodyLinebreak;

     if($this->data["Subject"])
      $alternativeContent.="  <title>".$this->data["Subject"][0]."</title>".$this->bodyLinebreak;

     $alternativeContent.=" </head>".$this->bodyLinebreak;
     $alternativeContent.=" <body>".$this->bodyLinebreak;
     $alternativeContent.=str_replace($this->bodyLinebreak, "<br/>".$this->bodyLinebreak, $content);
     $alternativeContent.=" </body>".$this->bodyLinebreak;
     $alternativeContent.="</html>".$this->bodyLinebreak;
    }
    //else use set content or no alternative content (false)
   }
   else if($contentType=="text/html") //html message
   {
    if($alternativeContent===true) //generate alternative text message
    {
     if(!($alternativeContent=preg_replace("/.*<body>(.+)<\/body>.*/is", "\\1", $content))) //get body from content
      $alternativeContent=$content; //no body, take whole content

     $alternativeContent=strip_tags(preg_replace("!<br/?>!", $this->bodyLinebreak, $alternativeContent));
    }
    //else use set content or no alternative content (false)

    //flip text
    $tmp=$content;
    $content=$alternativeContent;
    $alternativeContent=$tmp;
    unset($tmp);
   }
   else //unknown message type
   {
    $this->e->add("Unsupported message mime type: \"".$contentType."\"", "error", $this->filename);
    return false;
   }

   if($content&&$this->wordwrap) //wrap text message
    $content=wordwrap($content, $this->wordwrap, $this->bodyLinebreak);

   //add message attachment(s)
   if($content)
    if(!$this->attach($content, "text/plain", false, false, "message"))
     return false;

   if($alternativeContent)
    if(!$this->attach($alternativeContent, "text/html", false, false, "message"))
     return false;

   return true;
  }


  function embedFile($file, $alternativeContent=false)
  {
   //read embedded message attachment from file
   if(($content=FS::readS($file))===false)
   {
    $this->e->add("Unable to read embedded message attachment from file: \"".$file."\"", "error", $this->filename);
    return false;
   }

   return $this->embed($content, $alternativeContent);
  }

  function embed($content, $alternativeContent=false)
  {
   global $_path;

   if($this->mime->content($content)!="text/html")
   {
    $this->e->add("Embedded message mime type not text/html", "error", $this->filename);
    return false;
   }

   //add message
   if(!$this->message($content, $alternativeContent))
    return false;

   //detect linked files in HTML
   $files=Array();
   $regexps=Array(
    "/\<img .*src=\"([^\"]+)\"/im",
    "/\<img .*src=\'([^\"]+)\'/im",
    "/\<script .*src=\"([^\"]+)\"/im",
    "/\<script .*src=\'([^\"]+)\'/im",
    "/\<link .*href=\"([^\"]+)\"/im",
    "/\<link .*href=\'([^\"]+)\'/im",
   );

   foreach($regexps as $regexp)
   {
    if(preg_match_all($regexp, $content, $m))
    {
     foreach($m[1] as $file)
     {
      if(preg_match("![a-z]+://!", $file)) //remote file
       continue; //skip

      $location=$file;
      $file=FS::realPathS($file);

      if(isset($files[$file])) //file already in set
      {
       if(!in_array($location, $files[$file]["locations"])) //add new location
        $files[$file]["locations"][]=$location;
      }
      else //new file
      {
       $files[$file]=Array(
        "locations" => Array($location),
        //"name" => FS::filenameS($file),
        "path" => FS::filepathS($file),
        "ext" => FS::fileextS($file),
        "cid" => false,
       );
      }
     }
    }
   }

   //get css embedded images
   foreach($files as $cssfile => $info)
   {
    if(strtolower($info["ext"])=="css")
    {
     if(preg_match_all("/url\(([^\)]+)\)/mi", FS::readS($cssfile), $m))
     {
      foreach($m[1] as $file)
      {
       if(preg_match("![a-z]+://!", $file)) //remote file
        continue; //skip

       $location=$file;
       $file=FS::realPathS($info["path"].$file);

       if(isset($files[$file])) //file already in set
       {
        if(!in_array($location, $files[$file]["locations"])) //add new location
         $files[$file]["locations"][]=$location;
       }
       else //new file
       {
        $files[$file]=Array(
         "locations" => Array($location),
         //"name" => FS::filenameS($file),
         //"path" => FS::filepathS($file),
         //"ext" => FS::fileextS($file),
         "cid" => false,
        );
       }
      }
     }
    }
   }

   //add embedded files
   $success=true;
   foreach(array_keys($files) as $file)
   {
    if(!$this->attachFile($file, false, false, "embedded"))
     $success=false; //keep adding other files
    else //store CID
    {
     $entry=$this->vmax["Attachment"]-1;
     $files[$file]["cid"]=$this->data["Attachment"][$entry]["boundary"]; //get boundary from most recent entry
    }
   }

   //update HTML message and related CSS with CIDs
   for($c=0;$c<$this->vmax["Attachment"];$c++)
   {
    if($this->data["Attachment"][$c]["type"]=="message"&&$this->data["Attachment"][$c]["contentType"]=="text/html") //html message attachment found
    {
     foreach($files as $info)
      foreach($info["locations"] as $location)
       $this->data["Attachment"][$c]["content"]=str_replace($location, "cid:".$info["cid"], $this->data["Attachment"][$c]["content"]);
    }
    else if($this->data["Attachment"][$c]["contentType"]=="text/css") //css file
    {
     if($this->data["Attachment"][$c]["parent"]!==false&&$this->data["Attachment"][$this->data["Attachment"][$c]["parent"]]["contentType"]=="multipart/related") //css is related
     {
      foreach($files as $info)
       foreach($info["locations"] as $location)
        $this->data["Attachment"][$c]["content"]=str_replace("url(".$location.")", "url(cid:".$info["cid"].")", $this->data["Attachment"][$c]["content"]);
     }
    }
   }

   return $success;
  }


  /*
   * Output
   */

  function output($headers=false)
  {
   if(($header=$this->header())===false)
    return false;

   if(($body=$this->body())===false)
    return false;

   return $header.$this->headerLinebreak.$body;
  }

  function header()
  {
   if($this->e->any("fatal"))
    return false;

   //prep custom headers
   if(!isset($this->vmax["Message-ID"])||!$this->vmax["Message-ID"])
   {
    if($this->vmax["From"])
     $domain=preg_replace("/^[^@]+@/", "", $this->data["From"][0]["address"]);
    else
     $domain=$this->defaultDomain;

    if(!$this->set("Message-ID", "<".md5(uniqid(mt_rand(), true))."@".$domain.">"))
     return false;
   }

   if(!isset($this->vmax["Date"])||!$this->vmax["Date"])
    if(!$this->set("Date", date("D, j M Y H:i:s O")))
     return false;

   if(!isset($this->vmax["MIME-Version"])||!$this->vmax["MIME-Version"])
    if(!$this->set("MIME-Version", $this->mimeVersion))
     return false;

   if(!isset($this->vmax["User-Agent"])||!$this->vmax["User-Agent"])
    if(!$this->set("User-Agent", $this->userAgent))
     return false;


   //generate preset headers
   $headers=Array();
   foreach($this->presets as $key => $type)
   {
    if(!$this->vmax[$key]) //no values
     continue;

    switch($key)
    {
     case "Attachment":
     {
      break; //skip attachments in header
     }
     case "From":
     case "To":
     case "Cc":
     case "Bcc":
     {
      //prep values
      $values=Array();
      for($c=0;$c<$this->vmax[$key];$c++)
      {
       if($this->data[$key][$c]["name"]) //email+name
        $values[]="\"".$this->data[$key][$c]["name"]."\" <".$this->data[$key][$c]["address"].">";
       else //only email
        $values[]=$this->data[$key][$c]["address"];
      }

      $headers[]=$key.": ".implode(", ", $values);
      break;
     }
     case "Received":
     {
      for($c=0;$c<$this->vmax[$key];$c++)
      {
       //add Delivered-To header
       if($this->data[$key][$c]["Delivered-To"])
        $headers[]="Delivered-To: ".$this->data[$key][$c]["Delivered-To"];

       //add Received header
       $header=$key.": ";
       if($this->data[$key][$c]["from"])
        $header.="from ".$this->data[$key][$c]["from"]." by ".$this->data[$key][$c]["by"]." with ".$this->data[$key][$c]["with"];

       if($this->data[$key][$c]["text"])
        $header.=$this->data[$key][$c]["text"];

       if($this->data[$key][$c]["date"])
        $header.="; ".$this->data[$key][$c]["date"];

       $headers[]=$header;
      }

      break;
     }
     default:
     {
      if($type=="single")
       $headers[]=$key.": ".$this->data[$key][0];
      else if($type=="list")
       $headers[]=$key.": ".implode(", ", $this->data[$key]);
      else if($type=="multi")
      {
       for($c=0;$c<$this->vmax[$key];$c++)
        $headers[]=$key.": ".$this->data[$key][$c];
      }

      break;
     }
    }
   }


   //attachments headers
   if($this->vmax["Attachment"])
   {
    $header="Content-Type: ".$this->data["Attachment"][0]["contentType"];

    if($this->data["Attachment"][0]["contentType"]=="multipart/alternative"||$this->data["Attachment"][0]["contentType"]=="multipart/mixed")
     $header.="; boundary=".$this->boundaryPrefix.$this->data["Attachment"][0]["boundary"];

    if($this->data["Attachment"][0]["contentCharset"])
     $header.="; charset=".$this->data["Attachment"][0]["contentCharset"];

    $headers[]=$header;

    if($this->data["Attachment"][0]["contentEncoding"])
     $headers[]="Content-Transfer-Encoding: ".$this->data["Attachment"][0]["contentEncoding"];
   }

   //wrap headers
   if($this->wordwrap)
   {
    $l=sizeof($headers);
    for($c=0;$c<$l;$c++)
    {
     if(strlen($headers[$c])>$this->wordwrap) //line exceeds max length
     {
      //find line split point
      if(($pos=strrpos(substr($headers[$c], 0, $this->wordwrap), ";"))===false) //find last ; before max length
       if(($pos=strrpos(substr($headers[$c], 0, $this->wordwrap), " "))===false) //find last space before max length
        $pos=false; //no acceptable split point found

      if($pos!==false) //line split point found
      {
       //copy current line
       $headers=array_merge(array_slice($headers, 0, $c+1), array_slice($headers, $c));
       $l++;

       //split remainder from current line
       $headers[$c]=substr($headers[$c], 0, $pos);

       //add remainder and prefix to new line
       $headers[$c+1]="          ".substr($headers[$c+1], $pos);
      }
     }
    }
   }

   return implode($this->headerLinebreak, $headers).$this->headerLinebreak;
  }

  function body()
  {
   if($this->e->any("fatal"))
    return false;

   if(sizeof($this->data["Attachment"])==1)
    return $this->data["Attachment"][0]["content"].$this->bodyLinebreak;

   return $this->_bodyAttachments();
  }

  function _bodyAttachments($parent=0)
  {
   if(!isset($this->data["Attachment"][$parent])) //parent doesn't exist
    return "";

   //generate body for current sub attachments
   $body="";
   $l=sizeof($this->data["Attachment"]);
   for($c=0;$c<$l;$c++)
   {
    if($this->data["Attachment"][$c]["parent"]!==$parent) //only current parent
     continue;

    //open attachment
    $body.="--".$this->boundaryPrefix.$this->data["Attachment"][$parent]["boundary"].$this->bodyLinebreak;

    //content type
    $body.="Content-Type: ".$this->data["Attachment"][$c]["contentType"];

    if($this->data["Attachment"][$c]["contentType"]=="multipart/alternative"||$this->data["Attachment"][$c]["contentType"]=="multipart/related")
     $body.="; boundary=".$this->boundaryPrefix.$this->data["Attachment"][$c]["boundary"];

    if($this->data["Attachment"][$c]["contentCharset"])
     $body.="; charset=".$this->data["Attachment"][$c]["contentCharset"];

    if($this->data["Attachment"][$c]["name"])
     $body.="; name=\"".preg_replace("!^.*/([^/]+)$!", "\\1", $this->data["Attachment"][$c]["name"])."\"";

    $body.=$this->bodyLinebreak;

    //content encoding
    if($this->data["Attachment"][$c]["contentEncoding"])
     $body.="Content-Transfer-Encoding: ".$this->data["Attachment"][$c]["contentEncoding"].$this->bodyLinebreak;

    //content disposition
    if($this->data["Attachment"][$c]["contentType"]=="text/plain"&&$this->data["Attachment"][$c]["type"]=="message")
     $body.="Content-Disposition: inline".$this->bodyLinebreak;
    else if($this->data["Attachment"][$c]["name"])
    {
     $body.="Content-Disposition: attachment; filename=\"".preg_replace("!^.*/([^/]+)$!", "\\1", $this->data["Attachment"][$c]["name"])."\"".$this->bodyLinebreak;
     $body.="Content-ID: <".$this->data["Attachment"][$c]["boundary"].">".$this->bodyLinebreak;
    }

    //actual content
    $body.=$this->bodyLinebreak.$this->data["Attachment"][$c]["content"].$this->bodyLinebreak;

    //generate sub attachment
    $body.=$this->_bodyAttachments($c);
   }

   //close attachment
   if($body)
    $body.="--".$this->boundaryPrefix.$this->data["Attachment"][$parent]["boundary"]."--".$this->bodyLinebreak;

   return $body;
  }

 }

?>
