<?

 /*****************************************************************************
 * Filename: include/functions.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."plugin.php");

 function d()
 {
  echo "<pre>";
  foreach(func_get_args() as $arg)
  {
   if(is_string($arg)&&preg_match("/^SELECT/i", $arg))
    print preg_replace("/(SELECT |FROM |LEFT JOIN |WHERE | AND |GROUP BY |HAVING |ORDER BY |IF)/mi", "\n\\1", $arg)."\n\n";
   else
    var_dump($arg);
  }
  echo "</pre>";
 }

 function bt()
 {
  echo "<pre>";
  debug_print_backtrace();
  echo "</pre>";
 }

 //set_error_handler("jas_error_handler");
 //set_exception_handler("jas_error_handler");
 function jas_error_handler($number, $error="", $file=false, $line=false, $context=false)
 {
  if(is_object($number)) //exception
  {
   $e=$number;
   $number=-$e->getCode();
   $error=$e->getMessage();
   $file=$e->getFile();
   $line=$e->getLine();
  }

  switch($number)
  {
   case E_STRICT:
    $level="Strict";
   break;

   case E_DEPRECATED:
   case E_USER_DEPRECATED:
    $level="Deprecated";
   break;

   case E_NOTICE:
   case E_USER_NOTICE:
    $level="Notice";
   break;

   case E_WARNING:
   case E_USER_WARNING:
    $level="Warning";
   break;

   case E_NOTICE:
   case E_USER_NOTICE:
    $level="Notice";
   break;

   default:
   {
    if($number<=0)
     $level="Exception";
    else
     $level="Error";

    break;
   }
  }

  echo "<b>".$level."</b>: ".$error.($file?" in <b>".$file."</b> on line <b>".$line."</b>":"")."<br />\n";

  if($number<=0) //exception
  {
   $c=sizeof($e->getTrace());
   foreach($e->getTrace() as $bt)
    echo $c--.". <b>".(isset($bt["class"])?$bt["class"].$bt["type"]:"").$bt["function"]."</b>".(isset($bt["args"])?"(".implode(", ", $bt["args"]).")":"")." in <b>".$bt["file"]."</b> on line <b>".$bt["line"]."</b><br/>\n";

   exit;
  }
 }

 function request()
 {
  $keys=func_get_args();

  //cast to a string, inserting 0 in a switch results in it matching the first case
  switch((string)$keys[(sizeof($keys)-1)]) //check if last argument is method selection
  {
   case "POST":
   {
    array_pop($keys); //remove last "method" argument
    $r=&$_POST;
    break;
   }
   case "GET":
   {
    array_pop($keys); //remove last "method" argument
    $r=&$_GET;
    break;
   }
   case "COOKIE":
   {
    array_pop($keys); //remove last "method" argument
    $r=&$_COOKIE;
    break;
   }
   case "SESSION":
   {
    array_pop($keys); //remove last "method" argument
    $r=&$_SESSION;
    break;
   }
   case "SERVER":
   {
    array_pop($keys); //remove last "method" argument
    $r=&$_SERVER;
    break;
   }
   case "FILES":
   {
    array_pop($keys); //remove last "method" argument
    $r=&$_FILES;
    break;
   }
   case "REQUEST":
   {
    array_pop($keys); //remove last "method" argument
    $r=&$_REQUEST;
    break;
   }
   default: //autodetect
   {
    if(isset($_POST[$keys[0]]))
     $r=&$_POST;
    else if(isset($_GET[$keys[0]]))
     $r=&$_GET;
    else
     return NULL;

    break;
   }
  }

  foreach($keys as $key)
  {
   if(is_array($r)&&isset($r[$key]))
   {
    $tmp=$r[$key];
    unset($r); //remove reference to old predefined variable
    $r=&$tmp;
   }
   else
    return NULL;
  }
  return $r;
 }

 function array_type(&$array)
 {
  if(is_array($array))
  {
   $type="indexed";

   for($c=0;$c<sizeof($array);$c++)
   {
    if(!isset($array[$c]))
    {
     $type="associative";
     break;
    }
   }
  }
  else if(is_object($array)&&preg_match("/data/i", get_class($array)))
   $type="data";
  else
   $type="unknown";

  return $type;
 }

 function array_remove($array, $value)
 {
  switch(array_type($array))
  {
   case "indexed":
   {
    if(($index=array_search($value, $array))!==false) //found
    {
     if($index==0) //first entry
      array_shift($array);
     else if($index==sizeof($array)-1) //last entry
      array_pop($array);
     else
     {
      $array=array_merge(array_slice($array, 0, $index), array_slice($array, $index+1));
     }
    }

    break;
   }
   case "associative":
   {
    if(($index=array_search($value, $array))!==false) //found
     unset($array[$index]);

    break;
   }
   default:
    break;
  }

  return $array;
 }

 function get_argument_names($function, $object=false, $required=false)
 {
  if($object)
  {
   if(!method_exists($object, $function))
    return false;

   $ref=new ReflectionMethod($object, $function);
  }
  else
  {
   if(!function_exists($function))
    return false;

   $ref=new ReflectionFunction($function);
  }

  $args=Array();
  if($required) //only required arguments
  {
   foreach($ref->getParameters() as $arg)
    if(!$arg->isOptional())
     $args[]=$arg->name;
  }
  else //all arguments (including optional)
  {
   foreach($ref->getParameters() as $arg)
    $args[]=$arg->name;
  }

  return $args;
 }

?>
