<?

 /*****************************************************************************
 * Filename: include/language.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 if(!$_session->get("language"))
  $_session->set("language", Array("errors" => Array(), "files" => Array()));

 $_language=Array();
 $_language["errors"]=&$_session->getReference("language.errors");
 $_language["files"]=&$_session->getReference("language.files");

 class Language
 {
  function __construct($original=false, $language=false)
  {
   global $_language, $_config;

   if($original) //original language overwrite
    $this->original=$original;
   else //default original language
    $this->original="en";

   if($language) //language overwrite
    $this->language=$language;
   else if($_config&&($language=$_config->get("language"))) //configured language
    $this->language=$language;
   else //default language: use original (aka no translations)
    $this->language=$this->original;

   $this->sets=Array(); //loaded sets

   //set storage references
   $this->translations=&$_language["translations"];
   $this->errors=&$_language["errors"];
   $this->files=&$_language["files"];

   if($_config&&$_config->get("translationerrors"))
    $this->showErrors=true;
   else
    $this->showErrors=false;
  }


  function clear()
  {
   $this->translations=Array();
   $this->errors=Array();
   $this->files=Array();

   $this->sets=Array(); //loaded sets

   return true;
  }

  function setOriginal($original)
  {
   $this->original=$original;
   return true;
  }

  function setLanguage($language)
  {
   $this->language=$language;
   return true;
  }

  function showErrors($show)
  {
   $this->showErrors=($show?true:false);
   return true;
  }


  function getFilename($set, $language=false)
  {
   global $_path;

   if(!$language)
    $language=$this->language;

   //determine filename
   if(preg_match("/^plugin_(.+)$/", $set, $tmp))
   {
    $plugin=$tmp[1];

    if(isset($_path["project"])&&file_exists($_path["project"]["plugins"].$plugin."/language/".$language."/".$plugin.".xml"))
     $file=$_path["project"]["plugins"].$plugin."/language/".$language."/".$plugin.".xml";
    else if(file_exists($_path["plugins"].$plugin."/language/".$language."/".$plugin.".xml"))
     $file=$_path["plugins"].$plugin."/language/".$language."/".$plugin.".xml";
    else
     return false;
   }
   else if(preg_match("/^module_(.+)_(.+)$/", $set, $tmp))
   {
    $plugin=$tmp[1];
    $module=$tmp[2];

    if(isset($_path["project"])&&file_exists($_path["project"]["plugins"].$plugin."/language/".$language."/modules/".$module.".xml"))
     $file=$_path["project"]["plugins"].$plugin."/language/".$language."/modules/".$module.".xml";
    else if(file_exists($_path["plugins"].$plugin."/language/".$language."/modules/".$module.".xml"))
     $file=$_path["plugins"].$plugin."/language/".$language."/modules/".$module.".xml";
    else
     return false;
   }
   else if(preg_match("/^module_(.+)$/", $set, $tmp))
   {
    $module=$tmp[1];

    if(isset($_path["project"])&&file_exists($_path["project"]["include"]."language/".$language."/modules/".$module.".xml"))
     $file=$_path["project"]["include"]."language/".$language."/modules/".$module.".xml";
    else if(file_exists($_path["include"]."language/".$language."/modules/".$module.".xml"))
     $file=$_path["include"]."language/".$language."/modules/".$module.".xml";
    else
     return false;
   }
   else
   {
    if(isset($_path["project"])&&file_exists($_path["project"]["include"]."language/".$language."/".$set.".xml"))
     $file=$_path["project"]["include"]."language/".$language."/".$set.".xml";
    else if(file_exists($_path["include"]."language/".$language."/".$set.".xml"))
     $file=$_path["include"]."language/".$language."/".$set.".xml";
    else
     return false;
   }

   return $file;
  }

  function read($set, $language=false)
  {
   global $_path;

   //read sets from language object
   if(is_object($set)&&strtolower(get_class($set))=="language")
   {
    $object=$set;
    unset($set);

    //load missing sets in current object
    $tmp=$this->original;
    foreach($object->sets as $original => $languages)
    {
     if($this->original!=$original)
     {
      if(!$this->setOriginal($original))
      {
       $this->setOriginal($tmp);
       return false;
      }
     }

     if(!isset($this->sets[$this->original]))
      $this->sets[$this->original]=Array();

     foreach($languages as $language => $sets)
     {
      if(!isset($this->sets[$this->original][$language]))
       $this->sets[$this->original][$language]=Array();

      foreach(array_reverse($sets) as $set) //reverse to ensure reverse below is undone
      {
       if(!in_array($set, $this->sets[$this->original][$language])) //set not yet loaded
       {
        if(!$this->read($set, $language))
        {
         $this->setOriginal($tmp);
         return false;
        }
       }
      }
     }
    }

    $this->setOriginal($tmp);
    return true;
   }

   //read set from file/cache
   if(!$language)
    $language=$this->language;

   if($language==$this->original)
    return false;

   if(isset($this->translations["raw"][$this->original][$language][$set])) //set already read
   {
    if(!isset($this->sets[$this->original][$language]))
     $this->sets[$this->original][$language]=Array();

    if(!in_array($set, $this->sets[$this->original][$language])) //set not yet loaded
     array_unshift($this->sets[$this->original][$language], $set); //add to loaded sets, ensure last loaded translations are considered first

    return true;
   }

   if(!($file=$this->getFilename($set, $language)))
    return false;

   //read file
   if(!file_exists($file))
    return false;

   if(!($xml=@file_get_contents($file)))
    return false;

   //get original/translated language
   if(!preg_match("/<language original=\"([^\"]+)\" translated=\"([^\"]+)\"/", $xml, $m))
    return false;

   $original=$m[1];
   $translated=$m[2];

   if($original!=$this->original) //original language not expected input language
    return false;

   if($translated!=$language) //translated language not expected output language
    return false;

   if($original==$language) //original language same as output language
    return true; //don't bother parsing

   //get all translations
   if(!preg_match_all("/<translation original=\"([^\"]+)\" translated=\"([^\"]+)\"/", $xml, $m))
    return false;

   $l=sizeof($m[0]);
   for($c=0;$c<$l;$c++)
    if(!$this->setTranslation($set, $original, $translated, html_entity_decode($m[1][$c]), html_entity_decode($m[2][$c])))
     return false;

   if(!isset($this->files[$this->original][$language]))
    $this->files[$this->original][$language]=Array();

   $file=preg_replace("/^".preg_quote($_path["root"], "/")."/", "", $file, 1); //remove current root from file path (needed for cross page storing)
   $this->files[$this->original][$language][$set]=$file; //store file in session

   if(!isset($this->sets[$this->original][$language]))
    $this->sets[$this->original][$language]=Array();

   array_unshift($this->sets[$this->original][$language], $set); //add to loaded sets, ensure last loaded translations are considered first
   return true;
  }

  function updated($set, $language=false)
  {
   if(!$language)
    $language=$this->language;

   if(!($file=$this->getFilename($set, $language)))
    return false;

   return filemtime($file);
  }


  function setTranslation($set, $original, $language, $text, $translated)
  {
   $this->translations["raw"][$original][$language][$set][$text]=$translated;

   //determine type of translation for faster processing
   if(preg_match("/\|(translate|notranslate)\|/", $text)) //regexp
   {
    $regexp="/^".preg_quote($text, "/")."$/sm";
    $regexp=str_replace("\|translate\|", "(.*)", $regexp);
    $regexp=str_replace("\|notranslate\|", "(.*)", $regexp);
    $this->translations["regexp"][$original][$language][$set][$regexp]=$translated;

    preg_match_all("/\|(translate|notranslate)\|/", $text, $tmp);
    $this->translations["regexpTypes"][$original][$language][$set][$regexp]=$tmp[1];
   }
   else //simple translation
   {
    $this->translations["normal"][$original][$language][$set][$text]=$translated;
    $this->translations["lowercase"][$original][$language][$set][strtolower($text)]=strtolower($translated);
   }

   return true;
  }


  function t($text, $language=false)
  {
   global $_config;

   if(!$text) //no text
    return false;

   if(!$language)
    $language=$this->language;

   if($this->original==$language) //translation same as default language
    return $text; //display original text

   if(preg_match("/^[^\w]+$/", $text)) //no actual text
    return $text; //display original text

   if(!isset($this->sets[$this->original][$language])) //no translation loaded
    return $text; //display original text

   foreach($this->sets[$this->original][$language] as $set)
   {
    if(isset($this->translations["normal"][$this->original][$language][$set][$text])) //simple translation
     return $this->translations["normal"][$this->original][$language][$set][$text];

    $lowercase=strtolower($text);
    if(isset($this->translations["lowercase"][$this->original][$language][$set][$lowercase])) //simple lower case translation
     return $this->translations["lowercase"][$this->original][$language][$set][$lowercase];

    if(isset($this->translations["regexp"][$this->original][$language][$set])) //regexp translation
    {
     foreach(array_keys($this->translations["regexp"][$this->original][$language][$set]) as $regexp)
     {
      if(preg_match($regexp, $text, $tmp))
      {
       $newText=$this->translations["regexp"][$this->original][$language][$set][$regexp];
       for($c=1;$c<sizeof($tmp);$c++)
       {
        if($this->translations["regexpTypes"][$this->original][$language][$set][$regexp][$c-1]=="notranslate")
         $newText=str_replace("|".$c."|", $tmp[$c], $newText);
        else
         $newText=str_replace("|".$c."|", $this->t($tmp[$c], $language), $newText);
       }

       return $newText;
      }
     }
    }
   }

   //translation error
   if($this->showErrors)
   {
    //store translation error in session
    if(!isset($this->errors[$this->original][$language]))
     $this->errors[$this->original][$language]=Array();

    $this->errors[$this->original][$language][$text]=Array(
     "url" => (isset($_SERVER["HTTP_HOST"])?(isset($_SERVER["HTTPS"])&&$_SERVER["HTTPS"]?"https":"http")."://".$_SERVER["HTTP_HOST"].$_SERVER["REQUEST_URI"]:false),
     "sets" => $this->sets[$this->original][$language],
    );

    return "-!-".$text."-!-"; //display original text with error markers
   }

   return $text; //display original text
  }
 }
?>
