<?

 /*****************************************************************************
 * Filename: include/plugins/cms/modules/pluginConfigs.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class Plugin_Module_cms_pluginConfigs extends Module
 {
  function __construct(&$data)
  {
   global $_path, $_config;

   parent::__construct($data);

   if($this->data->module!="pluginConfigs")
    $this->extend("pluginConfigs");

   $this->data->title="Content Management System plugin configuration";
   $this->data->addField("module", "module", "all", "reload", 1, 25, false, "Default module");
   $this->data->addField("modules", "modules", "all", "", 1, 65536, "", "Allowed modules");

   $this->data->addField("action", "list", "all", "reload", 1, 25, false, "Default action");
   $this->data->listOptions("action", "", "view", "add", "copy", "edit", "move", "delete", "owner", "user");
   $this->data->addField("actions", "multilist", "all", "", 1, 65536, "", "Allowed actions");
   $this->data->listOptions("actions", "view", "add", "copy", "edit", "move", "delete", "owner", "user");

   $this->data->addField("extra", "list", "all", "reload", 1, 25, false, "Default extra option");
   $this->data->listOptions("extra", "", "item", "login", "logout", "password");
   $this->data->addField("extras", "multilist", "all", "", 1, 65536, "", "Allowed extra options");
   $this->data->listOptions("extras", "item", "login", "logout", "password");


   $this->data->addField("hideCategory", "boolean", "all", "", false, false, false, "Hide category on add");
   $this->data->addField("useConfig", "boolean", "all", "reload", false, false, false, "Use alternative config");
   $this->data->addField("configLabel", "label", "form", "", false, false, false, "");

   //add configCMS fields for configuration
   $this->configFields=Array();
   $d=Data::cache("configCMS", $this->data->action, $this->data->extra, Array("skipPostInit" => true, "skipCheckTables" => true));
   $this->data->l->read($d->l); //read configCMS translations

   foreach($d->check("use", "sql") as $field)
   {
    if( //all fields except default+control fields
     !$d->check("properties", "default", $field)&&
     !$d->check("properties", "control", $field)
    )
    {
     $this->data->addField($field,
      $d->field[$field]["type"],
      $d->field[$field]["use"],
      $d->field[$field]["properties"],
      $d->field[$field]["min"],
      $d->field[$field]["max"],
      ($_config->get("cms.".$field)!==NULL?$_config->get("cms.".$field):$d->field[$field]["defaultvalue"]), //use _config or default value
      "" //don't add title/text, already translated
     );

     //copy translated title/text
     $this->data->field[$field]["title"]=$d->field[$field]["title"];
     $this->data->field[$field]["text"]=$d->field[$field]["text"];

     if($d->field[$field]["type"]=="enum") //get enum options
      $this->data->enumOptions($field, $d->listOptions[$field]);

     $this->configFields[]=$field;
    }
   }

   $this->data->group("Modules", "module", "modules");
   $this->data->group("Actions", "action", "actions");
   $this->data->group("Extra options", "extra", "extras");
   $this->data->group("Configuration", array_merge(Array("hideCategory", "useConfig", "configLabel"), $this->configFields));
  }

  function postInit(){return $this->customizeFields();}
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}
  function movePostGet(){return $this->customizeFields();}
  function deletePostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   if($this->data->getValue("module"))
   {
    if(!$this->data->getValue("modules")||!is_array($this->data->getValue("modules")))
     $this->data->setValue("modules", Array($this->data->getValue("module")));
    else if(!in_array($this->data->getValue("module"), $this->data->getValue("modules")))
     $this->data->setValue("modules", array_merge(Array($this->data->getValue("module")), $this->data->getValue("modules")));
   }

   if($this->data->getValue("action"))
   {
    if(!$this->data->getValue("actions")||!is_array($this->data->getValue("actions")))
     $this->data->setValue("actions", Array($this->data->getValue("action")));
    else if(!in_array($this->data->getValue("action"), $this->data->getValue("actions")))
     $this->data->setValue("actions", array_merge(Array($this->data->getValue("action")), $this->data->getValue("actions")));
   }

   if($this->data->getValue("extra"))
   {
    if(!$this->data->getValue("extras")||!is_array($this->data->getValue("extras")))
     $this->data->setValue("extras", Array($this->data->getValue("extra")));
    else if(!in_array($this->data->getValue("extra"), $this->data->getValue("extras")))
     $this->data->setValue("extras", array_merge(Array($this->data->getValue("extra")), $this->data->getValue("extras")));
   }

   if($this->data->getValue("useConfig"))
   {
    $this->data->resetField("configLabel");
    foreach($this->configFields as $field)
     $this->data->resetField($field);
   }
   else
   {
    $this->data->field["configLabel"]["properties"]=";hidden";
    foreach($this->configFields as $field)
     $this->data->field[$field]["properties"]=";hidden";
   }

   return true;
  }

  function addPreSubmitErrorCheck(){return $this->checkFields();}
  function copyPreSubmitErrorCheck(){return $this->checkFields();}
  function editPreSubmitErrorCheck(){return $this->checkFields();}

  function checkFields() //not needed if postInit calls customizeFields
  {
   if($this->data->getValue("module")&&is_array($this->data->getValue("modules"))&&!in_array($this->data->getValue("module"), $this->data->getValue("modules")))
   {
    $this->data->e->add("Default module not in allowed modules list", "error");
    return false;
   }

   if($this->data->getValue("action")&&is_array($this->data->getValue("actions"))&&!in_array($this->data->getValue("action"), $this->data->getValue("actions")))
   {
    $this->data->e->add("Default action not in allowed actions list", "error");
    return false;
   }

   if($this->data->getValue("extra")&&is_array($this->data->getValue("extras"))&&!in_array($this->data->getValue("extra"), $this->data->getValue("extras")))
   {
    $this->data->e->add("Default extra option not in allowed extra options list", "error");
    return false;
   }

   return true;
  }
 }
?>
