<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/backend.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."data.php");
 require_once($_path["include"]."fs.php");

 class Plugin_Backend_unit_testing extends Base_Plugin_Backend
 {
  function main()
  {
   global $_path, $_benchmark;

   if($_benchmark) $_benchmark->start("Unit testing");

   $this->e->clear();

   $this->plugin->path["images"]=$_path["HTML"]["current"].$this->plugin->path["root"]."images/";
   $this->plugin->path["ut"]="ut/";

   $this->initUnits();

   $this->statistics=Array(
    "tests" => 0,
    "tested" => 0,
    "failed" => 0,
    "ok" => 0,
   );

   //parse input
   $this->runtests=false;
   if(($unit=str_replace("_", " ", request("unit")))&&isset($this->units[$unit]))
    $this->runtests=$unit;

   if($_benchmark) $_benchmark->stop();

   return true;
  }

  function output()
  {
   global $_benchmark;

   if($_benchmark) $_benchmark->start("Unit testing");

   //list & run unit tests
   $output="<div class=\"twothird\">\n";
   $output.=$this->list_tests_tree("UT", $this->runtests);
   $output.="</div>\n";

   //generate statistics
   $stats=Array();

   $stats["okImage"]=$this->plugin->path["images"]."ok.png";
   $stats["okReal"]=$this->statistics["ok"];

   if($this->statistics["tests"])
    $stats["okPercentage"]=round(($this->statistics["ok"]/$this->statistics["tests"])*100, 1);
   else
    $stats["okPercentage"]="0";

   $stats["skippedImage"]=$this->plugin->path["images"]."nottested.png";
   $stats["skippedReal"]=($this->statistics["tests"]-$this->statistics["tested"]);

   if($this->statistics["tests"])
    $stats["skippedPercentage"]=round((($this->statistics["tests"]-$this->statistics["tested"])/$this->statistics["tests"])*100, 1);
   else
    $stats["skippedPercentage"]="0";

   $stats["failedImage"]=$this->plugin->path["images"]."failed.png";
   $stats["failedReal"]=$this->statistics["failed"];

   if($this->statistics["tests"])
    $stats["failedPercentage"]=round(($this->statistics["failed"]/$this->statistics["tests"])*100, 1);
   else
    $stats["failedPercentage"]="0";

   if($this->statistics["ok"]==$this->statistics["tested"])
    $stats["totalImage"]=$this->plugin->path["images"]."ok.png";
   else if($this->statistics["failed"]==$this->statistics["tested"])
    $stats["totalImage"]=$this->plugin->path["images"]."failed.png";
   else if($this->statistics["tested"]==0)
    $stats["totalImage"]=$this->plugin->path["images"]."skipped.png";
   else
    $stats["totalImage"]=$this->plugin->path["images"]."partiallyfailed.png";

   $stats["totalPercentage"]=100;
   $stats["totalReal"]=$this->statistics["tests"];

   $output.="<div class=\"third\">\n";

   //render statistics
   $output.="<h3>".$this->plugin->l->t("Statistics")."</h3>\n";
   $output.="<table class=\"statistics\">\n";
   $output.=" <tr><td class=\"image\"><img src=\"".$stats["okImage"]."\" alt=\"".$this->plugin->l->t("OK")."\" class=\"status\"> ".$this->plugin->l->t("OK")."</td><td class=\"percentage nr\">".$stats["okPercentage"]."%</td><td class=\"real nr\">(".$stats["okReal"].")</td></tr>\n";
   $output.=" <tr><td class=\"image\"><img src=\"".$stats["skippedImage"]."\" alt=\"".$this->plugin->l->t("Skipped")."\" class=\"status\"> ".$this->plugin->l->t("Skipped")."</td><td class=\"percentage nr\">".$stats["skippedPercentage"]."%</td><td class=\"real nr\">(".$stats["skippedReal"].")</td></tr>\n";
   $output.=" <tr><td class=\"image\"><img src=\"".$stats["failedImage"]."\" alt=\"".$this->plugin->l->t("Failed")."\" class=\"status\"> ".$this->plugin->l->t("Failed")."</td><td class=\"percentage nr\">".$stats["failedPercentage"]."%</td><td class=\"real nr\">(".$stats["failedReal"].")</td></tr>\n";
   $output.=" <tr><td class=\"image\"><img src=\"".$stats["totalImage"]."\" alt=\"".$this->plugin->l->t("Total")."\" class=\"status\"> ".$this->plugin->l->t("Total")."</td><td class=\"percentage nr\">".$stats["totalPercentage"]."%</td><td class=\"real nr\">(".$stats["totalReal"].")</td></tr>\n";
   $output.="</table>\n";

   $output.="</div>\n";

   if($_benchmark) $_benchmark->stop();

   return $output;
  }

  function list_tests_tree($parent, $showtests=false)
  {
   $c=0;
   $tmp=Array();
   foreach(array_keys($this->units) as $unit)
   {
    //show all units for this parent unit
    if($this->units[$unit]->_getParentUnitName()!=$parent)
     continue;

     //run tests only for units where tests are shown
    if($this->runtests)
     if($showtests===true||$unit==$showtests)
      $this->units[$unit]->_run();

    //basic unit
    $tmp[$unit]=Array(
     "status" => "Not tested",
     "statusImage" => $this->plugin->path["images"]."nottested.png",
     "subunits" => $this->list_tests_tree($unit, ($unit==$showtests?true:$showtests)),
     "tests" => Array(),
    );

    //determine parent unit status based on this unit
    if($this->runtests)
    {
     switch($this->units[$unit]->results["status"])
     {
      case NULL: break;
      case UT_OK:
      {
       $tmp[$unit]["status"]="OK";
       $tmp[$unit]["statusImage"]=$this->plugin->path["images"]."ok.png";

       $parent=$this->units[$unit]->_getParentUnitName();

       if($parent!="UT")
       {
        if(!isset($this->units[$parent]->results["status"]))
         $this->units[$parent]->results["status"]=$this->units[$unit]->results["status"];
        else if($this->units[$parent]->results["status"]==UT_NOTESTS)
         $this->units[$parent]->results["status"]=UT_OK;
        else if($this->units[$parent]->results["status"]==UT_FAILED)
         $this->units[$parent]->results["status"]=UT_PARTIALLYFAILED;
       }
       break;
      }
      case UT_FAILED:
      {
       $tmp[$unit]["status"]="Failed";
       $tmp[$unit]["statusImage"]=$this->plugin->path["images"]."failed.png";

       $parent=$this->units[$unit]->_getParentUnitName();

       if($parent!="UT")
       {
        if(!isset($this->units[$parent]->results["status"]))
         $this->units[$parent]->results["status"]=$this->units[$unit]->results["status"];
        else if($this->units[$parent]->results["status"]==UT_NOTESTS)
         $this->units[$parent]->results["status"]=UT_FAILED;
        else if($this->units[$parent]->results["status"]==UT_OK)
         $this->units[$parent]->results["status"]=UT_PARTIALLYFAILED;
       }
       break;
      }
      case UT_PARTIALLYFAILED:
      {
       $tmp[$unit]["status"]="Partially failed";
       $tmp[$unit]["statusImage"]=$this->plugin->path["images"]."partiallyfailed.png";

       $parent=$this->units[$unit]->_getParentUnitName();

       if($parent!="UT")
       {
        if(!isset($this->units[$parent]->results["status"]))
         $this->units[$parent]->results["status"]=$this->units[$unit]->results["status"];
        else if($this->units[$parent]->results["status"]==UT_NOTESTS)
         $this->units[$parent]->results["status"]=UT_PARTIALLYFAILED;
        else if($this->units[$parent]->results["status"]==UT_OK)
         $this->units[$parent]->results["status"]=UT_PARTIALLYFAILED;
       }
       break;
      }
     }
    }

    //show individual unit tests
    if($showtests===true||$unit==$showtests)
    {
     $c=0;
     foreach($this->units[$unit]->_getTests() as $test)
     {
      $this->statistics["tests"]++;

      $tmp[$unit]["tests"][$c]=Array(
       "title" => str_replace("_", " ", $test),
       "status" => "Not tested",
       "error" => "",
       "info" => $this->units[$unit]->_getTestInfo($test),
       "statusImage" => $this->plugin->path["images"]."nottested.png",
      );

      if($this->runtests)
      {
       $tmp[$unit]["tests"][$c]["time"]=$this->units[$unit]->results["tests"][$test]["time"];

       switch($this->units[$unit]->results["tests"][$test]["status"])
       {
        case UT_OK:
        {
         $this->statistics["ok"]++;
         $this->statistics["tested"]++;

         $tmp[$unit]["tests"][$c]["status"]="OK";
         $tmp[$unit]["tests"][$c]["error"]=$this->units[$unit]->results["tests"][$test]["e"]->show();
         $tmp[$unit]["tests"][$c]["statusImage"]=$this->plugin->path["images"]."ok.png";
         break;
        }
        case UT_FAILED:
        {
         $this->statistics["failed"]++;
         $this->statistics["tested"]++;

         $tmp[$unit]["tests"][$c]["status"]="Failed";
         $tmp[$unit]["tests"][$c]["error"]=$this->units[$unit]->results["tests"][$test]["e"]->show();
         $tmp[$unit]["tests"][$c]["statusImage"]=$this->plugin->path["images"]."failed.png";
         break;
        }
        case UT_SKIPPED:
        {
         $tmp[$unit]["tests"][$c]["status"]="Skipped";
         $tmp[$unit]["tests"][$c]["error"]=$this->units[$unit]->results["tests"][$test]["e"]->show();
         break;
        }
       }
      }

      $c++;
     }
    }
   }

   if(!$tmp)
    return "";

   //render units + tests
   $output="<ul class=\"units tree\">\n";

   foreach($tmp as $unit => $v)
   {
    $output.=" <li class=\"unit\"><img src=\"".$v["statusImage"]."\" alt=\"".$v["status"]."\"> <a href=\"".$this->plugin->url()."unit=".str_replace(" ", "_", $unit)."\" title=\"".$unit."\">".$unit."</a>\n";

    if($v["tests"])
    {
     $output.="  <ul class=\"tests tree\">\n";

     foreach($v["tests"] as $v2)
     {
      $output.="   <li class=\"test\"><div class=\"tool\"><img src=\"".$v2["statusImage"]."\" alt=\"".$v2["status"]."\" class=\"status\"> ".$v2["title"]." ";
      $output.="<div class=\"tip\"><table class=\"info\">";

      $output.="<tr><td class=\"key\" style=\"width: 10%; white-space: nowrap;\">".$this->plugin->l->t("Unit test")."</td><td class=\"value\">".$v2["info"]["unit"]."::".$v2["info"]["test"]."</td></tr>";
      $output.="<tr><td class=\"key\" style=\"width: 10%;\">".$this->plugin->l->t("File")."</td><td class=\"value\">".$v2["info"]["file"].": ".$v2["info"]["start"]."</td></tr>";
      $output.="<tr><td class=\"key\" style=\"width: 10%;\">".$this->plugin->l->t("Time")."</td><td class=\"value\">".round($v2["time"], 3)."s</td></tr>";

      if($v2["error"])
       $output.="<tr><td class=\"key\" style=\"width: 10%;\">".$this->plugin->l->t("Errors")."</td><td class=\"value\">".$v2["error"]."</td></tr>";

      $output.="</table></div></li>\n";
     }

     $output.="  </ul>\n";
    }

    if($v["subunits"])
     $output.=$v["subunits"]."\n";

    $output.=" </li>\n";
   }

   $output.="</ul>\n";

   return $output;
  }


  function initUnits()
  {
   global $_path;

   require_once($this->plugin->path["root"]."ut.php");

   //get paths
   $paths=Array();

   //system plugins (includes unit_testing plugin)
   foreach(JAS::plugins("system") as $plugin)
    if(FS::existsS($_path["plugins"].$plugin."/".$this->plugin->path["ut"], "directory"))
     $paths[]=$_path["plugins"].$plugin."/".$this->plugin->path["ut"];

   //project plugins
   foreach(JAS::plugins("project") as $plugin)
    if(FS::existsS($_path["project"]["plugins"].$plugin."/".$this->plugin->path["ut"], "directory"))
     $paths[]=$_path["project"]["plugins"].$plugin."/".$this->plugin->path["ut"];

   foreach($paths as $path)
    foreach(FS::filesS($path) as $testFile)
     if(preg_match("/\.php$/", $testFile))
      require_once($testFile);

   foreach(get_declared_classes() as $class)
    if(is_subclass_of($class, "UT"))
     $this->units[str_replace("_", " ", str_replace("UT_", "", $class))]=new $class();

   ksort($this->units);
  }

 }

?>
