<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/api_data.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/api.php");
 require_once($_path["include"]."functions.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."jas.php");
 require_once($_path["api"]."data.php");

 class UT_API_Data extends UT_API
 {
  function Constructor()
  {
   global $_path;

   $object=new API_Data();

   if(!is_object($object)||get_class($object)!="API_Data")
   {
    $this->e->add("Constructor doesn't return a API_Data object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if($object->type!="Data")
   {
    $this->e->add("API type isn't Data");
    return UT_FAILED;
   }

   if($object->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if($object->encoding!="ISO-8859-1")
   {
    $this->e->add("API enconding isn't ISO-8859-1");
    return UT_FAILED;
   }

   if($object->contentType!="text/plain")
   {
    $this->e->add("API content type isn't text/plain");
    return UT_FAILED;
   }

   if($object->filename)
   {
    $this->e->add("API filename shouldn't be set");
    return UT_FAILED;
   }

   if(!isset($object->config))
   {
    $this->e->add("API configuration not set");
    return UT_FAILED;
   }

   if($object->mode!="server")
   {
    $this->e->add("API mode isn't server");
    return UT_FAILED;
   }

   //merge config
   $api=new API_Data("server", $this->_config(true)); //API enabled

   if(!$api->config["data"])
   {
    $this->e->add("API Data should be enabled");
    return UT_FAILED;
   }

   $api=new API_Data("server", $this->_config(false)); //API disabled

   if($api->config["data"])
   {
    $this->e->add("API Data should be disabled");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new API_Data();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("API_Data object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->mode="different";
   if($orig->mode==$copy->mode)
   {
    $this->e->add("API_Data object copy also modified original \"mode\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("API_Data object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_API::Copy_Constructor(); //run parent Copy_Constructor tests
  }


  function Output()
  {
   $api=new API_Data("server");

   $expected=new Errors();
   $api->output=$expected;

   if(!($output=$api->output(false)))
   {
    $this->e->add("API output not gathered");
    return UT_FAILED;
   }

   if(!$this->_compare($expected, unserialize($output)))
   {
    $this->e->add("API output doesn't match expected output");
    return UT_FAILED;
   }

   if(!$this->_compare($expected, $api->output))
   {
    $this->e->add("API output variable doesn't match original value");
    return UT_FAILED;
   }


   $expected=Data::cache("categories");
   $api->output=clone $expected;

   if(!($output=$api->output(false)))
   {
    $this->e->add("API output not gathered");
    return UT_FAILED;
   }

   if(!$this->_compare($expected, unserialize($output)))
   {
    $this->e->add("API output doesn't match expected output");
    return UT_FAILED;
   }

   if(!$this->_compare($expected, $api->output))
   {
    $this->e->add("API output variable doesn't match original value");
    return UT_FAILED;
   }


   $expected=Array("type" => "modules", "modules" => JAS::modules());
   $api->output=$expected;

   if(!($output=$api->output(false)))
   {
    $this->e->add("API output not gathered");
    return UT_FAILED;
   }

   if(!$this->_compare($expected, unserialize($output)))
   {
    $this->e->add("API output doesn't match expected output");
    return UT_FAILED;
   }

   if(!$this->_compare($expected, $api->output))
   {
    $this->e->add("API output variable doesn't match original value");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

 class UT_API_Data_Server extends UT_API_Data
 {
  function Constructor()
  {
   $object=new API_Data("server");

   if($object->mode!="server")
   {
    $this->e->add("API mode isn't server");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Error()
  {
   $api=new API_Data("server");

   if(!$api->e->add("Error", "error", "Text"))
   {
    $this->e->add("Unable to add error");
    return UT_FAILED;
   }

   if(!$api->e->any())
   {
    $this->e->add("API should contain errors");
    return UT_FAILED;
   }

   if(!$api->error())
   {
    $this->e->add("API error should be successful");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!($e=unserialize($api->output(false))))
   {
    $this->e->add("Unable to parse API output");
    return UT_FAILED;
   }

   if(!$this->_compare($api->e, $e))
    return UT_FAILED;

   return UT_OK;
  }


  function Version()
  {
   $api=new API_Data("server", $this->_config(true));

   //no header version
   $api->e->clear();
   $response=serialize(Array("type" => "fail"));

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "unsupported parse fail request")===false)
   {
    $this->e->add("First error should contain: unsupported parse fail request");
    return UT_FAILED;
   }


   //valid header version
   $api->e->clear();
   if(!$api->setRequestResponseHeaders(Array(
    "X-JAS-API" => $api->type."-".JAS::versionS(),
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "unsupported parse fail request")===false)
   {
    $this->e->add("First error should contain: unsupported parse fail request");
    return UT_FAILED;
   }


   //invalid header version
   $api->e->clear();
   if(!$api->setRequestResponseHeaders(Array(
    "X-JAS-API" => $api->type."-0.1a",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   if($api->getHeaderVersion()!="0.1a")
   {
    $this->e->add("Requested API version isn't 0.1a");
    return UT_FAILED;
   }

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "header version: \"0.1a\" doesn't match version: \"1.0\"")===false)
   {
    $this->e->add("First error should contain: header version: \"0.1a\" doesn't match version: \"1.0\"");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse_Data_Request()
  {
   $api=new API_Data("server", $this->_config(true));

   $d=Data::cache("files", "view", "all");

   $d->setValue("nr", 123);
   $d->setValue("file", "JAS & API");
   $d->setValue("fileData", "fpgmp9)");

   $request=serialize(clone $d);

   if(!$api->parse($request))
   {
    $this->e->add("Data request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Data parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="data")
   {
    $this->e->add("Data parse should set data command");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["data"])||get_class($api->commands[0]["data"])!="Data")
   {
    $this->e->add("Data parse no data object");
    return UT_FAILED;
   }

   if(!$this->_compare($d, $api->commands[0]["data"]))
    return UT_FAILED;

   if($api->commands[0]["count"])
   {
    $this->e->add("Data parse count incorrect");
    return UT_FAILED;
   }


   $api->clear();
   $request=serialize(clone $d);

   if(!$api->parse($request))
   {
    $this->e->add("Data request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->commands[0]["count"])
   {
    $this->e->add("Data parse count incorrect");
    return UT_FAILED;
   }

   $api->clear();
   $d->_count=true;
   $request=serialize(clone $d);

   if(!$api->parse($request))
   {
    $this->e->add("Data request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands[0]["count"])
   {
    $this->e->add("Data parse count incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Parse_Language_Request()
  {
   $api=new API_Data("server", $this->_config(true));

   $request=serialize(Array("type" => "language", "set" => "modules", "language" => "nl"));

   if(!$api->parse($request))
   {
    $this->e->add("Language request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Language parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="language")
   {
    $this->e->add("Language parse should set language command");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["language"])||get_class($api->commands[0]["language"])!="Language")
   {
    $this->e->add("Language parse no language object");
    return UT_FAILED;
   }

   if(!$this->_compare(new Language(), $api->commands[0]["language"]))
    return UT_FAILED;

   if($api->commands[0]["lang"]!="nl")
   {
    $this->e->add("Data parse lang incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["set"]!="modules")
   {
    $this->e->add("Data parse set incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Parse_Modules_Request()
  {
   $api=new API_Data("server", $this->_config(true));

   $request=serialize(Array("type" => "modules"));

   if(!$api->parse($request))
   {
    $this->e->add("Language request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if($api->commands[0]["modules"]!=Array())
   {
    $this->e->add("Modules parse should set modules");
    return UT_FAILED;
   }


   $api->clear();
   $request=serialize(Array("type" => "modules", "modules" => Array("categories", "users")));

   if(!$api->parse($request))
   {
    $this->e->add("Language request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if($api->commands[0]["modules"]!=Array("categories", "users"))
   {
    $this->e->add("Modules parse should set modules");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Parse_Projects_Request()
  {
   $api=new API_Data("server", $this->_config(true));

   $request=serialize(Array("type" => "projects"));

   if(!$api->parse($request))
   {
    $this->e->add("Language request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Projects parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="projects")
   {
    $this->e->add("Projects parse should set projects command");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Format_Data_Response()
  {
   global $_session;

   $api=new API_Data("server", $this->_config(true));

   $d=Data::cache("categories", "view", "all");
   $d->addField("base64", "file", "show", "", 0, 10000);

   $d->setValue("nr", 1);

   $d->setShowFields("nr", "name", "text", "base64Data");
   $d->addMutilation("name", "strip", 100);
   $d->sortOrder("nr-");

   if(!$api->addData($d, true))
   {
    $this->e->add("API add data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   $api->commands[0]["data"]->setValue("name", "JAS");
   $api->commands[0]["data"]->setValue("text", "multi
line & value");
   $api->commands[0]["data"]->setValue("base64Data", "fpgmp9)");

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   //clean data object
   $send=Array("module", "action", "extra", "dataformat", "vmax", "field", "sortOrder"); //send keys
   foreach($d as $key => $value) //remove all but send key
    if(!in_array($key, $send))
     unset($d->$key);

   $send=Array("use", "properties", "value", "valuetype", "operant", "operator", "mutilation"); //send field keys
   foreach(array_keys($d->field) as $field) //clean field array
    foreach(array_keys($d->field[$field]) as $key)
     if(!in_array($key, $send))
      unset($d->field[$field][$key]);

   $d->moduleClassName=false; //no class name so sleep/wakeup doesn't break
   $d->_count=true; //add count flag

   if(!$this->_compare($d, $api->output))
    return UT_FAILED;


   $api->clear();
   $_session->set("sessionStorage.data.unit_testing_testmodule"); //ensure no values in DB

   $d=Data::cache("unit_testing_testmodule", "add", "item", Array("db" => "session"));
   $d->setValue("name", "test");
   $d->setValue("catnr", 2);
   $d->setShowFields(); //only get nr

   if(!$api->addData($d, false))
   {
    $this->e->add("API add data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   //clean data object
   $send=Array("module", "action", "extra", "dataformat", "vmax", "field", "sortOrder"); //send keys
   foreach($d as $key => $value) //remove all but send key
    if(!in_array($key, $send))
     unset($d->$key);

   $send=Array("use", "properties", "value", "valuetype", "operant", "operator", "mutilation"); //send field keys
   foreach(array_keys($d->field) as $field) //clean field array
    foreach(array_keys($d->field[$field]) as $key)
     if(!in_array($key, $send))
      unset($d->field[$field][$key]);

   $d->moduleClassName=false; //no class name so sleep/wakeup doesn't break
   $d->_count=false; //add count flag

   if(!$this->_compare($d, $api->output))
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    return UT_FAILED;
   }

   $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
   return UT_OK;
  }

  function Format_Language_Response()
  {
   $l=new Language();
   $api=new API_Data("server", $this->_config(true));

   if(!$api->addLanguage("modules", "nl"))
   {
    $this->e->add("API add language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$l->read("modules", "nl", true))
   {
    $this->e->add("API read language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$this->_compare($l, $api->output))
    return UT_FAILED;

   return UT_OK;
  }

  function Format_Modules_Response()
  {
   $api=new API_Data("server", $this->_config(true));

   if(!$api->addModules(Array()))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->output["type"]!="modules")
   {
    $this->e->add("Response should set modules type");
    return UT_FAILED;
   }

   if(sizeof($api->output["modules"])<10)
   {
    $this->e->add("Response should contain at least 10 modules sections");
    return UT_FAILED;
   }

   foreach(Array("categories", "users", "files") as $module)
   {
    if(!in_array($module, $api->output["modules"]))
    {
     $this->e->add("Modules response doesn't contain module: ".$module);
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function Format_Projects_Response()
  {
   global $_session;

   $api=new API_Data("server", $this->_config(true));

   if(!$api->addProjects(false))
   {
    $this->e->add("API add projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->output["type"]!="projects")
   {
    $this->e->add("Response should set projects type");
    return UT_FAILED;
   }

   if(!$this->_compare(Array("type" => "projects", "projects" => JAS::projects(), "current" => $_session->get("project")), $api->output))
    return UT_FAILED;

   return UT_OK;
  }
 }


 class UT_API_Data_Client extends UT_API_Data
 {
  function Constructor()
  {
   $object=new API_Data("client");

   if($object->mode!="client")
   {
    $this->e->add("API mode isn't client");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Error()
  {
   $api=new API_Data("client", $this->_config(true));

   $e=new Errors();
   if(!$e->add("Error", "error", "Text"))
   {
    $this->e->add("Unable to add error");
    return UT_FAILED;
   }

   $response=serialize($e);

   if(!$api->parse($response))
   {
    $this->e->add("Error response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if($api->e->getErrors()!=1)
   {
    $this->e->add("Error parse should set 1 error");
    return UT_FAILED;
   }

   if(!$this->_compare($e, $api->e))
    return UT_FAILED;

   return UT_OK;
  }


  function Version()
  {
   $api=new API_Data("client", $this->_config(true));

   //valid XML version
   $api->e->clear();
   $response=serialize(Array("type" => "fail"));

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if($api->e->getLevel(0)!="warning")
   {
    $this->e->add("First error should contain: should be a warning");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "header version not set, using version: \"1.0\"")===false)
   {
    $this->e->add("First error should contain: header version not set, using version: \"1.0\"");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(1), "unsupported parse fail response")===false)
   {
    $this->e->add("Second error should contain: unsupported parse fail response");
    return UT_FAILED;
   }


   //valid header version
   $api->e->clear();
   if(!$api->setRequestResponseHeaders(Array(
    "X-JAS-API" => $api->type."-1.0",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   $response=serialize(Array("type" => "fail"));

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "unsupported parse fail response")===false)
   {
    $this->e->add("First error should contain: unsupported parse fail response");
    return UT_FAILED;
   }


   //invalid header version
   $api->e->clear();
   if(!$api->setRequestResponseHeaders(Array(
    "X-JAS-API" => $api->type."-0.1a",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   $response=serialize(Array("type" => "fail"));

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "header version: \"0.1a\" doesn't match version: \"1.0\"")===false)
   {
    $this->e->add("First error should contain: header version: \"0.1a\" doesn't match version: \"1.0\"");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Format_Data_Request()
  {
   $api=new API_Data("client", $this->_config(true));

   $d=Data::cache("categories", "view", "all");
   $d->addField("base64", "file", "show", "", 0, 10000);

   $d->setValue("nr", 1);
   $d->setValue("name", "JAS");
   $d->setValue("text", "multi
line & value");
   $d->setValue("base64Data", "fpgmp9)");

   $d->setShowFields("nr", "name");
   $d->addMutilation("name", "strip", 100);
   $d->sortOrder("nr-");

   if(!$api->addData($d, true))
   {
    $this->e->add("API add data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   //clean data object
   $send=Array("module", "action", "extra", "dataformat", "vmax", "field", "sortOrder"); //send keys
   foreach($d as $key => $value) //remove all but send key
    if(!in_array($key, $send))
     unset($d->$key);

   $send=Array("use", "properties", "value", "valuetype", "operant", "operator", "mutilation"); //send field keys
   foreach(array_keys($d->field) as $field) //clean field array
    foreach(array_keys($d->field[$field]) as $key)
     if(!in_array($key, $send))
      unset($d->field[$field][$key]);

   $d->moduleClassName=false; //no class name so sleep/wakeup doesn't break
   $d->_count=true; //add count flag

   if(!$this->_compare($d, $api->output))
    return UT_FAILED;

   return UT_OK;
  }

  function Format_Language_Request()
  {
   $api=new API_Data("client", $this->_config(true));

   if(!$api->addLanguage("modules", "nl"))
   {
    $this->e->add("API add language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array("type" => "language", "set" => "modules", "language" => "nl"), $api->output))
    return UT_FAILED;

   return UT_OK;
  }

  function Format_Modules_Request()
  {
   $api=new API_Data("client", $this->_config(true));

   if(!$api->addModules())
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array("type" => "modules", "modules" => Array()), $api->output))
    return UT_FAILED;


   $api->clear();

   if(!$api->addModules(Array("categories", "users")))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array("type" => "modules", "modules" => Array("categories", "users")), $api->output))
    return UT_FAILED;


   $api->clear();
   $d=Data::cache("categories", "view", "all");

   if(!$api->addModules(Array(&$d)))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array("type" => "modules", "modules" => Array("categories")), $api->output))
    return UT_FAILED;

   return UT_OK;
  }

  function Format_Projects_Request()
  {
   $api=new API_Data("client", $this->_config(true));

   if(!$api->addProjects())
   {
    $this->e->add("API add projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array("type" => "projects"), $api->output))
    return UT_FAILED;


   $api->clear();

   if(!$api->addProjects("jas"))
   {
    $this->e->add("API add projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$this->_compare(Array("type" => "projects", "project" => "jas"), $api->output))
    return UT_FAILED;

   return UT_OK;
  }


  function Parse_Data_Response()
  {
   $api=new API_Data("client", $this->_config(true));

   $d=Data::cache("files", "view", "all", Array("format" => "api"));

   $d->setValue("nr", 1);
   $d->setValue("file", "JAS & API");
   $d->setValue("fileData", "fpgmp9)");

   $d->setShowFields("nr", "name");
   $d->addMutilation("file", "strip", 100);
   $d->sortOrder("nr-");

   $response=serialize(clone $d);

   if(!$api->parse($response))
   {
    $this->e->add("Data response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Data parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="data")
   {
    $this->e->add("Data parse should set data command");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["data"])||get_class($api->commands[0]["data"])!="Data")
   {
    $this->e->add("Data parse no data object");
    return UT_FAILED;
   }

   if(!$this->_compare($d, $api->commands[0]["data"]))
    return UT_FAILED;

   return UT_OK;
  }


  function Parse_Language_Response()
  {
   $l=new Language();
   $api=new API_Data("client", $this->_config(true));

   if(!$l->read("modules", "nl", true))
   {
    $this->e->add("API read language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   $response=serialize($l);

   if(!$api->parse($response))
   {
    $this->e->add("Language response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Language parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="language")
   {
    $this->e->add("Language parse should set language command");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["language"])||get_class($api->commands[0]["language"])!="Language")
   {
    $this->e->add("Language parse no language object");
    return UT_FAILED;
   }

   if(!$this->_compare($l, $api->commands[0]["language"]))
    return UT_FAILED;

   if($api->commands[0]["lang"]!="nl")
   {
    $this->e->add("Language parse lang incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse_Modules_Response()
  {
   $api=new API_Data("client", $this->_config(true));

   $response=serialize(Array("type" => "modules", "modules" => Array("categories", "users")));

   if(!$api->parse($response))
   {
    $this->e->add("Modules response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["modules"])!=2)
   {
    $this->e->add("Response should contain 2 modules");
    return UT_FAILED;
   }


   $api->clear();

   $response=serialize(Array("type" => "modules", "modules" => Array(Data::cache("users"))));

   if(!$api->parse($response))
   {
    $this->e->add("Modules response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["modules"])!=1)
   {
    $this->e->add("Response should contain 1 modules");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["modules"][0])||get_class($api->commands[0]["modules"][0])!="Data")
   {
    $this->e->add("Modules parse no data object");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse_Projects_Response()
  {
   $api=new API_Data("client", $this->_config(true));

   $response=serialize(Array("type" => "projects", "projects" => Array("jas", "test"), "current" => "jas"));

   if(!$api->parse($response))
   {
    $this->e->add("Projects response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!=JAS::versionS())
   {
    $this->e->add("API version isn't ".JAS::versionS());
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Projects parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="projects")
   {
    $this->e->add("Projects parse should set projects command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["projects"])!=2)
   {
    $this->e->add("Response should contain 2 projects");
    return UT_FAILED;
   }

   if($api->commands[0]["activate"]!="jas")
   {
    $this->e->add("Response should set active project");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

?>
