<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/api_xml.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/api.php");
 require_once($_path["format"]."xml.php");
 require_once($_path["api"]."xml.php");

 class UT_API_XML extends UT_API
 {
  function Constructor()
  {
   global $_path;

   $object=new API_XML();

   if(!is_object($object)||get_class($object)!="API_XML")
   {
    $this->e->add("Constructor doesn't return a API_XML object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if($object->type!="XML")
   {
    $this->e->add("API type isn't XML");
    return UT_FAILED;
   }

   if($object->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if($object->encoding!="ISO-8859-1")
   {
    $this->e->add("API enconding isn't ISO-8859-1");
    return UT_FAILED;
   }

   if($object->contentType!="text/xml")
   {
    $this->e->add("API content type isn't text/xml");
    return UT_FAILED;
   }

   if($object->filename)
   {
    $this->e->add("API filename shouldn't be set");
    return UT_FAILED;
   }

   if(!isset($object->config))
   {
    $this->e->add("API configuration not set");
    return UT_FAILED;
   }

   if($object->mode!="server")
   {
    $this->e->add("API mode isn't server");
    return UT_FAILED;
   }

   if($object->templates!=$_path["api"]."xml/")
   {
    $this->e->add("API templates path incorrect");
    return UT_FAILED;
   }

   if(!file_exists($object->templates))
   {
    $this->e->add("Templates directory doesn't exist");
    return UT_FAILED;
   }

   if(!is_dir($object->templates))
   {
    $this->e->add("Templates directory isn't a directory");
    return UT_FAILED;
   }

   //merge config
   $api=new API_XML("server", $this->_config(true)); //API enabled

   if(!$api->config["xml"])
   {
    $this->e->add("API XML should be enabled");
    return UT_FAILED;
   }

   $api=new API_XML("server", $this->_config(false)); //API disabled

   if($api->config["xml"])
   {
    $this->e->add("API XML should be disabled");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new API_XML();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("API_XML object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->mode="different";
   if($orig->mode==$copy->mode)
   {
    $this->e->add("API_XML object copy also modified original \"mode\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("API_XML object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_API::Copy_Constructor(); //run parent Copy_Constructor tests
  }
 }


 class UT_API_XML_1_0 extends UT_API_XML
 {
  function ParseTemplate()
  {
   $api=new API_XML("server");

   if($api->version!="1.0")
   {
    $this->e->add("XML API version isn't 1.0");
    return UT_FAILED;
   }

   if(!$api->parseTemplate("jas"))
   {
    $this->e->add("Can't parse template \"jas\"");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->parseTemplate("errors"))
   {
    $this->e->add("Can't parse template \"errors\"");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->parseTemplate("data"))
   {
    $this->e->add("Can't parse template \"data\"");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->parseTemplate("modules"))
   {
    $this->e->add("Can't parse template \"modules\"");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->parseTemplate("projects"))
   {
    $this->e->add("Can't parse template \"projects\"");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->parseTemplate("language"))
   {
    $this->e->add("Can't parse template \"language\"");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->parseTemplate("foobar"))
   {
    $this->e->add("Parsing template \"foobar\" should fail");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }


 class UT_API_XML_Server extends UT_API_XML
 {
  function Constructor()
  {
   $object=new API_XML("server");

   if($object->mode!="server")
   {
    $this->e->add("API mode isn't server");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Error()
  {
   $xml=new Format_XML();
   $api=new API_XML("server");

   if(!$api->e->add("Error", "error", "Text"))
   {
    $this->e->add("Unable to add error");
    return UT_FAILED;
   }

   if(!$api->e->any())
   {
    $this->e->add("API should contain errors");
    return UT_FAILED;
   }

   if(!$api->error())
   {
    $this->e->add("API error should be successful");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$xml->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/errors"))!=1)
   {
    $this->e->add("Response should contain 1 error");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/errors[0]/error", "error")!="Error")
   {
    $this->e->add("Error respond error doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/errors[0]/error", "level")!="error")
   {
    $this->e->add("Error respond level doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/errors[0]/error", "extra")!="Text")
   {
    $this->e->add("Error respond extra doesn't match");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Version()
  {
   $api=new API_XML("server", $this->_config(true));

   //valid XML version
   $api->e->clear();
   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <fail/>
</jas>';

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "unsupported parse fail request")===false)
   {
    $this->e->add("First error should contain: unsupported parse fail request");
    return UT_FAILED;
   }


   //invalid XML version
   $api->e->clear();
   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="0.1a">
 <fail/>
</jas>';

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="0.1a")
   {
    $this->e->add("API version isn't 0.1a");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "unsupported version: \"0.1a\"")===false)
   {
    $this->e->add("First error should contain: unsupported version: \"0.1a\"");
    return UT_FAILED;
   }


   //valid header+XML version
   $api->e->clear();
   if(!$api->setRequestResponseHeaders(Array(
    "X-JAS-API" => $api->type."-1.0",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <fail/>
</jas>';

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "unsupported parse fail request")===false)
   {
    $this->e->add("First error should contain: unsupported parse fail request");
    return UT_FAILED;
   }


   //mismatch header+XML version
   $api->e->clear();
   if(!$api->setRequestResponseHeaders(Array(
    "X-JAS-API" => $api->type."-0.1a",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <fail/>
</jas>';

   if($api->getHeaderVersion()!="0.1a")
   {
    $this->e->add("Requested API version isn't 0.1a");
    return UT_FAILED;
   }

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "header version: \"0.1a\" doesn't match XML version: \"1.0\"")===false)
   {
    $this->e->add("First error should contain: header version: \"0.1a\" doesn't match XML version: \"1.0\"");
    return UT_FAILED;
   }

   //invalid header+XML version
   $api->e->clear();
   if(!$api->setRequestResponseHeaders(Array(
    "X-JAS-API" => $api->type."-0.1a",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="0.1a">
 <fail/>
</jas>';

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="0.1a")
   {
    $this->e->add("API version isn't 0.1a");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "unsupported version: \"0.1a\"")===false)
   {
    $this->e->add("First error should contain: unsupported version: \"0.1a\"");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse_Data_Request()
  {
   $api=new API_XML("server", $this->_config(true));

   $request='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <data module="files" action="view" extra="all">
  <row>
   <field name="nr" value="123" />
   <field name="file">JAS &amp; API</field>
   <field name="fileData">ZnBnbcZwOSk=</field>
  </row>
 </data>
</jas>';

   if(!$api->parse($request))
   {
    $this->e->add("Data request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Data parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="data")
   {
    $this->e->add("Data parse should set data command");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["data"])||get_class($api->commands[0]["data"])!="Data")
   {
    $this->e->add("Data parse no data object");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->module!="files")
   {
    $this->e->add("Data parse module incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->action!="view")
   {
    $this->e->add("Data parse action incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->extra!="all")
   {
    $this->e->add("Data parse extra incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["count"])
   {
    $this->e->add("Data parse count incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->getValue("nr")!=123)
   {
    $this->e->add("Data parse getValue nr incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->getValue("file")!="JAS & API")
   {
    $this->e->add("Data parse getValue name incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->getValue("fileData")!="fpgmp9)")
   {
    $this->e->add("Data parse getValue name incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Parse_Language_Request()
  {
   $api=new API_XML("server", $this->_config(true));

   $request='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <language set="modules" language="nl"></language>
</jas>';

   if(!$api->parse($request))
   {
    $this->e->add("Language request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Language parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="language")
   {
    $this->e->add("Language parse should set language command");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["language"])||get_class($api->commands[0]["language"])!="Language")
   {
    $this->e->add("Language parse no language object");
    return UT_FAILED;
   }

   if($api->commands[0]["lang"]!="nl")
   {
    $this->e->add("Data parse lang incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["set"]!="modules")
   {
    $this->e->add("Data parse set incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Parse_Modules_Request()
  {
   $api=new API_XML("server", $this->_config(true));

   $request='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <modules></modules>
</jas>';

   if(!$api->parse($request))
   {
    $this->e->add("Modules request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if($api->commands[0]["modules"]!=Array())
   {
    $this->e->add("Modules parse should set modules");
    return UT_FAILED;
   }


   $api->clear();
   $request='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <modules><module name="categories"/><module name="users"/></modules>
</jas>';

   if(!$api->parse($request))
   {
    $this->e->add("Modules request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if($api->commands[0]["modules"]!=Array("categories", "users"))
   {
    $this->e->add("Modules parse should set modules");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Parse_Projects_Request()
  {
   $api=new API_XML("server", $this->_config(true));

   $request='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <projects></projects>
</jas>';

   if(!$api->parse($request))
   {
    $this->e->add("Projects request failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Projects parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="projects")
   {
    $this->e->add("Projects parse should set projects command");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Format_Data_Response()
  {
   global $_session;

   $xml=new Format_XML();
   $api=new API_XML("server", $this->_config(true));

   $d=Data::cache("categories", "view", "all");
   $d->addField("base64", "file", "show", "", 0, 10000);

   $d->setValue("nr", 1);

   $d->setShowFields("nr", "name", "text", "base64Data");
   $d->addMutilation("name", "strip", 100);
   $d->sortOrder("nr-");

   if(!$api->addData($d, true))
   {
    $this->e->add("API add data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   $api->commands[0]["data"]->setValue("name", "JAS");
   $api->commands[0]["data"]->setValue("text", "multi
line & value");
   $api->commands[0]["data"]->setValue("base64Data", "fpgmp9)");

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$xml->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/data"))!=3)
   {
    $this->e->add("Response should contain 3 data sections");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data", "module")!="categories")
   {
    $this->e->add("Data response module doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data", "action")!="view")
   {
    $this->e->add("Data response action doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data", "extra")!="all")
   {
    $this->e->add("Data response extra doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data", "count")!="true")
   {
    $this->e->add("Data response count doesn't match");
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/data/row"))!=5)
   {
    $this->e->add("Response should contain 5 data row sections");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field[0]", "name")!="nr")
   {
    $this->e->add("Data response row name doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field[0]", "value")!="1")
   {
    $this->e->add("Data response row value doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field[1]", "name")!="name")
   {
    $this->e->add("Data response row name doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field[1]", "value")!="JAS")
   {
    $this->e->add("Data response row value doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field[2]", "name")!="text")
   {
    $this->e->add("Data response row name doesn't match");
    return UT_FAILED;
   }

   if($xml->getInner("jas/data/row/field[2]")!="multi
line &amp; value")
   {
    $this->e->add("Data response row value doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field[3]", "name")!="base64Data")
   {
    $this->e->add("Data response row name doesn't match");
    return UT_FAILED;
   }

   if($xml->getInner("jas/data/row/field[3]")!="ZnBnbcZwOSk=")
   {
    $this->e->add("Data response row value doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field[4]", "name")!="show_count")
   {
    $this->e->add("Data response row name doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field[4]", "value")!="1")
   {
    $this->e->add("Data response row value doesn't match");
    return UT_FAILED;
   }

   if($xml->getChildren("jas/data/show"))
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Response should not contain a data show sections");
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/data/sort"))!=1)
   {
    $this->e->add("Response should contain 1 data sort sections");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/sort/field", "name")!="nr-")
   {
    $this->e->add("Data response sort name doesn't match");
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/data/mutilations"))!=1)
   {
    $this->e->add("Response should contain 1 data mutilations sections");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/mutilations/mutilation", "name")!="name")
   {
    $this->e->add("Data response mutilations name doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/mutilations/mutilation", "type")!="strip")
   {
    $this->e->add("Data response mutilations type doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/mutilations/mutilation", "arg1")!="100")
   {
    $this->e->add("Data response mutilations arg1 doesn't match");
    return UT_FAILED;
   }


   $api->clear();
   $_session->set("sessionStorage.data.unit_testing_testmodule"); //ensure no values in DB

   $d=Data::cache("unit_testing_testmodule", "add", "item", Array("db" => "session"));
   $d->setValue("name", "test");
   $d->setValue("catnr", 2);
   $d->setShowFields(); //only get nr

   if(!$api->addData($d, false))
   {
    $this->e->add("API add data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$xml->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/data"))!=1)
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Response should contain 1 data sections");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data", "module")!="unit_testing_testmodule")
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Data response module doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data", "action")!="add")
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Data response action doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data", "extra")!="item")
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Data response extra doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data", "count"))
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Data response count shouldn't be set");
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/data/row"))!=1)
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Response should contain 1 data row sections");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field", "name")!="nr")
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Data response row name doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field", "value")!="1")
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Data response row value doesn't match");
    return UT_FAILED;
   }

   if($xml->getChildren("jas/data/show"))
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Response should not contain a data show sections");
    return UT_FAILED;
   }

   if($xml->getChildren("jas/data/sort"))
   {
    $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
    $this->e->add("Response should not contain a data sort sections");
    return UT_FAILED;
   }

   $_session->set("sessionStorage.data.unit_testing_testmodule"); //clean up DB
   return UT_OK;
  }

  function Format_Language_Response()
  {
   $xml=new Format_XML();
   $api=new API_XML("server", $this->_config(true));

   if(!$api->addLanguage("modules", "nl"))
   {
    $this->e->add("API add language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$xml->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/language"))<2)
   {
    $this->e->add("Response should contain at least 2 translations sections");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/language", "original")!="en")
   {
    $this->e->add("Language response original doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/language", "translated")!="nl")
   {
    $this->e->add("Language response translated doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/language", "set")!="modules")
   {
    $this->e->add("Language response set doesn't match");
    return UT_FAILED;
   }

   if(!$xml->getAttribute("jas/language/translation", "original"))
   {
    $this->e->add("Language response translation original not set");
    return UT_FAILED;
   }

   if(!$xml->getAttribute("jas/language/translation", "translated"))
   {
    $this->e->add("Language response translation translated not set");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Format_Modules_Response()
  {
   $xml=new Format_XML();
   $api=new API_XML("server", $this->_config(true));

   if(!$api->addModules(Array()))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$xml->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/modules"))<10)
   {
    $this->e->add("Response should contain at least 10 modules sections");
    return UT_FAILED;
   }

   foreach(Array("categories", "users", "files") as $module)
   {
    $found=false;
    $xml->reset();
    while($xml->search("module"))
    {
     if(isset($xml->v["attributes"]["name"])&&$xml->v["attributes"]["name"]==$module)
     {
      $found=true;
      break;
     }
    }

    if(!$found)
    {
     $this->e->add("Modules response doesn't contain module: ".$module);
     return UT_FAILED;
    }
   }

   return UT_OK;
  }


  function Format_Projects_Response()
  {
   $xml=new Format_XML();
   $api=new API_XML("server", $this->_config(true));

   if(!$api->addProjects(false))
   {
    $this->e->add("API add projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->commands())
   {
    $this->e->add("API commands failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$xml->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/projects"))<2)
   {
    $this->e->add("Response should contain at least 2 projects sections");
    return UT_FAILED;
   }

   $project="jas";
   $found=false;
   $xml->reset();
   while($xml->search("project"))
   {
    if(isset($xml->v["attributes"]["name"])&&$xml->v["attributes"]["name"]==$project)
    {
     $found=true;
     break;
    }
   }

   if(!$found)
   {
    $this->e->add("Projects response doesn't contain project: ".$project);
    return UT_FAILED;
   }

   return UT_OK;
  }
 }


 class UT_API_XML_Client extends UT_API_XML
 {
  function Constructor()
  {
   $object=new API_XML("client");

   if($object->mode!="client")
   {
    $this->e->add("API mode isn't client");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Error()
  {
   $api=new API_XML("client", $this->_config(true));

   $e=new Errors();
   if(!$e->add("Error", "error", "Text"))
   {
    $this->e->add("Unable to add error");
    return UT_FAILED;
   }

   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <errors>
  <error error="Error" level="error" extra="Text"/>
 </errors>
</jas>';

   if(!$api->parse($response))
   {
    $this->e->add("Data response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if($api->e->getErrors()!=1)
   {
    $this->e->add("Error parse should set 1 error");
    return UT_FAILED;
   }

   if(!$this->_compare($e, $api->e))
    return UT_FAILED;

   return UT_OK;
  }


  function Version()
  {
   $api=new API_XML("client", $this->_config(true));

   //valid XML version
   $api->e->clear();
   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <fail/>
</jas>';

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if($api->e->getLevel(0)!="warning")
   {
    $this->e->add("First error should contain: should be a warning");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "header version not set, using version: \"1.0\"")===false)
   {
    $this->e->add("First error should contain: header version not set, using version: \"1.0\"");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(1), "unsupported parse fail response")===false)
   {
    $this->e->add("Second error should contain: unsupported parse fail response");
    return UT_FAILED;
   }


   //invalid XML version
   $api->e->clear();
   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="0.1a">
 <fail/>
</jas>';

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if($api->e->getLevel(0)!="warning")
   {
    $this->e->add("First error should contain: should be a warning");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "header version not set, using version: \"1.0\"")===false)
   {
    $this->e->add("First error should contain: header version not set, using version: \"1.0\"");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(1), "XML version: \"0.1a\" doesn't match version: \"1.0\"")===false)
   {
    $this->e->add("Second error should contain: XML version: \"0.1a\" doesn't match version: \"1.0\"");
    return UT_FAILED;
   }


   //valid header+XML version
   $api->e->clear();
   if(!$api->setRequestResponseHeaders(Array(
    "X-JAS-API" => $api->type."-1.0",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <fail/>
</jas>';

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "unsupported parse fail response")===false)
   {
    $this->e->add("First error should contain: unsupported parse fail response");
    return UT_FAILED;
   }


   //mismatch header+XML version
   $api->e->clear();
   if(!$api->setRequestResponseHeaders(Array(
    "X-JAS-API" => $api->type."-1.0",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="0.1a">
 <errors/>
</jas>';

   if($api->getHeaderVersion()!="1.0")
   {
    $this->e->add("Requested API version isn't 1.0");
    return UT_FAILED;
   }

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "XML version: \"0.1a\" doesn't match version: \"1.0\"")===false)
   {
    $this->e->add("First error should contain: XML version: \"0.1a\" doesn't match version: \"1.0\"");
    return UT_FAILED;
   }


   //invalid header+XML version
   $api->e->clear();
   if(!$api->setRequestResponseHeaders(Array(
    "X-JAS-API" => $api->type."-0.1a",
   )))
   {
    $this->e->add("Unable to set request/response headers");
    return UT_FAILED;
   }

   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="0.1a">
 <errors/>
</jas>';

   if($api->parse($response))
   {
    $this->e->add("Error response should fail");
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(strpos($api->e->getError(0), "header version: \"0.1a\" doesn't match version: \"1.0\"")===false)
   {
    $this->e->add("First error should contain: header version: \"0.1a\" doesn't match version: \"1.0\"");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Format_Data_Request()
  {
   $xml=new Format_XML();
   $api=new API_XML("client", $this->_config(true));

   $d=Data::cache("categories", "view", "all");
   $d->addField("base64", "file", "show", "", 0, 10000);

   $d->setValue("nr", 1);
   $d->setValue("name", "JAS");
   $d->setValue("text", "multi
line & value");
   $d->setValue("base64Data", "fpgmp9)");

   $d->setShowFields("nr", "name");
   $d->addMutilation("name", "strip", 100);
   $d->sortOrder("nr-");

   if(!$api->addData($d, true))
   {
    $this->e->add("API add data failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$xml->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/data"))!=4)
   {
    $this->e->add("Request should contain 4 data sections");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data", "module")!="categories")
   {
    $this->e->add("Data request module doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data", "action")!="view")
   {
    $this->e->add("Data request action doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data", "extra")!="all")
   {
    $this->e->add("Data request extra doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data", "count")!="true")
   {
    $this->e->add("Data request count doesn't match");
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/data/row"))!=4)
   {
    $this->e->add("Request should contain 4 data row sections");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field[0]", "name")!="nr")
   {
    $this->e->add("Data request row name doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field[0]", "value")!="1")
   {
    $this->e->add("Data request row value doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field[1]", "name")!="name")
   {
    $this->e->add("Data request row name doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field[1]", "value")!="JAS")
   {
    $this->e->add("Data request row value doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field[2]", "name")!="text")
   {
    $this->e->add("Data request row name doesn't match");
    return UT_FAILED;
   }

   if($xml->getInner("jas/data/row/field[2]")!="multi
line &amp; value")
   {
    $this->e->add("Data request row value doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/row/field[3]", "name")!="base64Data")
   {
    $this->e->add("Data request row name doesn't match");
    return UT_FAILED;
   }

   if($xml->getInner("jas/data/row/field[3]")!="ZnBnbcZwOSk=")
   {
    $this->e->add("Data request row value doesn't match");
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/data/show"))!=2)
   {
    $this->e->add("Request should contain 2 data show sections");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/show/field[1]", "name")!="name")
   {
    $this->e->add("Data request show name doesn't match");
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/data/sort"))!=1)
   {
    $this->e->add("Request should contain 1 data sort sections");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/sort/field", "name")!="nr-")
   {
    $this->e->add("Data request sort name doesn't match");
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/data/mutilations"))!=1)
   {
    $this->e->add("Request should contain 1 data mutilations sections");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/mutilations/mutilation", "name")!="name")
   {
    $this->e->add("Data request mutilations name doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/mutilations/mutilation", "type")!="strip")
   {
    $this->e->add("Data request mutilations type doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/data/mutilations/mutilation", "arg1")!="100")
   {
    $this->e->add("Data request mutilations arg1 doesn't match");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Format_Language_Request()
  {
   $xml=new Format_XML();
   $api=new API_XML("client", $this->_config(true));

   if(!$api->addLanguage("modules", "nl"))
   {
    $this->e->add("API add language failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$xml->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/language"))!=0)
   {
    $this->e->add("Request should contain 0 modules sections");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/language", "language")!="nl")
   {
    $this->e->add("Language request translated doesn't match");
    return UT_FAILED;
   }

   if($xml->getAttribute("jas/language", "set")!="modules")
   {
    $this->e->add("Language request set doesn't match");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Format_Modules_Request()
  {
   $xml=new Format_XML();
   $api=new API_XML("client", $this->_config(true));

   if(!$api->addModules())
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$xml->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/modules"))!=0)
   {
    $this->e->add("Request should contain 0 modules sections");
    return UT_FAILED;
   }


   $api->clear();

   if(!$api->addModules(Array("categories", "users")))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$xml->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/modules"))!=2)
   {
    $this->e->add("Request should contain 2 modules sections");
    return UT_FAILED;
   }

   foreach(Array("categories", "users") as $module)
   {
    $found=false;
    $xml->reset();
    while($xml->search("module"))
    {
     if(isset($xml->v["attributes"]["name"])&&$xml->v["attributes"]["name"]==$module)
     {
      $found=true;
      break;
     }
    }

    if(!$found)
    {
     $this->e->add("Modules request doesn't contain module: ".$module);
     return UT_FAILED;
    }
   }


   $api->clear();
   $d=Data::cache("categories", "view", "all");

   if(!$api->addModules(Array(&$d)))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$xml->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/modules"))!=1)
   {
    $this->e->add("Request should contain 1 modules section");
    return UT_FAILED;
   }

   foreach(Array("categories") as $module)
   {
    $found=false;
    $xml->reset();
    while($xml->search("module"))
    {
     if(isset($xml->v["attributes"]["name"])&&$xml->v["attributes"]["name"]==$module)
     {
      $found=true;
      break;
     }
    }

    if(!$found)
    {
     $this->e->add("Modules request doesn't contain module: ".$module);
     return UT_FAILED;
    }
   }


   return UT_OK;
  }

  function Format_Projects_Request()
  {
   $xml=new Format_XML();
   $api=new API_XML("client", $this->_config(true));

   if(!$api->addProjects())
   {
    $this->e->add("API add projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$xml->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/projects"))!=0)
   {
    $this->e->add("Request should contain 0 projects sections");
    return UT_FAILED;
   }


   $api->clear();

   if(!$api->addProjects("jas"))
   {
    $this->e->add("API add projects failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$api->format())
   {
    $this->e->add("API format failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if(!$xml->parse($api->output(false)))
   {
    $this->e->add("Unable to parse API output");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(sizeof($xml->getChildren("jas/projects"))!=1)
   {
    $this->e->add("Request should contain 1 projects sections");
    return UT_FAILED;
   }

   $project="jas";
   $found=false;
   $xml->reset();
   while($xml->search("project"))
   {
    if(isset($xml->v["attributes"]["name"])&&$xml->v["attributes"]["name"]==$project)
    {
     $found=true;
     break;
    }
   }

   if(!$found)
   {
    $this->e->add("Projects request doesn't contain project: ".$project);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse_Data_Response()
  {
   $api=new API_XML("client", $this->_config(true));

   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <data module="files" action="view" extra="all" format="api">
  <row>
   <field name="nr" value="1" />
   <field name="file">JAS &amp; API</field>
   <field name="fileData">ZnBnbcZwOSk=</field>
   <field name="updated" value="2013-11-18 20:49:27" />
  </row>

  <sort>
   <field name="nr+" />
  </sort>
  <mutilations>
   <mutilation name="file" type="strip" arg1="100" />
  </mutilations>
 </data>
</jas>';

   if(!$api->parse($response))
   {
    $this->e->add("Data response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Data parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="data")
   {
    $this->e->add("Data parse should set data command");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->module!="files")
   {
    $this->e->add("Response should set files module");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->action!="view")
   {
    $this->e->add("Response should set view action");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->extra!="all")
   {
    $this->e->add("Response should set view extra");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->dataformat!="api")
   {
    $this->e->add("Response should set api format");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->getValue("file")!="JAS & API")
   {
    $this->e->add("Response set value incorrect for field: file");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->getValue("fileData")!="fpgmp9)")
   {
    $this->e->add("Response set value incorrect for field: fileData");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->getValue("nr")!="1")
   {
    $this->e->add("Response set value incorrect for field: nr");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->getValue("updated")!="2013-11-18 20:49:27")
   {
    $this->e->add("Response set value incorrect for field: updated");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->sortOrder!=Array("nr+"))
   {
    $this->e->add("Response set sort order fields incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->field["file"]["mutilation"]!=Array("strip" => Array(100)))
   {
    $this->e->add("Response set field mutilations incorrect");
    return UT_FAILED;
   }

   if($api->commands[0]["data"]->check("use", "show")!=Array("r", "nr", "updated", "file", "fileData"))
   {
    $this->e->add("Response set show fields incorrect");
    return UT_FAILED;
   }


   return UT_OK;
  }


  function Parse_Language_Response()
  {
   $api=new API_XML("client", $this->_config(true));

   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <language original="en" translated="nl" set="modules" updated="2009-07-27 14:55:42">
  <translation original="Test" translated="Test translated" />
  <translation original="Test |translate|" translated="|1| Test" />
  <translation original="|notranslate| test" translated="Test |1|" />
 </language>
</jas>';

   if(!$api->parse($response))
   {
    $this->e->add("Language response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Language parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="language")
   {
    $this->e->add("Language parse should set language command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["language"]->translations["normal"]["en"]["nl"])<3)
   {
    $this->e->add("Response should at least contain 3 translations");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["language"]->translations["lowercase"]["en"]["nl"])<1)
   {
    $this->e->add("Response should at least contain 1 lowercase translations");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["language"]->translations["regexp"]["en"]["nl"])<2)
   {
    $this->e->add("Response should at least contain 2 regexps");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse_Modules_Response()
  {
   $api=new API_XML("client", $this->_config(true));

   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
<modules>
 <module name="categories"/>
 <module name="users"/>
</modules>
</jas>';

   if(!$api->parse($response))
   {
    $this->e->add("Modules response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["modules"])!=2)
   {
    $this->e->add("Response should contain 2 modules");
    return UT_FAILED;
   }


   $api->clear();

   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <modules>
  <module name="users" title="Users">
   <fields>
    <field name="nr" type="nr" use="sql;form" properties="owner;identifier;hidden;global;default" min="0" default="usernr" title="nr"/>
    <field name="username" type="title" use="form;sql;show;login" properties="username;title" min="4" max="15" title="Username"/>
    <field name="password" type="password" use="sql;form;login" properties="password" min="7" title="Password"/>
    <field name="verifypassword" type="password" use="form" properties="password" min="7" title="Verify password"/>
    <field name="backend" type="boolean" use="form;sql;show" properties=";hidden" title="Backend login" text="If this user is allowed to log in to the JAS management tool"/>
    <field name="catnr" type="nr" use="sql;form" properties="rightsReference;global;default" min="1" default="1" title="catnr"/>
    <field name="usersAdditionalGroups" type="nr" use="form;show" properties="" min="0" default="usernr" title="Additional user groups" text="Select additional rights groups for this user"/>
   </fields>

   <references>
    <reference name="catnr" module="categories" from="catnr" to="nr" type="*:1"/>
    <reference name="usersAdditionalGroups" module="categories" from="nr" to="nr" type="*:*"/>
   </references>
  </module>
 </modules>
</jas>';

   if(!$api->parse($response))
   {
    $this->e->add("Modules response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["modules"])!=1)
   {
    $this->e->add("Response should contain 1 modules");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["modules"][0])||get_class($api->commands[0]["modules"][0])!="Data")
   {
    $this->e->add("Modules parse no data object");
    return UT_FAILED;
   }


   //module in command list
   $api->clear();
   if(!$api->addModules(Array("users")))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <modules>
  <module name="users" title="Users">
   <fields>
    <field name="nr" type="nr" use="sql;form" properties="owner;identifier;hidden;global;default" min="0" default="usernr" title="nr"/>
    <field name="username" type="title" use="form;sql;show;login" properties="username;title" min="4" max="15" title="Username"/>
    <field name="password" type="password" use="sql;form;login" properties="password" min="7" title="Password"/>
    <field name="verifypassword" type="password" use="form" properties="password" min="7" title="Verify password"/>
    <field name="backend" type="boolean" use="form;sql;show" properties=";hidden" title="Backend login" text="If this user is allowed to log in to the JAS management tool"/>
    <field name="catnr" type="nr" use="sql;form" properties="rightsReference;global;default" min="1" default="1" title="catnr"/>
    <field name="usersAdditionalGroups" type="nr" use="form;show" properties="" min="0" default="usernr" title="Additional user groups" text="Select additional rights groups for this user"/>
   </fields>

   <references>
    <reference name="catnr" module="categories" from="catnr" to="nr" type="*:1"/>
    <reference name="usersAdditionalGroups" module="categories" from="nr" to="nr" type="*:*"/>
   </references>
  </module>
 </modules>
</jas>';

   if(!$api->parse($response))
   {
    $this->e->add("Modules response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["modules"])!=1)
   {
    $this->e->add("Response should contain 1 modules");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["modules"][0])||get_class($api->commands[0]["modules"][0])!="Data")
   {
    $this->e->add("Modules parse no data object");
    return UT_FAILED;
   }

   //module definition in command list
   $api->clear();
   if(!$api->addModules(Array(Data::cache("users", false, false, Array("skipCheckTables" => true, "skipPostInit" => true)))))
   {
    $this->e->add("API add modules failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <modules>
  <module name="users" title="Users">
   <fields>
    <field name="nr" type="nr" use="sql;form" properties="identifier;hidden;global;default;owner" min="1" title="#"/>
    <field name="username" type="title" use="form;sql;show;login" properties="username;title" min="4" max="15" title="Username"/>
    <field name="password" type="password" use="sql;form;login" properties="password" min="7" title="Password"/>
    <field name="verifypassword" type="password" use="form" properties="password" min="7" title="Verify password"/>
    <field name="backend" type="boolean" use="form;sql;show" properties="" title="Backend login" text="If this user is allowed to log in to the JAS backend"/>
    <field name="catnr" type="nr" use="sql;form" properties="rightsReference;global;default" min="1" default="1" title="Category"/>
    <field name="added" type="datetime" use="sql;show" properties="noUpdate;default" default="now" title="Added"/>
    <field name="updated" type="datetime" use="sql;show" properties="forceUpdate;default" default="now" title="Last update"/>
    <field name="usersAdditionalGroups" type="nr" use="form;show" properties="" min="0" default="" title="Additional user groups" text="Select additional rights groups for this user"/>
   </fields>

   <references>
    <reference name="catnr" module="categories" from="catnr" to="nr" type="*:1"/>
    <reference name="usersAdditionalGroups" module="categories" from="nr" to="nr" type="*:*"/>
   </references>
  </module>
 </modules>
</jas>';

   if(!$api->parse($response))
   {
    $this->e->add("Modules response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Modules parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="modules")
   {
    $this->e->add("Modules parse should set modules command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["modules"])!=1)
   {
    $this->e->add("Response should contain 1 modules");
    return UT_FAILED;
   }

   if(!is_object($api->commands[0]["modules"][0])||get_class($api->commands[0]["modules"][0])!="Data")
   {
    $this->e->add("Modules parse no data object");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse_Projects_Response()
  {
   $api=new API_XML("client", $this->_config(true));

   $response='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
<projects>
 <project name="jas" current="true"/>
 <project name="test"/>
</projects>
</jas>';

   if(!$api->parse($response))
   {
    $this->e->add("Projects response failed");
    $this->e->add($api->e);
    return UT_FAILED;
   }

   if($api->version!="1.0")
   {
    $this->e->add("API version isn't 1.0");
    return UT_FAILED;
   }

   if(sizeof($api->commands)!=1)
   {
    $this->e->add("Projects parse should set 1 command");
    return UT_FAILED;
   }

   if($api->commands[0]["type"]!="projects")
   {
    $this->e->add("Projects parse should set projects command");
    return UT_FAILED;
   }

   if(sizeof($api->commands[0]["projects"])!=2)
   {
    $this->e->add("Response should contain 2 projects");
    return UT_FAILED;
   }

   if($api->commands[0]["activate"]!="jas")
   {
    $this->e->add("Response should set active project");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

?>
