<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/config.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."config.php");

 class UT_Config extends UT_Classes
 {
  function Constructor()
  {
   $object=new Config();

   if(!is_object($object)||get_class($object)!="Config")
   {
    $this->e->add("Constructor doesn't return a Config object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Config();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Config object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Config object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

 class UT_SU_Function extends UT_Config
 {
  function _getUsers()
  {
   global $_config;

   $d=Data::cache($_config->get("usermodule"), "view", "item");

   if(!($idField=$d->getID())) //no id field
   {
    $this->e->add("Get users failed, no ID field found");
    return false;
   }

   if(!($userField=$d->check("properties", "username"))) //no username field
   {
    $this->e->add("Get users failed, no username field found");
    return false;
   }

   if(sizeof($userField)!=1) //multiple username fields, not supported
   {
    $this->e->add("Get users failed, multiple username fields not supported");
    return false;
   }

   $userField=$userField[0];
   $d->setShowFields($userField);

   $d->clearValues();
   $d->setValue($idField, $_config->get("anonnr"), 0);
   $d->setValue($idField, $_config->get("rootnr"), 1);

   if(!$d->get())
   {
    $this->e->add("Get users failed, unable to gather users");
    $this->e->add($d->e);
    return false;
   }

   $users=Array();
   while($d->next())
   {
    if($d->v[$idField]==$_config->get("rootnr"))
     $users["root"]=Array("id" => $d->v[$idField], "username" => $d->v[$userField]);
    else if($d->v[$idField]==$_config->get("anonnr"))
     $users["anon"]=Array("id" => $d->v[$idField], "username" => $d->v[$userField]);
   }

   if(!isset($users["anon"])||!isset($users["root"]))
   {
    $this->e->add("Get users failed, anon/root doesn't exist");
    return false;
   }

   return $users;
  }

  function SU()
  {
   global $_config;

   $usernr=$_config->get("usernr"); //store usernr
   $_config->set("usernr", $_config->get("anonnr")); //become anon


   //normal scenario
   if(!$_config->su(true)) //become root
   {
    $this->e->add("Su failed, unable to become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su()) //revert
   {
    $this->e->add("Su failed, unable to revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("anonnr"))
   {
    $this->e->add("Su failed, user didn't revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }


   //become root twice fail
   if(!$_config->su(true)) //become root
   {
    $this->e->add("Su failed, unable to become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->su(true)) //become root again
   {
    $this->e->add("Su should fail to become root twice");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user should have remained root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su()) //revert
   {
    $this->e->add("Su failed, unable to revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("anonnr"))
   {
    $this->e->add("Su failed, user didn't revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }


   //revert twice fail
   if($_config->su()) //revert
   {
    $this->e->add("Su should fail to revert to original user when already original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("anonnr"))
   {
    $this->e->add("Su failed, user should have remained original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }


   $_config->set("usernr", $usernr); //restore usernr
   $_config->set("suusernr"); //ensure no leftovers
   return UT_OK;
  }

  function SUUser()
  {
   global $_config;

   if(!($users=$this->_getUsers()))
    return UT_FAILED;

   $usernr=$_config->get("usernr"); //store usernr

   //normal scenario
   $_config->set("usernr", $_config->get("rootnr")); //become root

   if(!$_config->su($users["root"]["username"])) //become root
   {
    $this->e->add("Su failed, unable to become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su()) //revert
   {
    $this->e->add("Su failed, unable to revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su($users["anon"]["username"])) //become anon
   {
    $this->e->add("Su failed, unable to become anon");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("anonnr"))
   {
    $this->e->add("Su failed, user didn't become anon");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su()) //revert
   {
    $this->e->add("Su failed, unable to revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }


   //normal scenario using ID
   $_config->set("usernr", $_config->get("rootnr")); //become root

   if(!$_config->su($users["root"]["id"])) //become root
   {
    $this->e->add("Su failed, unable to become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su()) //revert
   {
    $this->e->add("Su failed, unable to revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su($users["anon"]["id"])) //become anon
   {
    $this->e->add("Su failed, unable to become anon");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("anonnr"))
   {
    $this->e->add("Su failed, user didn't become anon");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su()) //revert
   {
    $this->e->add("Su failed, unable to revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }


   //not root fail
   $_config->set("usernr", $_config->get("anonnr")); //become anon

   if($_config->su($users["root"]["username"])) //become root
   {
    $this->e->add("Su should fail, non root users aren't allowed to change to a specific user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("anonnr"))
   {
    $this->e->add("Su failed, user changed");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->su($users["anon"]["username"])) //become anon
   {
    $this->e->add("Su should fail, non root users aren't allowed to change to a specific user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("anonnr"))
   {
    $this->e->add("Su failed, user changed");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }


   //su twice fail
   if(!$_config->su(true)) //become root
   {
    $this->e->add("Su failed, unable to become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->su($users["root"]["username"])) //become root
   {
    $this->e->add("Su should fail, su already used");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user changed");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->su($users["anon"]["username"])) //become anon
   {
    $this->e->add("Su should fail, su already used");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user changed");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }


   $_config->set("usernr", $usernr); //restore usernr
   $_config->set("suusernr"); //ensure no leftovers
   return UT_OK;
  }
 }


?>
