<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/errors.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."errors.php");

 class UT_Errors extends UT_Classes
 {
  function Constructor()
  {
   $object=new Errors();

   if(!is_object($object)||get_class($object)!="Errors")
   {
    $this->e->add("Constructor doesn't return a Errors object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Errors();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Errors object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->errors=-1;
   if($orig->errors==$copy->errors)
   {
    $this->e->add("Errors object copy also modified original \"errors\" variable", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test");
   if($orig->l->sets==$copy->l->sets)
   {
    $this->e->add("Errors object copy also modified original \"Language\" object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Add()
  {
   $error=new Errors();

   if(!$error->add("Test error", "error", "Extra text"))
   {
    $this->e->add("Unable to added error", "error");
    return UT_FAILED;
   }
   if($error->errors!==1)
   {
    $this->e->add("Error counter incorrect", "error");
    return UT_FAILED;
   }
   if($error->error[0]!="Test error")
   {
    $this->e->add("Error incorrect", "error");
    return UT_FAILED;
   }
   if($error->extra[0]!=="Extra text")
   {
    $this->e->add("Extra error info incorrect", "error");
    return UT_FAILED;
   }
   if($error->level($error->errorlevel[0])!="error")
   {
    $this->e->add("Error level incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Clear()
  {
   $error=new Errors();

   if(!$error->add("Test error"))
   {
    $this->e->add("Unable to added error", "error");
    return UT_FAILED;
   }

   $error->clear();

   if($error->errors)
   {
    $this->e->add("Error counter not cleared", "error");
    return UT_FAILED;
   }
   if($error->error)
   {
    $this->e->add("Error array not cleared", "error");
    return UT_FAILED;
   }
   if($error->extra)
   {
    $this->e->add("Extra array not cleared", "error");
    return UT_FAILED;
   }
   if($error->errorlevel)
   {
    $this->e->add("Error level array not cleared", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetErrors()
  {
   $error=new Errors();

   if(!$error->add("Test error"))
   {
    $this->e->add("Unable to added error", "error");
    return UT_FAILED;
   }

   if($error->getErrors()!==1)
   {
    $this->e->add("Error counter incorrect", "error");
    return UT_FAILED;
   }

   $error->add("Test error");
   if($error->getErrors()!==2)
   {
    $this->e->add("Error counter incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetError()
  {
   $error=new Errors();

   if(!$error->add("Test error"))
   {
    $this->e->add("Unable to added error", "error");
    return UT_FAILED;
   }

   if($error->getError(0)!="Test error")
   {
    $this->e->add("Returned error incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetExtra()
  {
   $error=new Errors();

   if(!$error->add("Test error", "error", "Extra text"))
   {
    $this->e->add("Unable to added error", "error");
    return UT_FAILED;
   }

   if($error->getExtra(0)!="Extra text")
   {
    $this->e->add("Returned extra error info incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetLevel()
  {
   $error=new Errors();

   if(!$error->add("Test error", "error"))
   {
    $this->e->add("Unable to added error", "error");
    return UT_FAILED;
   }

   if($error->getLevel(0)!="error")
   {
    $this->e->add("Returned error level incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Level()
  {
   $error=new Errors();

   foreach($error->errorType as $key => $level)
   {
    if($error->level($level)!==$key)
    {
     $this->e->add("Error level ".$level." converted incorrectly", "error");
     return UT_FAILED;
    }
    if($error->level($key)!==$level)
    {
     $this->e->add("Error level ".$level." converted back incorrectly", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function Log()
  {
   global $_config;

   if(!$_config->userRoot())
   {
    $this->e->add("Log tested need to be executed as root", "note");
    return UT_SKIPPED;
   }

   $d=Data::cache("log", "view", "all");

   if(!$d->tableExists())
   {
    $this->e->add("Log table not present", "warning");
    return UT_SKIPPED;
   }

   $error=new Errors();
   $error->logging="data";

   if(!$error->logErrorTypes)
   {
    $this->e->add("No loggable error types", "error");
    return UT_FAILED;
   }

   foreach($error->logErrorTypes as $level)
   {
    if(!$error->add("Unit testing generated ".$level, $level, "Test"))
     return UT_FAILED;

    //get latest log entry
    $d->setAction("view");
    $d->setExtra("all");

    $d->clearValues();
    $d->sortOrder("added-");
    $d->setValue("show_total", 1);

    if(!$d->get()||!$d->next())
    {
     $this->e->add("Unable to retrieve logged error from database", "error");
     return UT_FAILED;
    }

    if($d->v["title"]!="Unit testing generated ".$level)
    {
     $this->e->add("Error title logged incorrectly", "error");
     return UT_FAILED;
    }
    if($d->v["level"]!=$level)
    {
     $this->e->add("Error level logged incorrectly", "error");
     return UT_FAILED;
    }
    if($d->v["text"]!="Test")
    {
     $this->e->add("Error extra info logged incorrectly", "error");
     return UT_FAILED;
    }

    $d->setAction("delete");
    $d->setExtra("item");

    if(!$d->submit())
     $this->e->add("Unable to deleted logged error from database", "warning");
   }

   return UT_OK;
  }

  function Any()
  {
   $error=new Errors();

   if(!$error->add("Test error", "warning"))
   {
    $this->e->add("Unable to added error", "error");
    return UT_FAILED;
   }
   if(!$error->any())
   {
    $this->e->add("No errors found", "error");
    return UT_FAILED;
   }
   if(!$error->any("warning"))
   {
    $this->e->add("No warning errors found", "error");
    return UT_FAILED;
   }
   if(!$error->any("show"))
   {
    $this->e->add("No showable errors found", "error");
    return UT_FAILED;
   }
   if($error->any("fatal"))
   {
    $this->e->add("Fatal errors found", "error");
    return UT_FAILED;
   }

   if(!$error->add("Test error", "system"))
   {
    $this->e->add("Unable to added error", "error");
    return UT_FAILED;
   }
   if(!$error->any())
   {
    $this->e->add("No errors found", "error");
    return UT_FAILED;
   }
   if(!$error->any("fatal"))
   {
    $this->e->add("No fatal errors found", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Show()
  {
   $error=new Errors();

   if(!$error->add("Test error", "warning"))
   {
    $this->e->add("Unable to added error", "error");
    return UT_FAILED;
   }
   if(!$error->show())
   {
    $this->e->add("No errors found", "error");
    return UT_FAILED;
   }
   if(!$error->show("warning"))
   {
    $this->e->add("No warning errors found", "error");
    return UT_FAILED;
   }
   if(!$error->show("show"))
   {
    $this->e->add("No showable errors found", "error");
    return UT_FAILED;
   }
   if($error->show("fatal"))
   {
    $this->e->add("Fatal errors found", "error");
    return UT_FAILED;
   }

   if(!$error->add("Test error", "system"))
   {
    $this->e->add("Unable to added error", "error");
    return UT_FAILED;
   }
   if(!$error->show())
   {
    $this->e->add("No errors found", "error");
    return UT_FAILED;
   }
   if(!$error->show("fatal"))
   {
    $this->e->add("No fatal errors found", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

 }

?>
