<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/protocol.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["plugins"]."unit_testing/ut/script.php");
 require_once($_path["protocol"]."protocol.php");

 class UT_Protocol extends UT_Classes
 {
  function Constructor()
  {
   $object=new Protocol();

   if(!is_object($object)||get_class($object)!="Protocol")
   {
    $this->e->add("Constructor doesn't return a Protocol object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->connection)||get_class($object->connection)!="Socket")
   {
    $this->e->add("Object: \"connection\" isn't an Socket object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->listen)||get_class($object->listen)!="Socket")
   {
    $this->e->add("Object: \"listen\" isn't an Socket object", "error");
    return UT_FAILED;
   }

   if($object->user)
   {
    $this->e->add("Object user shouldn't be set", "error");
    return UT_FAILED;
   }

   if($object->group)
   {
    $this->e->add("Object group shouldn't be set", "error");
    return UT_FAILED;
   }

   if(!$object->linebreak)
   {
    $this->e->add("Object linebreak not set", "error");
    return UT_FAILED;
   }

   if(!$object->fork)
   {
    $this->e->add("Object fork not enabled", "error");
    return UT_FAILED;
   }

   $blacklist=Array(
    "Protocol",
    "main",
    "type",
    "setUserGroup",
   );

   foreach($blacklist as $function)
   {
    if(!in_array($function, $object->blacklist))
    {
     $this->e->add("Function: ".$function." not in blacklist", "error");
     return UT_FAILED;
    }
   }

   if($object->type)
   {
    $this->e->add("Protocol type shouldn't be set", "error");
    return UT_FAILED;
   }

   if(!$object->get("split"))
   {
    $this->e->add("Object value: split not set", "error");
    return UT_FAILED;
   }

   if($object->get("connection.default.port")!=23)
   {
    $this->e->add("Object value: connection.default.port not set to: 21", "error");
    return UT_FAILED;
   }

   if($object->get("connection.default.keepalive")!==true)
   {
    $this->e->add("Object value: connection.default.keepalive not set to: true", "error");
    return UT_FAILED;
   }

   if($object->get("connection.default.timeout")!==false)
   {
    $this->e->add("Object value: connection.default.timeout not set to: false", "error");
    return UT_FAILED;
   }

   if($object->get("connection.default.reread")!==false)
   {
    $this->e->add("Object value: connection.default.reread not set to: false", "error");
    return UT_FAILED;
   }

   if($object->get("mode")!="client")
   {
    $this->e->add("Object value: mode not set to: client", "error");
    return UT_FAILED;
   }

   if(!isset($object->commands["connect"]))
   {
    $this->e->add("Object command: connect not set", "error");
    return UT_FAILED;
   }

   if(!isset($object->commands["disconnect"]))
   {
    $this->e->add("Object command: disconnect not set", "error");
    return UT_FAILED;
   }

   if(!isset($object->commands["main"]))
   {
    $this->e->add("Object command: main not set", "error");
    return UT_FAILED;
   }

   //server mode
   $object=new Protocol(true);

   if($object->get("mode")!="server")
   {
    $this->e->add("Object value: mode not set to: server", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Protocol();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Protocol object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->linebreak=-1;
   if($orig->linebreak==$copy->linebreak)
   {
    $this->e->add("Protocol object copy also modified original \"linebreak\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Protocol object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->connection->ipv4=false;
   if($orig->connection->ipv4==$copy->connection->ipv4)
   {
    $this->e->add("Protocol object copy also modified original \"connection\" object", "error");
    return UT_FAILED;
   }

   $copy->listen->ipv4=false;
   if($orig->listen->ipv4==$copy->listen->ipv4)
   {
    $this->e->add("Protocol object copy also modified original \"listen\" object", "error");
    return UT_FAILED;
   }

   //connected
   $orig=new Protocol();
   $orig->set("url", "localhost:80");
   $orig->execute("connect");
   $copy=clone $orig;

   if($this->_compare($orig, $copy))
   {
    $this->e->add("Protocol object copy shouldn't match original", "error");
    return UT_FAILED;
   }

   $copy->linebreak=-1;
   if($orig->linebreak==$copy->linebreak)
   {
    $this->e->add("Protocol object copy also modified original \"linebreak\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Protocol object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->connection->ipv4=false;
   if($orig->connection->ipv4==$copy->connection->ipv4)
   {
    $this->e->add("Protocol object copy also modified original \"Connection\" object", "error");
    return UT_FAILED;
   }

   $copy->listen->ipv4=false;
   if($orig->listen->ipv4==$copy->listen->ipv4)
   {
    $this->e->add("Protocol object copy also modified original \"Listen\" object", "error");
    return UT_FAILED;
   }

   if($copy->connection->socket!==false)
   {
    $this->e->add("Protocol object copy \"Connection::socket\" variable should be reset", "error");
    return UT_FAILED;
   }

   $copy->connection->socket="resource";
   if($orig->connection->socket==$copy->connection->socket)
   {
    $this->e->add("Protocol object copy also modified original \"Connection::socket\" variable", "error");
    return UT_FAILED;
   }

   //listen (server mode)
   $orig=new Protocol(true);
   $orig->set("url", "localhost:10119"); //JAS (10, 1, 19) port, hopefully no-one's listening
   $orig->set("connection.default.keepalive", false);
   $orig->setFork(false);
   $orig->listen->setTimeout(0.0001);
   $orig->execute("connect");
   $copy=clone $orig;

   if($this->_compare($orig, $copy))
   {
    $this->e->add("Protocol object copy shouldn't match original", "error");
    return UT_FAILED;
   }

   $copy->linebreak=-1;
   if($orig->linebreak==$copy->linebreak)
   {
    $this->e->add("Protocol object copy also modified original \"linebreak\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Protocol object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->listen->ipv4=false;
   if($orig->listen->ipv4==$copy->listen->ipv4)
   {
    $this->e->add("Protocol object copy also modified original \"Connection\" object", "error");
    return UT_FAILED;
   }

   $copy->listen->ipv4=false;
   if($orig->listen->ipv4==$copy->listen->ipv4)
   {
    $this->e->add("Protocol object copy also modified original \"Listen\" object", "error");
    return UT_FAILED;
   }

   if($copy->listen->socket!==false)
   {
    $this->e->add("Protocol object copy \"Connection::socket\" variable should be reset", "error");
    return UT_FAILED;
   }

   $copy->listen->socket="resource";
   if($orig->listen->socket==$copy->listen->socket)
   {
    $this->e->add("Protocol object copy also modified original \"Connection::socket\" variable", "error");
    return UT_FAILED;
   }

   $script=new UT_Script();
   return $script->Copy_Constructor(); //run parent Copy_Constructor tests
  }


  function Clear()
  {
   $p=new Protocol();
   $p->connection->socket=@socket_create(AF_UNIX, SOCK_STREAM, 0);

   $p->clear();

   if($p->buffer!=="")
   {
    $this->e->add("Clear left buffer set", "error");
    return UT_FAILED;
   }

   if($p->pipeline!=="")
   {
    $this->e->add("Clear left pipeline set", "error");
    return UT_FAILED;
   }

   if($p->connection->socket!==false)
   {
    $this->e->add("Clear left connection socket set", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function ConnectionType()
  {
   $p=new Protocol();

   if(!$p->connectionType("tcp"))
   {
    $this->e->add("Connection type shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->connectionType!="tcp")
   {
    $this->e->add("Connection type doesn't match", "error");
    return UT_FAILED;
   }

   if(!$p->connectionType("udp"))
   {
    $this->e->add("Connection type shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->connectionType!="udp")
   {
    $this->e->add("Connection type doesn't match", "error");
    return UT_FAILED;
   }

   if(!$p->connectionType("file"))
   {
    $this->e->add("Connection type shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->connectionType!="file")
   {
    $this->e->add("Connection type doesn't match", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Parse()
  {
   $p=new Protocol();

   if(!$p->parse("test", "server send
client received"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!isset($p->commands["test"]))
   {
    $this->e->add("Parse should have set script: test", "error");
    return UT_FAILED;
   }

   if(sizeof($p->commands["test"])!=2)
   {
    $this->e->add("Parse should have set 2 commands for script: test", "error");
    return UT_FAILED;
   }

   if($p->commands["test"][0]["input"])
   {
    $this->e->add("Parse shouldn't have set server input", "error");
    return UT_FAILED;
   }

   if($p->commands["test"][0]["output"]!="send")
   {
    $this->e->add("Parse should have set server output", "error");
    return UT_FAILED;
   }

   if($p->commands["test"][1]["input"]!="received")
   {
    $this->e->add("Parse should have set client input", "error");
    return UT_FAILED;
   }

   if($p->commands["test"][1]["output"])
   {
    $this->e->add("Parse shouldn't have set client output", "error");
    return UT_FAILED;
   }

   //server
   $p=new Protocol(true);

   if(!$p->parse("test", "server send
client received"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!isset($p->commands["test"]))
   {
    $this->e->add("Parse should have set script: test", "error");
    return UT_FAILED;
   }

   if(sizeof($p->commands["test"])!=2)
   {
    $this->e->add("Parse should have set 2 commands for script: test", "error");
    return UT_FAILED;
   }

   if($p->commands["test"][0]["input"]!="send")
   {
    $this->e->add("Parse should have set server input", "error");
    return UT_FAILED;
   }

   if($p->commands["test"][0]["output"])
   {
    $this->e->add("Parse shouldn't have set server output", "error");
    return UT_FAILED;
   }

   if($p->commands["test"][1]["input"])
   {
    $this->e->add("Parse shouldn't have set client input", "error");
    return UT_FAILED;
   }

   if($p->commands["test"][1]["output"]!="received")
   {
    $this->e->add("Parse should have set client output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Listen()
  {
   $s=new Protocol(true);
   $s->fork=false; //don't fork client connections
   $s->listen->setBlocking(false); //don't block
   $s->listen->setTimeout(0); //don't wait

   $c=new Socket();
   $c->setBlocking(false); //don't block
   $c->setTimeout(0); //don't wait

   if($s->listen("localhost:2121"))
   {
    $this->e->add("Listen should fail as there are no waiting connections", "error");
    return UT_FAILED;
   }

   if(!$s->listen->socket)
   {
    $this->e->add("Listen should have open a listen socket", "error");
    $this->e->add($s->listen->e);
    return UT_FAILED;
   }

   if(!$c->connect("tcp://localhost:2121"))
   {
    $this->e->add("Client connect shouldn't fail", "error");
    $this->e->add($c->e);
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$c->socket)
   {
    $this->e->add("Connection should have opened a client connection", "error");
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$s->listen())
   {
    $this->e->add("Listen shouldn't fail", "error");
    $this->e->add($s->e);
    $s->listen->close();
    $c->close();
    return UT_FAILED;
   }

   if(!$s->connection->socket)
   {
    $this->e->add("Listen should have opened a client connection", "error");
    $s->listen->close();
    $c->close();
    return UT_FAILED;
   }

   if(!$c->close()) //close from client first so client gets TIME_WAIT, this way the server can immediatly reuse the port (i.e. no "unable to bind to port")
   {
    $this->e->add("Client connection socket close shouldn't have failed", "error");
    $this->e->add($c->e);
    $s->connection->close();
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$s->connection->close())
   {
    $this->e->add("Listen client connection socket close shouldn't have failed", "error");
    $this->e->add($s->connection->e);
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$s->listen->close())
   {
    $this->e->add("Listen socket close shouldn't have failed", "error");
    $this->e->add($s->listen->e);
    return UT_FAILED;
   }

   //fail
   if($s->listen("1.1.1.1:0"))
   {
    $this->e->add("Listen should fail", "error");
    $s->listen->close();
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Connect()
  {
   $s=new Protocol(true);
   $s->fork=false; //don't fork client connections
   $s->listen->setBlocking(false); //don't block
   $s->listen->setTimeout(0); //don't wait

   $c=new Protocol();
   $c->connection->setBlocking(false); //don't block
   $c->connection->setTimeout(0); //don't wait

   if($s->listen("localhost:2121"))
   {
    $this->e->add("Listen should fail as there are no waiting connections", "error");
    return UT_FAILED;
   }

   if(!$s->listen->socket)
   {
    $this->e->add("Listen should have open a listen socket", "error");
    $this->e->add($s->listen->e);
    return UT_FAILED;
   }

   if(!$c->connect("localhost:2121"))
   {
    $this->e->add("Client connect shouldn't fail", "error");
    $this->e->add($c->e);
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$c->connection->socket)
   {
    $this->e->add("Connection should have opened a client connection", "error");
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$s->listen())
   {
    $this->e->add("Listen shouldn't fail", "error");
    $this->e->add($s->e);
    $c->connection->close();
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$s->connection->socket)
   {
    $this->e->add("Listen should have opened a client connection", "error");
    $c->connection->close();
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$c->connection->close()) //close from client first so client gets TIME_WAIT, this way the server can immediatly reuse the port (i.e. no "unable to bind to port")
   {
    $this->e->add("Client connection socket close shouldn't have failed", "error");
    $this->e->add($c->e);
    $s->connection->close();
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$s->connection->close())
   {
    $this->e->add("Listen client connection socket close shouldn't have failed", "error");
    $this->e->add($s->connection->e);
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$s->listen->close())
   {
    $this->e->add("Listen socket close shouldn't have failed", "error");
    $this->e->add($s->listen->e);
    return UT_FAILED;
   }

   //fail
   if($c->connect(".1.1.1:0"))
   {
    $this->e->add("Connect should fail", "error");
    $c->connection->close();
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Disconnect()
  {
   $s=new Protocol(true);
   $s->fork=false; //don't fork client connections
   $s->listen->setBlocking(false); //don't block
   $s->listen->setTimeout(0); //don't wait

   $c=new Protocol();
   $c->connection->setBlocking(false); //don't block
   $c->connection->setTimeout(0); //don't wait

   if($s->listen("localhost:2121"))
   {
    $this->e->add("Listen should fail as there are no waiting connections", "error");
    return UT_FAILED;
   }

   if(!$s->listen->socket)
   {
    $this->e->add("Listen should have open a listen socket", "error");
    $this->e->add($s->listen->e);
    return UT_FAILED;
   }

   if(!$c->connect("localhost:2121"))
   {
    $this->e->add("Client connect shouldn't fail", "error");
    $this->e->add($c->e);
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$c->connection->socket)
   {
    $this->e->add("Connection should have opened a client connection", "error");
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$s->listen())
   {
    $this->e->add("Listen shouldn't fail", "error");
    $this->e->add($s->e);
    $c->connection->close();
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$s->connection->socket)
   {
    $this->e->add("Listen should have opened a client connection", "error");
    $c->connection->close();
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$c->disconnect()) //close from client first so client gets TIME_WAIT, this way the server can immediatly reuse the port (i.e. no "unable to bind to port")
   {
    $this->e->add("Client disconnect shouldn't have failed", "error");
    $this->e->add($c->e);
    $s->connection->close();
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$s->disconnect())
   {
    $this->e->add("Listen client disconnect shouldn't have failed", "error");
    $this->e->add($s->e);
    $s->listen->close();
    return UT_FAILED;
   }

   if(!$s->listen->close())
   {
    $this->e->add("Listen socket close shouldn't have failed", "error");
    $this->e->add($s->listen->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Read()
  {
   $s=new Protocol(true);
   $s->fork=false; //don't fork client connections
   $s->listen->setBlocking(false); //don't block
   $s->listen->setTimeout(0); //don't wait

   $c=new Protocol();
   $c->connection->setBlocking(false); //don't block
   $c->connection->setTimeout(0); //don't wait

   if($s->listen("localhost:2121"))
   {
    $this->e->add("Listen should fail as there are no waiting connections", "error");
    return UT_FAILED;
   }

   if(!$c->connect("localhost:2121"))
   {
    $this->e->add("Client connect shouldn't fail", "error");
    $this->e->add($c->e);
    return UT_FAILED;
   }

   if(!$s->listen())
   {
    $this->e->add("Listen shouldn't fail", "error");
    $this->e->add($s->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if(!$s->connection->write("test"))
   {
    $this->e->add("Write shouldn't fail", "error");
    $this->e->add($s->connection->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if($c->read()!="test")
   {
    $this->e->add("Read doesn't match", "error");
    $this->e->add($c->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if(!$c->connection->write("test"))
   {
    $this->e->add("Write shouldn't fail", "error");
    $this->e->add($c->connection->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if($s->read()!="test")
   {
    $this->e->add("Read doesn't match", "error");
    $this->e->add($c->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if(!$c->disconnect()) //close from client first so client gets TIME_WAIT, this way the server can immediatly reuse the port (i.e. no "unable to bind to port")
   {
    $this->e->add("Client disconnect shouldn't have failed", "error");
    $this->e->add($c->e);
    $s->connection->close();
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Write()
  {
   $s=new Protocol(true);
   $s->fork=false; //don't fork client connections
   $s->listen->setBlocking(false); //don't block
   $s->listen->setTimeout(0); //don't wait

   $c=new Protocol();
   $c->connection->setBlocking(false); //don't block
   $c->connection->setTimeout(0); //don't wait

   if($s->listen("localhost:2121"))
   {
    $this->e->add("Listen should fail as there are no waiting connections", "error");
    return UT_FAILED;
   }

   if(!$c->connect("localhost:2121"))
   {
    $this->e->add("Client connect shouldn't fail", "error");
    $this->e->add($c->e);
    return UT_FAILED;
   }

   if(!$s->listen())
   {
    $this->e->add("Listen shouldn't fail", "error");
    $this->e->add($s->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if(!$s->write("test"))
   {
    $this->e->add("Write shouldn't fail", "error");
    $this->e->add($s->connection->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if($c->read()!="test")
   {
    $this->e->add("Read doesn't match", "error");
    $this->e->add($c->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if(!$c->write("test"))
   {
    $this->e->add("Write shouldn't fail", "error");
    $this->e->add($c->connection->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if($s->read()!="test")
   {
    $this->e->add("Read doesn't match", "error");
    $this->e->add($c->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if(!$c->disconnect()) //close from client first so client gets TIME_WAIT, this way the server can immediatly reuse the port (i.e. no "unable to bind to port")
   {
    $this->e->add("Client disconnect shouldn't have failed", "error");
    $this->e->add($c->e);
    $s->connection->close();
    return UT_FAILED;
   }

   return UT_OK;
  }

  function ServerClient()
  {
   $s=new Protocol(true);
   $s->fork=false; //don't fork client connections
   $s->listen->setBlocking(false); //don't block
   $s->listen->setTimeout(0); //don't wait

   $c=new Protocol();
   $c->connection->setBlocking(false); //don't block
   $c->connection->setTimeout(0); //don't wait

   if($s->listen("localhost:2121"))
   {
    $this->e->add("Listen should fail as there are no waiting connections", "error");
    return UT_FAILED;
   }

   if(!$c->connect("localhost:2121"))
   {
    $this->e->add("Client connect shouldn't fail", "error");
    $this->e->add($c->e);
    return UT_FAILED;
   }

   if(!$s->listen())
   {
    $this->e->add("Listen shouldn't fail", "error");
    $this->e->add($s->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if(!$s->server("test"))
   {
    $this->e->add("Write shouldn't fail", "error");
    $this->e->add($s->connection->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if($c->server()!="test")
   {
    $this->e->add("Read doesn't match", "error");
    $this->e->add($c->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if(!$c->client("test"))
   {
    $this->e->add("Write shouldn't fail", "error");
    $this->e->add($c->connection->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if($s->client()!="test")
   {
    $this->e->add("Read doesn't match", "error");
    $this->e->add($c->e);
    $c->connection->close();
    return UT_FAILED;
   }

   if(!$c->disconnect()) //close from client first so client gets TIME_WAIT, this way the server can immediatly reuse the port (i.e. no "unable to bind to port")
   {
    $this->e->add("Client disconnect shouldn't have failed", "error");
    $this->e->add($c->e);
    $s->connection->close();
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetUserGroup()
  {
   $p=new Protocol(true);

   if($p->user)
   {
    $this->e->add("Protocol user shouldn't be set", "error");
    return UT_FAILED;
   }

   if($p->group)
   {
    $this->e->add("Protocol group shouldn't be set", "error");
    return UT_FAILED;
   }

   //set
   if(!$p->setUserGroup("user", "group"))
   {
    $this->e->add("Set user/group shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->user!="user")
   {
    $this->e->add("Protocol user doesn't match", "error");
    return UT_FAILED;
   }

   if($p->group!="group")
   {
    $this->e->add("Protocol group doesn't match", "error");
    return UT_FAILED;
   }

   //unset
   if(!$p->setUserGroup())
   {
    $this->e->add("Set user/group shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->user)
   {
    $this->e->add("Protocol user shouldn't be set", "error");
    return UT_FAILED;
   }

   if($p->group)
   {
    $this->e->add("Protocol group shouldn't be set", "error");
    return UT_FAILED;
   }

   //fail
   $p=new Protocol();

   if($p->setUserGroup("user", "group"))
   {
    $this->e->add("Set user/group should have failed", "error");
    return UT_FAILED;
   }

   if($p->user)
   {
    $this->e->add("Protocol user shouldn't be set", "error");
    return UT_FAILED;
   }

   if($p->group)
   {
    $this->e->add("Protocol group shouldn't be set", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Main()
  {
   return UT_SKIPPED;
  }


 }

?>
