<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/protocol_smtp.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/protocol.php");
 require_once($_path["protocol"]."smtp.php");

 class UT_Protocol_SMTP extends UT_Protocol
 {
  function Constructor()
  {
   global $_config;

   $object=new Protocol_SMTP();

   $config=$_config->merge("Email");

   if(!is_object($object)||get_class($object)!="Protocol_SMTP")
   {
    $this->e->add("Constructor doesn't return a Protocol_SMTP object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->connection)||get_class($object->connection)!="Socket")
   {
    $this->e->add("Object: \"connection\" isn't an Socket object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->listen)||get_class($object->listen)!="Socket")
   {
    $this->e->add("Object: \"listen\" isn't an Socket object", "error");
    return UT_FAILED;
   }

   if(!$object->linebreak)
   {
    $this->e->add("Object linebreak not set", "error");
    return UT_FAILED;
   }

   if(!$object->fork)
   {
    $this->e->add("Object fork not enabled", "error");
    return UT_FAILED;
   }

   $blacklist=Array(
    "Protocol",
    "main",
    "type",
    "setUserGroup",

    "Protocol_SMTP",
    "send",
   );

   foreach($blacklist as $function)
   {
    if(!in_array($function, $object->blacklist))
    {
     $this->e->add("Function: ".$function." not in blacklist", "error");
     return UT_FAILED;
    }
   }

   if($object->type!="SMTP")
   {
    $this->e->add("Protocol type not set to: SMTP", "error");
    return UT_FAILED;
   }

   if(!$object->get("split"))
   {
    $this->e->add("Object value: split not set", "error");
    return UT_FAILED;
   }

   if($object->get("connection.default.port")!=25)
   {
    $this->e->add("Object value: connection.default.port not set to: 25", "error");
    return UT_FAILED;
   }

   if($object->get("connection.default.keepalive")!==false)
   {
    $this->e->add("Object value: connection.default.keepalive not set to: false", "error");
    return UT_FAILED;
   }

   if($object->get("connection.default.timeout")!==false)
   {
    $this->e->add("Object value: connection.default.timeout not set to: false", "error");
    return UT_FAILED;
   }

   if($object->get("connection.default.reread")!==true)
   {
    $this->e->add("Object value: connection.default.reread not set to: true", "error");
    return UT_FAILED;
   }

   if($object->get("mode")!="client")
   {
    $this->e->add("Object value: mode not set to: client", "error");
    return UT_FAILED;
   }

   if(!isset($object->commands["connect"]))
   {
    $this->e->add("Object command: connect not set", "error");
    return UT_FAILED;
   }

   if(!isset($object->commands["disconnect"]))
   {
    $this->e->add("Object command: disconnect not set", "error");
    return UT_FAILED;
   }

   if(!isset($object->commands["main"]))
   {
    $this->e->add("Object command: main not set", "error");
    return UT_FAILED;
   }

   if(!isset($object->commands["hello"]))
   {
    $this->e->add("Object command: main not set", "error");
    return UT_FAILED;
   }

   if(!isset($object->commands["bye"]))
   {
    $this->e->add("Object command: main not set", "error");
    return UT_FAILED;
   }

   if($object->get("url")!="smtp://".$config["server"].":".$config["port"]."/")
   {
    $this->e->add("Object set url: doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($object->get("domain")!=$config["defaultDomain"])
   {
    $this->e->add("Object set domain: doesn't match expected output", "error");
    return UT_FAILED;
   }


   //server mode
   $object=new Protocol_SMTP(true, Array("defaultDomain" => "localhost"));

   if($object->get("mode")!="server")
   {
    $this->e->add("Object value: mode not set to: server", "error");
    return UT_FAILED;
   }

   if($object->get("defaultdomain")!="localhost")
   {
    $this->e->add("Object set default domain: doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($object->get("ehlo.type")!="SMTP")
   {
    $this->e->add("Object set ehlo type: doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($object->get("ehlo.size")!=0)
   {
    $this->e->add("Object set ehlo size: doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($object->get("ehlo.pipelining")!="PIPELINING")
   {
    $this->e->add("Object set ehlo pipelining: doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($object->get("ehlo.8bit")!="-8BITMIME")
   {
    $this->e->add("Object set ehlo 8bit: doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($object->get("250.ok")!="OK")
   {
    $this->e->add("Object set 250 ok: doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($object->get("354.go")!="go ahead")
   {
    $this->e->add("Object set 354 ok: doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Protocol_SMTP();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Protocol_SMTP object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->config=-1;
   if($orig->config==$copy->config)
   {
    $this->e->add("Protocol_SMTP object copy also modified original \"config\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Protocol_SMTP object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   //server mode
   $orig=new Protocol_SMTP(true);
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Protocol_SMTP object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Protocol_SMTP object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->config=-1;
   if($orig->config==$copy->config)
   {
    $this->e->add("Protocol_SMTP object copy also modified original \"config\" variable", "error");
    return UT_FAILED;
   }

   return parent::Copy_Constructor(); //run parent Copy_Constructor tests
  }


  function ResetRead()
  {
   $p=new Protocol_SMTP();

   $p->buffer="asdasd";
   $p->connection->reread["counter"]=1235;

   $p->resetRead();

   if($p->buffer!=="")
   {
    $this->e->add("ResetRead left buffer set", "error");
    return UT_FAILED;
   }

   if($p->connection->reread["counter"]!==0)
   {
    $this->e->add("ResetRead didn't reset reread counter", "error");
    return UT_FAILED;
   }

   return UT_SKIPPED;
  }


  function Send()
  {
   return UT_SKIPPED;
  }

  function Store()
  {
   return UT_SKIPPED;
  }

 }

?>
