<?

 /*****************************************************************************
 * Filename: include/protocol/http.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["protocol"]."protocol.php");
 require_once($_path["include"]."jas.php");

 class Protocol_HTTP extends Protocol
 {
  function __construct($server=false)
  {
   parent::__construct($server);

   //update blacklist
   $this->blacklist[]="setVar";
   $this->blacklist[]="setHeader";
   $this->blacklist[]="setCookie";
   $this->blacklist[]="compressVars";
   $this->blacklist[]="request";
   $this->blacklist[]="_responsePrepareg";
   $this->blacklist[]="_responseParseMethodg";
   $this->blacklist[]="_responseParseLocationg";
   $this->blacklist[]="_responseFileRequestg";
   $this->blacklist[]="_responseParseVarsg";
   $this->blacklist[]="_responseSetVarg";
   $this->blacklist[]="_responseServerVarsg";
   $this->blacklist[]="_responsePHPExecg";
   $this->blacklist[]="_responseCleanTmpg";
   $this->blacklist[]="_responseStatusg";

   $this->set("split", false); //don't auto split lines, handled by custom read function)

   $this->set("connection.default.port", 80);
   $this->set("connection.default.timeout", 5);

   //initial values
   $this->set("http.version", 1.1);
   $this->set("request.headers", Array());
   $this->set("request.body", "");
   $this->set("response.headers", Array());
   $this->set("response.body", "");

   $this->methods=Array("GET", "HEAD", "POST");

   //server init
   if($server)
   {
    $this->e->l->read("httpd");

    //server settings
    $this->serverString="JAS HTTPD/".JAS::versionS()." PHP/".phpversion();
    $this->serverRoot=getcwd();

    //mime database
    $this->mime=new Mime();

    if(!($this->sessionName=ini_get("session.name")))
     $this->sessionName="PHPSESSID";

    //get tmp folder
    if((!($this->tmpDir=ini_get("upload_tmp_dir")))||!is_dir($this->tmpDir))
    {
     if(!is_dir("/tmp/"))
     {
      $this->tmpDir="~/";
      $this->e->add("Can't determine tmp directory, using home", "system");
     }
     else
      $this->tmpDir="/tmp";
    }

    if($this->tmpDir[strlen($this->tmpDir)-1]!="/")
     $this->tmpDir.="/";

    //get upload limit
    $this->upload_max_filesize=ini_get("upload_max_filesize");
    if(preg_match("/^(\d+)(G|M|K|B)/i", $this->upload_max_filesize, $tmp))
    {
     switch(strtoupper($tmp[2]))
     {
      case "G":
       $this->upload_max_filesize=$tmp[1]*pow(10, 9);
      break;
      case "M":
       $this->upload_max_filesize=$tmp[1]*pow(10, 6);
      break;
      case "K":
       $this->upload_max_filesize=$tmp[1]*pow(10, 3);
      break;
      default:
       $this->upload_max_filesize=$tmp[1];
      break;
     }
    }
    else
     $this->upload_max_filesize=(int)$this->upload_max_filesize;
   }
   else //client
   {
    //client settings
    $this->userAgent="JAS HTTP/".JAS::versionS();
    $this->sessionName="PHPSESSID";
   }


   if(!$this->parse("main", "
set readcounter 0
client '|request.method| |request.location| HTTP/|http.version|'
else
 exit false
client |request.headers|[a-zA-Z0-9\-]+\:.+|
client
set linebreak false ||| don't send additional linebreak
client |request.body|
set linebreak true

get mode == server
 response

set readcounter 0
server 'HTTP/|http.version| |response.status|\d+| |response.statustext|'
else
 exit false
server |response.headers|[a-zA-Z0-9\-]+\:.+|
server
get request.method == HEAD ||| don't send/receive body
else
 set linebreak false ||| don't send additional linebreak
 server |response.body|
 set linebreak true
get mode == server
 log note '|connection.shost||:|connection.sport| |request.method| |request.location| - |response.status| - |response.length| (|connection.time|)'
")) return;

   $this->e->clear(); //clear parse overwrite errors
  }

  function __clone()
  {
   parent::__clone();

   //clone objects
   if(isset($this->mime))
    $this->mime=clone $this->mime;
  }


  function clear()
  {
   parent::clear();

   $this->vars=Array();
  }


  function setVar($key, $value=false)
  {
   if($value===false) //delete
   {
    //find matching keys
    foreach($this->vars as $c => $var)
     if($var["key"]==$key)
      unset($this->vars[$c]);
   }
   else if(is_array($value))
   {
    foreach($value as $k => $v)
     if(!$this->setVar($key."[".$k."]", $v))
      return false;
   }
   else if(is_object($value))
   {
    if(!$this->setVar($key, serialize($value)))
     return false;
   }
   else
    $this->vars[]=Array("key" => $key, "value" => $value);

   return true;
  }

  function getVar($key)
  {
   $values=Array();

   foreach($this->vars as $c => $var)
    if($var["key"]==$key)
     $values[]=$var["value"];

   if(sizeof($values)==1) //single value
    return $values[0];
   else if(sizeof($values)>1) //multiple values
    return $values;

   return false;
  }

  function setHeader($key=false, $value=false, $type=false, $start=false)
  {
   if(!$key&&$value===false)
   {
    $this->e->add($this->type." Protocol no header key/value", "system");
    return false;
   }

   if(!$type)
   {
    if($this->get("mode")=="server")
     $type="response";
    else
     $type="request";
   }

   if(!isset($this->values[$type]["headers"]))
   {
    $this->e->add($this->type." Protocol unknown header type: \"".$type."\"", "system");
    return false;
   }

   $headers=&$this->values[$type]["headers"];

   if($key) //key/value pair (no value deletes entry)
   {
    //find existing header
    $regexp="/^(".preg_quote($key, "/").")\s*:\s*(.*)$/i";
    foreach($headers as $c => $header)
    {
     if(preg_match($regexp, $header, $m)) //header found
     {
      if($value===false) //no value
      {
       $headers=array_merge(array_slice($headers, 0, $c), array_slice($headers, $c+1)); //remove entry
       return true;
      }

      if($value==$m[2]) //value didn't change
       return true; //done

      $headers[$c]=mb_convert_case($key, MB_CASE_TITLE).": ".$value; //update value
      return true;
     }
    }
    // else // not found, add below

    $header=mb_convert_case($key, MB_CASE_TITLE).": ".$value; //set key: value
   }
   else //value only
    $header=$value;

   //add either at start or end
   if($start)
    array_unshift($headers, $header);
   else
    $headers[]=$header;

   return true;
  }

  function getHeader($key, $type=false)
  {
   if(!$type)
   {
    if($this->get("mode")=="server")
     $type="request";
    else
     $type="response";
   }

   if(!isset($this->values[$type]["headers"]))
   {
    $this->e->add($this->type." Protocol unknown header type: \"".$type."\"", "system");
    return false;
   }

   $headers=&$this->values[$type]["headers"];

   //find header key
   $regexp="/^(".preg_quote($key, "/").")\s*:\s*(.*)$/i";
   foreach($headers as $header)
    if(preg_match($regexp, $header, $m))
     return $m[2];

   //$this->e->add($this->type." Protocol ".$type." header: ".$key." not found", "warning");
   return false;
  }


  function setCookie($key, $value=false, $type=false)
  {
   if(!$type)
   {
    if($this->get("mode")=="server")
     $type="response";
    else
     $type="request";
   }

   //set corresponding cookie header
   if($type=="response")
    $header="Set-Cookie";
   else //request
    $header="Cookie";

   if($cookies=$this->getHeader($header, $type)) //cookie header set
   {
    //find cookie key
    $found=false;
    $newCookies="";
    foreach(explode("; ", $cookies) as $cookie)
    {
     list($k, $v)=explode("=", $cookie);
     if($k==$key) //cookie key already exists
     {
      if($value!==false) //value set
       $newCookies.=$key."=".$value."; "; //set key/value

      $found=true;
     }
     else //key doesn't match: use existing key/value
      $newCookies.=$k."=".$v."; ";
    }

    if(!$found&&$value!==false) //key not found and has value
     $newCookies.=$key."=".$value."; ";

    if($newCookies) //cookies values left
     $cookies=substr($newCookies, 0, -2); //remove end ;
    else //no cookie values left (value=false could have removed last cookie key)
     $cookies="";
   }
   else //no cookies header set
   {
    if($value!==false) //value set
     $cookies=$key."=".$value; //set key/value
   }

   return $this->setHeader($header, $cookies, $type); //set/overwrite/unset cookie header
  }

  function getCookie($key, $type=false)
  {
   if(!$type)
   {
    if($this->get("mode")=="server")
     $type="request";
    else
     $type="response";
   }

   //set corresponding cookie header
   if($type=="response")
    $header="Set-Cookie";
   else //request
    $header="Cookie";

   if($cookies=$this->getHeader($header, $type)) //cookie header set
   {
    //find cookie key
    foreach(explode("; ", $cookies) as $cookie)
    {
     list($k, $v)=explode("=", $cookie);
     if($k==$key) //cookie key exists
      return $v;
    }
   }

   //$this->e->add($this->type." Protocol ".$type." cookie: ".$key." not found", "warning");
   return false;
  }


  function setSession($value=false, $type=false)
  {
   if(!$this->setCookie($this->sessionName, $value, $type))
    return false;

   return true;
  }

  function getSession($type=false)
  {
   if($session=$this->getCookie($this->sessionName, $type))
    return $session;

   return false;
  }


  function request($url, $method=false, $compression=false)
  {
   //get vars & hostname
   if(!($url=$this->connection->URL($url)))
   {
    $this->e->add($this->connection->e);
    return false;
   }

   if(!$method) //default: GET
    $method=$this->methods[0];
   else
    $method=strtoupper($method);

   if(!in_array($method, $this->methods))
   {
    $this->e->add($this->type." Protocol client request method not supported", "error");
    return false;
   }

   //connected to same host?
   if($this->connection->socket)
   {
    if($url["host"]!=$this->get("connection.host")|| //hostname changed
     ($url["port"]&&$url["port"]!=$this->get("connection.port"))|| //port changed
     (!$url["port"]&&$this->get("connection.default.port")!=$this->get("connection.port")) //not connect to default port
    )
     if(!$this->execute("disconnect"))
      return false;
   }

   //add / overwrite variables
   if($url["values"])
   {
    foreach($url["values"] as $var)
     if(!$this->setVar($var["key"], $var["value"]))
      return false;
   }

   //create request variables string
   $vars="";
   foreach($this->vars as $var)
   {
    if($var["value"])
     $vars.=$var["key"]."=".urlencode($var["value"])."&";
    else
     $vars.=$var["key"]."&";
   }

   if($vars)
    $vars=substr($vars, 0, -1); //remove end &

   //determin URL & body
   $location=$url["path"];

   if($method!="POST")
    $location.="?".$vars;
   else //POST
   {
    //compress vars body
    if($compression) //gzip vars
    {
     $this->e->add($this->type." Protocol client request compression may not be supported by the webserver", "debug", "For Apache please add the following configuration option: SetInputFilter DEFLATE");

     if(strlen($vars)>100) //don't bother compressing under 100 characters
     {
      $this->setHeader("Content-Encoding", "gzip");
      $vars=gzencode($vars);
     }
    }

    $this->set("request.body", $vars);
   }

   //setup basic client header
   $this->set("request.method", $method);
   $this->set("request.location", $location);

   $this->setHeader("User-Agent", $this->userAgent, false, true);

   if(preg_match("/[a-z]/i", $url["host"])) //hostname
    $this->setHeader("Host", $url["host"], false, true);

   $this->setHeader("Accept-Encoding", "gzip, deflate");

   if($method=="POST")
    $this->setHeader("Content-Type", "application/x-www-form-urlencoded");

   $this->setHeader("Content-Length", strlen($this->get("request.body")));

   //request keep-alive
   if($this->get("connection.default.keepalive"))
    $this->setHeader("Connection", "Keep-Alive");

   //fix URL var
   $this->set("url", $url["url"]);

   //connect and send request
   if(!$this->main())
    return false;

   //clear request
   $this->set("request.headers", Array());
   $this->set("request.body", "");

   //unarray http variables (caused by array read)
   if(is_array($this->get("http.version")))
   {
    $this->set("http.version", $this->get("http.version.0"));
    $this->set("response.status", $this->get("response.status.0"));
    $this->set("response.statustext", $this->get("response.statustext.0"));
   }

   //decompress body
   if($this->getHeader("Content-Encoding")=="gzip"&&$this->get("response.body"))
    $this->set("response.body", gzinflate(substr($this->get("response.body"), 10, -8))); //XXX: pre PHP 5.4 equivalent of gzdecode

   //check keep-alive response
   if($this->get("connection.keepalive")&&strtolower($this->getHeader("Connection"))!="keep-alive") //keep-alive requested but not allowed
    if(!$this->execute("disconnect")) //not done in main due to keepalive
     return false;

   //status checking
   switch($this->get("response.status"))
   {
    case "200": //OK
    {
     $this->e->add($this->type." Protocol server response: ".$this->get("response.statustext"). " (".$this->get("response.status").")", "note");
     break;
    }
    case "301": //page moved
    case "302":
    {
     $this->e->add($this->type." Protocol server response: ".$this->get("response.statustext"). " (".$this->get("response.status").")", "warning");
     $this->e->add($this->type." Protocol page moved to: ".$this->getHeader("location"), "note");
     return false;
    }
    default: //other errors
    {
     $this->e->add($this->type." Protocol server response: ".$this->get("response.statustext"). " (".$this->get("response.status").")", "error");
     return false;
    }
   }

   return true;
  }


  /*
   * Commands
   */

  function response()
  {
   //unarray http variables (caused by array read)
   if(is_array($this->get("http.version")))
   {
    $this->set("http.version", $this->get("http.version.0"));
    $this->set("request.method", $this->get("request.method.0"));
    $this->set("request.location", $this->get("request.location.0"));
   }

   //decompress body
   if($this->getHeader("Content-Encoding")=="gzip")
    $this->set("request.body", gzinflate(substr($this->get("request.body"), 10, -8))); //XXX: pre PHP 5.4 equivalent of gzdecode

   //check keep-alive request
   if($this->get("connection.default.keepalive")) //keep-alive allowed
   {
    if(strtolower($this->getHeader("Connection"))=="keep-alive")
     $this->set("connection.keepalive", true);
    else
     $this->set("connection.keepalive", false);
   }

   //clear response + status
   $this->set("response.headers", Array());
   $this->set("response.body", "");
   $this->set("response.length", 0);
   $this->_responseStatus();

   //setup basic server header
   $this->setHeader("Date", date("r"), false, true); //XXX: not GMT
   $this->setHeader("Server", $this->serverString, false, true);

   //parse request
   if(!$this->_responseParseMethod()||!$this->_responseParseLocation())
   {
    $this->_responsePrepare();
    return false;
   }

   //generate server body
   if(preg_match("/\.php$/", $this->get("request.filename"))) //PHP
   {
    if(!$this->_responseParseVars()) //_GET, _POST, _FILES, _COOKIE
    {
     $this->_responsePrepare();
     return false;
    }

    if(!$this->_responseServerVars()) //_SERVER
    {
     $this->_responsePrepare();
     return false;
    }

    if(!$this->_responsePHPExec())
    {
     $this->_responseCleanTmp();
     $this->_responseStatus(500);
     $this->_responsePrepare();
     return false;
    }

    $this->_responseCleanTmp();
   }
   else //normal files
   {
    if(!$this->_responseFileRequest())
    {
     $this->_responsePrepare();
     return false;
    }
   }

   //set status
   $this->_responseStatus(200);

   return $this->_responsePrepare();
  }

  function _responsePrepare()
  {
   //compress body
   if($this->getHeader("Accept-Encoding")=="gzip, deflate") //gzip body
   {
    if(strlen($this->get("response.body"))>100) //don't bother compressing under 100 characters
    {
     $this->setHeader("Content-Encoding", "gzip");
     $this->set("response.body", gzencode($this->get("response.body")));
    }
   }

   $this->set("response.length", strlen($this->get("response.body")));
   $this->setHeader("Content-Length", $this->get("response.length"));

   //set keep-alive response
   if($this->get("connection.keepalive"))
   {
    if($this->get("connection.timeout")&&$this->get("connection.timeout")>0.5)
     $this->setHeader("Keep-Alive", "timeout=".round($this->get("connection.timeout"), 1));

    $this->setHeader("Connection", "Keep-Alive");
   }

   return true;
  }

  function _responseParseMethod()
  {
   if(!in_array($this->get("request.method"), $this->methods))
   {
    $this->_responseStatus(405);
    return false;
   }

   return true;
  }

  function _responseParseLocation()
  {
   $location=explode("?", $this->get("request.location"));

   $filename=$location[0];
   $file=FS::realPathS($this->serverRoot.$filename);

   if(is_dir($file))
   {
    if($file[strlen($file)-1]!="/") //should end with /
    {
     $this->_responseStatus(301);
     $this->setHeader("Location", "http://".$this->getHeader("Host").$location[0]."/".(isset($location[1])?"?".$location[0]:"")); //redirect
     return false;
    }

    $filename.="index.php";
    $file.="index.php";
   }

   if(!file_exists($file))
   {
    $this->_responseStatus(404);
    return false;
   }

   if(!FS::inDirS($this->serverRoot, $file)) //tries to break out of web root?
   {
    $this->_responseStatus(403);
    return false;
   }

   if(sizeof($location)<2)
    $this->set("request.vars", "");
   else
    $this->set("request.vars", $location[1]);

   $this->set("request.filename", $filename);
   $this->set("request.directory", preg_replace("/[^\/]+$/", "", $file));
   $this->set("request.file", $file);
   return true;
  }

  function _responseFileRequest()
  {
   if(($body=@file_get_contents($this->get("request.file")))===false)
   {
    $this->_responseStatus(403);
    return false;
   }

   $this->setHeader("Last-Modified", date("r", filemtime($this->get("request.file"))));

   $contentType=$this->mime->filename($this->get("request.file"));

   if($contentType!="application/octet-stream")
    $this->setHeader("Content-Type", $contentType);

   $this->set("response.body", $body);
   return true;
  }

  /* PHP request */

  function _responseParseVars()
  {
   $this->_tmpfiles=Array();

   $this->_GET=Array();
   $this->_POST=Array();
   $this->_FILES=Array();
   $this->_COOKIE=Array();
   $this->_REQUEST=Array();

   if(preg_match_all("/([^?&=]+)(=([^?&=]+))?/", $this->get("request.vars"), $m))
   {
    $l=sizeof($m[1]);
    for($c=0;$c<$l;$c++)
     $this->_responseSetVar($this->_GET, $m[1][$c], $m[3][$c]);
   }

   if($this->get("request.method")=="POST")
   {
    if($this->getHeader("Content-Type")=="application/x-www-form-urlencoded")
    {
     if(preg_match_all("/([^?&=]+)(=([^?&=]+))?/", $this->get("request.body"), $m))
     {
      $l=sizeof($m[1]);
      for($c=0;$c<$l;$c++)
       $this->_responseSetVar($this->_POST, urldecode($m[1][$c]), urldecode($m[3][$c]));
     }
    }
    else if(preg_match("/^multipart\/form-data;\s*boundary=(.+)$/", $this->getHeader("Content-Type"), $m))
    {
     $boundary="--".$m[1];

     foreach(explode($boundary, $this->get("request.body")) as $var)
     {
      if(preg_match("/Content-Disposition: form-data; name=\"([^\\\"]+)\"(; filename=\"([^\\\"]+)\"[\n\r]+Content-Type: ([^\n\r]+))?/s", $var, $m, PREG_OFFSET_CAPTURE))
      {
       $key=$m[1][0];

       $linebreaksize=$m[0][1];
       $value=substr($var, (3*$linebreaksize)+strlen($m[0][0]), -($linebreaksize));

       $filename=(isset($m[3])?$m[3][0]:false);
       $type=(isset($m[4])?$m[4][0]:false);

       if(!$filename) // _POST
        $this->_responseSetVar($this->_POST, $key, $value);
       else // _FILES
       {
        $size=strlen($value);
        $tmpfile=$this->tmpDir.md5($this->get("connection.shost").$this->get("connection.sport").time());

        if($size>$this->upload_max_filesize)
        {
         $error=UPLOAD_ERR_INI_SIZE;
         $tmpfile="";
        }
        else if(isset($this->_POST["MAX_FILE_SIZE"])&&$size>$this->_POST["MAX_FILE_SIZE"])
        {
         $error=UPLOAD_ERR_FORM_SIZE;
         $tmpfile="";
        }
        else if(@file_put_contents($tmpfile, $value)===false)
        {
         $error=UPLOAD_ERR_CANT_WRITE;
         $tmpfile="";
        }
        else
         $error=UPLOAD_ERR_OK;

        $this->_tmpfiles[]=$tmpfile;

        if($pos=strpos($key, "[")) //array var[]
        {
         $pre=substr($key, 0, $pos);
         $post=substr($key, $pos);
        }
        else
        {
         $pre=$key;
         $post="";
        }

        $this->_responseSetVar($this->_FILES, $pre."[name]".$post, $filename);
        $this->_responseSetVar($this->_FILES, $pre."[type]".$post, $type);
        $this->_responseSetVar($this->_FILES, $pre."[tmp_name]".$post, $tmpfile);
        $this->_responseSetVar($this->_FILES, $pre."[size]".$post, $size);
        $this->_responseSetVar($this->_FILES, $pre."[error]".$post, $error);
       }
      }
     }
    }
   }

   if(preg_match_all("/\s?([^=;]+)=([^;]*);?/", $this->getHeader("Cookie"), $m))
   {
    $l=sizeof($m[0]);
    for($c=0;$c<$l;$c++)
     $this->_COOKIE[$m[1][$c]]=$m[2][$c];
   }

   if(!isset($this->_COOKIE[$this->sessionName]))
   {
    $this->_COOKIE[$this->sessionName]=md5($this->get("connection.shost").$this->get("connection.sport").time());
    $this->setSession($this->_COOKIE[$this->sessionName]);
   }

   $this->_REQUEST=array_merge($this->_COOKIE, $this->_GET, $this->_POST);
   return true;
  }

  function _responseSetVar(&$var, $key, $value)
  {
   if(preg_match_all("/([^\[]*)\[([^\]]*)\]/", $key, $m)) //array var[]
   {
    //convert var[] to (multi dimensional) array
    $key=$m[1][0];
    $var=&$var[$key];

    $l=sizeof($m[1]);
    for($c=0;$c<$l;$c++)
    {
     if(!isset($var))
      $var=Array();

     if($m[2][$c]!=="") //key set
      $entry=$m[2][$c]; //use key
     else //no key
      $entry=sizeof($var); //next entry

     if(!isset($var[$key])||!is_array($var[$key]))
      $var[$entry]=Array();

     $var=&$var[$entry]; //use entry
    }

    $var=$value;
   }
   else //normal var
    $var[$key]=$value;

   return true;
  }

  function _responseServerVars()
  {
   $this->_SERVER["HTTP_HOST"]=(isset($_SERVER["HOSTNAME"])?$_SERVER["HOSTNAME"]:"unknown");
   $this->_SERVER["SERVER_SIGNATURE"]="";
   $this->_SERVER["SERVER_SOFTWARE"]=$this->serverString;
   $this->_SERVER["SERVER_NAME"]="localhost";
   $this->_SERVER["SERVER_ADDR"]=$this->get("connection.host");
   $this->_SERVER["SERVER_PORT"]=$this->get("connection.port");
   $this->_SERVER["DOCUMENT_ROOT"]=$this->serverRoot;
   $this->_SERVER["PATH"]=$_SERVER["PATH"];

   $this->_SERVER["HTTP_HOST"]=$this->getHeader("Host");
   $this->_SERVER["HTTP_USER_AGENT"]=$this->getHeader("User-Agent");
   $this->_SERVER["HTTP_ACCEPT"]=$this->getHeader("Accept");
   $this->_SERVER["HTTP_ACCEPT_LANGUAGE"]=$this->getHeader("Accept-Language");
   $this->_SERVER["HTTP_ACCEPT_CHARSET"]=$this->getHeader("Accept-Charset");
   $this->_SERVER["HTTP_ACCEPT_ENCODING"]=$this->getHeader("Accept-Encoding");
   $this->_SERVER["HTTP_CACHE_CONTROL"]=$this->getHeader("Cache-Control");
   $this->_SERVER["HTTP_CONNECTION"]=$this->getHeader("Connection");
   $this->_SERVER["HTTP_TE"]=$this->getHeader("TE");
   $this->_SERVER["HTTP_REFERER"]=$this->getHeader("Referer");
   $this->_SERVER["HTTP_COOKIE"]=$this->getHeader("Cookie");
   $this->_SERVER["HTTP_COOKIE2"]=$this->getHeader("Cookie2");

   $this->_SERVER["CONTENT_LENGTH"]=$this->getHeader("Content-Length");
   $this->_SERVER["CONTENT_TYPE"]=$this->getHeader("Content-Type");

   $this->_SERVER["REMOTE_ADDR"]=$this->get("connection.shost");
   $this->_SERVER["REMOTE_PORT"]=$this->get("connection.sport");

   $this->_SERVER["SERVER_PROTOCOL"]="HTTP/".$this->get("http.version");
   $this->_SERVER["REQUEST_METHOD"]=$this->get("request.method");
   $this->_SERVER["QUERY_STRING"]=$this->get("request.vars");
   $this->_SERVER["REQUEST_URI"]=$this->get("request.location");
   $this->_SERVER["SCRIPT_FILENAME"]=$this->get("request.file");
   $this->_SERVER["SCRIPT_NAME"]=$this->get("request.filename");
   $this->_SERVER["PHP_SELF"]=$this->get("request.filename");
   $this->_SERVER["REQUEST_TIME"]=time();

   return true;
  }

  function _responsePHPExec()
  {
   if(!preg_match("/^(.*?)([^\/]+)$/", $this->get("request.file"), $m))
    return false;

   $dir=$m[1];
   $file=$m[2];

   $code="<?\n";
   $code.="ini_set(\"error_reporting\", E_ALL & ~E_STRICT);\n;";
   $code.="\$_GET=".preg_replace("/\n/", "", var_export($this->_GET, true)).";\n";
   $code.="\$_POST=".preg_replace("/\n/", "", var_export($this->_POST, true)).";\n";
   $code.="\$_FILES=".preg_replace("/\n/", "", var_export($this->_FILES, true)).";\n";
   $code.="\$_COOKIE=".preg_replace("/\n/", "", var_export($this->_COOKIE, true)).";\n";
   $code.="\$_REQUEST=".preg_replace("/\n/", "", var_export($this->_REQUEST, true)).";\n";
   $code.="\$_SERVER=".preg_replace("/\n/", "", var_export($this->_SERVER, true)).";\n";
   $code.="chdir('".$dir."');\n";
   $code.="include(\"".$file."\");\n";
   $code.="\n?>";

   $response="";
   $tmpfile=$this->tmpDir.md5(date("c").rand()).".php";

   file_put_contents($tmpfile, $code);
   $exec="php-cgi ".$tmpfile." 2>&1 ";

   exec($exec, $response, $ret);
   unlink($tmpfile);

   while($header=array_shift($response))
    if(preg_match("/^([^\:\s]+)\s*:\s*(.*)$/i", $header, $m))
     $this->setHeader($m[1], $m[2]);

   $this->set("response.body", implode("\n", $response));
   return true;
  }

  function _responseCleanTmp()
  {
   foreach($this->_tmpfiles as $tmpfile)
    unlink($tmpfile);
  }


  function _responseStatus($status=NULL)
  {
   switch($status)
   {
    case 200:
     $text="OK";
    break;
    case 204:
     $text="No Content";
    break;

    case 300:
     $text="Multiple Choices";
    break;
    case 301:
     $text="Moved Permanently";
    break;
    case 302:
     $text="Found";
    break;

    case 400:
     $text="Bad Request";
    break;
    case 401:
     $text="Unauthorized";
    break;
    case 402:
     $text="Payment Required";
    break;
    case 403:
     $text="Forbidden";
    break;
    case 404:
     $text="Not Found";
    break;
    case 405:
     $text="Method Not Allowed";
    break;

    case 500:
     $text="Internal Server Error";
    break;
    case 501:
     $text="Not Implemented";
    break;
    case 502:
     $text="Bad Gateway";
    break;
    case 503:
     $text="Service Unavailable";
    break;

    case NULL: //clear status
     $text=NULL;
    break;

    default:
     $text="Unknown";
    break;
   }

   $this->set("response.status", $status);
   $this->set("response.statustext", $text);
   return true;
  }


  function read()
  {
   if(!isset($this->pipeline))
    $this->pipeline="";

   if($this->get("readcounter")==0) //new request starts
    $this->buffer=$this->pipeline; //restore data from previous read run

   $this->set("readcounter", $this->get("readcounter")+1);
   switch($this->get("readcounter"))
   {
    case 1: //request
    {
     //grab request
     if(($pos=strpos($this->buffer, $this->linebreak))===false) //no request header end found in buffer
     {
      //read initial batch
      if(parent::read()===false)
       return false;

      if(($pos=strpos($this->buffer, $this->linebreak))===false) //no request header end found in buffer
      {
       $this->e->add($this->type." Protocol read request header failed", "error");
       return false;
      }
     }

     $request=substr($this->buffer, 0, $pos); //get request header

     if($pos+strlen($this->linebreak)<strlen($this->buffer)) //more than request header
      $this->buffer=substr($this->buffer, $pos+strlen($this->linebreak)); //remove request header from buffer
     else //only request header
      $this->buffer=""; //empty buffer

     return $request;
    }
    case 2: //header
    {
     //grab header
     if(($pos=strpos($this->buffer, $this->linebreak.$this->linebreak))===false) //no header end found in buffer
     {
      //read headers
      if(parent::read()===false)
       return false;

      //read header split
      if(($pos=strpos($this->buffer, $this->linebreak.$this->linebreak))===false) //no header end found in buffer
       if(parent::read()===false)
        return false;

      if(($pos=strpos($this->buffer, $this->linebreak.$this->linebreak))===false) //no header end found in buffer
      {
       $this->e->add($this->type." Protocol read header failed", "error");
       return false;
      }
     }

     $headers=explode($this->linebreak, substr($this->buffer, 0, $pos)); //get header array

     if($pos+strlen($this->linebreak)<strlen($this->buffer)) //more than header
      $this->buffer=substr($this->buffer, $pos+strlen($this->linebreak)); //remove header from buffer
     else //only header
      $this->buffer=""; //empty buffer

     return $headers;
    }
    case 3: //body/header split
    {
     if(strlen($this->linebreak)<strlen($this->buffer)) //more than header split
      $this->buffer=substr($this->buffer, strlen($this->linebreak)); //remove header split from buffer
     else //only header split
      $this->buffer=""; //empty buffer

     return "";
    }
    default: //body
    {
     if(($length=$this->getHeader("Content-Length"))!==false) //Content-Length
     {
      while(strlen($this->buffer)<$length) //keep reading till full body is received
       if(parent::read()===false) //or error occurs
        break;

      if($length<strlen($this->buffer)) //more than body
      {
       $body=substr($this->buffer, 0, $length); //get body
       $this->pipeline=substr($this->buffer, $length); //store rest of data for next read run
       $this->buffer=""; //empty buffer
      }
      else
      {
       $body=$this->buffer;
       $this->pipeline=""; //no data for next read
       $this->buffer=""; //empty buffer
      }
     }
     else if($this->getHeader("Transfer-Encoding")==="chunked") //chunky body
     {
      $body="";
      while(true) //read all chunks (till 0 chunk length is found)
      {
       if(!$this->buffer) //buffer empty, get chunk length
        if(parent::read()===false) //or error occurs
         break;

       if(!preg_match("/^([0-f]+)".$this->linebreak."/i", $this->buffer, $m)) //get chunk length
       {
        $this->e->add($this->type." Protocol read chunk failed", "error");
        return false;
       }

       $length=hexdec($m[1]); //convert hex body length to decimal
       $this->buffer=substr($this->buffer, strlen($m[1])+strlen($this->linebreak)); //remove chunk length from buffer

       if(!$length) //no further body content (for now)
        break; //done reading
       else //get chunk
       {
        while(strlen($this->buffer)<$length) //keep reading till full body is received
         if(parent::read()===false) //or error occurs
          break;

        if($length<strlen($this->buffer)) //more than body
        {
         $body.=substr($this->buffer, 0, $length); //append body chunk
         $this->buffer=substr($this->buffer, $length+strlen($this->linebreak));  //store rest of data for next chunk run
        }
        else
        {
         $body.=$this->buffer; //append body
         $this->buffer=""; //empty buffer
        }
       }
      }

      $this->pipeline=$this->buffer; //store rest of data for next read run
      $this->buffer=""; //empty buffer
     }
     else //no length
     {
      $body=false; //no buffer
      $this->pipeline=$this->buffer; //store rest of data for next read run
      $this->buffer=""; //ensure next read is blocking
     }

     return $body;
    }
   }

   return false;
  }
 }
?>
