<?

 /*****************************************************************************
 * Filename: include/protocol/smtp.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["protocol"]."protocol.php");
 require_once($_path["format"]."email.php");

 class Protocol_SMTP extends Protocol
 {
  function __construct($server=false, $config=false)
  {
   global $_config;

   parent::__construct($server);

   //update blacklist
   $this->blacklist[]="send";

   //get Email config
   $this->config=$_config->merge("Email", $config);

   $this->set("connection.default.port", 25);
   $this->set("connection.default.keepalive", false);
   $this->set("connection.default.timeout", false);
   $this->set("connection.default.reread", true);

   if($server)
   {
    $this->set("defaultdomain", $this->config["defaultDomain"]);

    $this->set("ehlo.type", "SMTP");
    $this->set("ehlo.size", 0);
    $this->set("ehlo.pipelining", "PIPELINING");
    $this->set("ehlo.8bit", "-8BITMIME");
    $this->set("250.ok", "OK");
    $this->set("354.go", "go ahead");
   }
   else //client
   {
    $this->set("url", "smtp://".$this->config["server"].":".$this->config["port"]."/");
    $this->set("domain", $this->config["defaultDomain"]);
   }

   if(!$this->parse("hello", "
set skipquitread false
server '220 |defaultdomain|[A-Za-z0-9_\-\.\s]+| |ehlo.type|E?SMTP|'
client 'EHLO |domain||'
 server '250-|defaultdomain|[a-z0-9_\-\.\s]+|'
 server '250-SIZE |ehlo.size|\d+|'
 server '250-|ehlo.pipelining|PIPELINING|'
 server '250|ehlo.8bit|\s?-?8BIT[A-Z0-9]+|'
 server '250 |250.ok||'
 else |||no OK received
  get mode == client
   resetRead |||ensure next read blocks
   server '250 |250.ok||'
else
 log error 'Hello failed: |output|'
 get mode == server
  server '500'
  set skipquitread true
 exit false

")) return;

   if(!$this->parse("bye", "
get skipquitread == true
else
 client QUIT
server '221 |defaultdomain|[a-z0-9_\-\.\s]+|'
")) return;

   if(!$this->parse("main", "
client 'MAIL FROM: <|from|([a-z0-9_\-\.]+)\@([a-z0-9_\-\.]+)\.([a-z0-9]+)|>'
 server '250 |250.ok|'
 else
  log error 'From |from| failed: |output|'
  exit false
else
 log error 'From failed: |output|'
 get mode == server
  server '500'
  set skipquitread true
 exit false

get mode == server
 set to |||set each in get mode
each to address
 client 'RCPT TO: <|address|([a-z0-9_\-\.]+)\@([a-z0-9_\-\.]+)\.([a-z0-9]+)|>'
  server '250 |250.ok|'
  else
   log error 'To |address| failed: |output|'
   exit false

client DATA
 server '354 |354.go|'
 else
  log error 'Data failed: |output|'
  exit false
else
 log error 'Data failed: |output|'
 get mode == server
  server '500'
  set skipquitread true
 exit false

set split false |||don't split lines
client |email|.*|
set split true |||split lines
get mode == server
 resetRead |||ensure next read blocks
client .
 server '250 |250.ok|'
 else
  log error 'Email failed: |output|'
  exit false
else
 log error 'Email failed: |output|'
 get mode == server
  server '500'
  set skipquitread true
 exit false

get mode == server
 store |from| |to| |email|
 log note 'Email received for |from|'
else
 log note 'Email send from |from|'
")) return;

   $this->e->clear(); //clear parse overwrite errors
  }

  function resetRead()
  {
   $this->buffer="";
   $this->connection->resetReRead();
  }

  function send($email)
  {
   if($this->get("mode")!="client")
   {
    $this->e->add("Sending email failed, not in client mode", "error");
    return false;
   }

   if(get_class($email)!="Format_Email")
   {
    $this->e->add("Sending email failed, incorrect email object", "error", "Should be Format_Email instead of ".get_class($email));
    return false;
   }


   //set from/to/cc/bcc
   $email->reset("From");
   if($email->next("From"))
    $this->set("from", $email->v["address"]);
   else
    $this->set("from");

   $to=Array();
   $email->reset("To");
   while($email->next("To"))
    $to[]=$email->v["address"];

   $email->reset("Cc");
   while($email->next("Cc"))
    $to[]=$email->v["address"];

   $email->reset("Bcc");
   while($email->next("Bcc"))
    $to[]=$email->v["address"];

   if($to)
    $this->set("to", $to);
   else
    $this->set("to");

   //set message
   $this->set("email", $email->output());

   //connect and send request
   if(!$this->main())
    return false;

   return true;
  }




  function store($from, $to, $email)
  {
   if($this->get("mode")=="server")
   {
    print "From: ".$from."\n";
    print "To: ".implode(", ", $to)."\n";
    print "------ Email ------\n".$email."\n";
    print "------ EOF ------\n";
   }

   return true;
  }
 }
?>
