<?

 /*****************************************************************************
 * Filename: include/storage/mysql.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."storage/storage.php");

 define("ALTER_FIELD_TYPE", 1);
 define("ALTER_FIELD_SIZE", 3);
 define("ALTER_FIELD_OPTIONS", 2);

/******************************************************************
 MySQL Class

 functions:
 + MySQL:              Contructor

******************************************************************/

 class System_Storage_mysql extends Storage
 {


/******************************************************************
 SQL
 Constructor, inits vars

 - module:   Part of website

******************************************************************/

  function __construct(&$d)
  {
   parent::__construct($d, "mysql");

   $this->restype=MYSQLI_ASSOC; //only get associative array indexes
   $this->dbcon=false;

   $this->collation="utf8_general_ci";
   $this->charset=preg_replace("/_.*$/", "", $this->collation);

   $this->sql=NULL;
   $this->clear();

   if(!($db=$this->getConfig()))
    return;

   if(!($this->dbcon=$this->_getConnection($db["hash"]))) //get existing connection
   {
    //init connection
    if(!$db["host"])
    {
     $this->e->add("Database configuration: \"".$this->data->db."\" incorrect: hostname not set", "system", false, ($this->data->module!="log")); //don't log
     return;
    }

    if(!$db["user"])
    {
     $this->e->add("Database configuration: \"".$this->data->db."\" incorrect: username not set", "system", false, ($this->data->module!="log")); //don't log
     return;
    }

    if(!$db["pass"])
    {
     $this->e->add("Database configuration: \"".$this->data->db."\" incorrect: password not set", "system", false, ($this->data->module!="log")); //don't log
     return;
    }

    if(!$db["name"])
    {
     $this->e->add("Database configuration: \"".$this->data->db."\" incorrect: database name not set", "system", false, ($this->data->module!="log")); //don't log
     return;
    }

    if(!$this->dbcon=@mysqli_connect($db["host"].($db["port"]?":".$db["port"]:""), $db["user"], $db["pass"]))
     $this->e->add("Unable to connect to database!", "system", "[".mysqli_connect_errno()."] ".mysqli_connect_error(), ($this->data->module!="log")); //don't log
    else if(!@mysqli_set_charset($this->dbcon, $this->charset))
     $this->e->add("Unable to set character set to utf8!", "system", "[".mysqli_errno($this->dbcon)."] ".mysqli_error($this->dbcon), ($this->data->module!="log")); //don't log
    else if(!@mysqli_select_db($this->dbcon, $db["name"]))
     $this->e->add("Unable to select database!", "system", "[".mysqli_errno($this->dbcon)."] ".mysqli_error($this->dbcon), ($this->data->module!="log")); //don't log

    $this->_setConnection($db["hash"], $this->dbcon); //store connection for later use
   }
  }


/******************************************************************
 clear
 Cleans up db variables

******************************************************************/
  function clear()
  {
   if($this->type=="fake")
    return parent::clear();

   if(!$this->e->any("fatal")&&preg_match("/^SELECT(.*)/i", $this->sql))
    mysqli_free_result($this->results);
   else
    $this->results=NULL;

   $this->resultsCounter=0;
   $this->res=NULL;

   $this->count=0;

   $this->txt=NULL;
   $this->sql=NULL;
  }


/******************************************************************
 exec
 Executes SQL query: $this->sql,
 either this function or this->sql() can be used to set $this->sql

 - sql: sets/overwrites $this->sql
 - txt: sets/overwrites $this->txt

 results can get from: $this->res

******************************************************************/
  function exec($sql, $txt=false)
  {
   if($this->e->any("fatal"))
    return false;

   if($this->type=="fake")
    return true;

   $this->sql=$sql;
   $this->txt=$txt;

   $this->results=NULL;

   if(!$this->dbcon)
   {
    $this->e->add("SQL error: Database connection not set up", "system", false, ($this->data->module!="log")); //don't log
    return false;
   }

   if(!$this->results=mysqli_query($this->dbcon, $this->sql))
   {
    $this->e->add("SQL error: ".$this->txt, "sql", $this->sql."\n\n".mysqli_error($this->dbcon), ($this->data->module!="log")); //don't log
    return false;
   }

   return true;
  }


/******************************************************************
 next
 Set next result row

******************************************************************/
  function next()
  {
   if($this->type=="fake")
    return parent::next();

   $this->res=NULL;

   if($this->e->any("fatal"))
    return false;

   if(!($this->res=mysqli_fetch_array($this->results, $this->restype)))
   {
    $this->clear(); //cleanup if no more results
    return false;
   }
   else
   {
    $this->resultsCounter++;
    return true;
   }
  }


  function insertID()
  {
   if($this->type=="fake")
    return 0;
   else
    return mysqli_insert_id($this->dbcon);
  }


  function mapOperator($operator, &$prefix, &$postfix)
  {
   $prefix="";
   $postfix="";
   switch($operator)
   {
    case "!=":
     return "<>";
    case "like":
     $prefix="%";
     $postfix="%";
     return " LIKE ";
    case "not like":
     $prefix="%";
     $postfix="%";
     return " NOT LIKE ";
    case "start like":
     $postfix="%";
     return " LIKE ";
    case "not start like":
     $postfix="%";
     return " NOT LIKE ";
    case "end like":
     $prefix="%";
     return " LIKE ";
    case "not end like":
     $prefix="%";
     return " NOT LIKE ";
   }

   return $operator;
  }


  function rightsField($action=false)
  {
   global $_config;

   $d=&$this->data; //D for short

   if($action==false)
    $action=$d->action;

   $sql="";

//Root user has full rights
   if($_config->userRoot())
    $sql.=$d->getRootRights()." ";
   else
   {
    if($refer=$d->getRightsReference())
     $rightsReference=$d->reference[$refer]["from"];
    else
     $rightsReference=false;

    if($action=="add"||$action=="copy")
    {
     if($rightsReference)
     {
      $sql.="IF((";
      $sql.="`categories`.`owner`=\"".$_config->get("usernr")."\" ";

      $sql.="), ".$d->getRootRights().", ";
     }
     else
      $sql.="(";
    }
    else if($fields=$d->check("properties", "owner"))
    {
//check that first and give the owner full rights
     $sql.="IF((";
     foreach($fields as $k)
      $sql.="`".$d->field[$k]["module"]."`.`".$k."`=\"".$_config->get("usernr")."\" OR ";

     $sql=substr($sql, 0, -3);

     $sql.="), ".$d->getRootRights().", ";
    }
    else
     $sql.="(";

//No category reference means no rights if
//Meaning no rights when not the owner
    if($rightsReference===false)
     $sql.="0";
    else
    {
     $sql.="IF(`userRights`.`rights` IS NULL, 0, `userRights`.`rights`) ";

     $l=sizeof($_config->get("groupnrs"));
     for($c=0;$c<$l;$c++)
      $sql.=" | IF(`groupRights".$c."`.`rights` IS NULL, 0, `groupRights".$c."`.`rights`)";
    }

    //rights4all
    if($action=="add"||$action=="copy")
    {
     foreach($d->check("properties", "allusers") as $k)
      $sql.="|".$d->field[$k]["defaultvalue"];
    }
    else
    {
     foreach($d->check("properties", "allusers") as $k)
     {
      if($d->check("use", "sql", $k))
       $sql.="|`".$d->field[$k]["module"]."`.`".$k."`";
      else
       $sql.="|".$d->field[$k]["defaultvalue"];
     }
    }

    $sql.=") ";
   }

   return $sql;
  }


  function from($action=false, $modules=false)
  {
   global $_config;

   $d=&$this->data; //D for short

   if($action==false)
    $action=$d->action;

   $joins="";
   if($modules) //gather joined modules and reference (latter only needed if in where clause)
   {
    $id=$d->getID();

    foreach($modules as $module) //extended modules first (in case references use extended fields)
     if(isset($d->extends[$module]))
      $joins.="LEFT JOIN `".$module."` ON `".$d->module."`.`".$id."`=`".$module."`.`".$id."` ";

    foreach($modules as $module)
    {
     $found=false;
     if(!isset($d->extends[$module])) //references
     {
      foreach(array_keys($d->reference) as $ref) //find referenced modules
      {
       if(($d->reference[$ref]["type"]=="0:*"||$d->reference[$ref]["type"]=="1:*")&&$d->reference[$ref]["module"]==$module) //0:* and 1:*
       {
        $joins.="LEFT JOIN `".$module."` ON `".$d->field[$d->reference[$ref]["from"]]["module"]."`.`".$d->reference[$ref]["from"]."`=`".$module."`.`".$d->reference[$ref]["to"]."` ";

        foreach(array_keys($d->getReferenceData($ref)->extends) as $extends)
         $joins.="LEFT JOIN `".$extends."` AS `".$extends."_extended` ON `".$module."`.`".$d->reference[$ref]["data"]->getID()."`=`".$extends."_extended`.`".$d->reference[$ref]["data"]->extends[$extends]->getID()."` ";

        $found=true;
        break;
       }
       else if(($d->reference[$ref]["type"]=="*:0"||$d->reference[$ref]["type"]=="*:1")&&$d->reference[$ref]["module"]==$module) //*:0 and *:1
       {
        $joins.="LEFT JOIN `".$module."` ON `".$d->field[$d->reference[$ref]["from"]]["module"]."`.`".$d->reference[$ref]["from"]."`=`".$module."`.`".$d->reference[$ref]["to"]."` ";

        foreach(array_keys($d->getReferenceData($ref)->extends) as $extends)
         $joins.="LEFT JOIN `".$extends."` AS `".$extends."_extended` ON `".$module."`.`".$d->reference[$ref]["data"]->getID()."`=`".$extends."_extended`.`".$d->reference[$ref]["data"]->extends[$extends]->getID()."` ";

        $found=true;
        break;
       }
       else if($d->reference[$ref]["type"]=="*:*"&&isset($d->reference[$ref]["data"])&&$d->reference[$ref]["data"]->module==$module) //*:*
       {
        //get from fields from m2m module
        $reffrom=$d->reference[$ref]["data"]->check("use", "from");
        $reffrom=$reffrom[0];
        $refto=$d->reference[$ref]["data"]->check("use", "to");
        $refto=$refto[0];

        //add m2m table
        $joins.="LEFT JOIN `".$module."` ON `".$d->field[$d->reference[$ref]["from"]]["module"]."`.`".$d->reference[$ref]["from"]."`=`".$module."`.`".$reffrom."` ";
        $joins.="LEFT JOIN `".$d->reference[$ref]["data"]->reference[$refto]["module"]."` ON `".$d->reference[$ref]["data"]->module."`.`".$refto."`=`".$d->reference[$ref]["module"]."`.`".$d->reference[$ref]["to"]."` ";

        foreach(array_keys($d->getReferenceData($ref)->getReferenceData($refto)->extends) as $extends)
         $joins.="LEFT JOIN `".$extends."` AS `".$extends."_extended` ON `".$d->reference[$ref]["data"]->reference[$refto]["data"]->module."`.`".$d->reference[$ref]["data"]->reference[$refto]["data"]->getID()."`=`".$extends."_extended`.`".$d->reference[$ref]["data"]->reference[$refto]["data"]->extends[$extends]->getID()."` ";

        $found=true;
        break;
       }
      }

      if(!$found)
       $joins.="LEFT JOIN `".$module."` ON `".$d->module."`.`".$id."`=`".$module."`.`".$id."` ";
     }
    }
   }

   if($_config->userRoot()||!($refer=$d->getRightsReference()))
   {
    $sql="FROM `".$d->module."` ";
    $sql.=$joins; //add joined modules
   }
   else
   {
    $rightsReference=$d->reference[$refer]["from"];

    if($action=="add")
     $sql="FROM `categories` ";
    else
    {
     $sql="FROM `".$d->module."` ";

     if($action=="copy"&&$d->module!="categories")
      $sql.="LEFT JOIN `categories` ON `categories`.`nr`=`".$d->field[$rightsReference]["module"]."`.`".$rightsReference."` ";
    }

    $sql.=$joins; //add joined modules

    if($d->module!="userRights")
    {
     $sql.="LEFT JOIN `userRights` ";

     if($d->module=="categories"||isset($d->extends["categories"])||$action=="add"||$action=="copy")
      $sql.="ON `userRights`.`catnr`=`categories`.`nr` ";
     else
      $sql.="ON `userRights`.`catnr`=`".$d->field[$rightsReference]["module"]."`.`".$rightsReference."` ";

     $sql.="AND `userRights`.`nr`=\"".$_config->get("usernr")."\" ";
    }

    $l=sizeof($_config->get("groupnrs"));
    for($c=0;$c<$l;$c++)
    {
     $sql.="LEFT JOIN `groupRights` AS `groupRights".$c."` ";

     if($d->module=="categories"||isset($d->extends["categories"])||$action=="add"||$action=="copy")
      $sql.="ON `groupRights".$c."`.`catnr`=`categories`.`nr` ";
     else
      $sql.="ON `groupRights".$c."`.`catnr`=`".$d->field[$rightsReference]["module"]."`.`".$rightsReference."` ";

     $sql.="AND `groupRights".$c."`.`nr`=\"".$_config->get("groupnrs.".$c)."\" ";
    }
   }

   return $sql;
  }


  function where($action=false)
  {
   global $_config;

   $d=&$this->data; //D for short

   if($action==false)
    $action=$d->action;

   if($_config->userRoot())
    $sql="WHERE ";
   else
   {
    if($d->module=="categories"||$d->module=="userRights"||$d->module=="groupRights")
     $rights=pow(2, $d->rights["category"][$action]);
    else
     $rights=pow(2, $d->rights["content"][$action]);

    $sql="WHERE ".$this->rightsField($action)." & ".$rights." AND ";
   }

   return $sql;
  }


  function order($skipReferences=false)
  {
   $d=&$this->data; //D for short

   $found=0;
   $sql="ORDER BY ";
   foreach($d->sortOrder as $k)
   {
    if(preg_match("/^(.*)(\-|\+)$/", $k, $field))
    {
     if(isset($d->reference[$field[1]])) //reference
     {
      if($d->reference[$field[1]]["type"]=="*:*") //*:* reference
      {
       if($skipReferences)
        continue;

       $dtmp=$d->getReferenceData($field[1]);
       $refto=$dtmp->check("use", "to");
       $refto=$refto[0];
       $dtmp=$dtmp->getReferenceData($refto); //to module

       if(!$dtmp->sortOrder) //no sorting in reference
        continue;

       foreach($dtmp->sortOrder as $k2)
       {
        if(preg_match("/^(.*)(\-|\+)$/", $k2, $field2))
        {
         if($dtmp->check("use", "sql", $field2[1]))
         {
          if($dtmp->field[$field2[1]]["module"]!=$dtmp->module) //extended
           $sql.="`".$dtmp->field[$field2[1]]["module"]."_extended`.`".$field2[1]."` ".($field2[2]=="-"?"DESC, ":"ASC, ");
          else
           $sql.="`".$dtmp->field[$field2[1]]["module"]."`.`".$field2[1]."` ".($field2[2]=="-"?"DESC, ":"ASC, ");

          $found++;
         }
        }
       }

       unset($dtmp);
      }
      else if($d->reference[$field[1]]["type"]=="0:*"||$d->reference[$field[1]]["type"]=="1:*") //0:* and 1:* references
      {
       if($skipReferences)
        continue;

       $dtmp=$d->getReferenceData($field[1]);

       if(!$dtmp->sortOrder) //no sorting in reference
        continue;

       foreach($dtmp->sortOrder as $k2)
       {
        if(preg_match("/^(.*)(\-|\+)$/", $k2, $field2))
        {
         if($dtmp->check("use", "sql", $field2[1]))
         {
          if($dtmp->field[$field2[1]]["module"]!=$dtmp->module) //extended
           $sql.="`".$dtmp->field[$field2[1]]["module"]."_extended`.`".$field2[1]."` ".($field2[2]=="-"?"DESC, ":"ASC, ");
          else
           $sql.="`".$dtmp->field[$field2[1]]["module"]."`.`".$field2[1]."` ".($field2[2]=="-"?"DESC, ":"ASC, ");

          $found++;
         }
        }
       }

       unset($dtmp);
      }
      else //*:0 and *:1 references
      {
       $dtmp=$d->getReferenceData($field[1]);

       if(!$dtmp->sortOrder||$skipReferences) //no sorting in reference or references skipped
       {
        //sort on field in "local" table: *:0 and *:1 allows reference and field to have the same name so if the reference doesn't have sort fields the local field is assumed
        if($d->check("use", "sql", $field[1]))
        {
         $sql.="`".$d->field[$field[1]]["module"]."`.`".$field[1]."` ";
         $sql.=($field[2]=="-"?"DESC, ":"ASC, ");
         $found++;
        }

        continue;
       }

       foreach($dtmp->sortOrder as $k2)
       {
        if(preg_match("/^(.*)(\-|\+)$/", $k2, $field2))
        {
         if($dtmp->check("use", "sql", $field2[1]))
         {
          if($dtmp->field[$field2[1]]["module"]!=$dtmp->module) //extended
           $sql.="`".$dtmp->field[$field2[1]]["module"]."_extended`.`".$field2[1]."` ".($field2[2]=="-"?"DESC, ":"ASC, ");
          else
           $sql.="`".$dtmp->field[$field2[1]]["module"]."`.`".$field2[1]."` ".($field2[2]=="-"?"DESC, ":"ASC, ");

          $found++;
         }
        }
       }

       unset($dtmp);
      }
     }
     else if($d->check("use", "sql", $field[1]))
     {
      $sql.="`".$d->field[$field[1]]["module"]."`.`".$field[1]."` ";
      $sql.=($field[2]=="-"?"DESC, ":"ASC, ");
      $found++;
     }
     else if($d->action=="search"&&$d->extra=="index"&&isset($d->fulltextIndex[$field[1]])) //full text index
     {
      if($d->check("use", "show", $field[1])) //already in show fields
       $sql.="`".$field[1]."` "; //use index name
      else //not shown
       $sql.="MATCH (`".$d->module."`.`".implode("`, `".$d->module."`.`", $d->fulltextIndex[$field[1]])."`) AGAINST (\"".addslashes($d->sqlValue("search"))."\") "; //use full match clause

      $sql.=($field[2]=="-"?"DESC, ":"ASC, ");
      $found++;
     }
    }
   }

   if($found)
    return substr($sql, 0, -2)." ";
   else
    return "";
  }


  function checkRights($action)
  {
   global $_config;

   if($_config->userRoot())
    return true;

   $d=&$this->data; //D for short

//Start query and set debug text
   $txt="Checking ".$action." rights: ".$d->title;

   switch($action)
   {
    case "add":
    case "copy":
    {
     $sql="SELECT ".$this->rightsField($action)."AS r ";

     if($refer=$d->getRightsReference())
     {
      $rightsReference=$d->reference[$refer]["from"];
      $sql.=$this->from($action);
      $sql.=$this->where($action)."(`categories`.`nr`=\"".addslashes($d->sqlValue($rightsReference))."\") ";
     }
     else //no rights reference
     {
      if($this->exec("SHOW TABLE STATUS LIKE \"".$d->module."\"", "Show table status")&&$this->next()&&!$this->res["Rows"]) //no rows in table
      {
       //check rights against rights fields, without using a query
       if($d->module=="categories"||$d->module=="userRights"||$d->module=="groupRights")
        $rights=pow(2, $d->rights["category"][$action]);
       else
        $rights=pow(2, $d->rights["content"][$action]);

       $userrights=0;
       foreach($d->check("properties", "allusers") as $k)
        $userrights|=$d->field[$k]["defaultvalue"];

       if(!($rights&$userrights))
       {
        $this->e->add("No ".$action." rights"."!", "hack");
        return false;
       }

       return true;
      }

      $sql.=$this->from($action);
      $sql.=$this->where($action)."(1) ";
     }

     break;
    }
    case "edit":
    case "delete":
    case "undelete":
    case "owner":
    case "rights":
    {
     $where="";
     $modules=Array();
     foreach($d->check("properties", "identifier") as $k)
     {
      if($d->check("use", "sql", $k))
      {
       if(!$d->check("valuetype", "none", $k))
       {
        $where.="`".$d->field[$k]["module"]."`.`".$k."`=\"".addslashes($d->sqlValue($k))."\" AND ";

        if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
         $modules[]=$d->field[$k]["module"];
       }
      }

      //filter out deleted items (or non deleted items for "undelete" action)
      foreach($d->check("properties", "deleted") as $k)
      {
       if($d->check("use", "sql", $k))
       {
        if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
         $modules[]=$d->field[$k]["module"];

        $where.="`".$d->field[$k]["module"]."`.`".$k."`=\"".($action=="undelete"?"true":"false")."\" AND ";
       }
      }
     }

     $id=$d->getID();

//select
     $sql="SELECT ".$this->rightsField($action)."AS r ";

//from
     $sql.=$this->from($action, $modules);

//where options
     $sql.=$this->where($action).$where;

     $sql=substr($sql, 0, -4);
     break;
    }
    default:
    {
     $this->e->add("No ".$action." rights"."!", "hack");
     return false;
    }
   }

   $sql.="LIMIT 0,1";

   if(!$this->exec($sql, $txt))
    return false;

   if(!$this->next())
   {
    if($action=="rights")
     $this->e->add("Not allowed to change rights"."!", "hack");
    else
     $this->e->add("No ".$action." rights"."!", "hack");

    return false;
   }

   return true;
  }


  function table()
  {
   $d=&$this->data; //D for short

   if(!$this->exec("SHOW TABLES LIKE \"".$d->module."\"", "Show tables"))
    return false;

   if($this->next())
    return true;

   return false;
  }


  function info()
  {
   global $_config;

   $d=&$this->data; //D for short

   if(!$this->exec("SHOW TABLE STATUS WHERE Name=\"".$d->module."\"", "Show table status"))
    return false;

   $results=Array();
   if($this->next())
   {
    $results["info_type"]=$this->res["Engine"]." ".$this->res["Version"];

    $results["info_rows"]=$this->res["Rows"];
    $results["info_insertID"]=$this->res["Auto_increment"];

    //look up SQL timezone offset
    $current=date_default_timezone_get();
    if($current!=$_config->get("sqltimezone"))
    {
     date_default_timezone_set($_config->get("sqltimezone"));
     $timezoneOffset=date("P");
     date_default_timezone_set($current);
    }
    else
     $timezoneOffset=date("P");

    //convert time zone to UTC
    if(!$this->exec("SELECT CONVERT_TZ('".$this->res["Create_time"]."', @@session.time_zone, '".$timezoneOffset."') as Create_time,  CONVERT_TZ('".$this->res["Update_time"]."', @@session.time_zone, '".$timezoneOffset."') as Update_time", "Convert create/update timezones"))
     return false;

    if($this->next())
    {
     $results["info_created"]=$this->res["Create_time"];
     $results["info_updated"]=$this->res["Update_time"];
    }

    $this->clear();
    $this->type="fake";
    $this->results=Array($results);

    return true;
   }

   return false;
  }


  function get($gather="data")
  {
   if($gather=="both")
   {
    if(!$this->get("count"))
     return false;

    if(!$this->get("data"))
     return false;

    return true;
   }

   global $_config;

   $d=&$this->data; //D for short

//Start query and set debug text
   $txt="Get ".$d->action.": ".$d->title;

   $id=$d->getID();

   if($d->check("valuetype", "set", "aggregation_field"))
   {
    $agg_field=$d->getValue("aggregation_field");
    $agg_type=($d->getValue("aggregation_type")=="average"?"AVG":strtoupper($d->getValue("aggregation_type")));
    $agg_method=$d->getValue("aggregation_method");

    if($agg_method=="before")
     $agg_method="CEIL";
    else if($agg_method=="after")
     $agg_method="FLOOR";
    else
     $agg_method="ROUND";

    $agg_interval=$d->getValue("aggregation_interval");
   }
   else
   {
    $agg_field=false;
    $agg_type=false;
    $agg_method=false;
    $agg_interval=false;
   }

//fields
   $select="SELECT ";
   $modules=Array();

   if($gather=="count"&&!$agg_field)
    $select.="COUNT(*) as show_count ";
   else
   {
    if($gather=="count"&&$agg_field) //count grouped result set
     $select.="COUNT(*) as show_count FROM (SELECT ";
    else if(!$agg_type&&$agg_field) //simple grouping, make sure result set is sorted first
     $select.="* FROM (SELECT ";

    if($id) //make sure rows are returned only once
     $select.="DISTINCT `".$d->module."`.`".$id."`, ";

    foreach($d->check("use", "sql") as $k)
    {
     if($k==$id) //skip id
      continue;
     else if($k=="r"&&$d->check("type", "rights", $k))
      $select.=$this->rightsField()."AS r, ";
     else if($d->check("use", "show", $k)||$d->check("type", "rights", $k)||(!$agg_type&&$agg_field==$k)) //also include aggregation field for simple grouping other wise group fails
     {
      if(isset($d->fulltextIndex[$k])) //full text index
      {
       if($d->action=="search"&&$d->extra=="index") //only for index search
        $select.="MATCH (`".$d->module."`.`".implode("`, `".$d->module."`.`", $d->fulltextIndex[$k])."`) AGAINST (\"".addslashes($d->sqlValue("search"))."\") AS `".$k."`, "; //add match clause as index name column
      }

      if(!$agg_field) //no aggregation
       $select.="`".$d->field[$k]["module"]."`.`".$k."`, ";
      else if($agg_field==$k) //aggregate by $agg_field field
      {
       if(!$agg_interval||$agg_interval==1) //no interval based aggregation
        $select.="`".$d->field[$k]["module"]."`.`".$k."`, ";
       else //interval based aggregation, round(value/interval)*interval
       {
        $type=$d->field[$agg_field]["type"];

        if($type=="date"||$type=="time"||$type=="datetime") //convert date/time before conversion
         $select.="FROM_UNIXTIME(".$agg_method."(UNIX_TIMESTAMP(`".$d->field[$agg_field]["module"]."`.`".$agg_field."`)/".$agg_interval.")*".$agg_interval.") AS `".$k."`, ";
        else //all other fields
         $select.="".$agg_method."(`".$d->field[$agg_field]["module"]."`.`".$agg_field."`/".$agg_interval.")*".$agg_interval." AS `".$k."`, ";
       }
      }
      else if($agg_type) //apply aggregation type to fields, otherwise simple grouping is applied
      {
       $type=$d->field[$k]["type"];

       if($agg_type=="COUNT") //use field to set count value
        $select.=$agg_type."(`".$d->field[$k]["module"]."`.`".$k."`) AS `".$k."`, ";
       else if(($type=="date"||$type=="time"||$type=="datetime")&&$agg_type!="SUM") //no sum for date/time
        $select.="FROM_UNIXTIME(".$agg_type."(UNIX_TIMESTAMP(`".$d->field[$k]["module"]."`.`".$k."`))) AS `".$k."`, ";
       else if($type=="float"||$type=="price")
        $select.=$agg_type."(`".$d->field[$k]["module"]."`.`".$k."`) AS `".$k."`, ";
       else if($type=="nr") //nr, only non references
       {
        //find reference link
        $found=false;
        foreach(array_keys($d->reference) as $ref)
         if($d->reference[$ref]["from"]==$k)
          $found=true;

        if(!$found) //normal nr
         //$select.="ROUND(".$agg_type."(`".$d->field[$k]["module"]."`.`".$k."`)) AS `".$k."`, ";
         $select.=$agg_type."(`".$d->field[$k]["module"]."`.`".$k."`) AS `".$k."`, "; //don't round average
        else //don't apply to references
         $select.="`".$d->field[$k]["module"]."`.`".$k."`, ";
       }
       else //normal, field aggregation not possible
        $select.="`".$d->field[$k]["module"]."`.`".$k."`, ";
      }
      else //no aggregation type (can still aggregate on field!)
       $select.="`".$d->field[$k]["module"]."`.`".$k."`, ";

      if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
       $modules[]=$d->field[$k]["module"];
     }
     else if($agg_field) //not a show/aggregation field but could be needed in order by (since the table is obscured by the group by when aggregating)
     {
      foreach($d->sortOrder as $sortField) //chec
      {
       if(preg_match("/^$k(\-|\+)$/", $sortField)) //field in sort field, add to select
       {
        $select.="`".$d->field[$k]["module"]."`.`".$k."`, ";

        if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
         $modules[]=$d->field[$k]["module"];

        break;
       }
      }
     }
    }

    $select=substr($select, 0, -2);
   }

//add full text index fields for index search
   if($d->action=="search"&&$d->extra=="index")
   {
    foreach(array_keys($d->fulltextIndex) as $index)
     if($d->check("use", "show", $index))
      $select.=", MATCH (`".$d->module."`.`".implode("`, `".$d->module."`.`", $d->fulltextIndex[$index])."`) AGAINST (\"".addslashes($d->sqlValue("search"))."\") AS `".$index."`"; //add match clause as index name column
   }

//where options
   $where=$this->where()."(";

   switch($d->extra)
   {
//Get by identifier (view, edit, etc)
    case "item":
    {
     $fields=Array();

     foreach($d->check("properties", "identifier") as $k)
     {
      if($d->check("use", "sql", $k))
      {
       if(!$d->check("valuetype", "none", $k))
       {
        $fields[]=$k;

        if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
         $modules[]=$d->field[$k]["module"];
       }
      }
     }


     if($d->vmax)
     {
      $l=sizeof($fields);
      for($d->vc=0;$d->vc<$d->vmax;$d->vc++)
      {
       $where.="(";
       for($c=0;$c<$l;$c++)
        $where.="`".$d->field[$fields[$c]]["module"]."`.`".$fields[$c]."`=\"".addslashes($d->sqlValue($fields[$c], $d->vc))."\" AND ";

       $where=preg_replace("/\s[A-Z]+\s$/", "", $where).") OR ";
      }

      $where=preg_replace("/\s[A-Z]+\s$/", "", $where);
     }
     else
      $where.="false";

     $d->vc=0;

     break;
    }

//List by category
    case "list":
    {
     $fields=Array();

     if(($refer=$d->getRightsReference())&&($k=$d->reference[$refer]["from"]))
     {
      if($d->check("use", "sql", $k))
      {
       if(!$d->check("valuetype", "none", $k))
       {
        $fields[]=$k;

        if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
         $modules[]=$d->field[$k]["module"];
       }
      }
     }


     if($d->vmax)
     {
      $l=sizeof($fields);
      for($d->vc=0;$d->vc<$d->vmax;$d->vc++)
      {
       $where.="(";
       for($c=0;$c<$l;$c++)
        $where.="`".$d->field[$fields[$c]]["module"]."`.`".$fields[$c]."`=\"".addslashes($d->sqlValue($fields[$c], $d->vc))."\" AND ";

       $where=preg_replace("/\s[A-Z]+\s$/", "", $where).") OR ";
      }

      $where=preg_replace("/\s[A-Z]+\s$/", "", $where);
     }
     else
      $where.="false";

     $d->vc=0;

     break;
    }

//All extra options (view->all, user->login, search: title, field, show, full, index, etc): value type = set
    default:
    {
     $fields=Array();

     foreach($d->check("valuetype", "set") as $k) //values should be set
     {
      if(isset($d->reference[$k])) //referenced field: get referenced module
      {
       $fields[]=$k;

       if($d->reference[$k]["type"]=="*:*")
       {
        if($d->reference[$k]["data"]->module!=$d->module&&!in_array($d->reference[$k]["data"]->module, $modules))
        {
         //add extended module if necessary
         if($d->field[$d->reference[$k]["from"]]["module"]!=$d->module&&!in_array($d->field[$d->reference[$k]["from"]]["module"], $modules))
          $modules[]=$d->field[$d->reference[$k]["from"]]["module"];

         $modules[]=$d->reference[$k]["data"]->module;
        }
       }
       else
       {
        if($d->reference[$k]["module"]!=$d->module&&!in_array($d->reference[$k]["module"], $modules))
         $modules[]=$d->reference[$k]["module"];
       }
      }
      else if($d->check("use", "sql", $k)) //normal field, get module
      {
       $fields[]=$k;

       if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
        $modules[]=$d->field[$k]["module"];
      }
     }

     if($d->action=="search"&&$d->extra=="index") //add full text index fields for index search
     {
      foreach(array_keys($d->fulltextIndex) as $index)
       if($d->check("valuetype", "set", $index))
        $fields[]=$index;
     }

     $l=sizeof($fields);

     $any=false;
     $open=false;
     for($d->vc=0;$d->vc<$d->vmax;$d->vc++)
     {
      for($c=0;$c<$l;$c++)
      {
       if(isset($d->field[$fields[$c]]["value"][$d->vc])&&isset($d->field[$fields[$c]]["operant"][$d->vc]))
       {
        if(!$open)
        {
         $where.="(";
         $open=true;
        }

        $prefix="";
        $postfix="";

        if($fields[$c]=="r")
         $where.=$this->rightsField($d->action)." ".$this->mapOperator($d->getOperator($fields[$c], $d->vc), $prefix, $postfix)." ".addslashes($d->sqlValue($fields[$c], $d->vc))." ".$d->getOperant($fields[$c], $d->vc)." ";
        else if((isset($d->reference[$fields[$c]])&&($d->reference[$fields[$c]]["type"]=="0:*"||$d->reference[$fields[$c]]["type"]=="1:*"||$d->reference[$fields[$c]]["type"]=="*:*"))) //0:*, 1:* and *:* reference
        {
         if($d->reference[$fields[$c]]["type"]=="*:*")
         {
          //get from/to fields from m2m module
          $refto=$d->reference[$fields[$c]]["data"]->check("use", "to");
          $refto=$refto[0];

          $where.="`".$d->reference[$fields[$c]]["data"]->module."`.`".$refto."`".$this->mapOperator($d->getOperator($fields[$c], $d->vc), $prefix, $postfix)."\"".$prefix.addslashes($d->sqlValue($fields[$c], $d->vc)).$postfix."\" ".$d->getOperant($fields[$c], $d->vc)." ";
         }
         else if($d->reference[$fields[$c]]["type"]=="0:*"||$d->reference[$fields[$c]]["type"]=="1:*")
          $where.="`".$d->reference[$fields[$c]]["module"]."`.`".$d->reference[$fields[$c]]["to"]."`".$this->mapOperator($d->getOperator($fields[$c], $d->vc), $prefix, $postfix)."\"".$prefix.addslashes($d->sqlValue($fields[$c], $d->vc)).$postfix."\" ".$d->getOperant($fields[$c], $d->vc)." ";
        }
        else if(isset($d->fulltextIndex[$fields[$c]])) //add match clause for all full text indexes
         $where.="MATCH (`".$d->module."`.`".implode("`, `".$d->module."`.`", $d->fulltextIndex[$fields[$c]])."`) AGAINST (\"".addslashes($d->sqlValue($fields[$c], $d->vc))."\") ".$d->getOperant($fields[$c], $d->vc)." ";
        else //normal fields (include *:0 and *:1 references)
         $where.="`".$d->field[$fields[$c]]["module"]."`.`".$fields[$c]."`".$this->mapOperator($d->getOperator($fields[$c], $d->vc), $prefix, $postfix)."\"".$prefix.addslashes($d->sqlValue($fields[$c], $d->vc)).$postfix."\" ".$d->getOperant($fields[$c], $d->vc)." ";

        if(!$any)
         $any=true;
       }
      }

      if($open)
      {
       $where=preg_replace("/(\s[A-Z]+\s)$/", ")\\1", $where);
       $open=false;
      }
     }

     $d->vc=0;

     if($any)
      $where=preg_replace("/\s[A-Z]+\s$/", "", $where);
     else
      $where.="true";

     break;
    }
   }

   $where.=") ";


   //filter out deleted items (or non deleted items for "undelete" actions)
   foreach($d->check("properties", "deleted") as $k)
   {
    if($d->check("use", "sql", $k))
    {
     if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
      $modules[]=$d->field[$k]["module"];

     $where.=" AND `".$d->field[$k]["module"]."`.`".$k."`=\"".($d->action=="undelete"||$d->extra=="deleted"?"true":"false")."\"";
    }
   }

   //get rights fields modules
   if(!$_config->userRoot())
   {
    if(($refer=$d->getRightsReference())&&($rightsReference=$d->reference[$refer]["from"]))
     if($d->field[$rightsReference]["module"]!=$d->module&&!in_array($d->field[$rightsReference]["module"], $modules))
      $modules[]=$d->field[$rightsReference]["module"];

    if($fields=$d->check("properties", "owner"))
    {
     foreach($fields as $k)
      if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
       $modules[]=$d->field[$k]["module"];
    }
   }

//get modules in order by clause
   if($gather=="data")
   {
    foreach($d->sortOrder as $k)
    {
     if(preg_match("/^(.*)(\-|\+)$/", $k, $field))
     {
      if(isset($d->reference[$field[1]])) //reference
      {
       if($d->reference[$field[1]]["type"]=="*:*") //*:* reference
       {
        $dtmp=$d->getReferenceData($field[1]);
        $refto=$dtmp->check("use", "to");
        $refto=$refto[0];
        $dtmp=$dtmp->getReferenceData($refto); //to module

        if($dtmp->sortOrder) //only added module if sort fields are defined
        {
         if($d->reference[$field[1]]["data"]->module!=$d->module&&!in_array($d->reference[$field[1]]["data"]->module, $modules))
          $modules[]=$d->reference[$field[1]]["data"]->module;
        }
       }
       else //0:*, 1:*, *:0 and *:1 references
       {
        $dtmp=$d->getReferenceData($field[1]);

        if($dtmp->sortOrder) //only added module if sort fields are defined
        {
         if($d->reference[$field[1]]["module"]!=$d->module&&!in_array($d->reference[$field[1]]["module"], $modules))
          $modules[]=$d->reference[$field[1]]["module"];
        }
        else if($d->reference[$field[1]]["type"]=="*:0"||$d->reference[$field[1]]["type"]=="*:1") //*:0 and *:1 references
        {
         //sort on field in "local" table: *:0 and *:1 allows reference and field to have the same name so if the reference doesn't have sort fields the local field is assumed
         if($d->check("use", "sql", $field[1]))
          if($d->field[$field[1]]["module"]!=$d->module&&!in_array($d->field[$field[1]]["module"], $modules))
           $modules[]=$d->field[$field[1]]["module"];
        }
       }
      }
      else if($d->check("use", "sql", $field[1])) //normal field
      {
       if($d->field[$field[1]]["module"]!=$d->module&&!in_array($d->field[$field[1]]["module"], $modules))
        $modules[]=$d->field[$field[1]]["module"];
      }

      unset($dtmp);
     }
    }
   }

//make proper query adding from with all modules used in select, where and order by clauses
   $sql=$select." ".$this->from(false, $modules).$where;

//aggregation using group by
   if($agg_field)
   {
    if($gather=="data"&&!$agg_type) //sort result set before simple grouping
     $sql.=$this->order().") as `".$d->module."` ";

    if(!$agg_interval||$agg_interval==1)
     $sql.="GROUP BY `".$d->field[$agg_field]["module"]."`.`".$agg_field."` ";
    else
    {
     $type=$d->field[$agg_field]["type"];

     if($type=="date"||$type=="time"||$type=="datetime")
      $sql.="GROUP BY FROM_UNIXTIME(".$agg_method."(UNIX_TIMESTAMP(`".$d->field[$agg_field]["module"]."`.`".$agg_field."`)/".$agg_interval.")*".$agg_interval.") ";
     else
      $sql.="GROUP BY ".$agg_method."(`".$d->field[$agg_field]["module"]."`.`".$agg_field."`/".$agg_interval.")*".$agg_interval." ";
    }

    if($gather=="count")
     $sql.=") as show_count ";
   }


   if($gather=="data")
   {
//sort
    $sql.=$this->order(($agg_field?true:false)); //skip reference sort on aggregation

//limit
    if($d->sqlValue("show_total")>0)
     $sql.="LIMIT ".$d->sqlValue("show_from").", ".$d->sqlValue("show_total");
   }

//d($sql);

//execute
   if(!$this->exec($sql, $txt))
    return false;

   if($gather=="count")
   {
    if(!$this->next())
     return false;

    $this->count=$this->res["show_count"];
    return true;
   }

   if($d->action=="search"&&$d->extra=="index") //make fake results with percentage relevance for full text indexes
   {
    $indexes=Array();
    foreach(array_keys($d->fulltextIndex) as $index)
     if($d->check("use", "show", $index))
      $indexes[]=$index;

    if(!($l=sizeof($indexes))) //no relevance score
     return true; //done

    $max=Array();
    for($c=0;$c<$l;$c++)
     $max[$indexes[$c]]=0;

    $results=Array();
    while($this->next())
    {
     //get highest relevance score for all full text indexes
     for($c=0;$c<$l;$c++)
      if($this->res[$indexes[$c]]>$max[$indexes[$c]])
       $max[$indexes[$c]]=$this->res[$indexes[$c]];

     //store fake results
     $results[]=$this->res;
    }

    $l2=sizeof($results);
    for($x=0;$x<$l2;$x++)
    {
     //set percentage relevance score
     for($c=0;$c<$l;$c++)
     {
      if($max[$indexes[$c]])
       $results[$x][$indexes[$c]]=($results[$x][$indexes[$c]]/$max[$indexes[$c]])*100;
      else
       $results[$x][$indexes[$c]]=0;
     }
    }

    $this->results=$results;
    $this->type="fake";
   }

   return true;
  }


  function treeUp($type="data")
  {
//use copy for gathering the data
   $d=clone $this->data;

//get reference+ID fields
   if(!($refer=$d->getSelfReference()))
   {
    $this->e->add("Unable to get tree: no self reference for module: \"".$d->module."\"", "module");
    return false;
   }

   $id=$d->getID();
   $refer=$d->reference[$refer]["from"];

   $d->setOperator($id, "="); //force operator

   $d->clearValues($id); //unset id field
   $d->setValue($id, $d->getValue($refer)); //set reference value to id field
   $d->clearValues($refer); //unset reference field

   //only return first row
   $d->setValue("show_from", 0);
   $d->setValue("show_total", 1);

   //show fields
   if($type=="count")
    $d->setShowFields($refer); //only get refer
   else if(!$d->check("use", "show", $refer)) //ensure refer field is shown
    $d->setShowFields(array_merge($d->check("use", "show"), Array($refer)));

//execute
   $results=Array();
   $db=new System_Storage_mysql($d);
   while(true)
   {
    if(!$db->get())
    {
     $this->e->add($db->e);
     break;
    }

    if(!$db->next()) //no results
     break; //done

    $results[]=$db->res; //add results

    if(!$db->res[$refer]) //no parent
     break; //done

    $d->setValue($id, $db->res[$refer]); //set parent for next run
   }

   //use fake storage
   $this->type="fake";
   $this->clear();
   $this->results=array_reverse($results); //reverse values

   if($type=="count"||$type=="both")
   {
    if(!$this->_setCount())
     return false;

    if($type=="count")
     return true;
   }

   if(!$this->_filterShowTotal())
    return false;

   return true;
  }

  function treeDown($type="data")
  {
//use copy for gathering the data
   $d=clone $this->data;

//get reference+ID fields
   if(!($refer=$d->getSelfReference()))
   {
    $this->e->add("Unable to get tree: no self reference for module: \"".$d->module."\"", "module");
    return false;
   }

   $id=$d->getID();
   $refer=$d->reference[$refer]["from"];

   $d->setOperator($id, "="); //force operator
   $d->setOperator($refer, "="); //force operator

   if(!$d->check("valuetype", "none", $refer)) //self reference based (i.e. only include sub entries in results)
   {
    $d->setValue($refer, $d->getValue($refer));
    $d->unsetValue($id);
   }
   else //id based (i.e. also include current (id) entry in results)
   {
    $d->unsetValue($refer);
    $d->setValue($id, $d->getValue($id));
   }

//show fields
   if($type=="count")
    $d->setShowFields($refer); //only get refer
   else if(!$d->check("use", "show", $refer)) //ensure refer field is shown
    $d->setShowFields(array_merge($d->check("use", "show"), Array($refer)));

//ensure output isn't limited
   if($show_total=$d->getValue("show_total"))
    $d->setValue("show_total", 0);

//execute
   $this->type="fake";
   $this->clear();

   $db=new System_Storage_mysql($d);
   if(($this->results=$db->_treeDownRecurse(0, $refer, $id))===false)
   {
    if($show_total) //restore show_total
     $d->setValue("show_total", $show_total);

    $this->e->add($db->e);
    return false;
   }

   if($show_total) //restore show_total
    $d->setValue("show_total", $show_total);

   if($type=="count"||$type=="both")
   {
    if(!$this->_setCount())
     return false;

    if($type=="count")
     return true;
   }

   if(!$this->_filterShowTotal())
    return false;

   return true;
  }

  function _treeDownRecurse($treeDepth, $refer, $id=false)
  {
   $d=&$this->data; //D for short

   $results=Array();

   if(!$this->get())
    return false;

   while($this->next())
   {
    $results[]=$this->res; //add results

//enforce tree depth limit
    if($d->field["treedepth"]["valuetype"]!="set"||$treeDepth<$d->getValue("treedepth"))
    {
     $d->setValue($refer, $this->res[$id]);
     $d->unsetValue($id);

     $db=new System_Storage_mysql($d);
     if(($results2=$db->_treeDownRecurse($treeDepth+1, $refer, $id))===false)
     {
      $this->e->add($db->e);
      return false;
     }

     $results=array_merge($results, $results2);
    }
   }

   return $results;
  }


  function lost()
  {
   global $_config;

   $d=&$this->data; //D for short

   if($refer=$d->getRightsReference())
    $rightsReference=$d->reference[$refer]["from"];
   else
    return false;


//Start query and set debug text
   $txt="Get ".$d->action.": ".$d->title;

//title fields
   $sql="SELECT ";
   $modules=Array();
   foreach($d->check("use", "sql") as $k)
   {
    if($d->check("properties", "title", $k)||$d->check("properties", "identifier", $k)||$k==$rightsReference)
    {
     $sql.="`".$d->field[$k]["module"]."`.`".$k."`, ";

     if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
      $modules[]=$d->field[$k]["module"];
    }
   }


//from
   $sql=substr($sql, 0, -2)." ".$this->from(false, $modules);

//where
   $c=0;
   $sql.="WHERE ";
   $sql.="`".$d->field[$rightsReference]["module"]."`.`".$rightsReference."` NOT IN (";
   $sql.="SELECT `categories`.`nr` FROM `categories` ";

   if($d->module!="categories"&&$d->module!="userRights")
    $sql.="WHERE `categories`.`module` LIKE \"%".$d->module."%\"";

   $sql.=")";

   if($d->module=="categories")
   {
    $id=$d->getID();
    $sql.=" AND `".$d->field[$id]["module"]."`.`".$id."`<>\"1\"";
   }

   return $this->exec($sql, $txt);
  }


  function unique()
  {
   $d=&$this->data; //D for short

   $txt="Check uniqueness: ".$d->title;

   $fields=func_get_args();
   if(is_array($fields[0]))
    $fields=$fields[0];

   $modules=Array();
   $where="WHERE ";
   foreach($fields as $field)
   {
    $where.="`".$d->field[$field]["module"]."`.`".$field."`=\"".addslashes($d->sqlValue($field))."\" AND ";

    if($d->field[$field]["module"]!=$d->module&&!in_array($d->field[$field]["module"], $modules))
     $modules[]=$d->field[$field]["module"];
   }

   if($id=$d->getID())
   {
    if($d->action!="add"&&$d->action!="copy")
     $where.="`".$d->field[$id]["module"]."`.`".$id."`<>\"".addslashes($d->sqlValue($id))."\" AND ";

    if($d->field[$id]["module"]!=$d->module&&!in_array($d->field[$id]["module"], $modules))
     $modules[]=$d->field[$id]["module"];
   }

   $sql="SELECT COUNT(*) as show_count ";

   $sql.=$this->from("unique", $modules);

   $sql.=substr($where, 0, -4);

   if(!$this->exec($sql, $txt))
    return false;

   if(!$this->next()||$this->res["show_count"])
    return false;

   return true;
  }


  function add()
  {
   global $_config;

   $d=&$this->data; //D for short

   if(!($values=$this->_getValues("add"))) //nothing to add... ?
    return true;

   foreach($values as $k => $v)
    if(strpos($v, "\"")!==false||strpos($v, "\\")!==false)
     $values[$k]=addslashes($v);

   $txt="Insert new ".$d->title;
   $sql="INSERT INTO `".$d->module."` (`".join("`, `", array_keys($values))."`) VALUES (\"".join("\", \"", $values)."\")";

   if(!$this->exec($sql, $txt))
    return false;

   if($insertID=$this->insertID())
    $d->setValue($d->getID(), $insertID, $d->vc);

   $this->e->add($d->title." inserted successfully", "note");
   return true;
  }


  function edit($looseFiltering=false)
  {
   $d=&$this->data; //D for short

   if(!($values=$this->_getValues("edit"))) //nothing to update
    return true;

   $txt="Update ".$d->title;
   $sql="UPDATE `".$d->module."` SET ";

   foreach($values as $k => $v)
    $sql.="`".$k."`=\"".addslashes($v)."\", ";

   $sql=substr($sql, 0, -2)." ";

   $sql.="WHERE ";

   if(!$looseFiltering) //identifier based
   {
    $found=false;
    foreach($d->check("properties", "identifier") as $k)
    {
     if($d->check("use", "sql", $k))
     {
      $sql.="`".$k."`=\"".addslashes($d->sqlValue($k))."\" AND ";
      $found=true;
     }
    }

    if(!$found)
    {
     $this->e->add("No identifiers set for updating!", "sql");
     return false;
    }

    $sql=substr($sql, 0, -4)." LIMIT 1";
   }
   else //looseFiltering: watch what you're doing!
   {
    $found=false;
    foreach($d->check("valuetype", "set") as $k)
    {
     if($d->check("module", $d->module, $k)&&$d->check("use", "sql", $k))
     {
      $sql.="`".$k."`=\"".addslashes($d->sqlValue($k))."\" AND ";
      $found=true;
     }
    }

    if(!$found)
    {
     $this->e->add("Update all not permitted!", "sql");
     return false;
    }

    $sql=substr($sql, 0, -4);
   }

   if(!$this->exec($sql, $txt))
    return false;

   $this->e->add($d->title." updated successfully", "note");
   return true;
  }


  function delete($looseFiltering=false)
  {
   global $_config;

   $d=&$this->data; //D for short

   $txt="Delete ".$d->title;
   $sql="DELETE FROM `".$d->module."` ";

   $sql.="WHERE ";

   if(!$looseFiltering) //identifier based
   {
    $found=false;
    foreach($d->check("properties", "identifier") as $k)
    {
     if($d->check("use", "sql", $k))
     {
      $sql.="`".$k."`=\"".addslashes($d->sqlValue($k))."\" AND ";
      $found=true;
     }
    }

    if(!$found)
    {
     $this->e->add("No identifiers set for deletion!", "sql");
     return false;
    }

    $sql=substr($sql, 0, -4)." LIMIT 1";
   }
   else //looseFiltering: watch what you're doing!
   {
    $found=false;
    foreach($d->check("valuetype", "set") as $k)
    {
     if($d->check("module", $d->module, $k)&&$d->check("use", "sql", $k))
     {
      $sql.="`".$k."`=\"".addslashes($d->sqlValue($k))."\" AND ";
      $found=true;
     }
    }

    if(!$found)
    {
     $this->e->add("Delete all not permitted!", "sql");
     return false;
    }

    $sql=substr($sql, 0, -4);
   }

   if(!$this->exec($sql, $txt))
    return false;

   $this->e->add($d->title." deleted successfully", "note");
   return true;
  }


  function create($submit=false)
  {
   $d=&$this->data;

   $txt="Create table ".$d->title;
   $sql="CREATE TABLE `".$d->module."` (";

   $keys="";

   $c=0;
   $autoincr=false;
   foreach($d->field as $k => $v)
   {
    if($d->field[$k]["module"]!=$d->module&&!$d->check("properties", "identifier", $k))
     continue;

    if($d->check("use", "sql", $k)&&!($k=="r"&&$v["type"]=="rights"))
    {
     $field=$this->getMySQLType($k);

     $sql.="`".$k."` ";
     $sql.=$field["type"]." ";

     if($field["collation"])
      $sql.="CHARACTER SET \"".$this->charset."\" COLLATE  \"".$this->collation."\" ";

     $sql.="NOT NULL ";

     if($d->check("properties", "identifier", $k))
     {
      $keys.="`".$k."`, ";

      if($d->check("type", "nr", $k)&&!$autoincr&&!$d->extends)
      {
       $autoincr=true;
       $sql.="auto_increment ";
      }
     }
     else if($field["default"])
      $sql.="default \"".$field["default"]."\" ";

     $sql.=", ";
     $c++;
    }
   }

   if($c==0)
   {
    $this->e->add("No fields for module \"".$d->title."\"", "error");
    return false;
   }

   if($keys!="")
    $sql.="PRIMARY KEY (".substr($keys, 0, -2).") ";
   else
    $sql=substr($sql, 0, -2)." ";

   foreach($d->index as $index => $fields)
    $sql.=", INDEX `".$index."` (`".join("`, `", $fields)."`) ";

   foreach($d->unique as $index => $fields)
    $sql.=", UNIQUE `".$index."` (`".join("`, `", $fields)."`) ";

   foreach($d->fulltextIndex as $index => $fields)
    $sql.=", FULLTEXT `".$index."` (`".join("`, `", $fields)."`) ";

   $sql.=") ENGINE=MyISAM CHARACTER SET \"".$this->charset."\" COLLATE  \"".$this->collation."\"";

   if($submit)
   {
    if(!$this->exec($sql, $txt))
     return false;

    $this->e->add("Table: \"".$d->module."\" created successfully", "note");
   }
   else
   {
    $this->e->add("Query for table: \"".$d->module."\": ".$sql, "note");
   }

   return true;
  }


  function alter($submit=false)
  {
   $d=&$this->data;

   $crossConvert=Array(
    "int" => Array("float", "varchar", "text", "bigint"),
    "float" => Array("int", "varchar", "text"),
    "varchar" => Array("text", "mediumtext", "longtext"),
    "text" => Array("mediumtext", "longtext"),
    "mediumtext" => Array("longtext"),
    "blob" => Array("mediumblob", "longblob"),
    "mediumblob" => Array("longblob"),
    "date" => Array("datetime"),
    "time" => Array("datetime"),
   );

   $txt="Get table status of: ".$d->title;
   $sql="SHOW TABLE STATUS WHERE Name=\"".$d->module."\"";

   if(!$this->exec($sql, $txt))
    return false;

   $collation=false;
   if($this->next())
    $collation=$this->res["Collation"];

   $txt="Get table columns of: ".$d->title;
   $sql="SHOW FULL COLUMNS FROM `".$d->module."`";

   if(!$this->exec($sql, $txt))
    return false;

   $fields=Array();
   $unusedFields=Array();
   $tmpFields=Array();

   foreach(array_keys($d->field) as $field)
    if($d->field[$field]["module"]==$d->module||$d->check("properties", "identifier", $field))
     $tmpFields[]=$field;

   while($this->next())
   {
    $fields[$this->res["Field"]]=$this->res;
    if(!in_array($this->res["Field"], $tmpFields))
     $unusedFields[]=$this->res["Field"];
   }
   unset($tmpFields);

   $txt="Get table indexes of: ".$d->title;
   $sql="SHOW INDEX FROM `".$d->module."`";

   if(!$this->exec($sql, $txt))
    return false;

   $indexMySQL=Array("primary" => Array(), "index" => Array(), "unique" => Array(), "fulltext" => Array());
   while($this->next())
   {
    if($this->res["Key_name"]=="PRIMARY")
    {
     $indexMySQL["primary"][]=$this->res["Column_name"];
    }
    else if($this->res["Index_type"]=="FULLTEXT")
    {
     if(!isset($indexMySQL["fulltext"][$this->res["Key_name"]]))
      $indexMySQL["fulltext"][$this->res["Key_name"]]=Array();

     $indexMySQL["fulltext"][$this->res["Key_name"]][]=$this->res["Column_name"];
    }
    else if($this->res["Non_unique"]) //index
    {
     if(!isset($indexMySQL["index"][$this->res["Key_name"]]))
      $indexMySQL["index"][$this->res["Key_name"]]=Array();

     $indexMySQL["index"][$this->res["Key_name"]][]=$this->res["Column_name"];
    }
    else //unique index
    {
     if(!isset($indexMySQL["unique"][$this->res["Key_name"]]))
      $indexMySQL["unique"][$this->res["Key_name"]]=Array();

     $indexMySQL["unique"][$this->res["Key_name"]][]=$this->res["Column_name"];
    }
   }

   $txt="Alter table ".$d->title;
   $sql="ALTER TABLE `".$d->module."` ";

   $c=0;
   $altered=false;
   $autoincr=false;

   $primaryKeyFields=Array();

   foreach($d->field as $k => $v)
   {
    if($d->field[$k]["module"]!=$d->module&&!$d->check("properties", "identifier", $k))
     continue;

    if($d->check("use", "sql", $k)&&!($k=="r"&&$v["type"]=="rights"))
    {
     if($d->check("properties", "identifier", $k))
      $primaryKeyFields[]=$k;

     $field=$this->getMySQLType($k);

     if(!isset($fields[$k])) //field doesn't exist in database, so create it
     {
      $altered=true;
      $renamed=false;

//XXX: rename checking needs enum extraction
      foreach($unusedFields as $key => $uf) //check all unused database field for similar properties
      {
       if($fields[$uf]["Type"]==$field["type"]) //found a matching field in unused database fields
       {
        $renamed=true;
        $sql.="CHANGE `".$uf."` `".$k."` ".$field["type"];
        $this->e->add("Renaming database field from: \"".$uf."\" to: \"".$k."\" in table: \"".$d->module."\"", "note");
        unset($unusedFields[$key]);
        break;
       }
      }

      if(!$renamed)
      {
       $sql.="ADD `".$k."` ".$field["type"];
       $this->e->add("Adding field: \"".$k."\" in table: \"".$d->module."\"", "note");
      }
     }
     //field exists
     else if($v["type"]=="enum"||$v["type"]=="boolean") //should be enum or boolean type
     {
      if(!preg_match("/^(\w+)\((.+)\)$/i", $fields[$k]["Type"], $current)) //get database field type
       $this->e->add("Can't determine current database type of field: \"".$k."\" in table: \"".$d->module."\"", "error");
      else if(!preg_match_all("/('([^']*)')+/", $current[2], $current)) //get database enum options
       $this->e->add("Can't determine current database enum options of field: \"".$k."\" in table: \"".$d->module."\"", "error");
      else
      {
       $current[0]=NULL;
       $current[ALTER_FIELD_TYPE]="enum";

       //compare enum/boolean options
       if($v["type"]=="boolean")
       {
        $booloptions=Array("false", "true");
        $newOptions=array_diff($booloptions, $current[ALTER_FIELD_OPTIONS]);
        $deletedOptions=array_diff($current[ALTER_FIELD_OPTIONS], $booloptions);
       }
       else
       {
        if(array_type($d->listOptions[$k])=="indexed")
        {
         $newOptions=array_diff($d->listOptions[$k], $current[ALTER_FIELD_OPTIONS]);
         $deletedOptions=array_diff($current[ALTER_FIELD_OPTIONS], $d->listOptions[$k]);
        }
        else //associative
        {
         $newOptions=array_diff(array_keys($d->listOptions[$k]), $current[ALTER_FIELD_OPTIONS]);
         $deletedOptions=array_diff($current[ALTER_FIELD_OPTIONS], array_keys($d->listOptions[$k]));
        }
       }

       if(sizeof($newOptions)) //new options to be added
       {
        $altered=true;
        $this->e->add("Adding new enum option(s): \"".join("\", \"", $newOptions)."\" for field: \"".$k."\" in table: \"".$d->module."\"", "note");

        if(isset($d->listOptions[$k]))
        {
         if(array_type($d->listOptions[$k])=="indexed")
          $newOptions=array_merge($d->listOptions[$k], $deletedOptions);
         else //associative
          $newOptions=array_merge(array_keys($d->listOptions[$k]), $deletedOptions);
        }

        $sql.="CHANGE `".$k."` `".$k."` enum(\"".join("\", \"", $newOptions)."\")";
       }

       if(sizeof($deletedOptions)) //old options, leave them in case they're in use
        $this->e->add("Leaving unused enum option(s): \"".join("\", \"", $deletedOptions)."\" for field: \"".$k."\" in table: \"".$d->module."\"", "note");
      }
     }
     //all but enum/boolean fields
     else if(!preg_match("/^(\w+)(\((\d+)\))?$/", $field["type"], $new)) //get module field type
      $this->e->add("Can't determine new database type of field: \"".$k."\" in table: \"".$d->module."\"", "error");
     else
     {
      if(!preg_match("/^(\w+)(\((\d+)\))?$/", $fields[$k]["Type"], $current)) //get database field type
       $this->e->add("Can't determine current database type of field: \"".$k."\" in table: \"".$d->module."\"", "error");
      else if($current[ALTER_FIELD_TYPE]!=$new[ALTER_FIELD_TYPE]) //database and module field type don't match
      {
       //try to cross convert field types
       if(isset($crossConvert[$current[ALTER_FIELD_TYPE]])&&in_array($new[ALTER_FIELD_TYPE], $crossConvert[$current[ALTER_FIELD_TYPE]]))
       {
        $altered=true;
        $sql.="CHANGE `".$k."` `".$k."` ".$field["type"];
        $this->e->add("Convert database type from: \"".$current[ALTER_FIELD_TYPE]."\" to: \"".$new[ALTER_FIELD_TYPE]."\" for field: \"".$k."\" in table: \"".$d->module."\"", "note");
       }
       else
        $this->e->add("Can't convert database type from: \"".$current[ALTER_FIELD_TYPE]."\" to: \"".$new[ALTER_FIELD_TYPE]."\" for field: \"".$k."\" in table: \"".$d->module."\"", "warning");
      }
      else if(isset($current[ALTER_FIELD_SIZE])&&isset($new[ALTER_FIELD_SIZE])&&$current[ALTER_FIELD_SIZE]!=$new[ALTER_FIELD_SIZE]) //type the same, check size change
      {
       //new type size is larger, resize
       if($current[ALTER_FIELD_SIZE]<$new[ALTER_FIELD_SIZE])
       {
        $altered=true;
        $sql.="CHANGE `".$k."` `".$k."` ".$field["type"];
        $this->e->add("Increase database field size from: \"".$current[ALTER_FIELD_SIZE]."\" to: \"".$new[ALTER_FIELD_SIZE]."\" for field: \"".$k."\" in table: \"".$d->module."\"", "note");
       }
       else if($current[ALTER_FIELD_SIZE]>$new[ALTER_FIELD_SIZE]) //new type size is smaller, leave it
        $this->e->add("Not shrinking database field size from: \"".$current[ALTER_FIELD_SIZE]."\" to: \"".$new[ALTER_FIELD_SIZE]."\" for field: \"".$k."\" in table: \"".$d->module."\"", "note");
      }

      //check if auto increment
      if(stristr($fields[$k]["Extra"], "auto_increment")) //database has auto increment set
      {
       if(!$d->check("properties", "identifier", $k)||!$d->check("type", "nr", $k)) //non identifier, remove auto increment
       {
        if(!$altered) //change if not already changed earlier, auto increment done below
        {
         $altered=true;
         $sql.="CHANGE `".$k."` `".$k."` ".$field["type"];
        }

        $this->e->add("Removing auto increment property for field: \"".$k."\" in table: \"".$d->module."\"", "note");
       }
      }
      else //no auto increment set in database
      {
       if($d->check("properties", "identifier", $k)&&$d->check("type", "nr", $k)&&!$d->extends) //non extending identifier, needs auto increment
       {
        if(!$altered) //change if not already changed earlier, auto increment done below
        {
         $altered=true;
         $sql.="CHANGE `".$k."` `".$k."` ".$field["type"];
        }

        $this->e->add("Adding auto increment property for field: \"".$k."\" in table: \"".$d->module."\"", "note");
       }
      }
     }

     if($field["collation"])
     {
      if(!isset($fields[$k])||$fields[$k]["Collation"]==$this->collation) //new field / collation stayed the same
      {
       if($altered)
        $sql.=" CHARACTER SET \"".$this->charset."\" COLLATE  \"".$this->collation."\"";
      }
      else //collation changed
      {
       $this->e->add("Changing collation from: \"".$fields[$k]["Collation"]."\" for field: \"".$k."\" to: \"".$this->collation."\" in table: \"".$d->module."\"", "note");

       if($altered)
        $sql.=" CHARACTER SET \"".$this->charset."\" COLLATE  \"".$this->collation."\"";
       else
       {
        $altered=true;
        $sql.="CHANGE `".$k."` `".$k."` ".$fields[$k]["Type"]; //don't change current field type
        $sql.=" CHARACTER SET \"".$this->charset."\" COLLATE  \"".$this->collation."\"";
       }
      }
     }

     if($altered)
     {
      $sql.=" NOT NULL ";

      if($d->check("properties", "identifier", $k))
      {
       if($d->check("type", "nr", $k)&&!$autoincr&&!$d->extends)
       {
        $autoincr=true;
        $sql.="auto_increment, ";
       }
       else
        $sql.=", ";
      }
      else if($field["default"])
       $sql.="default \"".$field["default"]."\", ";
      else
       $sql.=", ";

      $altered=false;
      $c++;
     }
    }
   }

   if(sizeof($unusedFields))
    $this->e->add("Not removing database field(s): \"".join("\", \"", $unusedFields)."\" in table: \"".$d->module."\"", "note");


   if($indexMySQL["primary"]&&(array_diff($indexMySQL["primary"], $primaryKeyFields)||array_diff($primaryKeyFields, $indexMySQL["primary"])))
   {
    $this->e->add("Dropping primary key (".implode(", ", $indexMySQL["primary"]).") in table: \"".$d->module."\"", "note");
    $sql.="DROP PRIMARY KEY, ";
    $c++;
   }

   if($primaryKeyFields&&(array_diff($indexMySQL["primary"], $primaryKeyFields)||array_diff($primaryKeyFields, $indexMySQL["primary"])))
   {
    $this->e->add("Adding primary key (".implode(", ", $primaryKeyFields).") in table: \"".$d->module."\"", "note");
    $sql.="ADD PRIMARY KEY (`".join("`, `", $primaryKeyFields)."`), ";
    $c++;
   }

   foreach($indexMySQL["index"] as $index => $fields)
   {
    if(!isset($d->index[$index])||array_diff($fields, $d->index[$index])||array_diff($d->index[$index], $fields))
    {
     $this->e->add("Dropping index: \"".$index."\" (".implode(", ", $fields).") in table: \"".$d->module."\"", "note");
     $sql.="DROP INDEX `".$index."`, ";
     $c++;
    }
   }
   foreach($d->index as $index => $fields)
   {
    if(!isset($indexMySQL["index"][$index])||array_diff($fields, $indexMySQL["index"][$index])||array_diff($indexMySQL["index"][$index], $fields))
    {
     $this->e->add("Adding index: \"".$index."\" (".implode(", ", $fields).") in table: \"".$d->module."\"", "note");
     $sql.="ADD INDEX `".$index."` (`".join("`, `", $fields)."`), ";
     $c++;
    }
   }

   foreach($indexMySQL["unique"] as $index => $fields)
   {
    if(!isset($d->unique[$index])||array_diff($fields, $d->unique[$index])||array_diff($d->unique[$index], $fields))
    {
     $this->e->add("Dropping unique index: \"".$index."\" (".implode(", ", $fields).") in table: \"".$d->module."\"", "note");
     $sql.="DROP INDEX `".$index."`, ";
     $c++;
    }
   }
   foreach($d->unique as $index => $fields)
   {
    if(!isset($indexMySQL["unique"][$index])||array_diff($fields, $indexMySQL["unique"][$index])||array_diff($indexMySQL["unique"][$index], $fields))
    {
     $this->e->add("Adding unique index: \"".$index."\" (".implode(", ", $fields).") in table: \"".$d->module."\"", "note");
     $sql.="ADD UNIQUE `".$index."` (`".join("`, `", $fields)."`), ";
     $c++;
    }
   }

   foreach($indexMySQL["fulltext"] as $index => $fields)
   {
    if(!isset($d->fulltextIndex[$index])||array_diff($fields, $d->fulltextIndex[$index])||array_diff($d->fulltextIndex[$index], $fields))
    {
     $this->e->add("Dropping fulltext index: \"".$index."\" (".implode(", ", $fields).") in table: \"".$d->module."\"", "note");
     $sql.="DROP INDEX `".$index."`, ";
     $c++;
    }
   }
   foreach($d->fulltextIndex as $index => $fields)
   {
    if(!isset($indexMySQL["fulltext"][$index])||array_diff($fields, $indexMySQL["fulltext"][$index])||array_diff($indexMySQL["fulltext"][$index], $fields))
    {
     $this->e->add("Adding fulltext index: \"".$index."\" (".implode(", ", $fields).") in table: \"".$d->module."\"", "note");
     $sql.="ADD FULLTEXT `".$index."` (`".join("`, `", $fields)."`), ";
     $c++;
    }
   }

   if($collation!=$this->collation)
   {
    $this->e->add("Changing collation from: \"".$collation."\" to: \"".$this->collation."\" in table: \"".$d->module."\"", "note");
    $sql.="CHARACTER SET \"".$this->charset."\" COLLATE  \"".$this->collation."\", ";
    $c++;
   }

   if($c>0)
   {
    $sql=substr($sql, 0, -2);

    if($submit)
    {
     if(!$this->exec($sql, $txt))
      return false;

     $this->e->add("Table: \"".$d->module."\" altered successfully", "note");
    }
    else
    {
     $this->e->add("Query for table: \"".$d->module."\": ".$sql, "note");
    }
   }
   else
    $this->e->add("No changes for table: \"".$d->module."\"", "note");

   return true;
  }


  function getMySQLType($field)
  {
   $d=&$this->data;
   $v=&$d->field[$field];

   $default=false;
   $collation=false;
   switch($v["type"])
   {
    case "nr":
    case "rights":
     if($d->check("properties", "big", $field))
      $type="bigint(20)";
     else
      $type="int(11)";
     $default=0;
    break;

    case "price":
    case "float":
     $type="double";
     $default=0;
    break;

    case "ip":
    case "ipv6":
     $type="varchar(40)";
     $collation=true;
    break;

    case "ipv4":
     $type.="int UNSIGNED";
     $default=0;
    break;

    case "password":
     if($d->check("properties", "nocrypt", $field))
      $type="varchar(".($v["max"]).")";
     else
      $type="varchar(255)";

     $default="";
     $collation=true;
    break;

    case "url":
    case "html":
    case "text":
    case "title":
    case "phone":
    case "domain":
    case "hostname":
    case "email":
    case "regexp":
    case "filename":
    case "alphabetic":
    case "captcha":

    case "list":
    case "multilist":
    case "module":
    case "modules":
    case "plugin":
    case "action":
    case "extra":
    case "project":

     if($v["max"]<=255)
      $type="varchar(".($v["max"]).")";
     else if($v["max"]<=pow(2,8))
      $type="tinytext";
     else if($v["max"]<=pow(2,16))
      $type="text";
     else if($v["max"]<=pow(2,24))
      $type="mediumtext";
     else if($v["max"]<=pow(2,32))
      $type="longtext";

     $default="";
     $collation=true;
    break;

    case "date":
     $type="date";
     $default="0000-00-00";
    break;

    case "time":
     $type="time";
     $default="00:00:00";
    break;

    case "datetime":
     $type="datetime";
     $default="0000-00-00 00:00:00";
    break;

    case "file":
    case "image":
    case "localfile":
    case "localdirectory":
     $type="varchar(255)";
     $default="";
     $collation=true;
    break;

    case "localfiles":
     $type="text";
     $default="";
     $collation=true;
    break;

    case "filedata":
     if($v["max"]<=pow(2,8))
      $type="tinyblob";
     else if($v["max"]<=pow(2,16))
      $type="blob";
     else if($v["max"]<=pow(2,24))
      $type="mediumblob";
     else if($v["max"]<=pow(2,32))
      $type="longblob";

     $default="";
    break;

    case "boolean":
     $type="enum('false','true')";
     $collation=true;
    break;

    case "enum":
     if(array_type($d->listOptions[$field])=="indexed")
      $type="enum('".join("','", $d->listOptions[$field])."')";
     else //associative
      $type="enum('".join("','", array_keys($d->listOptions[$field]))."')";

     $collation=true;
    break;

    case "version":
     $type="varchar(255)";
     $default="";
     $collation=true;
    break;

    default:
     $type=$v["type"];
     $default="";
    break;
   }

   $r=Array("type" => $type, "default" => $default, "collation" => $collation);
   return $r;
  }

 }

?>
