<?

 /*****************************************************************************
 * Filename: include/storage/storage.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."language.php");

 $_storageConnections=Array();

 class Storage
 {
  function __construct(&$d, $type)
  {
   $this->e=new Errors();

   $this->l=&$this->e->l;
   $this->l->read("storage");

   $this->type=$type;
   $this->realType=$type;

   $this->data=&$d;
  }

  function __clone()
  {
   //save original
   $tmpE=&$this->e;
   $tmpL=&$this->e->l;

   //restore original language (dereference), so it's cloned correctly
   unset($this->e->l);
   $this->e->l=$tmpL;

   //clone object
   unset($this->e);
   $this->e=clone $tmpE;

   //restore original
   unset($tmpE->l);
   $tmpE->l=&$tmpL;

   //update references
   unset($this->l);
   $this->l=&$this->e->l;
  }


  function _setConnection($hash, &$connection)
  {
   global $_storageConnections;

   if(isset($_storageConnections[$hash]))
    unset($_storageConnections[$hash]);

   $_storageConnections[$hash]=&$connection;
  }

  function _getConnection($hash)
  {
   global $_storageConnections;

   if(isset($_storageConnections[$hash]))
    return $_storageConnections[$hash];

   return false;
  }

  function getConfig()
  {
   global $_path, $_session;

   if(preg_match("!^([a-z0-9_]+)://!i", $this->data->db))
    $url=$this->data->db;
   else
   {
    require($_path["include"]."db.conf.php");

    if(file_exists($_path["project"]["include"]."db.conf.php"))
     require($_path["project"]["include"]."db.conf.php");

    if(!isset($db[$this->data->db]))
    {
     $this->e->add("Unknown database: \"".$this->data->db."\"", "system", false, ($this->data->module!="log")); //don't log
     return false;
    }

    if(!is_string($db[$this->data->db]))
    {
     $this->e->add("Incorrect database configuration: \"".$this->data->db."\"", "system", false, ($this->data->module!="log")); //don't log
     return false;
    }

    $url=$db[$this->data->db]; //uri: storage://user:pass@hostnane:port/database
   }

   //parse url
   if(!preg_match("!^(([a-z0-9_]+)://)?(([^@]+)@)?([a-z0-9\.\_\-]+|\[[0-F:]+\])?(:(\d+))?(/[^\?]*)?(\?.+)?$!i", $url, $m))
   {
    $this->e->add("Unable to parse database configuration: \"".$this->data->db."\"", "system", false, ($this->data->module!="log")); //don't log
    return false;
   }

   //basic database configuration
   $db=Array(
    "url" => $url,
    "hash" => md5($_session->get("project")."/".$url),
    "type" => false,
    "user" => false,
    "pass" => false,
    "host" => false,
    "port" => false,
    "name" => false,
    "path" => false,
    "vars" => Array(),
   );

   if($m[2])
    $db["type"]=strtolower($m[2]);

   if(isset($m[4])&&$m[4])
   {
    $tmp=explode(":", $m[4]);
    $db["user"]=$tmp[0];
    $db["pass"]=(isset($tmp[1])?$tmp[1]:false);
   }

   if(isset($m[5])&&$m[5])
   {
    if(preg_match("/^\[([0-F:]+)\]$/i", $m[5], $m2)) //ipv6 address
     $db["host"]=$m2[1];
    else //hostname / ipv4 address
     $db["host"]=$m[5];
   }

   if(isset($m[7])&&$m[7])
    $db["port"]=$m[7];

   if(isset($m[8])&&$m[8])
   {
    if(preg_match("!^/([^/]+)!", $m[8], $m2)) //detect database name
     $db["name"]=$m2[1]; //set first directory as database name

    $db["path"]=$m[8];
   }

   if(isset($m[9])&&$m[9])
   {
    $tmp=substr($m[9], 1); //remove ?
    foreach(explode("&", $tmp) as $kv) //get each key/value pair
    {
     $kv=explode("=", $kv);
     if(isset($kv[1]))
      $db["vars"][$kv[0]]=$kv[1];
     else
      $db["vars"][$kv[0]]=false;
    }
   }

   if(!$db["type"])
   {
    $this->e->add("Database type not defined for: \"".$this->data->db."\"", "system", false, ($this->data->module!="log")); //don't log
    return false;
   }

   if($this->type&&$db["type"]!=$this->type)
   {
    $this->e->add("Database: \"".$this->data->db."\" not a \"".$this->type."\" database", "system", false, ($this->data->module!="log")); //don't log
    return false;
   }

   return $db;
  }

  function clear()
  {
   $this->res=NULL;
   $this->results=Array();
   $this->resultsCounter=0;
   
   $this->count=0;
  }

  function next()
  {
   $this->res=NULL;

   if($this->e->any("fatal"))
    return false;

   if($this->resultsCounter<sizeof($this->results))
   {
    $this->res=$this->results[$this->resultsCounter];
    $this->resultsCounter++;
    return true;
   }
   else
   {
    $this->clear(); //cleanup if no more results
    return false;
   }
  }

  function getCount()
  {
   return $this->count;
  }


  function treeUp($gather="data")
  {
//use copy for gathering the data
   $d=clone $this->data;

//get reference+ID fields
   if(!($refer=$d->getSelfReference()))
   {
    $this->e->add("Unable to get tree: no self reference for module: \"".$d->module."\"", "module");
    return false;
   }

   $id=$d->getID();
   $refer=$d->reference[$refer]["from"];

   $d->setAction("view");
   $d->setExtra("all");
   $d->dataformat="sql";

   $d->setOperator($id, "="); //force operator

   $d->clearValues($id); //unset id field
   $d->setValue($id, $d->getValue($refer)); //set reference value to id field
   $d->clearValues($refer); //unset reference field

   //only return first row
   $d->setValue("show_from", 0);
   $d->setValue("show_total", 1);

   //show fields
   if($gather=="count")
    $d->setShowFields($refer); //only get refer
   else if(!$d->check("use", "show", $refer)) //ensure refer field is shown
    $d->setShowFields(array_merge($d->check("use", "show"), Array($refer)));

//execute
   $results=Array();
   while(true)
   {
    $d2=clone $d;
    if(!$d2->get())
    {
     $this->e->add($d2->e);
     return false;
    }

    if(!$d2->next()) //no results
     break; //done

    $results[]=$d2->v; //add results

    if(!$d2->v[$refer]) //no parent
     break; //done

    $d->setValue($id, $d2->v[$refer]); //set parent for next run
   }

   //use fake storage
   $this->type="fake";
   $this->clear();
   $this->results=array_reverse($results); //reverse values

   if($gather=="count"||$gather=="both")
   {
    if(!$this->_setCount())
     return false;

    if($gather=="count")
     return true;
   }

   if(!$this->_filterShowTotal())
    return false;

   return true;
  }

  function treeDown($gather="data")
  {
//use copy for gathering the data
   $d=clone $this->data;

//get reference+ID fields
   if(!($refer=$d->getSelfReference()))
   {
    $this->e->add("Unable to get tree: no self reference for module: \"".$d->module."\"", "module");
    return false;
   }

   $id=$d->getID();
   $refer=$d->reference[$refer]["from"];

   $d->setAction("view");
   $d->setExtra("all");
   $d->dataformat="sql";

   $d->setOperator($id, "="); //force operator
   $d->setOperator($refer, "="); //force operator

   if(!$d->check("valuetype", "none", $refer)) //self reference based (i.e. only include sub entries in results)
   {
    $d->field[$refer]["min"]=0; //allow top level gathering
    $d->setValue($refer, $d->getValue($refer));
    $d->unsetValue($id);
   }
   else //id based (i.e. also include current (id) entry in results)
   {
    $d->unsetValue($refer);
    $d->setValue($id, $d->getValue($id));
   }

//show fields
   if($gather=="count")
    $d->setShowFields($refer); //only get refer
   else if(!$d->check("use", "show", $refer)) //ensure refer field is shown
    $d->setShowFields(array_merge($d->check("use", "show"), Array($refer)));

//ensure output isn't limited
   if($show_total=$d->getValue("show_total"))
    $d->setValue("show_total", 0);

//execute
   $this->type="fake";
   $this->clear();

   $db=new Storage($d, "fake");
   if(($this->results=$db->_treeDownRecurse(0, $refer, $id))===false)
   {
    if($show_total) //restore show_total
     $d->setValue("show_total", $show_total);

    $this->e->add($db->e);
    return false;
   }

   if($show_total) //restore show_total
    $d->setValue("show_total", $show_total);

   if($gather=="count"||$gather=="both")
   {
    if(!$this->_setCount())
     return false;

    if($gather=="count")
     return true;
   }

   if(!$this->_filterShowTotal())
    return false;

   return true;
  }

  function _treeDownRecurse($treeDepth, $refer, $id=false)
  {
   $d=&$this->data; //D for short

   $results=Array();

   $d2=clone $d;
   if(!$d2->get())
   {
    $this->e->add($d2->e);
    return false;
   }

   while($d2->next())
   {
    $results[]=$d2->v; //add results

//enforce tree depth limit
    if($d->field["treedepth"]["valuetype"]!="set"||$treeDepth<$d->getValue("treedepth"))
    {
     $d->setValue($refer, $d2->v[$id]);
     $d->unsetValue($id);

     $db=new Storage($d, "fake");
     if(($results2=$db->_treeDownRecurse($treeDepth+1, $refer, $id))===false)
     {
      $this->e->add($db->e);
      return false;
     }

     $results=array_merge($results, $results2);
    }
   }

   return $results;
  }


  function unique()
  {
   $fields=func_get_args();
   if(is_array($fields[0]))
    $fields=$fields[0];

   $d=&$this->data; //D for short

   $id=$d->getID();

   $d2=clone $d;
   $d2->setAction("view");
   $d2->setExtra("all");
   $d2->dataformat="sql";

   $d2->clearValues();
   $d2->setShowFields();
   $d2->sortOrder();

   foreach($fields as $field)
    $d2->setValue($field, $d->sqlValue($field));

   if($d->action!="add")
   {
    $d2->setValue($id, $d->sqlValue($id));
    $d2->setOperator($id, "!=");
   }

   if(!$d2->get(false, true))
   {
    $this->e->add($d2->e);
    return false;
   }

   if($d2->getValue("show_count"))
    return false;

   return true;
  }

  function lost()
  {
   $d=&$this->data; //D for short

   $dc=new Data("categories", "view", "all");

   if($d->module!="categories"&&$d->module!="userRights")
   {
    $dc->setValue("module", $d->module);
    $dc->setOperator("module", "like");
   }

   $dcID=$dc->getID();
   $dc->setShowFields($dcID);

   if(!$dc->get())
   {
    $this->e->add($dc->e);
    return false;
   }

   $catRef=$d->getRightsReference();

   $d2=clone $d;
   $d2->setAction("view");
   $d2->setExtra("all");
   $d2->dataformat="sql";

   $d2->clearValues();
   $d2->setShowFields($d->check("properties", "title"));

   $c=0;
   if($d->module=="categories")
   {
    $d2->setValue($dcID, 1, $c);
    $d2->setOperator($dcID, "!=", $c);
    $c++;
   }

   while($dc->next())
   {
    $d2->setValue($catRef, $dc->v[$dcID], $c);
    $d2->setOperator($catRef, "!=", $c);
    $c++;
   }

   if(!$d2->get())
   {
    $this->e->add($d2->e);
    return false;
   }

   $this->type="fake";
   $this->clear();

   while($d2->next())
    $this->results[]=$d2->v;

   return true;
  }

  function _filterDataset(&$set, $gather="data")
  {
   $d=&$this->data; //D for short

   if($gather=="count")
   {
    foreach($d->check("use", "sql") as $k)
     if(
      ($k=="r"&&$d->check("type", "rights", $k))||
      ($d->check("properties", "identifier", $k)||$d->check("type", "rights", $k))
     )
      $showFields[]=$k;
   }
   else
   {
    $showFields=Array();
    foreach($d->check("use", "sql") as $k)
     if(
      ($k=="r"&&$d->check("type", "rights", $k))||
      ($d->check("use", "show", $k)||$d->check("properties", "identifier", $k)||$d->check("type", "rights", $k))
     )
      $showFields[]=$k;

    foreach($d->sortOrder as $field)
     if($field=preg_replace("/^([^\-\+]+)(\-|\+)?$/", "\\1", $field))
      if($d->check("use", "sql", $k))
       if(!in_array($field, $showFields))
        $showFields[]=$field;
   }

   $fields=Array();
   switch($d->extra)
   {
//Get by identifier (view, edit, etc)
    case "item":
    {
     foreach($d->check("properties", "identifier") as $k)
      if($d->check("use", "sql", $k))
       if($d->check("valuetype", "set", $k)||$d->check("valuetype", "request", $k)||$d->check("valuetype", "get", $k))
        $fields[]=$k;

     break;
    }

//List by category
    case "list":
    {
     $refer=$d->getRightsReference();
     $fields[]=$d->reference[$refer]["from"];
     break;
    }

//All extra options (view->all, user->login, search: title, field, show, full, etc): value type = set
    default:
    {
     foreach($d->check("valuetype", "set") as $k)
      if($d->check("use", "sql", $k))
       $fields[]=$k;

     break;
    }
   }


   $row=0;
   $resultsRow=0;
   $operant="AND";
   $l=sizeof($fields);
   $setSize=sizeof($set);

   for($s=0;$s<$setSize;$s++)
   {
    //filter out deleted items (or non deleted items for "deleted" and "undelete" actions
    $ignore=false;
    foreach($d->check("properties", "deleted") as $k)
    {
     if($d->check("use", "sql", $k))
     {
      if($d->action=="undelete"||$d->extra=="deleted") //show deleted/undelete
      {
       if(!isset($set[$s][$k])||$set[$s][$k]=="false") //entry not deleted
       {
        $ignore=true; //not relevant for data set
        break;
       }
      }
      else //show non deleted
      {
       if(isset($set[$s][$k])&&$set[$s][$k]=="true") //entry deleted
       {
        $ignore=true; //not relevant for data set
        break;
       }
      }
     }
    }

    if($ignore)
     continue;

    if($fields)
    {
     $allowed=false;
     for($d->vc=0;$d->vc<$d->vmax;$d->vc++)
     {
      $entryMatch=true; //entries normally match unless filter values don't match
      for($c=0;$c<$l;$c++)
      {
       if(isset($d->field[$fields[$c]]["value"][$d->vc])&&isset($d->field[$fields[$c]]["operant"][$d->vc]))
       {
        if(isset($set[$s][$fields[$c]]))
         $value=strtolower($set[$s][$fields[$c]]);
        else
         $value=NULL;

        $filter=strtolower($d->sqlValue($fields[$c], $d->vc));
        switch($d->getOperator($fields[$c], $d->vc))
        {
         case "=":
          $match=($value==$filter);
         break;

         case "!=":
          $match=($value!=$filter);
         break;

         case ">=":
          $match=($value>=$filter);
         break;

         case "<=":
          $match=($value<=$filter);
         break;

         case ">":
          $match=($value>$filter);
         break;

         case "<":
          $match=($value<$filter);
         break;

         case "&":
          $match=($value&$filter);
         break;

         case "|":
          $match=($value|$filter);
         break;

         case "like":
          $match=((strpos($value, $filter)!==false)?true:false);
         break;

         case "not like":
          $match=((strpos($value, $filter)!==false)?false:true);
         break;

         case "start like":
          $match=((strpos($value, $filter)===0)?true:false);
         break;

         case "not start like":
          $match=((strpos($value, $filter)===0)?false:true);
         break;

         case "end like":
          $match=((strrpos($value, $filter)===(strlen($value)-strlen($filter)))?true:false);
         break;

         case "not end like":
          $match=((strrpos($value, $filter)===(strlen($value)-strlen($filter)))?false:true);
         break;

         default:
          $match=false;
          $this->e->add("Unknown operator: \"".$d->getOperator($fields[$c], $d->vc)."\"", "sql");
         break;
        }

        if($d->extra=="item")
         $operant="OR";
        else
         $operant=$d->getOperant($fields[$c], $d->vc);

        if($operant=="OR")
        {
         if($match) //definit match
         {
          $entryMatch=true;
          break; //don't bother checking rest of the entry
         }
         else //see what else is in the entry
          $entryMatch=false;
        }
        else //AND
        {
         if(!$match) //definit mismatch
         {
          $entryMatch=false;
          break; //don't bother checking rest of the entry
         }
         else //see what else is in the entry
          $entryMatch=true;
        }
       }
      }

      if($operant=="OR")
      {
       if($entryMatch)
       {
        $allowed=true;
        break; //skip next entries
       }
       else
        $allowed=false;
      }
      else //AND
      {
       if(!$entryMatch)
       {
        $allowed=false;
        break; //skip next entries
       }
       else
        $allowed=true;
      }
     }

     if(!$allowed) //filter rows
      continue;
    }

    //filter fields
    $this->results[$row]=Array();
    foreach($showFields as $k)
    {
     if(!isset($set[$s][$k]))
      $this->results[$row][$k]=NULL;
     else
      $this->results[$row][$k]=$set[$s][$k];
    }

    $row++;
   }

   return true;
  }

  function _gatherFilterRights()
  {
   global $_config;

//XXX: add proper rights gathering

   $d=&$this->data; //D for short

   if($_config->userRoot())
    $rights=$d->getRootRights();
   else
    $rights=257; //read;

   if($l=sizeof($this->results))
    for($c=0;$c<$l;$c++)
     $this->results[$c]["r"]=$rights;

   return true;
  }

  function _sort($fields=false, $from=0, $to=false)
  {
   if(!$this->results)
    return true;

   $d=&$this->data; //D for short

   if($fields===false)
    $fields=$d->sortOrder;

   if(!$fields)
    return true;

   if($to===false)
    $to=sizeof($this->results);

   if(!preg_match("/^([^\-\+]+)(\-|\+)?$/", $fields[0], $field)) //shouldn't even come to here
    return false;

   if(isset($field[2])&&$field[2]=="-")
    $reverse=true;
   else
    $reverse=false;

   $field=$field[1];

   $this->_sortField($field, $reverse, $from, $to); //sort field for given range
   array_shift($fields); //unset current field from sort array (make sure we won't end up in an endless loop)

   if(sizeof($fields)==0) //no deeper sorting fields, so let's stop
    return true;

   $start=$from;
   $value=$this->results[$from][$field]; //get first value
   for($c=$from+1;$c<=$to;$c++)
   {
    if($c==$to||$this->results[$c][$field]!=$value) //check difference or end of range
    {
     //difference found, check sub fields in this range
     if(($c-$start)>1)//only for atleast 2 values
      $this->_sort($fields, $start, $c);

     if($c==$to)
      break;

     $value=$this->results[$c][$field];
     $start=$c; //set next starting point for next run
    }
   }

   return true;
  }

  function _sortField($field, $reverse=false, $from=false, $to=false)
  {
   $d=&$this->data; //D for short

   if($from===false)
    $from=0;
   if($to===false)
    $to=$d->vmax;

   $tmp=Array();
   $tmp2=Array();
   for($c=$from;$c<$to;$c++) //correct int sorting
   {
    $tmp[$c]=$this->results[$c][$field];
    $tmp2[$c]=$this->results[$c];
   }

   if($reverse)
    arsort($tmp); //sort on field, keeping keys
   else
    asort($tmp); //sort on field, keeping keys

   $z=0;
   $keys=array_keys($tmp);

   for($c=$from;$c<$to;$c++) //correct int sorting
    $this->results[$c]=$tmp2[$keys[$z++]];
  }


  function _filterShowTotal()
  {
   $d=&$this->data; //D for short

   if(!($show_total=$d->sqlValue("show_total", 0)))
    return true;

   $show_from=$d->sqlValue("show_from", 0);
   $show_total=$show_total+$show_from;

   $results=Array();
   for($c=$show_from;$c<$show_total;$c++)
    if(isset($this->results[$c]))
     $results[]=&$this->results[$c];

   $this->results=$results;
   return true;
  }

  function _setCount()
  {
   $this->count=sizeof($this->results);
   return true;
  }

  function _getValues($action)
  {
   global $_config;

   $d=&$this->data; //D for short

   $values=Array();

   switch($action)
   {
    case "add":
    {
     foreach($d->check("use", "sql") as $k)
     {
      if($k!="r"&&((!$d->getReferences($k)&&!$d->check("properties", "identifier", $k))||$d->dataformat=="sql"||$d->extends||$d->check("type", "text", $k))) //allow id insertion only for sql values, extending modues and text fields
      {
       if($d->module==$d->field[$k]["module"]||$d->check("properties", "identifier", $k))
       {
        if($d->check("properties", "owner", $k)&&!$d->check("properties", "identifier", $k)) //force usernr for owner field unless it's the identifier (forces correct id for extending modules
         $values[$k]=$_config->get("usernr");
        else
         $values[$k]=$d->sqlValue($k);
       }
      }
     }

     foreach(array_keys($d->reference) as $ref)
     {
      $k=$d->reference[$ref]["from"];

      if($d->field[$k]["module"]!=$d->module&&!$d->check("properties", "identifier", $k))
       continue;

      if(!$d->check("use", "sql", $k))
       continue;

      switch($d->reference[$ref]["type"])
      {
       case "1:1":
       case "*:0":
       case "*:1":
       {
        $values[$k]=$d->sqlValue($ref);
        break;
       }
       default: break;
      }
     }

     break;
    }

    case "edit":
    {
     foreach($d->check("use", "sql") as $k)
     {
      if($d->getReferences($k)) //references done below
       continue;

      if(
       $d->check("properties", "identifier", $k)|| //don't update identifiers
       $d->check("valuetype", "none", $k)|| //don't update non set fields
       !$d->check("module", $d->module, $k)|| //don't update fields from extended tables
       $k=="r" //don't update bogus r
      )
       continue;

      $values[$k]=$d->sqlValue($k);
     }

     foreach(array_keys($d->reference) as $ref)
     {
      if($d->reference[$ref]["type"]=="*:0"||$d->reference[$ref]["type"]=="*:1")
      {
       $k=$d->reference[$ref]["from"];

       if(!$d->check("use", "sql", $k)) //should be an SQL field
        continue;

       if(
        $d->check("properties", "identifier", $k)|| //don't update identifiers
        $d->check("valuetype", "none", $k)|| //don't update non set fields
        !$d->check("module", $d->module, $k)|| //don't update fields from extended tables
        $k=="r" //don't update bogus r
       )
        continue;

       $values[$k]=$d->sqlValue($ref);
       $found=true;
      }
     }

     break;
    }
   }

   return $values;
  }

  function checkRights($action){$this->e->add("Storage function: \"checkRights\" not defined", "system", false, ($this->data->module!="log")); return false;}
  function insertID(){$this->e->add("Storage function: \"insertID\" not defined", "system", false, ($this->data->module!="log")); return false;}
  function table(){$this->e->add("Storage function: \"table\" not defined", "system", false, ($this->data->module!="log")); return false;}
  function info(){$this->e->add("Storage function: \"info\" not defined", "system", false, ($this->data->module!="log")); return false;}
  function get(){$this->e->add("Storage function: \"get\" not defined", "system", false, ($this->data->module!="log")); return false;}
  function add(){$this->e->add("Storage function: \"add\" not defined", "system", false, ($this->data->module!="log")); return false;}
  function edit(){$this->e->add("Storage function: \"edit\" not defined", "system", false, ($this->data->module!="log")); return false;}
  function delete(){$this->e->add("Storage function: \"delete\" not defined", "system", false, ($this->data->module!="log")); return false;}
  function create(){$this->e->add("Storage function: \"create\" not defined", "system", false, ($this->data->module!="log")); return false;}
  function alter(){$this->e->add("Storage function: \"alter\" not defined", "system", false, ($this->data->module!="log")); return false;}

  //empty login/logout stubs, most cases these are not needed. Some DBs (xmli) however require special actions on login and logout
  function login(){return true;}
  function logout(){return true;}
 }

?>
