<?

 /*****************************************************************************
 * Filename: project/jas/include/backend.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."language.php");
 require_once($_path["include"]."plugin.php");
 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."jas.php");
 require_once($_path["include"]."fs.php");

 require_once($_path["project"]["include"]."backend/login.php");

 class Backend
 {
  function __construct($template)
  {
   $this->e=new Errors();
   $this->fs=new FS();
   $this->fs->e=&$this->e;

   $this->language=&$this->e->l;

   $this->pageVars=Array();

   $this->output="";
   $this->template=$template;

   $this->components=Array();

   $this->setTitle("", "");
   $this->search=Array();
   $this->stylesheets=Array();

   $this->login=new Backend_Login($this);
  }

  function setTitle($icon, $title)
  {
   $titles=func_get_args();
   array_shift($titles);

   if($titles&&is_array($titles[0]))
    $titles=$titles[0];

   $this->title=Array("icon" => $icon, "titles" => $titles);
   return true;
  }

  function determinePage()
  {
   //XXX: add page validation

   if(($p=request("p"))&&is_array($p))
    foreach($p as $page)
     $this->pageVars[]=$page;

   return true;
  }

  function search($request)
  {
   global $_config;

   $this->search=Array(
    "request" => $request,
    "inline" => true,
    "url" => "",
   );

   if(!$_config->get("api.xml")) //XML api disabled
    $this->search["inline"]=false;

   $args=func_get_args();
   array_shift($args);

   switch($request)
   {
    case "projects":
    {
     if($this->search["inline"])
      if(!$_config->get("api.projects")) //projects api disabled
       $this->search["inline"]=false;

     if(isset($args[0])) //prefix
      $this->search["prefix"]=$args[0];

     if(isset($args[1])) //suffix
      $this->search["suffix"]=$args[1];

     break;
    }
    case "modules":
    case "plugins":
    {
     if($this->search["inline"])
      if(!$_config->get("api.modules")) //modules api disabled
       $this->search["inline"]=false;

     $this->search["request"]=$request;

     if(isset($args[0])) //prefix
      $this->search["prefix"]=$args[0];

     if(isset($args[1])) //suffix
      $this->search["suffix"]=$args[1];

     break;
    }
    case "data":
    {
     if(sizeof($args)<3)
     {
      $this->e->add("Data search requires a module, a search type and a field", "system");
      return false;
     }

     if($this->search["inline"])
      if(!$_config->get("api.searching")) //search api disabled
       $this->search["inline"]=false;

     if($this->search["inline"])
      if(!$_config->get("api.searchingAllModules")) //search all modules not allowed
       if(!in_array($args[0], $_config->get("api.searchingModules"))) //search module not allowed
        $this->search["inline"]=false;

     $this->search["request"]=$request;
     $this->search["module"]=$args[0];
     $this->search["extra"]=$args[1];
     $this->search["field"]=$args[2];
     break;
    }
    case "normal":
    {
     $this->search["inline"]=false;
     break;
    }
    default:
    {
     $this->e->add("Unknown search request: \"".$request."\"", "system");
     return false;
    }
   }

   return true;
  }

  function searchURL($url)
  {
   if(!$this->search)
   {
    $this->e->add("Can't set search URL: \"".$url."\", search not active", "system");
    return false;
   }

   $this->search["url"]=$url;
   return true;
  }

  function addStyleSheet($file)
  {
   if(in_array($file, $this->stylesheets)) //already added
    return true;

   if(!$this->fs->exists($file))
   {
    $this->e->add("Style sheet not found: \"".$file."\"", "error");
    return false;
   }

   $this->stylesheets[]=$file;
   return true;
  }


  /*
   * Main logic
   */

  function main()
  {
   global $_path, $_config, $_session;

   //read template
   if(!$this->fs->exists($this->template))
   {
    $this->e->add("Template not found: \"".$this->template."\"", "system");
    return false;
   }

   if(!($this->output=$this->fs->read($this->template)))
   {
    if($this->output==="") //empty file
     $this->e->add("File: \"".$this->template."\" is empty", "error");

    return false;
   }

   //detect components
   if(!preg_match_all("/#([^#]+)#/i", $this->output, $m))
   {
    $this->e->add("No #markers# found in template: \"".$this->template."\"", "error");
    return false;
   }

   //set backend project
   if(request("backendProject")!==NULL)
    $_session->setBackendProject(request("backendProject"));

   //reload project configuration
   if(request("reloadConfig"))
   {
    $_session->frontend();
    $_config->reload();
    $_session->backend();
   }

   //login
   if(request("doLogin"))
    $this->login->main();

   $_session->frontend();

   //store history
   $hs=new History();
   $hs->lockProject(); //lock to current
   if(!$hs->isAlreadyStored())
    if(!$hs->store())
     $this->e->add($hs->e);

   //set project language + translation errors
   if($_config->get("language"))
    $this->language->setLanguage($_config->get("language"));
   if($_config->get("translationerrors")!==NULL)
    $this->language->showErrors($_config->get("translationerrors"));

   $_session->backend();

   //(re)load translations
   if(request("reloadTranslations"))
    $this->language->clear();

   $this->language->read("jas");

   //modrewrite
   if($_config->get("useModRewrite")&&!$_session->get("modRewriteActive")) //use mod rewrite force while not active
    $this->e->add("Mod rewrite used but not availble, either fix mod rewrite support or disable usage in the configuration", "warning");

   $this->rewrite=$_config->get("useModRewrite"); //set global _useModRewrite based on JAS config to make sure the project settings doesn't overwrite it

   //pages
   if(!$this->determinePage())
    return false;

   if(!$this->login->backendLoggedIn()||!$this->login->backendRootPasswordSet())
   {
    //clear all components
    foreach($m[1] as $component)
     $this->components[$component]=false;

    //set main component
    $this->components["main"]["object"]=&$this->login;

    //backend login / set root password
    $this->components["main"]["object"]->main();
   }
   else //logged in to backend
   {
    //assess components
    foreach($m[1] as $component)
    {
     if($component!="main") //normal component
      $path=explode("/", $component); //get path
     else //main component
     {
      if($this->pageVars) //page request
       $path=array_merge(Array("backend"), $this->pageVars); //use page path
      else
       $path=Array("backend", "home"); //home page
     }

     $this->components[$component]=false;

     //init component object
     if($path[0]=="backend")
     {
      if(sizeof($path)<2)
      {
       $this->e->add("Backend section requires at least 1 argument", "error", "E.g. backend.login");
       continue;
      }

      //determine backend
      $file=$_path["project"]["include"]."backend/".$path[1].".php";
      if(!$this->fs->exists($file))
      {
       $this->e->add("Template component: \"".$path[1]."\" not found", "error", $file);
       continue;
      }

      require_once($file);

      $class="Backend_".ucfirst($path[1]);
      if(!class_exists($class))
      {
       $this->e->add("Template component class not defined: \"".$class."\"", "error");
       continue;
      }

      //load backend
      $this->components[$component]["object"]=new $class($this);

      if(sizeof($path)<3) //no arguments
       $this->components[$component]["arguments"]=false;
      else //arguments
       $this->components[$component]["arguments"]=array_slice($path, 2);
     }
     else if($path[0]=="plugin")
     {
      if(sizeof($path)<2)
      {
       $this->e->add("Plugin section requires at least 1 argument", "error", "E.g. plugin.skeleton");
       continue;
      }

      //load plugin
      $_session->frontend();

      if(!in_array($path[1], JAS::plugins())) //plugin not found
      {
       $_session->backend();
       continue; //skip
      }

      $this->components[$component]["plugin"]=new Plugin($path[1]);

      $_session->backend();


      if(sizeof($path)<3) //no arguments
      {
       $this->components[$component]["frontend"]=false;
       $this->components[$component]["arguments"]=false;
      }
      else //arguments
      {
       if($path[2]=="frontend") //show plugin frontend
       {
        $this->components[$component]["frontend"]=true;
        $this->components[$component]["arguments"]=array_slice($path, 3);
       }
       else //arguments
       {
        $this->components[$component]["frontend"]=false;
        $this->components[$component]["arguments"]=array_slice($path, 2);
       }
      }
     }
     //else //special marker

    }

    //main logic
    foreach(array_keys($this->components) as $component)
    {
     if($this->components[$component])
     {
      if(isset($this->components[$component]["object"])) //backend object
      {
       if($this->components[$component]["arguments"]) //arguments provided
        call_user_func_array(Array($this->components[$component]["object"], "main"), $this->components[$component]["arguments"]); //call with proper arguments
       else //no arguments
        $this->components[$component]["object"]->main(); //call directly
      }
      else if(isset($this->components[$component]["plugin"])) //plugin
      {
       $args=Array(($this->components[$component]["frontend"]?"frontend":"backend"), "main"); //determine frontend/backend main call

       if($this->components[$component]["arguments"]) //arguments provided
        foreach($this->components[$component]["arguments"] as $arg)
         $args[]=$arg;

       $_session->frontend();

       if(!call_user_func_array(Array($this->components[$component]["plugin"], "call"), $args)) //call with proper arguments
       {
        $this->e->add($this->components[$component]["plugin"]->e); //store errors
        $this->components[$component]=false; //don't render output
       }

       $_session->backend();
      }
     }
    }
   }

   return true;
  }

  function &component($component)
  {
   if(isset($this->components[$component]))
    if(isset($this->components[$component]["object"])) //backend object
     return $this->components[$component]["object"];

   return NULL;
  }


  /*
   * Render output
   */

  function output()
  {
   global $_path, $_session, $_config;

   //set theme from cookie
   if($theme=request("backendTheme", "COOKIE"))
    if(preg_match("/^\w+$/", $theme)) 
     if($this->fs->exists($_path["project"]["style"].$theme.".css"))
      $this->addStyleSheet($_path["project"]["style"].$theme.".css");

   //fix relative paths in template
   $this->output=preg_replace("!(<[^>]+(href|src)=)(['\"])([^/#][^'\"\:]+)(\?[^'\"]+)?(['\"])!i", "\\1\\3".$_path["HTML"]["prefix"].$_path["project"]["templates"]."\\4\\5\\6", $this->output);

   //render components
   foreach(array_keys($this->components) as $component)
   {
    if(!$this->components[$component]) //skip special component
     continue;

    if(isset($this->components[$component]["object"])) //render backend output
     $content=$this->components[$component]["object"]->output();
    else if(isset($this->components[$component]["plugin"])) //render plugin output
    {
     $_session->frontend();

     if(($content=$this->components[$component]["plugin"]->call(($this->components[$component]["frontend"]?"frontend":"backend"), "output"))===false)
      $this->e->add($this->components[$component]["plugin"]->e);

     $_session->backend();
    }
    else
     $content=false;

    $this->output=str_replace("#".$component."#", $content, $this->output);
   }

   foreach(array_keys($this->components) as $component)
   {
    if($this->components[$component]) //skip backend/plugin component
     continue;

    switch($component)
    {
     case "script":
     {
      $content=" <script type=\"text/javascript\"><!--\n var browser=\"".$_session->get("browser")."\";\n var path=\"".$_path["HTML"]["prefix"].$_path["project"]["root"]."\";\n var rootpath=\"".$_path["HTML"]["prefix"].$_path["root"]."\";\n --></script>\n";
      break;
     }
     case "root":
     {
      $content=$_path["HTML"]["current"];
      break;
     }
     case "errors":
     {
      $content=$this->e->show();
      break;
     }
     case "stylesheets":
     {
      $content="";
      foreach($this->stylesheets as $file)
       $content.="  <link rel=\"stylesheet\" type=\"text/css\" media=\"all\" href=\"".$_path["HTML"]["prefix"].$file."\"/>\n";

      if(!$content) //ensure nbsp isn't rendered below
       $content.=" ";

      break;
     }
     case "title":
     {
      $content=$_config->get("title");
      break;
     }
     case "version":
     {
      if($this->login->backendLoggedIn()) //version information only for logged in users
       $content="v".JAS::versionS().(JAS::versionS("tag")?" ".JAS::versionS("tag"):"").(JAS::versionS("development")?"*":"");
      else
       $content="";

      break;
     }
     case "title/page":
     {
      $content=implode(" / ", $this->title["titles"]);
      break;
     }
     case "title/page/strip":
     {
      $content=strip_tags(implode(" - ", $this->title["titles"]));
      break;
     }
     case "title/icon":
     {
      $content=$this->title["icon"];
      break;
     }
     case "benchmark":
     {
      if($this->login->backendLoggedIn())
       $content="#benchmark#"; //leave in output, handled in index.php

      break;
     }
     default:
     {
      $content=false;
      break;
     }
    }

    $this->output=str_replace("#".$component."#", ($content?$content:"&nbsp; "), $this->output);
   }

   return $this->output;
  }

 }
?>
