<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/categories.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["include"]."plugin.php");
 require_once($_path["include"]."cache.php");
 require_once($_path["include"]."data.php");

 class Backend_Categories extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->categories=Array();
   $this->path="data/";

   $this->catnr=false;
  }

  function main($section=false)
  {
   global $_session, $_config;

   //section
   if(!parent::main($section))
    return false;

   $_session->frontend();

   $cache=false;
   if($_config->get("cache.memory"))
   {
    $cache=new Cache("memory"); //autodetect
    if($cache->e->any("fatal")) //cache init failed
     $cache=false;
    else
    {
     $cacheKey="JAS.".$_session->get("project").".".$_config->get("usernr").".Categories.tree";

     if($cache->exists($cacheKey))
     {
      if(!($lastUpdate=$cache->lastUpdate("categories"))) //get database updated time
       $this->e->add($cache->e);
      else if($cache->time($cacheKey)<$lastUpdate) //cache older than last database update
       $cache->remove($cacheKey); //remove stale cache
      else //cache newer than last database update
      {
       if(!($this->categories=$cache->read($cacheKey))) //get cached version
        $this->e->add($cache->e);
      }
     }
    }
   }

   if(!$this->categories)
   {
    $d=Data::cache("categories", "tree", "down");
    $d->setShowFields("name", "text", "catnr");

    $d->clearValues();
    $d->setValue("catnr", 0);

    if(!$d->get())
    {
     $this->e->add($d->e);
     $_session->backend();
     return false;
    }

    while($d->next())
    {
     $this->categories[$d->v("id")]=$d->v;
     $this->categories[$d->v("id")]["icon"]="category";
    }

    if($cache) //caching
     if(!$cache->save($cacheKey, $this->categories)) //store categories list
      $this->e->add($cache->e);
   }

   if(($tmp=request("catnr"))&&!is_array($tmp))
    $this->catnr=$tmp;

   $_session->backend();

   return true;
  }


  /*
   * Output
   */

  function output_quick()
  {
   global $_session;

   //list categories
   $_session->frontend();

   if(!$this->categories)
   {
    $this->menuItem("categories", $this->language->t("Categories"), $this->url("data/"), $this->language->t("Data"), "category");
    $r=$this->menu("categories");
   }
   else
   {
    //get first id
    $first=key($this->categories);

    //get selected entries
    $selected=Array();
    if($this->catnr)
    {
     $tmp=$this->catnr;
     while(isset($this->categories[$tmp]))
     {
      $selected[]=$tmp;

      if(!($tmp=$this->categories[$tmp]["catnr"])||$tmp==$first)
       break;
     }
    }

    //set preselected flag
    $catnr=false;
    foreach(array_reverse($this->categories, true) as $id => $v)
    {
     if($catnr==$v["catnr"]) //same level
      $this->categories[$id]["preselected"]=true;
     else //other level
     {
      $this->categories[$id]["preselected"]=false;

      if(in_array($id, $selected)) //current selected
       $catnr=$v["catnr"];
     }
    }

    //generate menus
    foreach($this->categories as $id => $v)
     $this->menuItem("categories_".(!$v["catnr"]?$first:$v["catnr"]), $v["name"], ($this->path=="data/"?$this->url($this->path.$id."/"):$this->url($this->path, Array("catnr" => $id))), $v["text"], $v["icon"], Array("menu" => "categories_".$id, "class" => "popup".($v["preselected"]?" preselected":""), (in_array($id, $selected)?"selected":false)));

    $r=$this->menu("categories_".$first);
   }

   $_session->backend();

   return $r;
  }

 }
?>
