<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/data.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["include"]."data.php");

 class Backend_Data extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->module=false;
   $this->nr=false;
  }

  function main($module=false, $nr=false)
  {
   global $_session;

   $_session->frontend();

   if(preg_match("/^\d+$/", $module)) //nr set as first var
   {
    $nr=$module; //use as nr
    $module=false;
   }

   if(!$module) //category
   {
    $this->section="category";

    $d=Data::cache("categories", "view", "all");

    if($titles=$d->check("properties", "title"))
     $this->backend->search("data", $d->module, "title", $titles[0]);

    if(request("search")) //show nr or search results
    {
     $d->setAction("search");
     $d->setExtra("full");

     $d->setShowFields();
     $d->setValue("show_total", 2);
     $d->setRequestValue("search");

     if($d->get()&&$d->vmax==1) //only 1 entry
      Headers::redirectS($this->url("data/".$d->getValue($d->getID())."/"));
     else
      $this->section="search";
    }
    else if($nr) //nr set
     $this->nr=$nr;
    else //no category, get first
    {
     $d->setAction("view");
     $d->setExtra("all");

     $d->field["catnr"]["min"]=0; //overwrite minimum catnr
     $d->setShowFields();

     $d->clearValues();
     $d->setValue("catnr", 0);
     $d->setValue("show_total", 1);

     if(!$d->get())
     {
      $this->e->add($d->e);
      $_session->backend();
      return false;
     }

     $this->nr=$d->getValue("nr");
    }
   }
   else
   {
    $this->section="item";
    $this->module=$module;
    $this->nr=$nr;
   }

   //modify quick categories list selected items
   if($this->nr)
    if($quick=$this->backend->component("backend/categories/quick")) //categories list found
     $quick->catnr=$this->nr;

   $_session->backend();
   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session;

   $_session->frontend();

   //crumbtrail
   $d=Data::cache("categories", "tree", "up");
   $d->setShowFields("name");

   $d->clearValues();
   $d->setValue("catnr", $this->nr);

   if(!$d->get()||!$d->next()) //skip top category
   {
    $this->backend->setTitle($this->icon("data_dim"),
     $this->link($this->language->t("Data"), Array("data/")));

    $this->e->add($d->e);
    $_session->backend();
    return false;
   }

   $path=Array($this->link($this->language->t("Data"), Array("data/")));
   while($d->next())
    $path[]=$this->link($d->v["name"], Array("data/".$d->v["nr"]."/"));

   $this->backend->setTitle($this->icon("data_dim"), $path);

   //get category information
   $d->setAction("view");
   $d->setExtra("item");
   $d->setShowFields("name", "text", "moduleLock", "module");

   $d->clearValues();
   $d->setValue($d->getID(), $this->nr);

   if(!$d->get()||!$d->next())
   {
    $this->e->add($d->e);
    $_session->backend();
    return false;
   }

   //get modules lists
   $r="";
   $modules=Array("categories");
   foreach($d->v["module"] as $module)
    if(!in_array($module, $modules))
     $modules[]=$module;

   foreach($modules as $module)
   {
    $tmp=Data::cache($module, "view", "list");

    if(!($field=$tmp->getRightsReference())) //no rights reference
     continue;

    $tmp->clearValues();
    $tmp->setValue($field, $this->nr);

    if(!($list=$this->viewList($tmp, Array("get" => 0))))
     continue;

    //link header/pager to modules page
    if($this->backend->rewrite)
    {
     $list=preg_replace("!href=\"([^\"]*)data/\d+/(.*)\"!", "href=\"\\1modules/".$tmp->module."/\\2\"", $list);
     $list=preg_replace("!action=\"([^\"]*)data/\d+/(.*)\"!", "action=\"\\1modules/".$tmp->module."/\\2\"", $list);
    }
    else
    {
     $list=preg_replace("!href=\"([^\"]*)p\[\]=data&p\[\]=\d+(.*)\"!", "href=\"\\1p[]=modules&p[]=".$tmp->module."&p[]=\\2\"", $list);
     $list=preg_replace("!action=\"([^\"]*)p\[\]=data&p\[\]=\d+(.*)\"!", "action=\"\\1p[]=modules&p[]=".$tmp->module."&p[]=\\2\"", $list);
    }

    if($module=="categories") //show data list for category rather than just category
    {
     if($this->backend->rewrite)
      $list=preg_replace("!href=\"([^\"]*)data/categories/(\d+)/\"!", "href=\"\\1data/\\2/\"", $list);
     else
      $list=preg_replace("!href=\"([^\"]*)p\[\]=data&p\[\]=categories&p\[\]=(\d+)\"!", "href=\"\\1p[]=data&p[]=\\2\"", $list);
    }

    $r.="<h2>".$this->link($tmp->l->t($tmp->title), Array("modules/".$tmp->module."/", Array("catnr" => $this->nr)))."</h2>\n";
    $r.=$list;
   }

   $r="<div class=\"twothird\">\n".$r."</div>\n";

   //show category info
   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("info_dim", "small")." ".$this->language->t("Info")."</h3>\n";
   $r.=$this->viewItem($d, Array("show" => "normal"))."\n";

   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->cmsOptions($d, "add;edit;move;delete;owner;rights", true)."\n";
   $r.="</div>\n";

   $_session->backend();
   return $r;
  }

  function output_search()
  {
   global $_session;

   $_session->frontend();

   $this->backend->setTitle($this->icon("data_dim"),
    $this->link($this->language->t("Data"), Array("data/")),
    $this->language->t("Search results"));

   //search categories
   $d=Data::cache("categories", "search", "full");

   $d->clearValues();
   $d->setRequestValue("search");

   $list=$this->viewList($d, Array("get" => 0));

   if($this->backend->rewrite)
    $list=preg_replace("!href=\"([^\"]*)data/categories/(\d+)/\"!", "href=\"\\1data/\\2/\"", $list);
   else
    $list=preg_replace("!href=\"([^\"]*)p\[\]=data&p\[\]=categories&p\[\]=(\d+)\"!", "href=\"\\1p[]=data&p[]=\\2\"", $list);

   $r="<div class=\"twothird rights\">\n";
   $r.=$list;
   $r.="</div>\n";

   if($d->e->any("fatal"))
   {
    $_session->backend();
    return $r;
   }

   //show category info
   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->cmsOptions($d, "add", true)."\n";
   $r.="</div>\n";

   $_session->backend();
   return $r;
  }

  function output_item()
  {
   global $_session;

   $_session->frontend();

   $d=Data::cache($this->module, "view", "item");
   $d->setValue($d->getID(), $this->nr);

   //set default title
   $this->backend->setTitle($this->icon("data_dim"),
    $this->link($this->language->t("Data"), Array("data/")),
    $this->link($d->l->t($d->title), Array("modules/".$d->module."/")));

   if(!($r=$this->viewItem($d, Array("show" => "normal", "get" => 1))))
   {
    $_session->backend();
    return false;
   }

   //set title after viewItem got data
   $this->backend->setTitle($this->icon("data_dim"),
    $this->link($this->language->t("Data"), Array("data/")),
    $this->link($d->l->t($d->title), Array("modules/".$d->module."/")),
    $this->link(Mutilate::stripS(strip_tags($d->v("title")), 24), $this->url()));

   $r="<div class=\"twothird\">\n".$r."</div>\n";

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("info_dim", "small")." ".$this->language->t("Info")."</h3>\n";
   $r.=$this->viewItem($d, Array("show" => "default"))."\n";

   $r.="<h3>".$this->icon("link_dim", "small")." ".$this->language->t("Reference")."</h3>\n";
   $r.=$this->viewItem($d, Array("show" => "reference"))."\n";

   $vars=false;
   if($d->module=="categories")
    $vars=Array("catnr" => $d->getValue("nr"));
   else if($d->getValue("catnr"))
    $vars=Array("catnr" => $d->getValue("catnr"));

   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->cmsOptions($d, "add;copy;edit;move;owner;rights;delete", true, $vars)."\n";
   $r.="</div>\n";

   $_session->backend();
   return $r;
  }


  /*
   * Support
   */

 }
?>
