<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/export.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/modules.php");
 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."headers.php");
 require_once($_path["include"]."history.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."jas.php");
 require_once($_path["api"]."csv.php");
 require_once($_path["api"]."data.php");
 require_once($_path["api"]."xml.php");

 class Backend_Export extends Backend_Modules
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);
  }

  function main($module=false, $nr=false)
  {
   global $_session, $_config;

   $_session->frontend();

   //check module
   if($module)
   {
    if(!in_array($module, JAS::modules())) //invalid module
     $this->e->add("Module: \"".$module."\" not found", "error");
    else //valid module
    {
     //modify quick categories list url
     if($quick=$this->backend->component("backend/categories/quick")) //categories list found
      $quick->path=false; //current page

     $d=Data::cache($module, "view", "all");
     
     if($d->e->any("fatal"))
      $this->e->add($d->e);
     else
     {
      $id=$d->getID();
      $this->module=$module;

      foreach($d->check("use", "sql") as $field)
       if(!$d->check("use", "show", $field))
        $d->field[$field]["use"].=";show";

      if(request("s")) //export
      {
       if($d->field[$id]["valuetype"]=="request") //individual items selected
        $d->setExtra("item");
       else if(request("search"))
       {
        $d->setAction("search");
        if($d->fulltextIndex) //full text index exists
        {
         $d->setExtra("index");

         //default sort on index desc
         $sort=Array();
         foreach(array_keys($d->fulltextIndex) as $index)
          $sort[].=$index."-";

         $d->sortOrder($sort);
        }
        else //normal search
         $d->setExtra("full");

        $d->setRequestValue("search");
       }
       else if(($catnr=request("catnr"))&&($field=$d->getRightsReference())) //category filter
       {
        $d->setExtra("list");
        $d->setValue($field, $catnr);
       }
       //else //all


       //sort
       if($sort=request("sort"))
        $d->sortOrder(explode(";", $sort));

       //unsorted self reference, use tree
       if($d->extra=="all"&&($self=$d->getSelfReference())&&request("sort")===NULL)
       {
        if($d->field[$self]["valuetype"]=="default")
         $d->setValue($self, 0);

        $d->setShowFields(array_merge($d->check("use", "show"), Array("treedepth")));

        $d->setAction("tree");
        $d->setExtra("down");
       }

       //ensure all entries are exported
       $d->field["show_total"]["min"]=false;
       $d->field["show_total"]["max"]=false;
       $d->setValue("show_total", 0);

       if(!$d->get())
        $this->e->add($d->e);
       else if(!$d->vmax)
        $this->e->add("No items to export for module: \"".$module."\"", "error");
       else
       {
        //export
        $filename=$d->module."-".date("YmdHis");
        if(request("type")=="XML")
        {
         $filename.=".xml";
         $api=new API_XML();
        }
        else if(request("type")=="Data")
        {
         $filename.=".txt";
         $api=new API_Data();
        }
        else
        {
         $filename.=".csv";
         $api=new API_CSV();
        }

        $api->addData($d);
        $api->format();
        if(!($output=$api->output(false)))
         $this->e->add("Export failed for module: \"".$module."\"", "error");
        else
        {
         Headers::fileS($filename, true, strlen($output));
         print $output;
         exit;
        }
       }
      }

      $this->section="list";

      //set inline search options
      if($titles=$d->check("properties", "title"))
       $this->backend->search("data", $this->module, "title", $titles[0]);
     }
    }
   }

   if(!$this->module)//no module set
   {
    $this->backend->search("modules"); //inline search all modules

    //search
    if($search=request("search"))
    {
     foreach(Backend_Modules::modules(true) as $module => $v) //filter modules list
      if($v["enabled"])
       if(stristr($module, $search)||stristr($v["title"], $search))
        $this->modules[$module]=$v;

     if(sizeof($this->modules)==1) //only one module
      Headers::redirectS($this->url("export/".key($this->modules)."/"));
    }
    else
    {
     foreach(Backend_Modules::modules(true) as $module => $v) //filter modules list
      if($v["enabled"])
       $this->modules[$module]=$v;
    }
   }

   $_session->backend();

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session, $_config;

   $_session->frontend();

   $this->backend->setTitle($this->icon("export_dim"),
    $this->link($this->language->t("Export"), Array("export/")));

   $this->tableRow("modules", true);
   $this->tableColumn("modules", $this->language->t("Module"), false, false, Array("class" => "module"));
   $this->tableColumn("modules", $this->language->t("Type"), false, false, Array("class" => "type"));

   //modules list
   $enabled=0;
   foreach($this->modules as $module => $v)
   {
    $this->tableRow("modules");
    $this->tableColumn("modules", $v["title"], $this->url("export/".$module."/"), $v["title"]);

    if($v["type"]=="plugin")
     $this->tableColumn("modules", $v["pluginfo"]["title"], $this->url("plugins/".$v["plugin"]."/"), $v["plugin"]." v".$v["pluginfo"]["version"]." ".$v["pluginfo"]["status"].($v["pluginfo"]["description"]?" - ".$v["pluginfo"]["description"]:""));
    else
     $this->tableColumn("modules", $this->language->t(ucfirst($v["type"])));
   }

   $this->tableRow("modules", false, Array("class" => "total"));

   if(!$this->modules)
    $this->tableColumn("modules", $this->language->t("No modules"), false, false, Array("colspan" => 2));
   else
   {
    $this->tableColumn("modules", $this->language->t("Total"));
    $this->tableColumn("modules", sizeof($this->modules), false, false, Array("class" => "nr"));
   }

   $r="<div class=\"twothird\">\n";
   $r.=$this->table("modules", "export");
   $r.="</div>\n";

   //actions
   $this->actionsMenu();

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="</div>\n";

   $_session->backend();

   return $r;
  }

  function output_list()
  {
   global $_session, $_config;

   $_session->frontend();

   $d=Data::cache($this->module, "view", "all");

   $this->backend->setTitle($this->icon("export_dim"),
    $this->link($this->language->t("Export"), Array("export/")),
    $this->link($d->l->t($d->title), Array("export/".$d->module."/")));

   if(request("search"))
    $d->setAction("search");
   else if(($catnr=request("catnr"))&&($field=$d->getRightsReference())) //category filter
   {
    $d->setExtra("list");
    $d->setValue($field, $catnr);
   }

   $list=$this->viewList($d, Array("checkboxes", "nopagerform", "get" => 0));

   $r="<div class=\"twothird\">\n";
   $r.=$list;
   $r.="</div>\n";

   if($d->e->any("fatal"))
   {
    $_session->backend();
    return $r;
   }

   $r.="<div class=\"third\">\n";

   if(request("search"))
    $r.=$this->form->input("hidden", "search", strip_tags(request("search")));
   else if(request("catnr"))
    $r.=$this->form->input("hidden", "catnr", strip_tags(request("catnr")));

   if(request("sort"))
    $r.=$this->form->input("hidden", "sort", strip_tags(request("sort")));

   //export submit
   $r.="<h3>".$this->icon("export_dim", "small")." ".$this->language->t("Export")."</h3>\n";
   $r.=$this->field($this->language->t("Items"), $this->form->select(false, "type", Array("CSV", "Data", "XML"), request("type"), "export")." ".$this->form->button("s", $this->icon("export", "small")." ".$this->language->t("Export"), false, false, "submit"));

   //actions
   $this->actionsMenu($d);

   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");

   $r.="</div>\n";

   //make full page part of the form
   $r=$this->form($r);

   $_session->backend();

   return $r;
  }

 }
?>
