<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/logs.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["include"]."mutilate.php");

 class Backend_Logs extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->filter=Array();
  }

  function main($filter=false)
  {
   global $_session, $_config;

   $_session->frontend();

   if(preg_match("/^\d+$/", $filter))
    $this->filter["usernr"]=$filter;
   else if(preg_match(REGEXP_DATA_IP, $filter))
    $this->filter["ip"]=$filter;
   else if(in_array($filter, $this->e->errorType))
    $this->filter["level"]=$filter;

   $d=Data::cache("log");

   if(!$_config->userRoot())
   {
    $this->e->add("Logs only permitted for root user", "warning");
    $this->section="error";
    $_session->backend();
    return false;
   }
   else if($d->e->any("fatal"))
   {
    $this->e->add($d->e);
    $this->section="error";
    $_session->backend();
    return false;
   }

   //set inline search options
   if($titles=$d->check("properties", "title"))
    $this->backend->search("data", "log", "title", $titles[0]);

   //search redirect
   if(request("search"))
   {
    $d->setAction("search");
    if($d->fulltextIndex) //full text index exists
     $d->setExtra("index");
    else //normal search
     $d->setExtra("full");

    $d->setShowFields();
    $d->setValue("show_total", 2);
    $d->setRequestValue("search");

    if($d->get()&&$d->vmax==1) //only 1 entry
     Headers::redirectS($this->url("data/log/".$d->getValue($d->getID())."/"));
   }

   $_session->backend();

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session, $_config;

   $_session->frontend();

   $d=Data::cache("log", "view", "all");
   $d->addMutilation("title", "strip", 60);

   $this->backend->setTitle($this->icon("log_dim"),
    $this->link($this->language->t("Logs"), Array("logs/")));

   if(request("search"))
    $d->setAction("search");

   foreach($this->filter as $field => $value)
    $d->setValue($field, $value);

   $r="<div class=\"twothird\">\n";
   $r.=$this->viewList($d, Array("get" => 0));
   $r.="</div>\n";

   //level statistics
   $d->setAction("view");
   $d->setExtra("all");

   $d->clearValues();
   $d->setShowFields("level", "title");
   $d->setValue("aggregation_field", "level");
   $d->setValue("aggregation_type", "count");

   if($d->get())
   {
    $d->resort(Array("title-"));

    while($d->next())
     $this->statsEntry("level", ($d->v["level"]?$this->link(ucfirst($d->v["level"]), Array("logs/".$d->v["level"]."/"), $this->language->t("Show ".$d->v["level"]." logs")):$this->language->t("Unknown")), $d->v["title"], Array("class" => $d->v["level"]));
   }

   //user statistics
   $d->clearValues();
   $d->setShowFields("usernr", "title");
   $d->setValue("aggregation_field", "usernr");
   $d->setValue("aggregation_type", "count");
   $d->sortOrder("title-");

   $du=$d->getReferenceData("usernr");
   $userTitle=$du->check("properties", "title")[0];
   $userID=$du->getID();
   $du->setShowFields($userTitle);

   $c=0;
   if($d->get(1))
   {
    $d->resort(Array("title-"));

    while($d->next()&&$c++<10)
     $this->statsEntry("users", ($d->v["usernr"]?$this->link(Mutilate::stripS($d->v["usernr"][$userTitle], 11), Array("logs/".$d->v["usernr"][$userID]."/"), $this->language->t("Show ".$d->v["usernr"][$userTitle]." logs")):$this->language->t("Unknown")), $d->v["title"]);
   }

   //level statistics
   $d->clearValues();
   $d->setShowFields("ip", "title");
   $d->setValue("aggregation_field", "ip");
   $d->setValue("aggregation_type", "count");

   $c=0;
   if($d->get())
   {
    $d->resort(Array("title-"));

    while($d->next()&&$c++<10)
     $this->statsEntry("ip", ($d->v["ip"]?$this->link(preg_replace("/^(([a-z0-9]+:){3}).*$/i", "\\1", $d->v["ip"]), Array("logs/".$d->v["ip"]."/"), $this->language->t("Show ".$d->v["ip"]." logs")):$this->language->t("Unknown")), $d->v["title"]);
   }

   //show statistics
   if($c)
   {
    $r.="<div class=\"third logs\">\n";
    $r.="<h3>".$this->icon("log_dim", "small")." ".$this->language->t("Level statistics")."</h3>\n";
    $r.=$this->stats("level");
    $r.="<h3>".$this->icon("user_dim", "small")." ".$this->language->t("User statistics")."</h3>\n";
    $r.=$this->stats("users");
    $r.="<h3>".$this->icon("online_dim", "small")." ".$this->language->t("IP statistics")."</h3>\n";
    $r.=$this->stats("ip");
    $r.="</div>\n";
   }

   $_session->backend();

   return $r;
  }

  function output_error()
  {
   return "";
  }
 }
?>
