<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/projects.directories.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["include"]."cms.php");
 require_once($_path["include"]."jas.php");
 require_once($_path["include"]."fs.php");

 class Backend_Projects_Directories extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->fs=new FS();

   $this->data=false;
   $this->jasRootUser=false;
  }


  function data()
  {
   global $_session, $_path;

   $this->data=Data::cache("empty", "add", "item", Array("skipCheckTables" => true));
   $this->data->addField("backendProject", "regexp", "all", "", 2, 25, $_session->get("backendProject"), "Project name");
   $this->data->regexps("backendProject", REGEXP_DATA_PROJECT);

   $this->data->addField("exists", "list", "", "", false, 1000000, false, "Existing directories");
   $this->data->addField("required", "list", "", "", false, 1000000, false, "Required directories");
   $this->data->listOptions("required", "root", "include");

   $this->data->addField("directories", "multilist", "all", "", false, 1000000, false, "Directories");

   $this->data->setRequestValues();

   $project=$this->data->getValue("backendProject");

   $exists=Array();
   $directories=Array();
   foreach($_path["project"] as $k => $dir)
   {
    if($project)
     $dir=str_replace("jas", $project, $dir);

    $directories[$k]=$dir;

    if($this->fs->exists($dir))
     $exists[]=$k;
   }

   $this->data->listOptions("directories", $directories);
   $this->data->listOptions("exists", $exists);

   if($this->data->getValue("directories"))
    $this->data->setValue("directories", array_unique(array_merge($this->data->listOptions["required"], $this->data->listOptions["exists"], $this->data->getValue("directories"))));
   else
    $this->data->setValue("directories", array_unique(array_merge($this->data->listOptions["required"], $this->data->listOptions["exists"])));
  }


  function status(&$text)
  {
   $this->data();

   if($this->data->check("valuetype", "request", "backendProject")&&$this->data->getValue("backendProject")&&!$this->data->checkField("backendProject"))
   {
    $this->e->add($this->data->e);

    $text="Project name invalid";
    return "Failure";
   }

   $missing=Array();
   foreach($this->data->listOptions["required"] as $dir)
    if(!in_array($dir, $this->data->listOptions["exists"]))
     $missing[]=$dir;

   if($missing)
   {
    if(in_array("root", $missing)) //project directory doesn't exist
     $text="Project doesn't exist";
    else
    {
     $text="Required project directories missing: ";
     foreach($missing as $dir)
      $text.=$dir." (".$this->data->listOptions["directories"][$dir]."), ";

     $text=substr($text, 0, -2);
    }

    return "Failure";
   }

   if(!$this->fs->exists($this->data->listOptions["directories"]["include"].".htaccess"))
   {
    $text="Required include .htaccess missing";
    return "Failure";
   }

   $text="";
   return "Done";
  }

  function main($submit=true)
  {
   global $_session, $_config, $_path;

   $this->jasRootUser=$_config->userRoot(); //get jas project user root status

   if(!$this->jasRootUser)
   {
    $this->e->add("Project directories can only be setup by the JAS root user", "error");
    return false;
   }

   $this->data();

   if($this->data->check("valuetype", "request", "backendProject")&&!$this->data->checkField("backendProject")) //new project, validate
   {
    $this->section="project"; //show project name submission
    $this->data->field["directories"]["use"]=""; //don't submit directories
   }
   else
   {
    if($this->data->check("valuetype", "request", "backendProject")) //new project
     $this->data->field["backendProject"]["properties"].=";hidden";
    else //existing project
     $this->data->field["backendProject"]["use"]=""; //don't submit project

    if(request("s")&&$submit)
    {
     $success=true;
     foreach($this->data->getValue("directories") as $k)
      if(!$this->fs->exists($this->data->listOptions["directories"][$k]))
       if(!$this->fs->mkdir($this->data->listOptions["directories"][$k], true))
        $success=false;

     if(!$this->fs->exists($this->data->listOptions["directories"]["include"].".htaccess"))
      if(!$this->fs->copy($_path["include"].".htaccess", $this->data->listOptions["directories"]["include"]))
       $success=false;

     if($success)
     {
      if($this->data->check("valuetype", "request", "backendProject")) //new project
      {
       $this->e->add("Project: \"".$this->data->getValue("backendProject")."\" successfully created", "note");
       $_session->setBackendProject($this->data->getValue("backendProject")); //activate in backend

       $language=$_config->get("language");

       $_session->frontend();
       $_config->set("title", ucfirst($this->data->getValue("backendProject"))); //set fake title
       $_config->set("language", $language); //copy language from backend
       $_config->set("failed", false); //ensure configuration isn't loaded
       $_config->set("rootnr", 1); //make root
       $_config->set("usernr", 1); //make root
       $_session->backend();
      }
      else
       $this->e->add("Directories for project: \"".$this->data->getValue("backendProject")."\" successfully created", "note");

      return "next";
     }
     else //failed
      $this->e->add($this->fs->e);
    }
   }

   $this->e->add($this->data->e);
   return true;
  }


  /*
   * Output
   */

  function output()
  {
   if(!$this->jasRootUser)
    return "&nbsp; ";

   $cms=new CMS();
   $cms->l->read("jas");
   $cms->setDataForm($this->data, $this->form);

   if($this->section=="project")
   {
    $form=$cms->field("backendProject");
    $form.=$this->field($this->language->t("Project"), $this->form->button("confirm", $this->icon("add", "small")." ".$this->language->t("Create"), false, false, "submit"));
   }
   else //show directories
   {
    $directories=Array();
    foreach($this->data->listOptions["directories"] as $k => $dir)
     $directories[$k]=(in_array($k, $this->data->listOptions["exists"])?"<i>".$dir."</i>":(in_array($k, $this->data->listOptions["required"])?"<b>".$dir."</b>":$dir));

    $this->data->listOptions("directories", $directories);

    $form="";
    foreach($this->data->check("use", "form") as $field)
     $form.=$cms->field($field);

    $form.=$this->confirm(Array("history", (!$this->jasRootUser?"disabled":false)));
   }

   $r=$this->form($form, $this->url("projects/setup/directories/"));

   return $r;
  }

 }
?>
