<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/projects.modules.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["project"]["include"]."backend/modules.php");
 require_once($_path["include"]."cms.php");
 require_once($_path["include"]."jas.php");

 class Backend_Projects_Modules extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->data=false;
   $this->jasRootUser=false;
  }


  function data()
  {
   global $_session;

   $this->data=Data::cache("empty", "add", "item", Array("skipCheckTables" => true));

   $this->data->addField("exists", "list", "", "", false, 1000000, false, "Existing modules");
   $this->data->addField("required", "list", "", "", false, 1000000, false, "Required modules");
   $this->data->listOptions("required", "config", "userRights", "groupRights", "users", "categories");

   $this->data->addField("modules", "multilist", "all", "", false, 1000000, false, "Modules");

   $this->data->setRequestValues();

   $project=$this->data->getValue("backendProject");

   $_session->frontend();

   $exists=Array();
   $modules=Array();
   foreach(Backend_Modules::modules() as $module => $v)
   {
    $modules[$module]=$v["title"];

    if($v["enabled"])
     $exists[]=$module;
   }

   $this->data->listOptions("modules", $modules);
   $this->data->listOptions("exists", $exists);

   if($this->data->getValue("modules"))
    $this->data->setValue("modules", array_unique(array_merge($this->data->listOptions["required"], $this->data->listOptions["exists"], $this->data->getValue("modules"))));
   else
    $this->data->setValue("modules", array_unique(array_merge($this->data->listOptions["required"], $this->data->listOptions["exists"])));

   $_session->backend();
  }


  function status(&$text)
  {
   $this->data();

   $missing=Array();
   foreach($this->data->listOptions["required"] as $module)
    if(!in_array($module, $this->data->listOptions["exists"]))
     $missing[]=$module;

   if($missing)
   {
    $text="Required project modules missing: ";
    foreach($missing as $module)
     $text.=$module." (".$this->data->listOptions["modules"][$module]."), ";

    $text=substr($text, 0, -2);

    return "Failure";
   }

   $text="";
   return "Done";
  }

  function main($submit=true)
  {
   global $_session, $_config;

   $this->jasRootUser=$_config->userRoot(); //get jas project user root status
   if(!$this->jasRootUser)
   {
    $this->e->add("Project modules can only be setup by the JAS root user", "error");
    return false;
   }

   $this->data();

   $_session->frontend();

   if(request("s")&&$submit)
   {
    $_session->set("tableExists.".$_session->get("project").".".$this->data->db); //clear tableExists cache

    $success=true;
    foreach(Backend_Modules::modules() as $module => $v)
    {
     if(in_array($module, $this->data->getValue("modules")))
     {
      $d=Data::cache($module, "database", "create", Array("skipCheckTables" => true, "skipSetRequest" => true));

      if($d->tableExists())
       $d->setExtra("alter");

      $_config->su(true);
      if(!$d->submit())
      {
       $_config->su(false);
       $this->e->add($d->e);
       $success=false;
      }
      else //create *:* references
      {
       $_config->su(false);
       foreach(array_keys($d->reference) as $ref)
       {
        if($d->reference[$ref]["type"]=="*:*")
        {
         $dr=$d->getReferenceData($ref);

         $dr->setAction("database");

         if(!$dr->tableExists())
          $dr->setExtra("create");
         else
          $dr->setExtra("alter");

         $_config->su(true);
         if(!$dr->submit())
         {
          $_config->su(false);
          $this->e->add($dr->e);
          $success=false;
         }
         else
          $_config->su(false);

         unset($dr);
        }
       }
      }
     }
    }

    if($success)
    {
     $this->e->add("Modules for project: \"".$_session->get("project")."\" successfully created", "note");
     $_session->backend();
     return "next";
    }
    else
    {
     $_session->backend();
     return false;
    }
   }

   $_session->backend();
   return true;
  }


  /*
   * Output
   */

  function output()
  {
   if(!$this->jasRootUser)
    return "&nbsp; ";

   $cms=new CMS();
   $cms->l->read("jas");
   $cms->setDataForm($this->data, $this->form);

   $modules=Array();
   foreach($this->data->listOptions["modules"] as $k => $dir)
    $modules[$k]=(in_array($k, $this->data->listOptions["exists"])?"<i>".$dir."</i>":(in_array($k, $this->data->listOptions["required"])?"<b>".$dir."</b>":$dir));

   $this->data->listOptions("modules", $modules);

   $form="";
   foreach($this->data->check("use", "form") as $field)
    $form.=$cms->field($field);

   $form.=$this->confirm(Array("history", (!$this->jasRootUser?"disabled":false)));

   $r=$this->form($form, $this->url("projects/setup/modules/"));

   return $r;
  }

 }
?>
