<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/users.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["project"]["include"]."backend/modules.php");
 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."headers.php");
 require_once($_path["include"]."history.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."jas.php");

 class Backend_Users extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->nr=false;
   $this->module=false;
  }

  function main($nr=false)
  {
   global $_session, $_config;

   $_session->frontend();

   $this->module=$_config->get("usermodule");

   //modify quick categories list url
   if($quick=$this->backend->component("backend/categories/quick")) //categories list found
    $quick->path=false; //current page

   $d=Data::cache($this->module, "view", "item");

   if($d->e->any("fatal"))
    $this->e->add($d->e);
   else
   {
    $id=$d->getID();

    if(request("confirm"))
    {
     if($d->field[$id]["valuetype"]!="request")
      $this->e->add("No ".$d->title." items selected to change the users", "warning");
     else
      $this->e->add("Are you sure you wish to change the users for ".$d->vmax." ".$d->title." item(s)?", "warning");
    }
    else if(request("s")&&$d->field[$id]["valuetype"]=="request") //values submitted
    {
     //set page as CMS in history
     $hs=new History();
     if(!$hs->cms())
      $this->e->add($hs->e);

     $d->submit();
     $this->e->add($d->e);
    }

    if($nr)
    {
     $this->nr=(int)$nr;
     $this->section="item";

     //set inline search options
     $d=Data::cache("categories", "view", "item");
     if($titles=$d->check("properties", "title"))
      $this->backend->search("data", $d->module, "title", $titles[0]);
    }
    else
    {
     $this->section="list";

     //set inline search options
     if($titles=$d->check("properties", "title"))
      $this->backend->search("data", $this->module, "title", $titles[0]);

     //search redirect
     if(request("search"))
     {
      $d->setAction("search");
      if($d->fulltextIndex) //full text index exists
       $d->setExtra("index");
      else //normal search
       $d->setExtra("full");

      $d->setShowFields();
      $d->setValue("show_total", 2);
      $d->setRequestValue("search");

      if($d->get()&&$d->vmax==1) //only 1 entry
       Headers::redirectS($this->url("users/".$d->getValue($d->getID())."/"));
     }
    }
   }

   $_session->backend();

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session, $_config;

   $_session->frontend();

   $d=Data::cache($this->module, "view", "all");

   $fields=Array();
   if($field=$d->getRightsReference())
    $fields[]=$field;
   if(isset($d->field["usersAdditionalGroups"]))
    $fields[]="usersAdditionalGroups";
   if(isset($d->field["backend"]))
    $fields[]="backend";

   $this->backend->setTitle($this->icon("user_dim"),
    $this->link($d->l->t($d->title), Array("users/")));

   if(request("search"))
    $d->setAction("search");
   else if(($catnr=request("catnr"))&&($field=$d->getRightsReference())) //category filter
    $d->setValue($field, $catnr);

   $list=$this->viewList($d, Array("get" => 1, "fields" => $fields, "class" => "userrights"));

   if($this->backend->rewrite)
   {
    $list=preg_replace("!href=\"([^\"]*)data/".$d->module."/(\d+)/\"!", "href=\"\\1users/\\2/\"", $list);
    $list=preg_replace("!href=\"([^\"]*)data/categories/(\d+)/\"!", "href=\"\\1rights/\\2/\"", $list);
   }
   else
   {
    $list=preg_replace("!href=\"([^\"]*)p\[\]=data&p\[\]=".$d->module."&p\[\]=(\d+)\"!", "href=\"\\1p[]=users&p[]=\\2\"", $list);
    $list=preg_replace("!href=\"([^\"]*)p\[\]=data&p\[\]=categories&p\[\]=(\d+)\"!", "href=\"\\1p[]=rights&p[]=\\2\"", $list);
   }

   $r="<div class=\"twothird\">\n";
   $r.=$list;
   $r.="</div>\n";

   if($d->e->any("fatal"))
   {
    $_session->backend();
    return $r;
   }

   //generate module owners sub menu
   $modules=Array();
   foreach(Backend_Modules::modules(true) as $module => $v)
    if($v["enabled"]&&$v["owner"])
     $this->menuItem("owners", $v["title"], $this->url("owner/".$module."/"), $this->language->t("Change ".$v["title"]." owners"), "module");

   //actions
   $this->menuItem("actions", $this->language->t("Owners"), $this->url("owner/"), $this->language->t("Change owners"), "owner", Array("menu" => "owners", "class" => "popup"));
   $this->menuItem("actions", $this->language->t("Rights"), $this->url("rights/"), $this->language->t("Change rights"), "rights");

   //get groups (user categories)
   $dc=Data::cache("categories", "view", "all");
   $dc->setShowFields("name");
   $dc->sortOrder("name");

   $dc->clearValues();
   $dc->setValue("module", "users");
   $dc->setOperator("module", "like");

   $dc->get();

   $groups=Array();
   while($dc->next())
   {
    $groups[$dc->v["nr"]]=Array(
     "title" => $dc->v["name"],
     "value" => 0,
    );
   }

   //group statistics
   $d=Data::cache($this->module, "view", "all");
   $catnr=$d->getRightsReference();
   $title=$d->check("properties", "title")[0];

   $d->clearValues();
   $d->setShowFields($catnr, $title);
   $d->setValue("aggregation_field", $catnr);
   $d->setValue("aggregation_type", "count");
   $d->sortOrder($catnr);

   $dg=$d->getReferenceData($catnr);
   $groupTitle=$dg->check("properties", "title")[0];
   $groupID=$dg->getID();
   $dg->setShowFields($groupTitle);

   if($d->get(1))
   {
    while($d->next())
    {
     $nr=($d->v[$catnr]?$d->v[$catnr][$groupID]:$d->getValue($catnr, $d->vc-1));

     if(isset($groups[$nr]))
      $groups[$nr]["value"]+=$d->v[$title];
     else
      $groups[$nr]=Array(
       "title" => ($d->v[$catnr]?$d->v[$catnr][$groupTitle]:$this->language->t("Unknown")),
       "value" => $d->v[$title],
      );
    }
   }

   //determine additional group statistics
   if($da=$d->getReferenceData("usersAdditionalGroups"))
   {
    $catnr="categories";
    $title="users";

    $da->setAction("view");
    $da->setExtra("all");

    $da->clearValues();
    $da->setShowFields($catnr, $title);
    $da->setValue("aggregation_field", $catnr);
    $da->setValue("aggregation_type", "count");
    $da->sortOrder($catnr);

    $dg=$da->getReferenceData($catnr);
    $groupTitle=$dg->check("properties", "title")[0];
    $groupID=$dg->getID();
    $dg->setShowFields($groupTitle);

    if($da->get(1))
    {
     while($da->next())
     {
      $nr=($da->v[$catnr]?$da->v[$catnr][$groupID]:$da->getValue($catnr, $da->vc-1));

      if(isset($groups[$nr]))
       $groups[$nr]["value"]+=$da->v[$title];
      else
       $groups[$nr]=Array(
        "title" => ($da->v[$catnr]?$da->v[$catnr][$groupTitle]:$this->language->t("Unknown")),
        "value" => $da->v[$title],
       );
     }
    }
   }

   foreach($groups as $nr => $group)
    $this->statsEntry("groups", $this->link($group["title"], Array("users/", Array("catnr" => $nr)), $this->language->t("Show users in group: ".$group["title"])), $group["value"]);

   //show actions+statistics
   $r.="<div class=\"third users\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="<h3>".$this->icon("group_dim", "small")." ".$this->language->t("Group statistics")."</h3>\n";
   $r.=$this->stats("groups");
   $r.="</div>\n";

   $_session->backend();

   return $r;
  }

  function output_item()
  {
   global $_session;

   $_session->frontend();

   //set default title
   $this->backend->setTitle($this->icon("user_dim"),
    $this->link($this->language->t("Users"), Array("users/")));

   $d=Data::cache($this->module, "view", "item");
   $d->setValue($d->getID(), $this->nr);

   $r="<div class=\"twothird rights\">\n";
   $r.=$this->viewItem($d, Array("show" => "normal", "get" => 1))."<br/>";

   //set title after viewItem got data
   $this->backend->setTitle($this->icon("user_dim"),
    $this->link($d->l->t($d->title), Array("users/")),
    $this->link(Mutilate::stripS(strip_tags($d->v("title")), 24), $this->url()));

   //categories
   $dc=Data::cache("categories", "view", "all");

   if(request("search"))
    $dc->setAction("search");
   else if(($catnr=request("catnr"))&&($field=$dc->getRightsReference())) //category filter
    $dc->setValue($field, $catnr);

   $list=$this->viewList($dc, Array("get" => 1, "rights", "user" => $this->nr));

   if($this->backend->rewrite)
    $list=preg_replace("!href=\"([^\"]*)data/".$dc->module."/(\d+)/\"!", "href=\"\\1rights/\\2/\"", $list);
   else
    $list=preg_replace("!href=\"([^\"]*)p\[\]=data&p\[\]=".$dc->module."&p\[\]=(\d+)\"!", "href=\"\\1p[]=rights&p[]=\\2\"", $list);

   $r.=$list;
   $r.="</div>\n";


   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("info_dim", "small")." ".$this->language->t("Info")."</h3>\n";
   $r.=$this->viewItem($d, Array("show" => "default"))."\n";

   $r.="<h3>".$this->icon("link_dim", "small")." ".$this->language->t("Reference")."</h3>\n";
   $r.=$this->viewItem($d, Array("show" => "reference"))."\n";

   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->cmsOptions($d, "add;edit;move;rights;delete", true)."\n";
   $r.="</div>\n";

   $_session->backend();

   return $r;
  }


 }
?>
