

 /*****************************************************************************
 * Filename: project/jas/script/search.js
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


if(typeof(onload)!='function')
 alert("Script not loaded: functions.js");
else if(typeof(setGetElementFunctions)!="function")
 alert("Script not loaded: getelements.js");
else if(typeof(HTTP)!="function")
 alert("Script not loaded: http.js");
else
 onload("initSearch();");

function initSearch()
{
 //get search block
 var search;
 if(!(search=document.getElementById("search")))
  return false;

 //get search field and hidden options
 var options=Array();
 var inputs=search.getElementsByTagName("input");
 for(var c=0;c<inputs.length;c++)
 {
  if(inputs[c].type.toLowerCase()=="text"&&inputs[c].name.toLowerCase()=="search") //search field
   search.field=inputs[c];
  else if(inputs[c].type.toLowerCase()=="hidden"&&inputs[c].name.toLowerCase()=="_options") //search options
  {
   //parse ; separated list of key=value options
   var tmp=inputs[c].value.split(";");
   for(var x=0;x<tmp.length;x++)
   {
    var tmp2=tmp[x].split("=");
    options[tmp2[0]]=tmp2[1];
   }
  }
 }

 if(search.field&&typeof(options["request"])!="undefined") //field+options found
 {
  //set options and events
  search.options=options;
  search.field.onkeyup=searchRequest;
  search.field.onfocus=searchShow;
  search.field.onblur=function(){setTimeout("searchHide();", 100);};

  //create results list
  var results=document.createElement("ul");
  results.className="results";
  search.appendChild(results);
  search.results=results;
 }
}


function searchShow()
{
 var search=document.getElementById("search");

 search.results.style.display="block";
}
function searchHide()
{
 var search=document.getElementById("search");

 if(search.field!=document.activeElement) //only hide if focus hasn't returned
  search.results.style.display="none";
}


function searchRequest(e)
{
 var search=document.getElementById("search");
 var options=search.results.getElementsByTagName("li");

 //check if up/down error has been pressed
 if(e.keyCode==38) //up
 {
  select=options.length-1;
  for(var c=0;c<options.length;c++)
  {
   if(options[c].className=="selected")
   {
    if(c-1>=0)
     select=c-1;

    break;
   }
  }

  setSearchValue(options[select]);
  return true;
 }
 else if(e.keyCode==40) //down
 {
  select=0;
  for(var c=0;c<options.length;c++)
  {
   if(options[c].className=="selected")
   {
    if(c+1<options.length)
     select=c+1;

    break;
   }
  }

  setSearchValue(options[select]);
  return true;
 }

 //handle request
 var stop=false;
 var http=new HTTP();

 switch(search.options["request"])
 {
  case "projects":
  {
   if(search.field.value.length<1)
   {
    setSearchResults(Array());
    break;
   }

   //get all projects (just once)
   if(!search.projects)
   {
    searchLoading();
    search.projects=Array();
    if(!http.projects()&&http.error)
    {
     stop=true;
     break;
    }

    while(v=http.next())
     search.projects.push(v["name"]);
   }

   //search regexp, with prefix/suffix
   var regexp=search.field.value.toLowerCase()
   if(typeof(search.options["prefix"])!="undefined")
    regexp="^"+search.options["prefix"]+".*"+regexp;
   if(typeof(search.options["suffix"])!="undefined")
    regexp=regexp+".*"+search.options["suffix"]+"$";

   regexp=new RegExp(regexp, "i");

   //handle results
   var results=Array();
   for(var c=0;c<search.projects.length;c++)
    if(search.projects[c].match(regexp))
     results.push(search.projects[c]);

   setSearchResults(results);

   break;
  }
  case "modules":
  {
   if(search.field.value.length<1)
   {
    setSearchResults(Array());
    break;
   }

   //get all modules (just once)
   if(!search.modules)
   {
    searchLoading();
    search.modules=Array();
    if(!http.modules()&&http.error)
    {
     stop=true;
     break;
    }

    while(v=http.next())
     search.modules.push(v["name"]);
   }

   //search regexp, with prefix/suffix
   var regexp=search.field.value.toLowerCase()
   if(typeof(search.options["prefix"])!="undefined")
    regexp="^"+search.options["prefix"]+".*"+regexp;
   if(typeof(search.options["suffix"])!="undefined")
    regexp=regexp+".*"+search.options["suffix"]+"$";

   regexp=new RegExp(regexp, "i");

   //handle results
   var results=Array();
   for(var c=0;c<search.modules.length;c++)
    if(search.modules[c].match(regexp))
     results.push(search.modules[c]);

   setSearchResults(results);

   break;
  }
  case "plugins":
  {
   if(search.field.value.length<1)
   {
    setSearchResults(Array());
    break;
   }

   //get all modules (just once)
   if(!search.plugins)
   {
    searchLoading();
    search.plugins=Array();
    if(!http.modules()&&http.error)
    {
     stop=true;
     break;
    }

    var m;
    while(v=http.next())
     if(m=v["name"].match(/^(.+)_[a-z]+$/i)) //get plugin name from module: plugin_module
      if(!search.plugins.in_array(m[1])) //plugin not yet in results
       search.plugins.push(m[1]);
   }

   //search regexp, with prefix/suffix
   var regexp=search.field.value.toLowerCase()
   if(typeof(search.options["prefix"])!="undefined")
    regexp="^"+search.options["prefix"]+".*"+regexp;
   if(typeof(search.options["suffix"])!="undefined")
    regexp=regexp+".*"+search.options["suffix"]+"$";

   regexp=new RegExp(regexp, "i");

   //handle results
   var results=Array();
   for(var c=0;c<search.plugins.length;c++)
    if(search.plugins[c].match(regexp))
     results.push(search.plugins[c]);

   setSearchResults(results);

   break;
  }
  case "data":
  {
   //validate input and options
   if(search.field.value.length<2)
   {
    setSearchResults(Array());
    break;
   }

   if(!http.setValue("search", search.field.value))
    break;

   if(!http.setValue("show_total", 10))
    break;

   if(!http.setValue("aggregation_field", search.options["field"])) //show unique values only
    break;

   if(!http.show(Array(search.options["field"])))
    break;

   //get data
   searchLoading();
   if(!http.data(search.options["module"], "search", search.options["extra"])&&http.error)
   {
    stop=true;
    break;
   }

   //handle results
   var results=Array();
   while(v=http.next())
    results.push(v[search.options["field"]]);

   setSearchResults(results);

   break;
  }
  default:
  {
   stop=true;
   break;
  }
 }

 if(stop) //failed to get data
 {
  //disabled search triggers
  var search=document.getElementById("search");

  search.field.onkeyup="";
  search.field.onfocus="";
  search.field.onblur="";

  setSearchResults(Array());
 }

 return true;
}

function setSearchResults(results)
{
 var search=document.getElementById("search");

 //clear results list
 while(search.results.firstChild)
  search.results.removeChild(search.results.firstChild);

 //add new options to results list
 var option;
 for(var c=0;c<results.length;c++)
 {
  option=document.createElement("li");
  option.innerHTML=results[c];
  option.onclick=setSearchValue;

  search.results.appendChild(option);
 }
}

function searchLoading()
{
 var search=document.getElementById("search");

 //clear results list
 while(search.results.firstChild)
  search.results.removeChild(search.results.firstChild);

 //add new options to results list
 var option=document.createElement("li");
 option.innerHTML="Loading...";
 option.className="loading";

 search.results.appendChild(option);
 search.results.style.display="block";
}


function setSearchValue(option)
{
 var search=document.getElementById("search");

 if(!option.innerHTML) //onclick
 {
  search.field.value=this.innerHTML;
  search.field.form.submit();
  return true;
 }

 //clear selected flag of other options
 var options=option.parentNode.getElementsByTagName("li");
 for(var c=0;c<options.length;c++)
 {
  if(options[c].className=="selected")
  {
   options[c].className="";
   break;
  }
 }

 //set option to search field and mark options as selected
 search.field.value=option.innerHTML;
 search.field.focus();
 option.className="selected";

 return true;
}
