<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/format_xml.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2015 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/format.php");
 require_once($_path["format"]."xml.php");

 class UT_Format_XML extends UT_Format
 {
  var $testXML="<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>
  <test ut=\"xml\">
   <!-- comment -->
   <unit test=\"asd\">
   #marker#
   <!shortcomment>
 </test>";

  var $testTag=Array( //tag validation array, based on testXML
   Array(
    "name" => "?xml",
    "type" => "short",
    "outer" => Array(
     "start" => 0,
     "length" => 43,
    ),
    "attributes" => Array(
     "version" => "1.0",
     "encoding" => "ISO-8859-1",
    ),
    "parent" => false,
   ),

   Array(
    "name" => "test",
    "type" => "open",
    "inner" => Array(
     "start" => 61,
     "length" => 74,
    ),
    "outer" => Array(
     "start" => 46,
     "length" => 96,
    ),
    "attributes" => Array(
     "ut" => "xml",
    ),
    "parent" => false,
    "close" => 6,
   ),

   Array(
    "name" => "comment",
    "type" => "!--short",
    "outer" => Array(
     "start" => 65,
     "length" => 16,
    ),
    "attributes" => Array(
    ),
    "parent" => 1,
   ),

   Array(
    "name" => "unit",
    "type" => "short",
    "outer" => Array(
     "start" => 85,
     "length" => 17,
    ),
    "attributes" => Array(
     "test" => "asd",
    ),
    "parent" => 1,
   ),

   Array(
    "name" => "marker",
    "type" => "marker",
    "outer" => Array(
     "start" => 106,
     "length" => 8,
    ),
    "attributes" => Array(
    ),
    "parent" => 1,
   ),

   Array(
    "name" => "shortcomment",
    "type" => "!short",
    "outer" => Array(
     "start" => 118,
     "length" => 15,
    ),
    "attributes" => Array(
    ),
    "parent" => 1,
   ),

   Array(
    "name" => "test",
    "type" => "close",
    "open" => 1,
   ),
  );


  function Constructor()
  {
   $object=new Format_XML();

   if(!is_object($object)||get_class($object)!="Format_XML")
   {
    $this->e->add("Constructor doesn't return a Format_XML object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if($object->onlyCommentsAndMarkers)
   {
    $this->e->add("\"OnlyCommentsAndMarkers\" isn't false", "error");
    return UT_FAILED;
   }
   if(!$object->exactPath)
   {
    $this->e->add("\"ExactPath\" isn't true", "error");
    return UT_FAILED;
   }


   $object=new Format_XML(true, false, true);

   if(!$object->onlyCommentsAndMarkers)
   {
    $this->e->add("\"OnlyCommentsAndMarkers\" isn't true", "error");
    return UT_FAILED;
   }
   if($object->exactPath)
   {
    $this->e->add("\"ExactPath\" isn't false", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Format_XML();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Format_XML object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->contentType=-1;
   if($orig->contentType==$copy->contentType)
   {
    $this->e->add("Format_XML object copy also modified original \"contentType\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Format_XML object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_Format::Copy_Constructor(); //run parent Copy_Constructor tests
  }


  function _validateTags($tag, $expected=false)
  {
   $validTypes=Array(
    "open", "!open", "!--open",
    "close", "!close", "!--close",
    "short", "!short", "!--short",
    "marker",
   );

   $l=sizeof($tag);
   for($c=0;$c<$l;$c++)
   {
    //name validation
    if(!isset($tag[$c]["name"]))
    {
     $this->e->add("Tag validation failed: no name for tag: \"".$c."\"", "error");
     return UT_FAILED;
    }

    //type validation
    if(!isset($tag[$c]["type"]))
    {
     $this->e->add("Tag validation failed: no type for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if(!in_array($tag[$c]["type"], $validTypes))
    {
     $this->e->add("Tag validation failed: invalid type \"".$tag[$c]["type"]."\" for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }

    if(strstr($tag[$c]["type"], "close"))
    {
     if(!isset($tag[$c]["open"]))
     {
      $this->e->add("Tag validation failed: no open reference for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }

     continue; //no more validation for close tag
    }

    //outer validation
    if(!isset($tag[$c]["outer"]))
    {
     $this->e->add("Tag validation failed: no outer array for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if(!isset($tag[$c]["outer"]["start"]))
    {
     $this->e->add("Tag validation failed: no outer start for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if(!isset($tag[$c]["outer"]["length"]))
    {
     $this->e->add("Tag validation failed: no outer end for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }

    //parent validations
    if(!isset($tag[$c]["parent"]))
    {
     $this->e->add("Tag validation failed: no parent for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if($tag[$c]["parent"]<0)
    {
     $this->e->add("Tag validation failed: invalid parent \"".$tag[$c]["parent"]."\" for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if($tag[$c]["parent"]>=$l)
    {
     $this->e->add("Tag validation failed: parent \"".$tag[$c]["parent"]."\" doesn't exist for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if($tag[$c]["parent"]>$c)
    {
     $this->e->add("Tag validation failed: parent \"".$tag[$c]["parent"]."\" can't be after tag for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if($tag[$c]["parent"]===$c)
    {
     $this->e->add("Tag validation failed: parent \"".$tag[$c]["parent"]."\" can't be the same tag for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }

    //attributes validations
    if(!isset($tag[$c]["attributes"]))
    {
     $this->e->add("Tag validation failed: no attributes for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if(!is_array($tag[$c]["attributes"]))
    {
     $this->e->add("Tag validation failed: attributes should be an array for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }

    //open tag specific validations
    if(strstr($tag[$c]["type"], "open"))
    {
     //inner validation
     if(!isset($tag[$c]["inner"]))
     {
      $this->e->add("Tag validation failed: no inner array for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if(!isset($tag[$c]["inner"]["start"]))
     {
      $this->e->add("Tag validation failed: no inner start for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if(!isset($tag[$c]["inner"]["length"]))
     {
      $this->e->add("Tag validation failed: no inner end for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }

     //close reference validations
     if(!isset($tag[$c]["close"]))
     {
      $this->e->add("Tag validation failed: no close reference for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if($tag[$c]["close"]<0)
     {
      $this->e->add("Tag validation failed: invalid close reference \"".$tag[$c]["close"]."\" for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if($tag[$c]["close"]>=$l)
     {
      $this->e->add("Tag validation failed: close reference \"".$tag[$c]["close"]."\" doesn't exist for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if($tag[$c]["close"]<$c)
     {
      $this->e->add("Tag validation failed: close reference \"".$tag[$c]["close"]."\" can't be before tag for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if($tag[$c]["close"]==$c)
     {
      $this->e->add("Tag validation failed: close reference \"".$tag[$c]["close"]."\" can't be the same tag for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
    }
   }

   if($expected&&is_array($expected))
   {
    for($c=0;$c<$l;$c++)
    {
     //more tags then expected
     if(!isset($expected[$c]))
     {
      $this->e->add("Tag validation failed: tag \"".$tag[$c]["name"]."\" (".$c.") not expected", "error");
      return UT_FAILED;
     }

     //name validation
     if($tag[$c]["name"]!=$expected[$c]["name"])
     {
      $this->e->add("Tag validation failed: name \"".$tag[$c]["name"]."\" doesn't match for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }

     //type validation
     if($tag[$c]["type"]!=$expected[$c]["type"])
     {
      $this->e->add("Tag validation failed: type \"".$tag[$c]["type"]."\" doesn't match \"".$expected[$c]["type"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }

     if(strstr($tag[$c]["type"], "close"))
     {
      if($tag[$c]["open"]!=$expected[$c]["open"])
      {
       $this->e->add("Tag validation failed: open reference \"".$tag[$c]["open"]."\" doesn't match for tag: \"".$expected[$c]["open"]."\" (".$c.")", "error");
       return UT_FAILED;
      }

      continue; //no more validation for close tag
     }

     //outer validation
     if($tag[$c]["outer"]["start"]!=$expected[$c]["outer"]["start"])
     {
      $this->e->add("Tag validation failed: outer start \"".$tag[$c]["outer"]["start"]."\" doesn't match \"".$expected[$c]["outer"]["start"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if($tag[$c]["outer"]["length"]!=$expected[$c]["outer"]["length"])
     {
      $this->e->add("Tag validation failed: outer length \"".$tag[$c]["outer"]["length"]."\" doesn't match \"".$expected[$c]["outer"]["length"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }

     //parent validations
     if($tag[$c]["parent"]!==$expected[$c]["parent"])
     {
      $this->e->add("Tag validation failed: parent \"".$tag[$c]["parent"]."\" doesn't match \"".$expected[$c]["parent"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }

     //attributes validations
     if($tag[$c]["attributes"]!=$expected[$c]["attributes"])
     {
      foreach(array_keys($expected[$c]["attributes"]) as $k)
      {
        $this->e->add("Tag validation failed: attribute \"".$k."\" value \"".$expected[$c]["attributes"][$k]."\" expected for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
       if(!isset($tag[$c]["attributes"][$k]))
        $this->e->add("Tag validation failed: attribute \"".$k."\" value \"".$expected[$c]["attributes"][$k]."\" expected for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
       else if($tag[$c]["attributes"][$k]!=$expected[$c]["attributes"][$k])
        $this->e->add("Tag validation failed: attribute \"".$k."\" value \"".$tag[$c]["attributes"][$k]."\" doesn't match \"".$expected[$c]["attributes"][$k]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      }

      foreach(array_keys($tag[$c]["attributes"]) as $k)
      {
       if(!isset($expected[$c]["attributes"][$k]))
        $this->e->add("Tag validation failed: attribute \"".$k."\" value \"".$tag[$c]["attributes"][$k]."\" not expected for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      }

      return UT_FAILED;
     }

     //open tag specific validations
     if(strstr($tag[$c]["type"], "open"))
     {
      //inner validations
      if($tag[$c]["inner"]["start"]!=$expected[$c]["inner"]["start"])
      {
       $this->e->add("Tag validation failed: inner start \"".$tag[$c]["inner"]["start"]."\" doesn't match \"".$expected[$c]["inner"]["start"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
       return UT_FAILED;
      }
      if($tag[$c]["inner"]["length"]!=$expected[$c]["inner"]["length"])
      {
       $this->e->add("Tag validation failed: inner length \"".$tag[$c]["inner"]["length"]."\" doesn't match \"".$expected[$c]["inner"]["length"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
       return UT_FAILED;
      }

      //close validations
      if($tag[$c]["close"]!=$expected[$c]["close"])
      {
       $this->e->add("Tag validation failed: close reference \"".$tag[$c]["close"]."\" doesn't match \"".$expected[$c]["close"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
       return UT_FAILED;
      }
     }
    }


    $l2=sizeof($expected);
    for($c=$l;$c<$l2;$c++) //check rest of expected array, i.e. more tags expected
    {
     $this->e->add("Tag validation failed: tag \"".$expected[$c]["name"]."\" (".$c.") expected", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }



  function Parse()
  {
   $xml=new Format_XML(false, true);
   if(!$xml->parse($this->testXML))
   {
    $this->e->add("Parsing test XML failed");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->original!=$this->testXML)
   {
    $this->e->add("Original XML not set correctly");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML not set correctly");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   $test="var test=1;

   function lala()
   {
    test++;
   }";

   $xml->clear();
   $xml->e->clear();
   if($xml->parse($test))
   {
    $this->e->add("Parser should fail on java script");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->original!=$test)
   {
    $this->e->add("Original XML should not be set");
    return UT_FAILED;
   }

   if($xml->output!=$test)
   {
    $this->e->add("XML should not be set");
    return UT_FAILED;
   }

   if($xml->data)
   {
    $this->e->add("XML tag variable should be empty");
    return UT_FAILED;
   }

   if($xml->vmax!==0)
   {
    $this->e->add("XML tags counter should be 0");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Read()
  {
   global $_path;

   $xml=new Format_XML(false, true);
   if(!$xml->read($_path["api"]."xml/1.0/jas.xml"))
   {
    $this->e->add("Parsing file failed");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->filename!=$_path["api"]."xml/1.0/jas.xml")
   {
    $this->e->add("Filename parameter not set after loading");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data)==UT_FAILED)
    return UT_FAILED;


   $xml->clear();
   $xml->e->clear();
   if($xml->read($_path["root"]."jakdsajsdkhahjsdhajksdasd.lkjasd"))
   {
    $this->e->add("Parser should not find file: \"".$_path["root"]."jakdsajsdkhahjsdhajksdasd.lkjasd\"");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Reparse()
  {
   $xml=new Format_XML(false, true);

   if($xml->reparse())
   {
    $this->e->add("XML should not be initialized");
    return UT_FAILED;
   }

   $xml->parse($this->testXML);
   $xml->delete("test");

   if(!$xml->reparse())
   {
    $this->e->add("XML should reinitialize");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=$xml->original)
   {
    $this->e->add("XML should be the same as original");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;

   return UT_OK;
  }


  function ReparseOutput()
  {
   $xml=new Format_XML(false, true);

   if($xml->reparse(true))
   {
    $this->e->add("XML should not be initialized");
    return UT_FAILED;
   }

   $xml->parse($this->testXML);
   $xml->output=preg_replace("!</test>!", "", $xml->output);

   if(!$xml->reparse(true))
   {
    $this->e->add("XML should reparse");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->data[1]["type"]!="short")
   {
    $this->e->add("XML test tag should now be a short tag");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function ParseInternalChecks()
  {
   $xml=new Format_XML(false, true);

   //correct type for short sub tag with same name
   $xml->parse("<tag open><tag short/></tag>");

   if($xml->data[0]["type"]!="open")
   {
    $this->e->add("XML parse failed", "error", "first tag should be a open tag");
    return UT_FAILED;
   }

   if($xml->data[1]["type"]!="short")
   {
    $this->e->add("XML parse failed", "error", "second tag should be a short tag");
    return UT_FAILED;
   }


   //correct type for short sub tag with same name
   $xml->parse("<tag short/><tag open></tag>");

   if($xml->data[0]["type"]!="short")
   {
    $this->e->add("XML parse failed", "error", "first tag should be a short tag");
    return UT_FAILED;
   }

   if($xml->data[1]["type"]!="open")
   {
    $this->e->add("XML parse failed", "error", "second tag should be a open tag");
    return UT_FAILED;
   }


   //correct type for short sub tag with same name (comments)
   $xml->parse("<!tag open><!tag open><!tag short><!/tag><!/tag>");

   if($xml->data[0]["type"]!="!open")
   {
    $this->e->add("XML parse failed", "error", "first tag should be a open tag");
    return UT_FAILED;
   }

   if($xml->data[1]["type"]!="!open")
   {
    $this->e->add("XML parse failed", "error", "second tag should be a open tag");
    return UT_FAILED;
   }

   if($xml->data[2]["type"]!="!short")
   {
    $this->e->add("XML parse failed", "error", "third tag should be a short tag");
    return UT_FAILED;
   }

   //correct location for marker in short tag
   $xml->parse("<!tag open><!tag #marker#><!/tag>");

   if($xml->data[2]["name"]!="marker")
   {
    $this->e->add("XML parse failed", "error", "third tag should be marker");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function ParseMultilineTags()
  {
   $xml=new Format_XML(false, true);
   $testXML=new Format_XML(false, true);

//attributes multiline
   if(!$xml->parse("<tag
key1=\"value1\"
key2=\"value2\"
key3=\"value3\">"))
   {
    $this->e->add("XML parse shouldn't fail");
    return UT_FAILED;
   }

   $testXML->parse("<tag key1=\"value1\" key2=\"value2\" key3=\"value3\">");

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;

   if(!$xml->parse("<tag
key1=\"value1\"
key2=\"value2\"
key3=\"value3\"><tag2><tag3>"))
   {
    $this->e->add("XML parse shouldn't fail");
    return UT_FAILED;
   }

   $testXML->parse("<tag key1=\"value1\" key2=\"value2\" key3=\"value3\"><tag2><tag3>");

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;

//name on separate line
   if($xml->parse("<
tag
key1=\"value1\"
key2=\"value2\"
key3=\"value3\">"))
   {
    $this->e->add("XML parse should fail");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, Array())==UT_FAILED)
    return UT_FAILED;

   if(!$xml->parse("<
tag
key1=\"value1\"
key2=\"value2\"
key3=\"value3\"><tag2><tag3>"))
   {
    $this->e->add("XML parse shouldn't fail");
    return UT_FAILED;
   }

   $testXML->parse("< tag key1=\"value1\" key2=\"value2\" key3=\"value3\"><tag2><tag3>");

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;

//marker multiline
   if($xml->parse("#marker
key1=\"value1\"
key2=\"value2\"
key3=\"value3\"#"))
   {
    $this->e->add("XML parse should fail");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, Array())==UT_FAILED)
    return UT_FAILED;

   if(!$xml->parse("#
marker
key1=\"value1\"
key2=\"value2\"
key3=\"value3\"#<tag2><tag3>"))
   {
    $this->e->add("XML parse shouldn't fail");
    return UT_FAILED;
   }

   $testXML->parse("_ marker key1=\"value1\" key2=\"value2\" key3=\"value3\"_<tag2><tag3>");

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function InternalSetFailures()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//add text before begin
   if($xml->_set(-1, 0, "asd"))
   {
    $this->e->add("XML _set should fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//add text after end
   if($xml->_set(strlen($this->testXML)+1, 0, "asd"))
   {
    $this->e->add("XML _set should fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//add text ending after end
   if($xml->_set(strlen($this->testXML)-1, 10, "asd"))
   {
    $this->e->add("XML _set should fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//add array
   if($xml->_set(0, 0, Array("asd")))
   {
    $this->e->add("XML _set should fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//add object
   if($xml->_set(0, 0, $this))
   {
    $this->e->add("XML _set should fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//break tags test, i.e. replace < or >...
   return UT_OK;
  }


  function InternalSetText()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//add text at begin
   if(!$xml->_set(0, 0, "asd"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="asd".$this->testXML)
   {
    $this->e->add("XML should include the new value");
    return UT_FAILED;
   }

   $diff=3; //length difference
   //adjust outer+inner start position of all tags
   $testTag=$this->testTag;
   $l=sizeof($testTag);
   for($c=0;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
     $testTag[$c]["outer"]["start"]+=$diff;

    if(isset($testTag[$c]["inner"]))
     $testTag[$c]["inner"]["start"]+=$diff;
   }

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//edit text at begin
   if(!$xml->_set(0, 3, "trala"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="trala".$this->testXML)
   {
    $this->e->add("XML should include the updated value");
    return UT_FAILED;
   }

   $diff=5; //length difference
   //adjust outer+inner start position of all tags
   $testTag=$this->testTag;
   $l=sizeof($testTag);
   for($c=0;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
     $testTag[$c]["outer"]["start"]+=$diff;

    if(isset($testTag[$c]["inner"]))
     $testTag[$c]["inner"]["start"]+=$diff;
   }

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//delete text at begin
   if(!$xml->_set(0, 5, ""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text at end
   if(!$xml->_set(strlen($xml->output), 0, "asd"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML."asd")
   {
    $this->e->add("XML should include the new value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//edit text at end
   if(!$xml->_set(strlen($xml->output)-3, 3, "trala"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML."trala")
   {
    $this->e->add("XML should include the updated value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//delete text at end
   if(!$xml->_set(strlen($xml->output)-5, 5, ""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text after <test>
   if(!$xml->_set(61, 0, "asd"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, 61)."asd".substr($this->testXML, 61))
   {
    $this->e->add("XML should include the new value");
    return UT_FAILED;
   }

   $diff=3; //length difference
   $testTag=$this->testTag;

   //adjust outer+inner length of test tag
   $testTag[1]["outer"]["length"]+=$diff;
   $testTag[1]["inner"]["length"]+=$diff;

   //adjust outer+inner start position off all tags after test tag
   $l=sizeof($testTag);
   for($c=2;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
     $testTag[$c]["outer"]["start"]+=$diff;

    if(isset($testTag[$c]["inner"]))
     $testTag[$c]["inner"]["start"]+=$diff;
   }

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//edit text after <test>
   if(!$xml->_set(61, 3, "trala"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, 61)."trala".substr($this->testXML, 61))
   {
    $this->e->add("XML should include the updated value");
    return UT_FAILED;
   }

   $diff=5; //length difference
   $testTag=$this->testTag;

   //adjust outer+inner length of test tag
   $testTag[1]["outer"]["length"]+=$diff;
   $testTag[1]["inner"]["length"]+=$diff;

   //adjust outer+inner start position off all tags after test tag
   $l=sizeof($testTag);
   for($c=2;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
     $testTag[$c]["outer"]["start"]+=$diff;

    if(isset($testTag[$c]["inner"]))
     $testTag[$c]["inner"]["start"]+=$diff;
   }

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//delete text after <test>
   if(!$xml->_set(61, 5, ""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function InternalSetTags()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//add tag at begin
   if(!$xml->_set(0, 0, "<asd>"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<asd>".$this->testXML)
   {
    $this->e->add("XML should include the new value");
    return UT_FAILED;
   }

   $diff=5; //length difference
   $testTag=$this->testTag;

   //adjust outer+inner start position off all tags
   $l=sizeof($testTag);
   for($c=0;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
     $testTag[$c]["outer"]["start"]+=$diff;

    if(isset($testTag[$c]["inner"]))
     $testTag[$c]["inner"]["start"]+=$diff;

    if(isset($testTag[$c]["parent"])&&$testTag[$c]["parent"]!==false)
     $testTag[$c]["parent"]++; //1 tag added

    if(isset($testTag[$c]["open"]))
     $testTag[$c]["open"]++; //1 tag added
    if(isset($testTag[$c]["close"]))
     $testTag[$c]["close"]++; //1 tag added
   }

   //add asd tag at begin of array
   array_unshift($testTag, Array(
    "name" => "asd",
    "type" => "short",
    "outer" => Array(
     "start" => 0,
     "length" => $diff,
    ),
    "attributes" => Array(
    ),
    "parent" => false,
   ));

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//overwrite tag at begin
   if(!$xml->_set(0, 5, "<trala>"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<trala>".$this->testXML)
   {
    $this->e->add("XML should include the updated value");
    return UT_FAILED;
   }

   $diff=7; //length difference
   $testTag=$this->testTag;

   //adjust outer+inner start position off all tags
   $l=sizeof($testTag);
   for($c=0;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
     $testTag[$c]["outer"]["start"]+=$diff;

    if(isset($testTag[$c]["inner"]))
     $testTag[$c]["inner"]["start"]+=$diff;

    if(isset($testTag[$c]["parent"])&&$testTag[$c]["parent"]!==false)
     $testTag[$c]["parent"]++; //1 tag added

    if(isset($testTag[$c]["open"]))
     $testTag[$c]["open"]++; //1 tag added
    if(isset($testTag[$c]["close"]))
     $testTag[$c]["close"]++; //1 tag added
   }

   //add trala tag at begin of array
   array_unshift($testTag, Array(
    "name" => "trala",
    "type" => "short",
    "outer" => Array(
     "start" => 0,
     "length" => $diff,
    ),
    "attributes" => Array(
    ),
    "parent" => false,
   ));

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//delete tag at begin
   if(!$xml->_set(0, 7, ""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add tag at end
   if(!$xml->_set(strlen($xml->output), 0, "<asd>"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML."<asd>")
   {
    $this->e->add("XML should include the new value");
    return UT_FAILED;
   }

   $diff=5; //length difference
   $testTag=$this->testTag;

   //add asd tag at end of array
   array_push($testTag, Array(
    "name" => "asd",
    "type" => "short",
    "outer" => Array(
     "start" => strlen($this->testXML),
     "length" => $diff,
    ),
    "attributes" => Array(
    ),
    "parent" => false,
   ));

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//overwrite tag at end
   if(!$xml->_set(strlen($xml->output)-5, 5, "<trala>"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML."<trala>")
   {
    $this->e->add("XML should include the updated value");
    return UT_FAILED;
   }

   $diff=7; //length difference
   $testTag=$this->testTag;

   //add asd tag at end of array
   array_push($testTag, Array(
    "name" => "trala",
    "type" => "short",
    "outer" => Array(
     "start" => strlen($this->testXML),
     "length" => $diff,
    ),
    "attributes" => Array(
    ),
    "parent" => false,
   ));

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//delete tag at end
   if(!$xml->_set(strlen($xml->output)-7, 7, ""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add tag after <test>
   if(!$xml->_set(61, 0, "<asd>"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, 61)."<asd>".substr($this->testXML, 61))
   {
    $this->e->add("XML should include the new value");
    return UT_FAILED;
   }

   $diff=5; //length difference
   $testTag=$this->testTag;

   //adjust outer+inner end position+length and close reference of test tag
   $testTag[1]["outer"]["length"]+=$diff;
   $testTag[1]["inner"]["length"]+=$diff;
   $testTag[1]["close"]++; //1 tag added

   //adjust outer+inner position off all tags after new tag
   $l=sizeof($testTag);
   for($c=2;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
     $testTag[$c]["outer"]["start"]+=$diff;

    if(isset($testTag[$c]["inner"]))
     $testTag[$c]["inner"]["start"]+=$diff;
   }

   //add asd tag after <test>
   array_splice($testTag, 2, 0, Array(Array(
    "name" => "asd",
    "type" => "short",
    "outer" => Array(
     "start" => 61,
     "length" => $diff,
    ),
    "attributes" => Array(
    ),
    "parent" => 1,
   )));

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//edit tag after <test>
   if(!$xml->_set(61, 5, "<trala>"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, 61)."<trala>".substr($this->testXML, 61))
   {
    $this->e->add("XML should include the updated value");
    return UT_FAILED;
   }

   $diff=7; //length difference
   $testTag=$this->testTag;

   //adjust outer+inner end position+length and close reference of test tag
   $testTag[1]["outer"]["length"]+=$diff;
   $testTag[1]["inner"]["length"]+=$diff;
   $testTag[1]["close"]++; //1 tag added

   //adjust outer+inner position off all tags after new tag
   $l=sizeof($testTag);
   for($c=2;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
     $testTag[$c]["outer"]["start"]+=$diff;

    if(isset($testTag[$c]["inner"]))
     $testTag[$c]["inner"]["start"]+=$diff;
   }

   //add trala tag after <test>
   array_splice($testTag, 2, 0, Array(Array(
    "name" => "trala",
    "type" => "short",
    "outer" => Array(
     "start" => 61,
     "length" => $diff,
    ),
    "attributes" => Array(
    ),
    "parent" => 1,
   )));

   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//delete tag after <test>
   if(!$xml->_set(61, 7, ""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;


   if($this->_validateTags($xml->data, $testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function InternalSetAttributes()
  {
   $xml=new Format_XML(false, true);
   $testXML=new Format_XML(false, true);

//delete argument from tag
   $xml->parse("<tag key1=\"value1\" key2=\"value2\">text</tag>");

   if(!$xml->_set(4, 14, ""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag key2=\"value2\">text</tag>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//add argument to short tag
   $xml->parse("<tag key1=\"value1\" key2=\"value2\"/>");

   if(!$xml->_set(18, 0, " key3=\"value3\""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag key1=\"value1\" key3=\"value3\" key2=\"value2\"/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//edit argument in tag
   $xml->parse("<tag key1=\"value1\" key2=\"value2\">text</tag>");

   if(!$xml->_set(11, 6, "new"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag key1=\"new\" key2=\"value2\">text</tag>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//edit argument key in short tag
   $xml->parse("<tag key1=\"value1\" key2=\"value2\"/>");

   if(!$xml->_set(19, 4, "newkey2"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag key1=\"value1\" newkey2=\"value2\"/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//key only arguments


//delete argument from tag
   $xml->parse("<tag key1 key2>text</tag>");

   if(!$xml->_set(4, 5, ""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag key2>text</tag>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//add argument to short tag
   $xml->parse("<tag key1 key2/>");

   if(!$xml->_set(9, 0, " key3"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag key1 key3 key2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//edit argument in tag
   $xml->parse("<tag key1 key2>text</tag>");

   if(!$xml->_set(5, 4, "new"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag new key2>text</tag>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//edit argument key in short tag
   $xml->parse("<tag key1 key2/>");

   if(!$xml->_set(10, 4, "newkey2"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag key1 newkey2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function InternalSetMarkers()
  {
   $xml=new Format_XML(false, true);
   $testXML=new Format_XML(false, true);

//add key before markers
   $xml->parse("<tag key1=\"#marker1#\" key2=\"#marker2#\" key3=\"#marker3#\"/><tag2/>");

   if(!$xml->_set(4, 0, " newkey"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag newkey key1=\"#marker1#\" key2=\"#marker2#\" key3=\"#marker3#\"/><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//add key after markers
   $xml->parse("<tag key1=\"#marker1#\" key2=\"#marker2#\" key3=\"#marker3#\"/><tag2/>");

   if(!$xml->_set(55, 0, " newkey"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag key1=\"#marker1#\" key2=\"#marker2#\" key3=\"#marker3#\" newkey/><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete marker
   $xml->parse("<tag key1=\"#marker1#\" key2=\"#marker2#\" key3=\"#marker3#\"/><tag2/>");

   if(!$xml->_set(11, 9, ""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag key1=\"\" key2=\"#marker2#\" key3=\"#marker3#\"/><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete markers
   $xml->parse("<tag key1=\"#marker1#\" key2=\"#marker2#\" key3=\"#marker3#\"/><tag2/>");

   if(!$xml->_set(28, 26, ""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag key1=\"#marker1#\" key2=\"\"/><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//replace markers
   $xml->parse("<tag key1=\"#marker1#\" key2=\"#marker2#\" key3=\"#marker3#\"/><tag2/>");

   if(!$xml->_set(28, 9, "value"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag key1=\"#marker1#\" key2=\"value\" key3=\"#marker3#\"/><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//add markers
   $xml->parse("<tag key1=\"#marker1#\" key2=\"#marker2#\" key3=\"#marker3#\"/><tag2/>");

   if(!$xml->_set(38, 0, " key2a=\"#marker2a#\""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag key1=\"#marker1#\" key2=\"#marker2#\" key2a=\"#marker2a#\" key3=\"#marker3#\"/><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//add markers with multiple tags
   $xml->parse("<wrap><tag key1=\"#marker1#\" key2=\"#marker2#\" key3=\"#marker3#\"/><tag2>test</tag2></wrap>");

   if(!$xml->_set(44, 0, " key2a=\"#marker2a#\""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<wrap><tag key1=\"#marker1#\" key2=\"#marker2#\" key2a=\"#marker2a#\" key3=\"#marker3#\"/><tag2>test</tag2></wrap>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//rename marker
   $xml->parse("<tag key1=\"#marker1#\" key2=\"#marker2#\" key3=\"#marker3#\"/><tag2/>");

   if(!$xml->_set(28, 9, "#newmarker#"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag key1=\"#marker1#\" key2=\"#newmarker#\" key3=\"#marker3#\"/><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//key only arguments


//add key before markers
   $xml->parse("<tag #marker1# #marker2# #marker3#/><tag2/>");

   if(!$xml->_set(4, 0, " newkey"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag newkey #marker1# #marker2# #marker3#/><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//add key after markers
   $xml->parse("<tag #marker1# #marker2# #marker3#/><tag2/>");

   if(!$xml->_set(34, 0, " newkey"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag #marker1# #marker2# #marker3# newkey/><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete marker
   $xml->parse("<tag #marker1# #marker2# #marker3#/><tag2/>");

   if(!$xml->_set(4, 10, ""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag #marker2# #marker3#/><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete markers
   $xml->parse("<tag #marker1# #marker2# #marker3#/><tag2/>");

   if(!$xml->_set(15, 19, ""))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag #marker1# /><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//replace markers
   $xml->parse("<tag #marker1# #marker2# #marker3#/><tag2/>");

   if(!$xml->_set(15, 9, "value"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag #marker1# value #marker3#/><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//add markers
   $xml->parse("<tag #marker1# #marker2# #marker3#/><tag2/>");

   if(!$xml->_set(24, 0, " #marker2a#"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag #marker1# #marker2# #marker2a# #marker3#/><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//add markers with multiple tags
   $xml->parse("<wrap><tag #marker1# #marker2# #marker3#/><tag2>test</tag2></wrap>");

   if(!$xml->_set(30, 0, " #marker2a#"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<wrap><tag #marker1# #marker2# #marker2a# #marker3#/><tag2>test</tag2></wrap>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//rename marker
   $xml->parse("<tag #marker1# #marker2# #marker3#/><tag2/>");

   if(!$xml->_set(15, 9, "#newmarker#"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag #marker1# #newmarker# #marker3#/><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//add 1 marker in tag
   $xml->parse("<tag key1=\"\"/><tag2/>");

   if(!$xml->_set(11, 0, "#marker#"))
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->output!="<tag key1=\"#marker#\"/><tag2/>")
   {
    $this->e->add("XML should be the new value");
    return UT_FAILED;
   }

   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetFailures()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//replace non existing tag
   if($xml->setReplace("asd", "asd"))
   {
    $this->e->add("XML should not set value for non existing tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//replace non existing tag
   if($xml->setReplace(-1, "asd"))
   {
    $this->e->add("XML should not set value for negative key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//replace non existing tag
   if($xml->setReplace($xml->vmax, "asd"))
   {
    $this->e->add("XML should not set value for non existing key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function ShortToBlock()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//convert short tag to block
   if(!$xml->_shortToBlock("test/unit"))
   {
    $this->e->add("XML should convert short tag to block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=preg_replace("/<unit[^>]+>/", "\\0</unit>", $this->testXML))
   {
    $this->e->add("XML should include the new close tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//convert short comment to block
   if(!$xml->_shortToBlock("test/shortcomment"))
   {
    $this->e->add("XML should convert short comment to block comment");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=preg_replace("/<!shortcomment[^>]*>/", "\\0<!/shortcomment>", $this->testXML))
   {
    $this->e->add("XML should include the new close short comment");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->parse("<test/>");

//convert <short tag /> to block
   if(!$xml->_shortToBlock("test"))
   {
    $this->e->add("XML should convert short tag to block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!="<test></test>")
   {
    $this->e->add("XML should include the new close tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse("<test></test>");

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->parse("<test asd />");

//convert <short tag /> to block
   if(!$xml->_shortToBlock("test"))
   {
    $this->e->add("XML should convert short tag to block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!="<test asd></test>")
   {
    $this->e->add("XML should include the new close tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse("<test asd></test>");

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->parse($this->testXML);

//convert block to block
   if($xml->_shortToBlock("test"))
   {
    $this->e->add("XML should not convert block tag to block tag");
    return UT_FAILED;
   }

//convert comment to block
   if($xml->_shortToBlock("test/comment"))
   {
    $this->e->add("XML should not convert comment tag to block tag");
    return UT_FAILED;
   }

//convert marker to block
   if($xml->_shortToBlock("test/marker"))
   {
    $this->e->add("XML should not convert marker tag to block tag");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetRoot()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//replace inside of tag with text
   if(!$xml->setInside(false, "asd"))
   {
    $this->e->add("XML should replace the inside of the existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!="asd")
   {
    $this->e->add("XML should include the new value inside of the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace inside of tag with text leaving a copy of the original
   if(!$xml->setInsideCopy(false, "asd"))
   {
    $this->e->add("XML should copy the existing open tag and replace inside of the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!="asd".$this->testXML)
   {
    $this->e->add("XML should include a copy of the open tag and the new value inside of the original open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text
   if(!$xml->setReplace(false, "asd"))
   {
    $this->e->add("XML should replace the existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!="asd")
   {
    $this->e->add("XML should include the new value instead of the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setBefore(false, "asd"))
   {
    $this->e->add("XML should copy the existing open tag and replace the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!="asd".$this->testXML)
   {
    $this->e->add("XML should include a copy of the open tag and the new value instead of the original open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setAfter(false, "asd"))
   {
    $this->e->add("XML should copy the existing open tag and replace the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML."asd")
   {
    $this->e->add("XML should include a copy of the open tag and the new value instead of the original open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace inside of tag with tag
   if(!$xml->setInside(false, "<asd test=\"asd\">"))
   {
    $this->e->add("XML should replace inside of the existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!="<asd test=\"asd\">")
   {
    $this->e->add("XML should include the new value inside of the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace inside of tag with tag leaving a copy of the original
   if(!$xml->setInsideCopy(false, "<asd test=\"asd\">"))
   {
    $this->e->add("XML should copy the existing open tag and replace inside of the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!="<asd test=\"asd\">".$this->testXML)
   {
    $this->e->add("XML should include a copy of the open tag and the new value inside of the original open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag
   if(!$xml->setReplace(false, "<asd test=\"asd\">"))
   {
    $this->e->add("XML should replace the existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!="<asd test=\"asd\">")
   {
    $this->e->add("XML should include the new value instead of the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag leaving a copy of the original
   if(!$xml->setBefore(false, "<asd test=\"asd\">"))
   {
    $this->e->add("XML should copy the existing open tag and replace the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!="<asd test=\"asd\">".$this->testXML)
   {
    $this->e->add("XML should include the a copy of the open tag and the new value instead of the original open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag leaving a copy of the original
   if(!$xml->setAfter(false, "<asd test=\"asd\">"))
   {
    $this->e->add("XML should copy the existing open tag and replace the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML."<asd test=\"asd\">")
   {
    $this->e->add("XML should include the a copy of the open tag and the new value instead of the original open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetBlock()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//replace inside of tag with text
   if(!$xml->setInside("test", "asd"))
   {
    $this->e->add("XML should replace the inside of the existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["inner"]["start"])."asd".substr($this->testXML, $this->testTag[1]["inner"]["start"]+$this->testTag[1]["inner"]["length"]))
   {
    $this->e->add("XML should include the new value inside of the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace inside of tag with text leaving a copy of the original
   if(!$xml->setInsideCopy("test", "asd"))
   {
    $this->e->add("XML should copy the existing open tag and replace inside of the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["inner"]["start"])."asd".substr($this->testXML, $this->testTag[1]["inner"]["start"]+$this->testTag[1]["inner"]["length"]).substr($this->testXML, $this->testTag[1]["outer"]["start"]))
   {
    $this->e->add("XML should include a copy of the open tag and the new value inside of the original open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text
   if(!$xml->setReplace("test", "asd"))
   {
    $this->e->add("XML should replace the existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"])."asd")
   {
    $this->e->add("XML should include the new value instead of the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setBefore("test", "asd"))
   {
    $this->e->add("XML should copy the existing open tag and replace the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[1]["outer"]["start"]))
   {
    $this->e->add("XML should include a copy of the open tag and the new value instead of the original open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setAfter("test", "asd"))
   {
    $this->e->add("XML should copy the existing open tag and replace the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"]+$this->testTag[1]["outer"]["length"])."asd".substr($this->testXML, $this->testTag[1]["outer"]["start"]+$this->testTag[1]["outer"]["length"]))
   {
    $this->e->add("XML should include a copy of the open tag and the new value instead of the original open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace inside of tag with tag
   if(!$xml->setInside("test", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should replace inside of the existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["inner"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[1]["inner"]["start"]+$this->testTag[1]["inner"]["length"]))
   {
    $this->e->add("XML should include the new value inside of the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace inside of tag with tag leaving a copy of the original
   if(!$xml->setInsideCopy("test", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should copy the existing open tag and replace inside of the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["inner"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[1]["inner"]["start"]+$this->testTag[1]["inner"]["length"]).substr($this->testXML, $this->testTag[1]["outer"]["start"]))
   {
    $this->e->add("XML should include a copy of the open tag and the new value inside of the original open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag
   if(!$xml->setReplace("test", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should replace the existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"])."<asd test=\"asd\">")
   {
    $this->e->add("XML should include the new value instead of the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag leaving a copy of the original
   if(!$xml->setBefore("test", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should copy the existing open tag and replace the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[1]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the open tag and the new value instead of the original open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag leaving a copy of the original
   if(!$xml->setAfter("test", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should copy the existing open tag and replace the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"]+$this->testTag[1]["outer"]["length"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[1]["outer"]["start"]+$this->testTag[1]["outer"]["length"]))
   {
    $this->e->add("XML should include the a copy of the open tag and the new value instead of the original open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetShort()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//replace inside of tag with text
   if(!$xml->setInside("test/unit", "asd"))
   {
    $this->e->add("XML should replace the inside of the existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=preg_replace("/<(unit[^>]+)/", "<\\1>asd</unit", $this->testXML))
   {
    $this->e->add("XML should include the new value inside of the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace inside of tag with text leaving a copy of the original
   if(!$xml->setInsideCopy("test/unit", "asd"))
   {
    $this->e->add("XML should copy the existing short tag and replace inside of the original short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=preg_replace("/<(unit[^>]+)/", "<\\1>asd</unit><\\1", $this->testXML))
   {
    $this->e->add("XML should include a copy of the short tag and the new value inside of the original short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text
   if(!$xml->setReplace("test/unit", "asd"))
   {
    $this->e->add("XML should replace the existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[3]["outer"]["start"]+$this->testTag[3]["outer"]["length"]))
   {
    $this->e->add("XML should include the new value instead of the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setBefore("test/unit", "asd"))
   {
    $this->e->add("XML should copy the existing short tag and replace the original short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[3]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the short tag and the new value instead of the original short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setAfter("test/unit", "asd"))
   {
    $this->e->add("XML should copy the existing short tag and replace the original short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"]+$this->testTag[3]["outer"]["length"])."asd".substr($this->testXML, $this->testTag[3]["outer"]["start"]+$this->testTag[3]["outer"]["length"]))
   {
    $this->e->add("XML should include the a copy of the short tag and the new value instead of the original short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag
   if(!$xml->setReplace("test/unit", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should replace the existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[3]["outer"]["start"]+$this->testTag[3]["outer"]["length"]))
   {
    $this->e->add("XML should include the new value instead of the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag leaving a copy of the original
   if(!$xml->setBefore("test/unit", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should copy the existing short tag and replace the original short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[3]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the short tag and the new value instead of the original short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag leaving a copy of the original
   if(!$xml->setAfter("test/unit", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should copy the existing short tag and replace the original short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"]+$this->testTag[3]["outer"]["length"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[3]["outer"]["start"]+$this->testTag[3]["outer"]["length"]))
   {
    $this->e->add("XML should include the a copy of the short tag and the new value instead of the original short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetComment()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//replace tag with text
   if(!$xml->setReplace("test/comment", "asd"))
   {
    $this->e->add("XML should replace the existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[2]["outer"]["start"]+$this->testTag[2]["outer"]["length"]))
   {
    $this->e->add("XML should include the new value instead of the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setBefore("test/comment", "asd"))
   {
    $this->e->add("XML should copy the existing comment tag and replace the original comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[2]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the comment tag and the new value instead of the original comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setAfter("test/comment", "asd"))
   {
    $this->e->add("XML should copy the existing comment tag and replace the original comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"]+$this->testTag[2]["outer"]["length"])."asd".substr($this->testXML, $this->testTag[2]["outer"]["start"]+$this->testTag[2]["outer"]["length"]))
   {
    $this->e->add("XML should include the a copy of the comment tag and the new value instead of the original comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag
   if(!$xml->setReplace("test/comment", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should replace the existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[2]["outer"]["start"]+$this->testTag[2]["outer"]["length"]))
   {
    $this->e->add("XML should include the new value instead of the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag leaving a copy of the original
   if(!$xml->setBefore("test/comment", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should copy the existing comment tag and replace the original comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[2]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the comment tag and the new value instead of the original comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag leaving a copy of the original
   if(!$xml->setAfter("test/comment", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should copy the existing comment tag and replace the original comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"]+$this->testTag[2]["outer"]["length"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[2]["outer"]["start"]+$this->testTag[2]["outer"]["length"]))
   {
    $this->e->add("XML should include the a copy of the comment tag and the new value instead of the original comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetShortComment()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//replace inside of tag with text
   if(!$xml->setInside("test/shortcomment", "asd"))
   {
    $this->e->add("XML should replace the inside of the existing short comment");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=preg_replace("/<(!shortcomment[^>]*)/", "<\\1>asd<!/shortcomment", $this->testXML))
   {
    $this->e->add("XML should include the new value inside of the short comment");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace inside of tag with text leaving a copy of the original
   if(!$xml->setInsideCopy("test/shortcomment", "asd"))
   {
    $this->e->add("XML should copy the existing short comment and replace inside of the original short comment");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=preg_replace("/<(!shortcomment[^>]*)/", "<\\1>asd<!/shortcomment><\\1", $this->testXML))
   {
    $this->e->add("XML should include a copy of the short comment and the new value inside of the original short comment");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text
   if(!$xml->setReplace("test/shortcomment", "asd"))
   {
    $this->e->add("XML should replace the existing short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[5]["outer"]["start"]+$this->testTag[5]["outer"]["length"]))
   {
    $this->e->add("XML should include the new value instead of the short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setBefore("test/shortcomment", "asd"))
   {
    $this->e->add("XML should copy the existing short comment tag and replace the original short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[5]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the short comment tag and the new value instead of the original short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setAfter("test/shortcomment", "asd"))
   {
    $this->e->add("XML should copy the existing short comment tag and replace the original short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"]+$this->testTag[5]["outer"]["length"])."asd".substr($this->testXML, $this->testTag[5]["outer"]["start"]+$this->testTag[5]["outer"]["length"]))
   {
    $this->e->add("XML should include the a copy of the short comment tag and the new value instead of the original short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag
   if(!$xml->setReplace("test/shortcomment", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should replace the existing short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[5]["outer"]["start"]+$this->testTag[5]["outer"]["length"]))
   {
    $this->e->add("XML should include the new value instead of the short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag leaving a copy of the original
   if(!$xml->setBefore("test/shortcomment", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should copy the existing short comment tag and replace the original short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[5]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the short comment tag and the new value instead of the original short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag leaving a copy of the original
   if(!$xml->setAfter("test/shortcomment", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should copy the existing short comment tag and replace the original short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"]+$this->testTag[5]["outer"]["length"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[5]["outer"]["start"]+$this->testTag[5]["outer"]["length"]))
   {
    $this->e->add("XML should include the a copy of the short comment tag and the new value instead of the original short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetMarker()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//replace tag with text
   if(!$xml->setReplace("test/marker", "asd"))
   {
    $this->e->add("XML should replace the existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[4]["outer"]["start"]+$this->testTag[4]["outer"]["length"]))
   {
    $this->e->add("XML should include the new value instead of the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setBefore("test/marker", "asd"))
   {
    $this->e->add("XML should copy the existing marker tag and replace the original marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[4]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the marker tag and the new value instead of the original marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setAfter("test/marker", "asd"))
   {
    $this->e->add("XML should copy the existing marker tag and replace the original marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"]+$this->testTag[4]["outer"]["length"])."asd".substr($this->testXML, $this->testTag[4]["outer"]["start"]+$this->testTag[4]["outer"]["length"]))
   {
    $this->e->add("XML should include the a copy of the marker tag and the new value instead of the original marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag
   if(!$xml->setReplace("test/marker", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should replace the existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[4]["outer"]["start"]+$this->testTag[4]["outer"]["length"]))
   {
    $this->e->add("XML should include the new value instead of the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag leaving a copy of the original
   if(!$xml->setBefore("test/marker", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should copy the existing marker tag and replace the original marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[4]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the marker tag and the new value instead of the original marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with tag leaving a copy of the original
   if(!$xml->setAfter("test/marker", "<asd test=\"asd\">"))
   {
    $this->e->add("XML should copy the existing marker tag and replace the original marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"]+$this->testTag[4]["outer"]["length"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[4]["outer"]["start"]+$this->testTag[4]["outer"]["length"]))
   {
    $this->e->add("XML should include the a copy of the marker tag and the new value instead of the original marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetMultiple()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   if(!$xml->setReplace("test/comment", "asd"))
   {
    $this->e->add("XML should add the new value to the existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!$xml->setBefore("test/shortcomment", "asd"))
   {
    $this->e->add("XML should add the new value to the existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!$xml->setAfter("test/shortcomment", "asd"))
   {
    $this->e->add("XML should add the new value to the existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!$xml->setReplace("test/unit", "asd"))
   {
    $this->e->add("XML should add the new value to the existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!$xml->setBefore("test", "asd"))
   {
    $this->e->add("XML should add the new value to the existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!$xml->setAfter("test", "asd"))
   {
    $this->e->add("XML should add the new value to the existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetArray()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   $values=Array(
    "unit" => "Unit replacement",
    "comment" => "Comment replacement",
    "shortcomment" => "Short comment replacement",
    "marker" => "Marker replacement",
   );


//replace inside of tag with array
   if(!$xml->setInside("test", $values))
   {
    $this->e->add("XML should replace inside of an existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Unit!s", $xml->output))
   {
    $this->e->add("XML Unit replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Comment!s", $xml->output))
   {
    $this->e->add("XML Comment replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Short comment!s", $xml->output))
   {
    $this->e->add("XML Short comment replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Marker!s", $xml->output))
   {
    $this->e->add("XML Marker replacement should exist in the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace inside of tag with array leaving a copy of the original
   if(!$xml->setInsideCopy("test", $values))
   {
    $this->e->add("XML should copy an existing open tag and replace inside of the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Unit!s", $xml->output))
   {
    $this->e->add("XML Unit replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Comment!s", $xml->output))
   {
    $this->e->add("XML Comment replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Short comment!s", $xml->output))
   {
    $this->e->add("XML Short comment replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Marker!s", $xml->output))
   {
    $this->e->add("XML Marker replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<unit!s", $xml->output))
   {
    $this->e->add("XML unit tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!-- comment!s", $xml->output))
   {
    $this->e->add("XML comment tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!shortcomment!s", $xml->output))
   {
    $this->e->add("XML short comment tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+#marker!s", $xml->output))
   {
    $this->e->add("XML marker tag should exist in the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with array
   if(!$xml->setReplace("test", $values))
   {
    $this->e->add("XML should replace an existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!Unit replacement!", $xml->output))
   {
    $this->e->add("XML Unit replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment replacement!", $xml->output))
   {
    $this->e->add("XML Comment replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment replacement!", $xml->output))
   {
    $this->e->add("XML Short comment replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker replacement!", $xml->output))
   {
    $this->e->add("XML Marker replacement should exist in the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setBefore("test", $values))
   {
    $this->e->add("XML should copy an existing open tag and replace the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!Unit replacement!", $xml->output))
   {
    $this->e->add("XML Unit replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment replacement!", $xml->output))
   {
    $this->e->add("XML Comment replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment replacement!", $xml->output))
   {
    $this->e->add("XML Short comment replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker replacement!", $xml->output))
   {
    $this->e->add("XML Marker replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<unit!s", $xml->output))
   {
    $this->e->add("XML unit tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!-- comment!s", $xml->output))
   {
    $this->e->add("XML comment tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!shortcomment!s", $xml->output))
   {
    $this->e->add("XML short comment tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+#marker!s", $xml->output))
   {
    $this->e->add("XML marker tag should exist in the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setAfter("test", $values))
   {
    $this->e->add("XML should copy an existing open tag and replace the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!Unit replacement!", $xml->output))
   {
    $this->e->add("XML Unit replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment replacement!", $xml->output))
   {
    $this->e->add("XML Comment replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment replacement!", $xml->output))
   {
    $this->e->add("XML Short comment replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker replacement!", $xml->output))
   {
    $this->e->add("XML Marker replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<unit!s", $xml->output))
   {
    $this->e->add("XML unit tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!-- comment!s", $xml->output))
   {
    $this->e->add("XML comment tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!shortcomment!s", $xml->output))
   {
    $this->e->add("XML short comment tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+#marker!s", $xml->output))
   {
    $this->e->add("XML marker tag should exist in the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetMultiArray()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   $values=Array(
    Array(
     "unit" => "Unit1 replacement",
     "comment" => "Comment1 replacement",
     "shortcomment" => "Short comment1 replacement",
     "marker" => "Marker1 replacement",
    ),
    Array(
     "unit" => "Unit2 replacement",
     "comment" => "Comment2 replacement",
     "shortcomment" => "Short comment2 replacement",
     "marker" => "Marker2 replacement",
    ),
   );


//replace inside of tag with array
   if(!$xml->setInside("test", $values))
   {
    $this->e->add("XML should replace inside of an existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Unit1!s", $xml->output))
   {
    $this->e->add("XML Unit1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Comment1!s", $xml->output))
   {
    $this->e->add("XML Comment1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Short comment1!s", $xml->output))
   {
    $this->e->add("XML Short comment1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Marker1!s", $xml->output))
   {
    $this->e->add("XML Marker1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Unit2!s", $xml->output))
   {
    $this->e->add("XML Unit2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Comment2!s", $xml->output))
   {
    $this->e->add("XML Comment2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Short comment2!s", $xml->output))
   {
    $this->e->add("XML Short comment2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Marker2!s", $xml->output))
   {
    $this->e->add("XML Marker2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace inside of tag with array leaving a copy of the original
   if(!$xml->setInsideCopy("test", $values))
   {
    $this->e->add("XML should copy an existing open tag and replace inside of the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Unit1!s", $xml->output))
   {
    $this->e->add("XML Unit1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Comment1!s", $xml->output))
   {
    $this->e->add("XML Comment1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Short comment1!s", $xml->output))
   {
    $this->e->add("XML Short comment1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Marker1!s", $xml->output))
   {
    $this->e->add("XML Marker1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Unit2!s", $xml->output))
   {
    $this->e->add("XML Unit2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Comment2!s", $xml->output))
   {
    $this->e->add("XML Comment2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Short comment2!s", $xml->output))
   {
    $this->e->add("XML Short comment2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Marker2!s", $xml->output))
   {
    $this->e->add("XML Marker2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<unit!s", $xml->output))
   {
    $this->e->add("XML unit tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!-- comment!s", $xml->output))
   {
    $this->e->add("XML comment tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!shortcomment!s", $xml->output))
   {
    $this->e->add("XML short comment tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+#marker!s", $xml->output))
   {
    $this->e->add("XML marker tag should exist in the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with array
   if(!$xml->setReplace("test", $values))
   {
    $this->e->add("XML should replace an existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!Unit1 replacement!", $xml->output))
   {
    $this->e->add("XML Unit1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment1 replacement!", $xml->output))
   {
    $this->e->add("XML Comment1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment1 replacement!", $xml->output))
   {
    $this->e->add("XML Short comment1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker1 replacement!", $xml->output))
   {
    $this->e->add("XML Marker1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Unit2 replacement!", $xml->output))
   {
    $this->e->add("XML Unit2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment2 replacement!", $xml->output))
   {
    $this->e->add("XML Comment2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment2 replacement!", $xml->output))
   {
    $this->e->add("XML Short comment2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker2 replacement!", $xml->output))
   {
    $this->e->add("XML Marker2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setBefore("test", $values))
   {
    $this->e->add("XML should copy an existing open tag and replace the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!Unit1 replacement!", $xml->output))
   {
    $this->e->add("XML Unit1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment1 replacement!", $xml->output))
   {
    $this->e->add("XML Comment1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment1 replacement!", $xml->output))
   {
    $this->e->add("XML Short comment1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker1 replacement!", $xml->output))
   {
    $this->e->add("XML Marker1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Unit2 replacement!", $xml->output))
   {
    $this->e->add("XML Unit2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment2 replacement!", $xml->output))
   {
    $this->e->add("XML Comment2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment2 replacement!", $xml->output))
   {
    $this->e->add("XML Short comment2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker2 replacement!", $xml->output))
   {
    $this->e->add("XML Marker2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<unit!s", $xml->output))
   {
    $this->e->add("XML unit tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!-- comment!s", $xml->output))
   {
    $this->e->add("XML comment tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!shortcomment!s", $xml->output))
   {
    $this->e->add("XML short comment tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+#marker!s", $xml->output))
   {
    $this->e->add("XML marker tag should exist in the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//replace tag with text leaving a copy of the original
   if(!$xml->setAfter("test", $values))
   {
    $this->e->add("XML should copy an existing open tag and replace the original open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!Unit1 replacement!", $xml->output))
   {
    $this->e->add("XML Unit1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment1 replacement!", $xml->output))
   {
    $this->e->add("XML Comment1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment1 replacement!", $xml->output))
   {
    $this->e->add("XML Short comment1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker1 replacement!", $xml->output))
   {
    $this->e->add("XML Marker1 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Unit2 replacement!", $xml->output))
   {
    $this->e->add("XML Unit2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment2 replacement!", $xml->output))
   {
    $this->e->add("XML Comment2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment2 replacement!", $xml->output))
   {
    $this->e->add("XML Short comment2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker2 replacement!", $xml->output))
   {
    $this->e->add("XML Marker2 replacement should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<unit!s", $xml->output))
   {
    $this->e->add("XML unit tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!-- comment!s", $xml->output))
   {
    $this->e->add("XML comment tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!shortcomment!s", $xml->output))
   {
    $this->e->add("XML short comment tag should exist in the open tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+#marker!s", $xml->output))
   {
    $this->e->add("XML marker tag should exist in the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddBeforeFailures()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//AddBefore non existing tag
   if($xml->addBefore("asd", "asd"))
   {
    $this->e->add("XML should not addBefore value for non existing tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddBefore non existing tag
   if($xml->addBefore(-1, "asd"))
   {
    $this->e->add("XML should not addBefore value for negative key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddBefore non existing tag
   if($xml->addBefore($xml->vmax, "asd"))
   {
    $this->e->add("XML should not addBefore value for non existing key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddBefore()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//add text at start
   if(!$xml->addBefore(false, "asd"))
   {
    $this->e->add("XML should addBefore value at start of xml");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!="asd".$this->testXML)
   {
    $this->e->add("XML should include the new value before the start");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before test tag
   if(!$xml->addBefore("test", "asd"))
   {
    $this->e->add("XML should addBefore value for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[1]["outer"]["start"]))
   {
    $this->e->add("XML should include the new value before the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before short tag
   if(!$xml->addBefore("test/unit", "asd"))
   {
    $this->e->add("XML should addBefore value for existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[3]["outer"]["start"]))
   {
    $this->e->add("XML should include the new value before the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before comment tag
   if(!$xml->addBefore("test/comment", "asd"))
   {
    $this->e->add("XML should addBefore value for existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[2]["outer"]["start"]))
   {
    $this->e->add("XML should include the new value before the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before shortcomment tag
   if(!$xml->addBefore("test/shortcomment", "asd"))
   {
    $this->e->add("XML should addBefore value for existing shortcomment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[5]["outer"]["start"]))
   {
    $this->e->add("XML should include the new value before the shortcomment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before marker tag
   if(!$xml->addBefore("test/marker", "asd"))
   {
    $this->e->add("XML should addBefore value for existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[4]["outer"]["start"]))
   {
    $this->e->add("XML should include the new value before the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddAfterFailures()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//AddAfter non existing tag
   if($xml->addAfter("asd", "asd"))
   {
    $this->e->add("XML should not addAfter value for non existing tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddAfter non existing tag
   if($xml->addAfter(-1, "asd"))
   {
    $this->e->add("XML should not addAfter value for negative key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddAfter non existing tag
   if($xml->addAfter($xml->vmax, "asd"))
   {
    $this->e->add("XML should not addAfter value for non existing key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddAfter()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//add text at end
   if(!$xml->addAfter(false, "asd"))
   {
    $this->e->add("XML should addAfter value at end of xml");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML."asd")
   {
    $this->e->add("XML should include the new value at the end");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before test tag
   if(!$xml->addAfter("test", "asd"))
   {
    $this->e->add("XML should addAfter value for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"]+$this->testTag[1]["outer"]["length"])."asd".substr($this->testXML, $this->testTag[1]["outer"]["start"]+$this->testTag[1]["outer"]["length"]))
   {
    $this->e->add("XML should include the new value before the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before short tag
   if(!$xml->addAfter("test/unit", "asd"))
   {
    $this->e->add("XML should addAfter value for existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"]+$this->testTag[3]["outer"]["length"])."asd".substr($this->testXML, $this->testTag[3]["outer"]["start"]+$this->testTag[3]["outer"]["length"]))
   {
    $this->e->add("XML should include the new value before the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before comment tag
   if(!$xml->addAfter("test/comment", "asd"))
   {
    $this->e->add("XML should addAfter value for existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"]+$this->testTag[2]["outer"]["length"])."asd".substr($this->testXML, $this->testTag[2]["outer"]["start"]+$this->testTag[2]["outer"]["length"]))
   {
    $this->e->add("XML should include the new value before the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before shortcomment tag
   if(!$xml->addAfter("test/shortcomment", "asd"))
   {
    $this->e->add("XML should addAfter value for existing shortcomment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"]+$this->testTag[5]["outer"]["length"])."asd".substr($this->testXML, $this->testTag[5]["outer"]["start"]+$this->testTag[5]["outer"]["length"]))
   {
    $this->e->add("XML should include the new value before the shortcomment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before marker tag
   if(!$xml->addAfter("test/marker", "asd"))
   {
    $this->e->add("XML should addAfter value for existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"]+$this->testTag[4]["outer"]["length"])."asd".substr($this->testXML, $this->testTag[4]["outer"]["start"]+$this->testTag[4]["outer"]["length"]))
   {
    $this->e->add("XML should include the new value before the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddInsideAtBeginFailures()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//AddInsideAtBegin non existing tag
   if($xml->addInsideAtBegin("asd", "asd"))
   {
    $this->e->add("XML should not addInsideAtBegin value for non existing tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtBegin non existing tag
   if($xml->addInsideAtBegin(-1, "asd"))
   {
    $this->e->add("XML should not addInsideAtBegin value for negative key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtBegin non existing tag
   if($xml->addInsideAtBegin($xml->vmax, "asd"))
   {
    $this->e->add("XML should not addInsideAtBegin value for non existing key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtBegin comment tag
   if($xml->addInsideAtBegin("test/comment", "asd"))
   {
    $this->e->add("XML should not addInsideAtBegin value for comment tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtBegin marker tag
   if($xml->addInsideAtBegin("test/marker", "asd"))
   {
    $this->e->add("XML should not addInsideAtBegin value for marker tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddInsideAtBegin()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//add text at start
   if(!$xml->addInsideAtBegin(false, "asd"))
   {
    $this->e->add("XML should addInsideAtBegin value at begin of xml");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!="asd".$this->testXML)
   {
    $this->e->add("XML should include the new value before the xml");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before test tag
   if(!$xml->addInsideAtBegin("test", "asd"))
   {
    $this->e->add("XML should addInsideAtBegin value for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["inner"]["start"])."asd".substr($this->testXML, $this->testTag[1]["inner"]["start"]))
   {
    $this->e->add("XML should include the new value before the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before unit tag
   if(!$xml->addInsideAtBegin("test/unit", "asd"))
   {
    $this->e->add("XML should addInsideAtBegin value for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=preg_replace("/<(unit[^>]+)/", "<\\1>asd</unit", $this->testXML))
   {
    $this->e->add("XML should include the new value before the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before shortcomment tag
   if(!$xml->addInsideAtBegin("test/shortcomment", "asd"))
   {
    $this->e->add("XML should addInsideAtBegin value for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=preg_replace("/<(!shortcomment[^>]*)/", "<\\1>asd<!/shortcomment", $this->testXML))
   {
    $this->e->add("XML should include the new value before the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddInsideAtEndFailures()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//AddInsideAtEnd non existing tag
   if($xml->addInsideAtEnd("asd", "asd"))
   {
    $this->e->add("XML should not addInsideAtEnd value for non existing tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtEnd non existing tag
   if($xml->addInsideAtEnd(-1, "asd"))
   {
    $this->e->add("XML should not addInsideAtEnd value for negative key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtEnd non existing tag
   if($xml->addInsideAtEnd($xml->vmax, "asd"))
   {
    $this->e->add("XML should not addInsideAtEnd value for non existing key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtEnd comment tag
   if($xml->addInsideAtEnd("test/comment", "asd"))
   {
    $this->e->add("XML should not addInsideAtEnd value for comment tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtEnd marker tag
   if($xml->addInsideAtEnd("test/marker", "asd"))
   {
    $this->e->add("XML should not addInsideAtEnd value for marker tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddInsideAtEnd()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//add text at end
   if(!$xml->addInsideAtEnd(false, "asd"))
   {
    $this->e->add("XML should addInsideAtEnd value at end of xml");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML."asd")
   {
    $this->e->add("XML should include the new value at end");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before test tag
   if(!$xml->addInsideAtEnd("test", "asd"))
   {
    $this->e->add("XML should addInsideAtEnd value for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["inner"]["start"]+$this->testTag[1]["inner"]["length"])."asd".substr($this->testXML, $this->testTag[1]["inner"]["start"]+$this->testTag[1]["inner"]["length"]))
   {
    $this->e->add("XML should include the new value before the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before unit tag
   if(!$xml->addInsideAtEnd("test/unit", "asd"))
   {
    $this->e->add("XML should addInsideAtEnd value for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=preg_replace("/<(unit[^>]+)/", "<\\1>asd</unit", $this->testXML))
   {
    $this->e->add("XML should include the new value before the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add text before shortcomment tag
   if(!$xml->addInsideAtEnd("test/shortcomment", "asd"))
   {
    $this->e->add("XML should addInsideAtEnd value for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->output!=preg_replace("/<(!shortcomment[^>]*)/", "<\\1>asd<!/shortcomment", $this->testXML))
   {
    $this->e->add("XML should include the new value before the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetAttributeFailures()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//SetAttribute root tag
   if($xml->setAttribute(false, "asd"))
   {
    $this->e->add("XML should not setAttribute value for root tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//SetAttribute non existing tag
   if($xml->setAttribute("asd", "asd"))
   {
    $this->e->add("XML should not setAttribute value for non existing tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//SetAttribute non existing tag
   if($xml->setAttribute(-1, "asd"))
   {
    $this->e->add("XML should not setAttribute value for negative key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//SetAttribute non existing tag
   if($xml->setAttribute($xml->vmax, "asd"))
   {
    $this->e->add("XML should not setAttribute value for non existing key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }

  function SetAttributeBlock()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//add attribute for open tag
   if(!$xml->setAttribute("test", "lala", "asd"))
   {
    $this->e->add("XML should set attribute for existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test ut=\"xml\" lala=\"asd\">!", $xml->output))
   {
    $this->e->add("New XML attribute should exist in the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//edit attribute for open tag
   if(!$xml->setAttribute("test", "lala", "asdasdads"))
   {
    $this->e->add("XML should set attribute for existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test ut=\"xml\" lala=\"asdasdads\">!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be changed in the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete attribute for open tag
   if(!$xml->setAttribute("test", "ut"))
   {
    $this->e->add("XML should set attribute for existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test lala=\"asdasdads\">!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be removed in the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete attribute for open tag
   if(!$xml->setAttribute("test", "lala"))
   {
    $this->e->add("XML should set attribute for existing open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test>!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be removed in the open tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetAttributeShort()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//add attribute for short tag
   if(!$xml->setAttribute("test/unit", "lala", "asd"))
   {
    $this->e->add("XML should set attribute for existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<unit test=\"asd\" lala=\"asd\">!", $xml->output))
   {
    $this->e->add("New XML attribute should exist in the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//edit attribute for short tag
   if(!$xml->setAttribute("test/unit", "lala", "asdasdads"))
   {
    $this->e->add("XML should set attribute for existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<unit test=\"asd\" lala=\"asdasdads\">!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be changed in the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete attribute for short tag
   if(!$xml->setAttribute("test/unit", "lala"))
   {
    $this->e->add("XML should set attribute for existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<unit test=\"asd\">!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be removed in the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete attribute for short tag
   if(!$xml->setAttribute("test/unit", "test"))
   {
    $this->e->add("XML should set attribute for existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<unit>!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be removed in the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//add attribute for short ?xml tag
   if(!$xml->setAttribute("?xml", "lala", "asd"))
   {
    $this->e->add("XML should set attribute for existing short ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\?xml version=\"1.0\" encoding=\"ISO-8859-1\" lala=\"asd\"\?>!", $xml->output))
   {
    $this->e->add("New XML attribute should exist in the short ?xml tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//edit attribute for short ?xml tag
   if(!$xml->setAttribute("?xml", "lala", "asdasdads"))
   {
    $this->e->add("XML should set attribute for existing short ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\?xml version=\"1.0\" encoding=\"ISO-8859-1\" lala=\"asdasdads\"\?>!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be changed in the short ?xml tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete attribute for short ?xml tag
   if(!$xml->setAttribute("?xml", "lala"))
   {
    $this->e->add("XML should set attribute for existing short ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\?xml version=\"1.0\" encoding=\"ISO-8859-1\"\?>!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be removed in the short ?xml tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete attribute for short ?xml tag
   if(!$xml->setAttribute("?xml", "encoding"))
   {
    $this->e->add("XML should set attribute for existing short ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\?xml version=\"1.0\"\?>!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be removed in the short ?xml tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete attribute for short ?xml tag
   if(!$xml->setAttribute("?xml", "version"))
   {
    $this->e->add("XML should set attribute for existing short ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\?xml\?>!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be removed in the short ?xml tag");
    return UT_FAILED;
   }

   /*
    * Skip: <?xml?> name is interpreted as ?xml?
    *
   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;
   */

   return UT_OK;
  }


  function SetAttributeComment()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//add attribute for comment tag
   if(!$xml->setAttribute("test/comment", "lala", "asd"))
   {
    $this->e->add("XML should set attribute for existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\!-- comment lala=\"asd\" -->!", $xml->output))
   {
    $this->e->add("New XML attribute should exist in the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//edit attribute for comment tag
   if(!$xml->setAttribute("test/comment", "lala", "asdasdads"))
   {
    $this->e->add("XML should set attribute for existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\!-- comment lala=\"asdasdads\" -->!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be changed in the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete attribute for comment tag
   if(!$xml->setAttribute("test/comment", "lala"))
   {
    $this->e->add("XML should set attribute for existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\!-- comment -->!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be removed in the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetAttributeShortComment()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//add attribute for short comment tag
   if(!$xml->setAttribute("test/shortcomment", "lala", "asd"))
   {
    $this->e->add("XML should set attribute for existing short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\!shortcomment lala=\"asd\">!", $xml->output))
   {
    $this->e->add("New XML attribute should exist in the short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//edit attribute for short comment tag
   if(!$xml->setAttribute("test/shortcomment", "lala", "asdasdads"))
   {
    $this->e->add("XML should set attribute for existing short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\!shortcomment lala=\"asdasdads\">!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be changed in the short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete attribute for short comment tag
   if(!$xml->setAttribute("test/shortcomment", "lala"))
   {
    $this->e->add("XML should set attribute for existing short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\!shortcomment>!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be removed in the short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetAttributeMarker()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//add attribute for marker tag
   if(!$xml->setAttribute("test/marker", "lala", "asd"))
   {
    $this->e->add("XML should set attribute for existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!#marker lala=\"asd\"#!", $xml->output))
   {
    $this->e->add("New XML attribute should exist in the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//edit attribute for marker tag
   if(!$xml->setAttribute("test/marker", "lala", "asdasdads"))
   {
    $this->e->add("XML should set attribute for existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!#marker lala=\"asdasdads\"#!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be changed in the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete attribute for marker tag
   if(!$xml->setAttribute("test/marker", "lala"))
   {
    $this->e->add("XML should set attribute for existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!#marker#!", $xml->output))
   {
    $this->e->add("Existing XML attribute should be removed in the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function RenameFailures()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//SetAttribute root tag
   if($xml->rename(false, "lala"))
   {
    $this->e->add("XML should not rename value for root tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//SetAttribute non existing tag
   if($xml->rename("asd", "lala"))
   {
    $this->e->add("XML should not rename value for non existing tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//SetAttribute non existing tag
   if($xml->rename(-1, "lala"))
   {
    $this->e->add("XML should not rename value for negative key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//SetAttribute non existing tag
   if($xml->rename($xml->vmax, "lala"))
   {
    $this->e->add("XML should not rename value for non existing key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function Rename()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//rename first
   if(!$xml->rename("?xml", "?lala"))
   {
    $this->e->add("XML should rename existing first tag");
    return UT_FAILED;
   }

   if($xml->output!=str_replace("<?xml", "<?lala", $this->testXML))
   {
    $this->e->add("XML tag ?xml should be renamed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//rename open
   if(!$xml->rename("test", "lala"))
   {
    $this->e->add("XML should rename existing open tag");
    return UT_FAILED;
   }

   if($xml->output!=str_replace("<test", "<lala", str_replace("</test", "</lala", $this->testXML)))
   {
    $this->e->add("XML tag test should be renamed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//rename short
   if(!$xml->rename("test/unit", "lala"))
   {
    $this->e->add("XML should rename existing open tag");
    return UT_FAILED;
   }

   if($xml->output!=str_replace("<unit", "<lala", $this->testXML))
   {
    $this->e->add("XML tag unit should be renamed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//rename comment
   if(!$xml->rename("test/comment", "lala"))
   {
    $this->e->add("XML should rename existing comment tag");
    return UT_FAILED;
   }

   if($xml->output!=str_replace("<!-- comment", "<!-- lala", $this->testXML))
   {
    $this->e->add("XML tag comment should be renamed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//rename short comment
   if(!$xml->rename("test/shortcomment", "lala"))
   {
    $this->e->add("XML should rename existing short comment tag");
    return UT_FAILED;
   }

   if($xml->output!=str_replace("<!shortcomment", "<!lala", $this->testXML))
   {
    $this->e->add("XML tag should be renamed");
    return UT_FAILED;
   }


   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//rename marker
   if(!$xml->rename("test/marker", "lala"))
   {
    $this->e->add("XML should rename existing marker tag");
    return UT_FAILED;
   }

   if($xml->output!=str_replace("#marker#", "#lala#", $this->testXML))
   {
    $this->e->add("XML tag should be renamed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function DeleteFailures()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//delete root tag
   if($xml->delete(false))
   {
    $this->e->add("XML should not delete root tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//delete non existing tag
   if($xml->delete("asd"))
   {
    $this->e->add("XML should not delete value for non existing tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//delete non existing tag
   if($xml->delete(-1))
   {
    $this->e->add("XML should not delete value for negative key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//delete non existing tag
   if($xml->delete($xml->vmax, "asd"))
   {
    $this->e->add("XML should not delete value for non existing key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function Delete()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//delete first
   if(!$xml->delete("?xml"))
   {
    $this->e->add("XML should delete existing first tag");
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, $this->testTag[0]["outer"]["start"]+$this->testTag[0]["outer"]["length"]))
   {
    $this->e->add("XML first tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//delete open
   if(!$xml->delete("test"))
   {
    $this->e->add("XML should delete existing open tag");
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"]).substr($this->testXML, $this->testTag[1]["outer"]["start"]+$this->testTag[1]["outer"]["length"]))
   {
    $this->e->add("XML open tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//delete short
   if(!$xml->delete("test/unit"))
   {
    $this->e->add("XML should delete existing short tag");
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"]).substr($this->testXML, $this->testTag[3]["outer"]["start"]+$this->testTag[3]["outer"]["length"]))
   {
    $this->e->add("XML short tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//delete comment
   if(!$xml->delete("test/comment"))
   {
    $this->e->add("XML should delete existing comment tag");
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"]).substr($this->testXML, $this->testTag[2]["outer"]["start"]+$this->testTag[2]["outer"]["length"]))
   {
    $this->e->add("XML comment tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//delete short comment
   if(!$xml->delete("test/shortcomment"))
   {
    $this->e->add("XML should delete existing short comment tag");
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"]).substr($this->testXML, $this->testTag[5]["outer"]["start"]+$this->testTag[5]["outer"]["length"]))
   {
    $this->e->add("XML short comment tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//delete marker
   if(!$xml->delete("test/marker"))
   {
    $this->e->add("XML should delete existing marker tag");
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"]).substr($this->testXML, $this->testTag[4]["outer"]["start"]+$this->testTag[4]["outer"]["length"]))
   {
    $this->e->add("XML marker tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function DeleteMultiple()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//delete marker
   if(!$xml->delete("test/marker"))
   {
    $this->e->add("XML should delete existing marker tag");
    return UT_FAILED;
   }

   if(preg_match("!#marker#!", $xml->output))
   {
    $this->e->add("XML marker tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete short comment
   if(!$xml->delete("test/shortcomment"))
   {
    $this->e->add("XML should delete existing short comment tag");
    return UT_FAILED;
   }

   if(preg_match("!<\!shortcomment>!", $xml->output))
   {
    $this->e->add("XML short comment tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete comment
   if(!$xml->delete("test/comment"))
   {
    $this->e->add("XML should delete existing comment tag");
    return UT_FAILED;
   }

   if(preg_match("!<\!-- comment -->!", $xml->output))
   {
    $this->e->add("XML comment tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   $xml->reparse();

//delete short
   if(!$xml->delete("test/unit"))
   {
    $this->e->add("XML should delete existing short tag");
    return UT_FAILED;
   }

   if(preg_match("!<unit test=\"asd\">!", $xml->output))
   {
    $this->e->add("XML short tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete open
   if(!$xml->delete("test"))
   {
    $this->e->add("XML should delete existing open tag");
    return UT_FAILED;
   }

   if(preg_match("!<test ut=\"xml\">!", $xml->output))
   {
    $this->e->add("XML open tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


//delete first
   if(!$xml->delete("?xml"))
   {
    $this->e->add("XML should delete existing first tag");
    return UT_FAILED;
   }

   if(preg_match("!\?xml!", $xml->output))
   {
    $this->e->add("XML first tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   if(trim($xml->output))
   {
    $this->e->add("XML should be empty");
    return UT_FAILED;
   }


   return UT_OK;
  }


  function BlankFailures()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//blank root
   if($xml->blank(false))
   {
    $this->e->add("XML should not blank root tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//blank non existing tag
   if($xml->blank("asd"))
   {
    $this->e->add("XML should not blank value for non existing tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//blank non existing tag
   if($xml->blank(-1))
   {
    $this->e->add("XML should not blank value for negative key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//blank non existing tag
   if($xml->blank($xml->vmax))
   {
    $this->e->add("XML should not blank value for non existing key number");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//blank short tag
   if($xml->blank("?xml"))
   {
    $this->e->add("XML should not blank value for a short tag");
    return UT_FAILED;
   }

   if($xml->output!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->data, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function Blank()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//blank open
   if(!$xml->blank("test"))
   {
    $this->e->add("XML should blank existing open tag");
    return UT_FAILED;
   }

   if($xml->output!=substr($this->testXML, 0, $this->testTag[1]["inner"]["start"]).substr($this->testXML, $this->testTag[1]["inner"]["start"]+$this->testTag[1]["inner"]["length"]))
   {
    $this->e->add("XML open tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new Format_XML(false, true);
   $testXML->parse($xml->output);

   if($this->_validateTags($xml->data, $testXML->data)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function GetPathAndKey()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   if($xml->getPathAndKey("asd"))
   {
    $this->e->add("XML should not get path and key for non existing tag");
    return UT_FAILED;
   }

   if(!($key=$xml->getPathAndKey("test/unit")))
   {
    $this->e->add("XML should get path and key for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key[0]!="test/unit")
   {
    $this->e->add("XML path incorrect");
    return UT_FAILED;
   }

   if($key[1]!=3)
   {
    $this->e->add("XML key incorrect");
    return UT_FAILED;
   }

   if(!($key=$xml->getPathAndKey(3)))
   {
    $this->e->add("XML should get path and key for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key[0]!="test[0]/unit[0]")
   {
    $this->e->add("XML path incorrect");
    return UT_FAILED;
   }

   if($key[1]!=3)
   {
    $this->e->add("XML key incorrect");
    return UT_FAILED;
   }

   $xml->next(); //test
   $xml->next(); //test->comment
   $xml->next(); //test->unit

   //fail
   if($xml->getPathAndKey(false))
   {
    $this->e->add("XML shouldn't get path and key for false");
    return UT_FAILED;
   }

   if($xml->getPathAndKey(-1))
   {
    $this->e->add("XML shouldn't get path and key for non-existing tag");
    return UT_FAILED;
   }

   if($xml->getPathAndKey(234523))
   {
    $this->e->add("XML shouldn't get path and key for non-existing tag");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetKey()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   if($xml->getKey("asd"))
   {
    $this->e->add("XML should not get key for non existing tag");
    return UT_FAILED;
   }

   if(($key=$xml->getKey("test/unit"))===false)
   {
    $this->e->add("XML should get key for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3)
   {
    $this->e->add("XML key incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetName()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   if($xml->getName("asd"))
   {
    $this->e->add("XML should not get name for non existing tag");
    return UT_FAILED;
   }

   if(!($path=$xml->getName(3)))
   {
    $this->e->add("XML should get name for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($path!="unit")
   {
    $this->e->add("XML name incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function GetPath()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   if($xml->getPath("asd"))
   {
    $this->e->add("XML should not get path for non existing tag");
    return UT_FAILED;
   }

   if(!($path=$xml->getPath(3)))
   {
    $this->e->add("XML should get path for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($path!="test[0]/unit[0]")
   {
    $this->e->add("XML path incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetInner()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   //non-existing tag
   if($xml->getInner("asd"))
   {
    $this->e->add("XML should not get inner for non existing tag");
    return UT_FAILED;
   }


   //short tag
   if($xml->getInner("test/unit"))
   {
    $this->e->add("XML should not get inner for short tag");
    return UT_FAILED;
   }


   //open tag
   if(($inner=$xml->getInner("test"))===false)
   {
    $this->e->add("XML should get inner for open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<unit test=\"asd\">!", $inner))
   {
    $this->e->add("XML inner should contain unit tag");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetOuter()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   //non-existing tag
   if($xml->getOuter("asd"))
   {
    $this->e->add("XML should not get outer for non existing tag");
    return UT_FAILED;
   }


   //open tag
   if(($outer=$xml->getOuter("test"))===false)
   {
    $this->e->add("XML should get outer for open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<unit test=\"asd\">!", $outer))
   {
    $this->e->add("XML outer should contain unit tag");
    return UT_FAILED;
   }


   //short tag
   if(($outer=$xml->getOuter("test/unit"))===false)
   {
    $this->e->add("XML should get outer for short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!^<unit test=\"asd\">$!", $outer))
   {
    $this->e->add("XML outer should contain unit tag");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetAttributes()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   if($xml->getAttributes("asd"))
   {
    $this->e->add("XML should not get attributes for non existing tag");
    return UT_FAILED;
   }

   if(($attributes=$xml->getAttributes("test"))===false)
   {
    $this->e->add("XML should get attributes for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!isset($attributes["ut"])||$attributes["ut"]!="xml")
   {
    $this->e->add("XML attributes incorrect");
    return UT_FAILED;
   }

   //single quotes
   $xml->parse(str_replace("\"", "'", $this->testXML));

   if($xml->getAttributes("asd"))
   {
    $this->e->add("XML should not get attributes for non existing tag");
    return UT_FAILED;
   }

   if(($attributes=$xml->getAttributes("test"))===false)
   {
    $this->e->add("XML should get attributes for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!isset($attributes["ut"])||$attributes["ut"]!="xml")
   {
    $this->e->add("XML attributes incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetAttribute()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   if($xml->getAttribute("asd", "asd"))
   {
    $this->e->add("XML should not get attribute value for non existing tag");
    return UT_FAILED;
   }

   if($xml->getAttribute("test", "asd"))
   {
    $this->e->add("XML should not get attribute value for non existing attribute");
    return UT_FAILED;
   }

   if(($attribute=$xml->getAttribute("test", "ut"))===false)
   {
    $this->e->add("XML should get value for existing attribute");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($attribute!="xml")
   {
    $this->e->add("XML attribute incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetChildren()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   if($xml->getChildren("asd"))
   {
    $this->e->add("XML should not get children for non existing tag");
    return UT_FAILED;
   }

   if(!($childs=$xml->getChildren("test")))
   {
    $this->e->add("XML should get children for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($childs[0]!=2)
   {
    $this->e->add("XML children incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetOuterXML()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   //non-existing tag
   if($xml->getOuterXML("asd"))
   {
    $this->e->add("XML should not get outer XML object for non existing tag");
    return UT_FAILED;
   }


   //open tag
   if(!($xml2=$xml->getOuterXML("test")))
   {
    $this->e->add("XML should get outer XML object for open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!is_object($xml2)||get_class($xml2)!="Format_XML")
   {
    $this->e->add("XML sub XML object incorrect");
    return UT_FAILED;
   }

   //check structure
   $xml3=new Format_XML(false, true);
   if(!$xml3->parse($xml2->output()))
   {
    $this->e->add("XML should parse inner XML");
    $this->e->add($xml2->e);
    return UT_FAILED;
   }

   if($this->_validateTags($xml2->data, $xml3->data)==UT_FAILED)
    return UT_FAILED;


   //short tag
   if(!($xml2=$xml->getOuterXML("test/unit")))
   {
    $this->e->add("XML should get outer XML object for short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!is_object($xml2)||get_class($xml2)!="Format_XML")
   {
    $this->e->add("XML sub XML object incorrect");
    return UT_FAILED;
   }

   //check structure
   $xml3=new Format_XML(false, true);
   if(!$xml3->parse($xml2->output()))
   {
    $this->e->add("XML should parse inner XML");
    $this->e->add($xml2->e);
    return UT_FAILED;
   }

   if($this->_validateTags($xml2->data, $xml3->data)==UT_FAILED)
    return UT_FAILED;

   return UT_OK;
  }

  function GetInnerXML()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   //non-existing
   if($xml->getInnerXML("asd"))
   {
    $this->e->add("XML should not get inner XML object for non existing tag");
    return UT_FAILED;
   }


   //short tag
   if($xml->getInnerXML("test/unit"))
   {
    $this->e->add("XML should not get inner XML object for short tag");
    return UT_FAILED;
   }


   //open tag
   if(!($xml2=$xml->getInnerXML("test")))
   {
    $this->e->add("XML should get inner XML object for open tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!is_object($xml2)||get_class($xml2)!="Format_XML")
   {
    $this->e->add("XML sub XML object incorrect");
    return UT_FAILED;
   }

   //check structure
   $xml3=new Format_XML(false, true);
   if(!$xml3->parse($xml2->output()))
   {
    $this->e->add("XML should parse inner XML");
    $this->e->add($xml2->e);
    return UT_FAILED;
   }

   if($this->_validateTags($xml2->data, $xml3->data)==UT_FAILED)
    return UT_FAILED;

   return UT_OK;
  }

  function GetType()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   if($xml->getType("asd"))
   {
    $this->e->add("XML should not get type for non existing tag");
    return UT_FAILED;
   }

   if(($type=$xml->getType("test"))===false)
   {
    $this->e->add("XML should get type for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($type!="open")
   {
    $this->e->add("XML type incorrect");
    return UT_FAILED;
   }

   if(($type=$xml->getType("test/unit"))===false)
   {
    $this->e->add("XML should get type for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($type!="short")
   {
    $this->e->add("XML type incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetPosition()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//?xml
   if($xml->getPosition("?xml", "outer", "start")!=0)
   {
    $this->e->add("XML should get outer start position for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("?xml", "outer", "length")!=43)
   {
    $this->e->add("XML should get outer length position for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("?xml", "outer", "end")!=0+43)
   {
    $this->e->add("XML should get outer end position for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("?xml", "outer")!=Array("start" => 0, "length" => 43, "end" => 0+43))
   {
    $this->e->add("XML should get outer positions for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("?xml")!=Array("outer" => Array("start" => 0, "length" => 43, "end" => 0+43)))
   {
    $this->e->add("XML should get positions for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

//test
   if($xml->getPosition("test", "outer", "start")!=46)
   {
    $this->e->add("XML should get outer start position for test tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test", "outer", "length")!=96)
   {
    $this->e->add("XML should get outer length position for test tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test", "outer", "end")!=46+96)
   {
    $this->e->add("XML should get outer end position for test tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test", "outer")!=Array("start" => 46, "length" => 96, "end" => 46+96))
   {
    $this->e->add("XML should get outer positions for test tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test", "inner", "start")!=61)
   {
    $this->e->add("XML should get inner start position for test tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test", "inner", "length")!=74)
   {
    $this->e->add("XML should get inner length position for test tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test", "inner", "end")!=61+74)
   {
    $this->e->add("XML should get inner end position for test tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test", "inner")!=Array("start" => 61, "length" => 74, "end" => 61+74))
   {
    $this->e->add("XML should get inner positions for test tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test")!=Array(
    "outer" => Array("start" => 46, "length" => 96, "end" => 46+96),
    "inner" => Array("start" => 61, "length" => 74, "end" => 61+74)
   ))
   {
    $this->e->add("XML should get positions for test tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

//comment
   if($xml->getPosition("test/comment", "outer", "start")!=65)
   {
    $this->e->add("XML should get outer start position for test->comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/comment", "outer", "length")!=16)
   {
    $this->e->add("XML should get outer length position for test->comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/comment", "outer", "end")!=65+16)
   {
    $this->e->add("XML should get outer end position for test->comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/comment", "outer")!=Array("start" => 65, "length" => 16, "end" => 65+16))
   {
    $this->e->add("XML should get outer positions for test->comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/comment")!=Array("outer" => Array("start" => 65, "length" => 16, "end" => 65+16)))
   {
    $this->e->add("XML should get positions for test->comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

//unit
   if($xml->getPosition("test/unit", "outer", "start")!=85)
   {
    $this->e->add("XML should get outer start position for test->unit tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/unit", "outer", "length")!=17)
   {
    $this->e->add("XML should get outer length position for test->unit tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/unit", "outer", "end")!=85+17)
   {
    $this->e->add("XML should get outer end position for test->unit tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/unit", "outer")!=Array("start" => 85, "length" => 17, "end" => 85+17))
   {
    $this->e->add("XML should get outer positions for test->unit tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/unit")!=Array("outer" => Array("start" => 85, "length" => 17, "end" => 85+17)))
   {
    $this->e->add("XML should get positions for test->unit tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

//marker
   if($xml->getPosition("test/marker", "outer", "start")!=106)
   {
    $this->e->add("XML should get outer start position for test->marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/marker", "outer", "length")!=8)
   {
    $this->e->add("XML should get outer length position for test->marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/marker", "outer", "end")!=106+8)
   {
    $this->e->add("XML should get outer end position for test->marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/marker", "outer")!=Array("start" => 106, "length" => 8, "end" => 106+8))
   {
    $this->e->add("XML should get outer positions for test->marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/marker")!=Array("outer" => Array("start" => 106, "length" => 8, "end" => 106+8)))
   {
    $this->e->add("XML should get positions for test->marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

//shortcomment
   if($xml->getPosition("test/shortcomment", "outer", "start")!=118)
   {
    $this->e->add("XML should get outer start position for test->shortcomment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/shortcomment", "outer", "length")!=15)
   {
    $this->e->add("XML should get outer length position for test->shortcomment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/shortcomment", "outer", "end")!=118+15)
   {
    $this->e->add("XML should get outer end position for test->shortcomment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/shortcomment", "outer")!=Array("start" => 118, "length" => 15, "end" => 118+15))
   {
    $this->e->add("XML should get outer positions for test->shortcomment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("test/shortcomment")!=Array("outer" => Array("start" => 118, "length" => 15, "end" => 118+15)))
   {
    $this->e->add("XML should get positions for test->shortcomment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }


   //non-existing tag
   if($xml->getPosition("non-existing", "outer", "start")!==false)
   {
    $this->e->add("XML shouldn't get outer start position for non-existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("non-existing", "outer", "length")!==false)
   {
    $this->e->add("XML shouldn't get outer length position for non-existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("non-existing", "outer", "end")!==false)
   {
    $this->e->add("XML shouldn't get outer end position for non-existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("non-existing", "outer")!==false)
   {
    $this->e->add("XML shouldn't get outer positions for non-existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("non-existing", "inner", "start")!==false)
   {
    $this->e->add("XML shouldn't get inner start position for non-existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("non-existing", "inner", "length")!==false)
   {
    $this->e->add("XML shouldn't get inner length position for non-existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("non-existing", "inner", "end")!==false)
   {
    $this->e->add("XML shouldn't get inner end position for non-existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("non-existing", "inner")!==false)
   {
    $this->e->add("XML shouldn't get inner positions for non-existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   //no outer position (close tag)
   if($xml->getPosition(6, "outer", "start")!==false)
   {
    $this->e->add("XML shouldn't get outer start position for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition(6, "outer", "length")!==false)
   {
    $this->e->add("XML shouldn't get outer length position for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition(6, "outer", "end")!==false)
   {
    $this->e->add("XML shouldn't get outer end position for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition(6, "outer")!==false)
   {
    $this->e->add("XML shouldn't get outer positions for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   //no inner position
   if($xml->getPosition("?xml", "inner", "start")!==false)
   {
    $this->e->add("XML shouldn't get inner start position for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("?xml", "inner", "length")!==false)
   {
    $this->e->add("XML shouldn't get inner length position for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("?xml", "inner", "end")!==false)
   {
    $this->e->add("XML shouldn't get inner end position for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("?xml", "inner")!==false)
   {
    $this->e->add("XML shouldn't get inner positions for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   //no fake positions
   if($xml->getPosition("?xml", "fake", "start")!==false)
   {
    $this->e->add("XML shouldn't get fake start position for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("?xml", "fake", "length")!==false)
   {
    $this->e->add("XML shouldn't get fake length position for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("?xml", "fake", "end")!==false)
   {
    $this->e->add("XML shouldn't get fake end position for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
   if($xml->getPosition("?xml", "fake")!==false)
   {
    $this->e->add("XML shouldn't get fake positions for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   //no outer fake position
   if($xml->getPosition("?xml", "outer", "fake")!==false)
   {
    $this->e->add("XML shouldn't get outer fake position for ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Current()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   if($xml->current("asd"))
   {
    $this->e->add("XML should not current non existing tag");
    return UT_FAILED;
   }

   if(($key=$xml->current("test"))===false)
   {
    $this->e->add("XML should current existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=1||$xml->vc!=2)
   {
    $this->e->add("XML current incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Next()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//?test
   if(($key=$xml->next())===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=0||$xml->vc!=1)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="?xml")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test
   if(($key=$xml->next())===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=1||$xml->vc!=2)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->comment
   if(($key=$xml->next())===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=2||$xml->vc!=3)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="comment")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->unit
   if(($key=$xml->next())===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->vc!=4)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="unit")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->marker
   if(($key=$xml->next())===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=4||$xml->vc!=5)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="marker")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->shortcomment
   if(($key=$xml->next())===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=5||$xml->vc!=6)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="shortcomment")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

   if($xml->next()!==false)
   {
    $this->e->add("XML should not goto next non existing tag");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function NextFilters()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//comments, markers, close tags
//
//?ml
   if(($key=$xml->next(true, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=0||$xml->vc!=1)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="?xml")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test
   if(($key=$xml->next(true, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=1||$xml->vc!=2)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->comment
   if(($key=$xml->next(true, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=2||$xml->vc!=3)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="comment")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->unit
   if(($key=$xml->next(true, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->vc!=4)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="unit")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->marker
   if(($key=$xml->next(true, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=4||$xml->vc!=5)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="marker")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->shortcomment
   if(($key=$xml->next(true, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=5||$xml->vc!=6)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="shortcomment")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }


//test (close)
   if(($key=$xml->next(true, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=6||$xml->vc!=7)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

   if($xml->next(true, true, true)!==false)
   {
    $this->e->add("XML should not goto next non existing tag");
    return UT_FAILED;
   }


//markers, close tags
//
//?xml
   if(($key=$xml->next(false, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=0||$xml->vc!=1)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="?xml")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test
   if(($key=$xml->next(false, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=1||$xml->vc!=2)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->unit
   if(($key=$xml->next(false, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->vc!=4)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="unit")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->marker
   if(($key=$xml->next(false, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=4||$xml->vc!=5)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="marker")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test (close)
   if(($key=$xml->next(false, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=6||$xml->vc!=7)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

   if($xml->next(false, true, true)!==false)
   {
    $this->e->add("XML should not goto next non existing tag");
    return UT_FAILED;
   }

//close tags
//
//?xml
   if(($key=$xml->next(false, false, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=0||$xml->vc!=1)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="?xml")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test
   if(($key=$xml->next(false, false, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=1||$xml->vc!=2)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->unit
   if(($key=$xml->next(false, false, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->vc!=4)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="unit")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test (close)
   if(($key=$xml->next(false, false, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=6||$xml->vc!=7)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

   if($xml->next(false, false, true)!==false)
   {
    $this->e->add("XML should not goto next non existing tag");
    return UT_FAILED;
   }

//normal tags
//
//?xml
   if(($key=$xml->next(false, false, false))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=0||$xml->vc!=1)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="?xml")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test
   if(($key=$xml->next(false, false, false))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=1||$xml->vc!=2)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->unit
   if(($key=$xml->next(false, false, false))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->vc!=4)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="unit")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

   if($xml->next(false, false, false)!==false)
   {
    $this->e->add("XML should not goto next non existing tag");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Previous()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

//shortcomment
   if(($key=$xml->previous())===false)
   {
    $this->e->add("XML should goto previous existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=5||$xml->vc!=2)
   {
    $this->e->add("XML goto previous incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="shortcomment")
   {
    $this->e->add("XML goto previous incorrect key");
    return UT_FAILED;
   }

   if($xml->v["name"]!="shortcomment")
   {
    $this->e->add("XML goto previous incorrect value");
    return UT_FAILED;
   }

//test->marker
   if(($key=$xml->previous())===false)
   {
    $this->e->add("XML should goto previous existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=4||$xml->vc!=3)
   {
    $this->e->add("XML goto previous incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="marker")
   {
    $this->e->add("XML goto previous incorrect key");
    return UT_FAILED;
   }

   if($xml->v["name"]!="marker")
   {
    $this->e->add("XML goto previous incorrect value");
    return UT_FAILED;
   }

//test->unit
   if(($key=$xml->previous())===false)
   {
    $this->e->add("XML should goto previous existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->vc!=4)
   {
    $this->e->add("XML goto previous incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="unit")
   {
    $this->e->add("XML goto previous incorrect key");
    return UT_FAILED;
   }

   if($xml->v["name"]!="unit")
   {
    $this->e->add("XML goto previous incorrect value");
    return UT_FAILED;
   }

//test->comment
   if(($key=$xml->previous())===false)
   {
    $this->e->add("XML should goto previous existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=2||$xml->vc!=5)
   {
    $this->e->add("XML goto previous incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="comment")
   {
    $this->e->add("XML goto previous incorrect key");
    return UT_FAILED;
   }

   if($xml->v["name"]!="comment")
   {
    $this->e->add("XML goto previous incorrect value");
    return UT_FAILED;
   }

//test
   if(($key=$xml->previous())===false)
   {
    $this->e->add("XML should goto previous existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=1||$xml->vc!=6)
   {
    $this->e->add("XML goto previous incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto previous incorrect key");
    return UT_FAILED;
   }

   if($xml->v["name"]!="test")
   {
    $this->e->add("XML goto previous incorrect value");
    return UT_FAILED;
   }

//?xml
   if(($key=$xml->previous())===false)
   {
    $this->e->add("XML should goto previous existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=0||$xml->vc!=7)
   {
    $this->e->add("XML goto previous incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="?xml")
   {
    $this->e->add("XML goto previous incorrect key");
    return UT_FAILED;
   }

   if($xml->v["name"]!="?xml")
   {
    $this->e->add("XML goto previous incorrect value");
    return UT_FAILED;
   }

   if($xml->previous()!==false)
   {
    $this->e->add("XML should not goto previous non existing tag");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Search()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   if(($key=$xml->search("unit"))===false)
   {
    $this->e->add("XML search should find existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->vc!=4)
   {
    $this->e->add("XML search incorrect");
    return UT_FAILED;
   }


   $xml->reset();

   if(($key=$xml->search("shortcomment"))===false)
   {
    $this->e->add("XML search should find existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=5||$xml->vc!=6)
   {
    $this->e->add("XML search incorrect");
    return UT_FAILED;
   }

   if($xml->search("shortcomment")!==false)
   {
    $this->e->add("XML search should not find unique tag twice");
    return UT_FAILED;
   }


   $xml->reset();

   if($xml->search("asd")!==false)
   {
    $this->e->add("XML should not search non existing tag");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SearchFilters()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   if(($key=$xml->search("unit", false, false, false))===false)
   {
    $this->e->add("XML search should find existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->vc!=4)
   {
    $this->e->add("XML search incorrect");
    return UT_FAILED;
   }


   $xml->reset();

   if(($key=$xml->search("test", false, false, true))===false)
   {
    $this->e->add("XML search should find existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=1||$xml->vc!=2)
   {
    $this->e->add("XML search incorrect");
    return UT_FAILED;
   }

   if(($key=$xml->search("test", false, false, true))===false)
   {
    $this->e->add("XML search should find existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=6||$xml->vc!=7)
   {
    $this->e->add("XML search incorrect");
    return UT_FAILED;
   }


   $xml->reset();

   if($xml->search("comment", false, false, false)!==false)
   {
    $this->e->add("XML search should not find comment tag");
    return UT_FAILED;
   }


   $xml->reset();

   if($xml->search("shortcomment", false, false, false)!==false)
   {
    $this->e->add("XML search should not find comment tag");
    return UT_FAILED;
   }


   $xml->reset();

   if($xml->search("marker", false, false, false)!==false)
   {
    $this->e->add("XML search should not find marker");
    return UT_FAILED;
   }


   return UT_OK;
  }

  function Reset()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   $xml->next();
   $xml->reset();

   if($xml->vc!==0)
   {
    $this->e->add("XML reset incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Output()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   if($xml->output(false)!=$this->testXML)
   {
    $this->e->add("XML output incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Display()
  {
   $xml=new Format_XML(false, true);
   $xml->parse($this->testXML);

   $display=$xml->display();
   $display=preg_replace("!<[^>]+>!", "", $display);
   $display=html_entity_decode($display);
   $display=preg_replace("!>[^<]*<!", "><", $display);
   $display=trim($display);

   $testXML=preg_replace("!>[^<]*<!", "><", $this->testXML);

   if($display!=$testXML)
   {
    $this->e->add("XML display incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function BugShortTag()
  {
   $xml=new Format_XML(false, true);

   if(!$xml->parse("<test1/><!test2/>"))
   {
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->getName(0)!="test1")
   {
    $this->e->add("First tag should be: test1");
    return UT_FAILED;
   }

   if($xml->getName(1)!="test2")
   {
    $this->e->add("Second tag should be: test2");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

?>
